//  Translate a raw correlator dump into a MS
//
//
//  $Id: j2ms2.cc,v 1.19.2.2 2018/08/21 12:31:13 verkout Exp $
//
//  $Log: j2ms2.cc,v $
//  Revision 1.19.2.2  2018/08/21 12:31:13  verkout
//  HV: * j2ms2 cleanup + drag kicking/screaming into 21st century
//          - help text clarified + updated to reflect current reality
//          - lis file parsing now supports "-- eo:... fo:..." on-the-fly
//            changing of experiment- and/or filler options
//          - GhostbusterFilter is now OFF by default; with SFXC it basically
//            didn't do anything at a lot of computational cost (also these days
//            it's not necessary; it was for MarkIV correlator equipment
//            peculiarities)
//
//  Revision 1.19.2.1  2018/06/07 15:28:38  verkout
//  HV: * start fixing lot of bugs in j2ms2
//          - support reading "eo:..." and using options from lines in lis file
//
//  Revision 1.19  2016/06/23 13:02:22  jive_cc
//  HV: * moved codebase to casacore 2.x compatibility - the namespace is now
//        casacore::  in stead of casa::. A lot of improvement ......
//      * now link against vex parser which understands $BITSTREAMS and $THREADS
//
//  Revision 1.18  2016-01-11 10:46:44  jive_cc
//  HV: * Add explanation about buffer specific options to '-h' output
//
//  Revision 1.17  2014-07-01 12:13:19  jive_cc
//  HV: * Added some documentation about experiment-options that
//        users can pass to j2ms2
//
//  Revision 1.16  2013-06-12 07:39:21  jive_cc
//  HV: * By popular demand - j2ms2 now exits with non-zero value in case of
//        exceptional exceptions being raised from within the guts of the prgrm.
//
//  Revision 1.15  2011-10-12 12:45:22  jive_cc
//  HV: * DtdLUT (DataTransferDescriptor Lookup Table) removed and
//        changed into 'labels' for more genericity
//      * can now be built either as part of AIPS++/CASA [using makefile]
//        or can be built as standalone set of libs + binaries and linking
//        against casacore [using Makefile]
//        check 'code/
//
//  Revision 1.14  2007-05-25 12:00:01  jive_cc
//  HV: - switched to using casacore::Regex
//      - filter/ghostbuster is now on by default
//
//  Revision 1.13  2007/03/01 14:29:16  jive_cc
//  HV: mainly cosmetic changes, made sure stuff is better to read,
//      removed unused code
//
//  Revision 1.12  2006/03/02 14:21:36  verkout
//  HV: * Removed remaining instances of strstream, replaced with strstream * Got rid of (home grown) Container class!!!! YES! Finally replaced with STLs vector/list, whichever is appropriate * Added pure virtual method to hduGenerator - class writers MUST return an ID which (uniquely) identifies their specific Generator * mapping of generator-ID to actual Generator is now built automagically rather than by ms2uvfitsConverter, class writers should look in Registrar.h for details
//
//  Revision 1.11  2006/02/10 08:59:11  verkout
//  HV: * Removed dependency on libExceptionObject and libConnection * Removed Utilities/AIPSExceptionObject.h * Switched to throwing/catching of exceptions derived from std::exception * Removed strstream (maybe some still left) * Removed conditional includes (Scott Meyers Effective C++ trick) because it... well.. sucks * Start support for PCInt datafiles * Removed packags RAW and replaced it with JCCS
//
//  Revision 1.10  2006/01/13 12:59:55  verkout
//  HV: CASAfied version of the implement
//
//  Revision 1.9  2005/03/15 15:01:04  verkout
//  HV: Added scan-selection possibility for *.lis files. -v reads the first/last scan to process from the 4th and 5th column from the lis file. -V is compatibility mode: in this case the scanIDs from column 4/5 are not used; j2ms2 just translates everything
//
//  Revision 1.8  2003/02/14 15:48:43  verkout
//  HV:   std::string does not have automatic conversion to
//        (const char*). Fits-stuff does need (const char*)
//        so had to add the odd .c_str() here and there.
//
//  Revision 1.7  2002/11/15 08:46:10  verkout
//  HV: Added pointer to j2ms2 'homepage'.
//
//  Revision 1.6  2002/08/06 09:26:30  verkout
//  HV: * Added support for Huibs streamlining .lis file
//  * Sort of dropped support for Albert's format of data
//  * Added support for passing options to experiment and/or filler
//  * Make use of STL stuff, like vector/string rather than char**
//
//  Revision 1.5  2001/07/09 07:55:55  verkout
//  HV: Some mods
//  - Enabled passing of options from j2ms2 commandline to an experiment
//  - Enables passing of options from j2ms2 commandline to the JIVEMSFiller
//  - Needed more libs to link correctly
//
//  Revision 1.4  2001/05/30 11:55:58  verkout
//  HV: Made changes to accomodate MSv2.0 and moved all code from the 'vlbi' branch to the 'jive' branch.
//
//  Revision 1.3  2000/03/28 07:39:13  verkout
//  HV: By popular demand: added the cmd-line opetion -f to j2ms2 = read buffer specifications from a file.
//
#include <casacore/casa/aips.h>
#include <casacore/casa/Utilities/Regex.h>

#include <jive/Utilities/jexcept.h>
#include <jive/j2ms/Visibility.h>
#include <jive/labels/CorrelationCode.h>
#include <jive/j2ms/Scan.h>
#include <jive/j2ms/VEXperiment.h>
#include <jive/j2ms/JIVEMSFiller.h>
#include <jive/j2ms/subjob.h>

#include <iostream>
#include <sstream>
#include <fstream>

#include <signal.h>
#include <stdlib.h>
#include <libgen.h>
#include <ctype.h>


// STL-stuff
#include <string>
#include <vector>

using std::string;
using std::ostream;
using std::ifstream;
using std::istringstream;
using std::ostringstream;
using std::vector;
using std::cout;
using std::cerr;
using std::endl;

using namespace casacore;

#ifndef BUILDTIME
#define BUILDTIME "<no buildtime info present>"
#endif

// Apr 2018: These are now needed outside main() as well
const Regex        filleropt( "^fo:.+" );
const Regex        experopt( "^eo:.+" );

// Stuff for processing Huib's .lis files...
typedef struct _job_t {
	string           experName;
	string           vexfileName;
	string           profileName;
	string           outputMSName;
	vector<subjob_t> jobList;
} job_t;



void strtolower( string& s ) {
	string::iterator   cur = s.begin();

	while( cur!=s.end() ) {
		*cur = ::tolower( *cur );
		cur++;
	}
	return;
}


// We want to have an exception thrown whenever signal's are
// generated...
void throwOnSignal( int signr ) {
    THROW_JEXCEPT("Signal #" << signr << " raised. Stopping!");
}


//  Read a list of buffers from file.
//  Assume:
//
//  1) one buffer/line
//  2) all characters following # (inclusive the #-sign itself) are discarded on the same line
//  3) empty lines are discarded
Bool  readListFromFile( const char* fname, vector<subjob_t>& buflist ) {
    char     linebuf[1024];
    char     outbuf[1024];
    Bool     retval;
    char*    ptr;
    char*    optr;
    ifstream strm( fname );
    
    //  Init return values to default
	buflist.clear();
    retval  = False;
    
    if( !strm || strm.bad() || strm.fail() || !strm.good() ) {
		cout << "readListFromFile: Drat! Failed to open file "
		   	 << ((fname==0)?("<NULL>"):(fname)) << "!" << endl;
		return retval;
    }
    
    //  Ok. Read the file!
    retval = True;
    while( !strm.eof() ) {
		strm.getline( linebuf, sizeof(linebuf) );

		//  Rub out comments
		if( (ptr=strchr(linebuf, '#'))!=0 ) 
			*ptr = '\0';
		
		//  Filter out whitespace....
		ptr  = linebuf;
		optr = outbuf;
		while( *ptr ) {
			if( !strchr(" \t\f\r", *ptr) ) {
				*optr = *ptr;
				optr++;
			}
			ptr++;
		}
		*optr = '\0';

		//  If there still are characters left, it must be a buffername!
		if( strlen(outbuf) ) 
			buflist.push_back( subjob_t(outbuf) );
    }
    strm.close();
    return retval;
}


// Read *.lis file as per HJvL definition.
//  in backwardcompatibility mode (compat==true)
//   the "first" and "last" scan info is discarded.
//   Otherwise only scans from first->last (inclusive) 
//   are written to the MS
bool readLisFile( const char* lisfile, job_t& job, bool compat ) {
    char     linebuf[2048];
    Bool     retval;
	bool     firstlineread;
    char*    ptr;
    char*    optr;
    ifstream strm( lisfile );
    
    //  Init return values to default
	job     = job_t();
    retval  = False;
    
    if( !strm || strm.bad() || strm.fail() || !strm.good() ) {
		cout << "readLisFile: Drat! Failed to open file "
		   	 << ((lisfile==0)?("<NULL>"):(lisfile)) << "!" << endl;
		return retval;
    }
    
    retval        = True;
	firstlineread = false;
    while( retval==True && !strm.eof() ) {
		strm.getline( linebuf, sizeof(linebuf)-1 );

		//  Rub out comments
		if( (ptr=strchr(linebuf, '#'))!=0 )
			*ptr = '\0';
		
		//  Skip leading whitespace....
		optr  = linebuf;
		while( *optr && strchr(" \t\f\r", *optr) ) 
			optr++;

		//  If no characters left, nothing to do but to read another line...
		if( !strlen(optr) )
			continue;

		// What can the line be?
        if( !firstlineread ) {
            // First line of lis file is special;
            // it sets some important variables
			istringstream flstrm( optr );

			flstrm >> job.experName >> job.vexfileName
				   >> job.profileName >> job.outputMSName;

			if( job.experName.size()==0 ) {
				retval = False;
				cout << "readLisFile: experiment name is empty?!" << endl;
			} else {
                // Provide defaults as per HJvL for these items if
                // they were not set in the lisfile
				if( job.vexfileName.size()==0 ) {
					job.vexfileName = job.experName+".vix";
					strtolower( job.vexfileName );
				}
				if( job.profileName.size()==0 )
					job.profileName = "Prod";
				if( job.outputMSName.size()==0 )
					job.outputMSName = job.experName+job.profileName+".ms";
			}
			firstlineread = true;
            // Can immediately continue with reading next line
            continue;
        }

        // Ending up here, we've seen the first line so now we're
        // parsing job lines.
        // If that line does NOT start with '+' we're not even going to
        // look at it
        if( *optr!='+' )
            continue;
        // OK this line /may/ describe a job to append to the MS
        char*         cmdLineOpts( 0 );
        string        plus;
        string        jobname;
        string        dummy_pass;
        string        fscan;
        string        escan;
        // HV: Apr 2018 - New lis file format: the line may include extra
        //                command line options like "eo:setup_ref_station=XX"
        //                and the likes! Yikes!
        //                Those options are - if present - found after the
        //                standard "--" sentinel
        if( (ptr=strstr(optr, "--"))!=0 ) {
            // Only allow "old style" processing up to "--"
            *ptr = '\0';
            // Anything after "--" is taken to be as if typed on the
            // j2ms2 command line
            cmdLineOpts = ptr+2;
        }

        // Do old style parsing of first part of the line
        subjob_t*     subjob;
        istringstream sjstrm( optr );

        sjstrm >> plus >> jobname >> dummy_pass >> fscan >> escan;

        // Depending on compatibility mode create the subjob
        if( compat )
            subjob = new subjob_t(jobname);
        else
            subjob = new subjob_t(jobname, fscan, escan);

        // And add "command line style parsing"
        // (inspired by "main(...)")
        // Note that over here we filter ONLY eo:<bla> and fo:<bla>
        static char const*  delims = " \t";
        while( (ptr=strtok(cmdLineOpts, delims))!=0 ) {
            string*         dst = 0;
            const String    tmp(ptr);

            // Check if it's fo: or eo:
            if( tmp.matches(filleropt) )
                dst = &subjob->fo;
            else if( tmp.matches(experopt) )
                dst = &subjob->eo;

            // Only do something if there is a destination string 
            // NOTE: dst != 0 implies "fo:" matched or "eo:" matched
            //       which implies strchr(.., ':') will always succeed
            if( dst ) {
                char const* valptr = ::strchr(ptr, ':') + 1;
                // String multiple option values together into
                // a ;-separated string
                if( dst->size() )
                    *dst += ';';
                *dst += valptr;
            }
            // As per strtok(3):
            // First time call with string-to-be-tokenized,
            // after that call with NULL
            cmdLineOpts = 0;
        }
        // Now the subjob-to-do can be appended to the job's todo-list
        job.jobList.push_back( *subjob );
        delete subjob;
    }
    strm.close();
    return retval;
}

void Usage( void ) {
    cout <<
"Usage: j2ms2 [-h] [-o <outputMS>] [-d <domain>] [-f <buflistfile>] " << endl <<
"             [-(v|V) <lisfile>] [eo:<exper opts>]* [fo:<filler opts>]*" << endl <<
"             [<buffer>]*" << endl <<
"       note: '*' means option may be present 0 or more times" << endl <<
"" << endl <<
"Options:" << endl <<
"  -h                output this message, exit succesfully" << endl <<
"  -o <outputMS>     override output MeasurementSet name to <outputMS>" << endl <<
"                    Defaults to auto-deduced  <experiment code>.ms" << endl <<
"  -(v|V) <lisfile>  use indicated lisfile to create the MeasurementSet." << endl <<
"                    Use -V for backward compatibility mode" << endl <<
"                    See http://www.jive.eu/~verkout/lisfile.txt" << endl <<
"                    for the lisfile format and justification" << endl <<
"  -d <domain>       'time' or 'frequency' - write data in this domain in the" << endl <<
"                    output MeasurementSet, j2ms2 will perform FFT as necessary" << endl <<
"                    based on what domain <buffer> contains" << endl <<
"  -f <buflistfile>  one <buffer> per line for easy multi-<buffer> translation" << endl <<
"  eo:<exper opts>   experiment options in key=value format:" << endl <<
"                    - eo:setup_ref_station=<station>" << endl <<
"                      Set the reference station whose frequency setup from the" << endl <<
"                      VEX file will be used (SFXC mixed bandwidth correlation)" << endl <<
"                    - eo:verbose" << endl <<
"                      Enable verbose logging of some steps, time+expr variable" << endl <<
"                    - eo:avgchannel=N" << endl <<
"                      Average down UniBoard spectral data by a factor of N" << endl <<
"  fo:<filler opts>  options for the filler, in key=value format." << endl <<
"                    At the moment used to enable filters:" << endl <<
"                    - fo:filter/source=<commaseparated sourcelist>" << endl <<
"                      only write visibilites for sources in the list" << endl <<
"                    - fo:filter/ghostbuster[=<flag>[,<flag>]*]" << endl <<
"                      do not write data for which a station has" << endl <<
"                      no associated scan (ghostdata)" << endl <<
"                      The filter supports extra flags:" << endl <<
"                        relaxed   accept data if a station has no scans at all" << endl <<
"                                  default is to delete if station has no scans" << endl <<
"                        data_good honour the 'data good' property from the VEX" << endl <<
"                                  file (will be added to nominal scan start)" << endl <<
"  <buffer>          visibility buffer identifier, typically:" << endl <<
"                      for SFXC:      '<job>/<scan>.cor'" << endl <<
"                      for UniBoard:  '<job>.json'" << endl <<
"" << endl <<
"For some formats, per-<buffer> specific options can be given by suffixing the" << endl <<
"buffer name with '%<comma separated options>'" << endl <<
"" << endl <<
"For UniBoard data format there exist at least the following options:" << endl <<
"      debug[=N]    set debug level to 1 (without '=N') or N; higher number" << endl <<
"                   gives more debug output" << endl <<
"      normalize=0  switch OFF normalization (default is ON)" << endl <<
"" << endl <<
"   e.g.:  j2ms2 F15L1-edit.json%normalize=0,debug=2" << endl <<
"" << endl <<
"If no <buffer>s, <lisfile> or <buflistfile> are given on the command line," << endl <<
"j2ms2 will run interactively, prompting the user to type <buffer>s manually" << endl <<
"until the special case 'quit' is entered, to, well, err, quit ..." << endl <<
"" << endl <<
"See http://www.jive.eu/~verkout/j2ms2.html for more detail." << endl <<
"" << endl <<
"";
    return;

#if 0
    cout << "Usage:" << endl 
         << "    j2ms2 [-h] [-o <outputMS>] [-d <domain>] [-f <buflistfile>] [-(v|V) <lisfile>] [eo:<exper opts>]* [fo:<filler opts>]* [<buffers>]*\n\n"
         << "  '*' means option may be present 0 or more times" << endl
         << "Options:\n"
         << "  -h               output this message, exit succesfully" << endl
         << "  -(v|V) <lisfile> use indicated lisfile, -V for backward compat mode" << endl
         << "  <domain>         'time' or 'frequency' - write data in this domain in the output MS" << endl
         << "  <buflistfile>    one buffer per line" << endl
         << "  <exper opts>     experiment options in key=value format, see below" << endl
         << "  <filler opts>    options for the filler, in key=value format, see below" << endl
         << "  <buffers>        visibility buffer identifier, typically:" << endl
         << "                    for SFXC:      '<job>/<scan>.cor'" << endl
         << "                    for UniBoard:  '<job>.json'" << endl
         << "\n"
         << "<exper opts>: per experiment settable options:" << endl
         << "  eo:setup_ref_station=<station>   Set the reference station whose\n"
         << "                                   frequency setup from the VEX file\n"
         << "                                   will be used. (Mixed bandwidth SFXC)\n"
         << "  eo:avgchannel=N                  Average down UniBoard spectral data\n"
         << "                                   by a factor of N\n"
         << "\n"
         << "<filler opts> can be used to switch on filter(s):\n"
         << "  fo:filter/source=<commaseparated list of sources to accept>\n"
         << "  fo:filter/ghostbuster[=<option>[,<option>]*]\n"
         << "              throw out data for which a station has no associated scan (ghostdata)\n"
         << "              options (comma-separated list for >1 options enabled):" << endl
         << "                  relaxed   if a station has no scans at all, accept visibilities." << endl
         << "                            the default is to delete visibilites if station has no scans at all" << endl
         << "                  data_good honour the 'data good' property from the VEX file (will be added" << endl
         << "                            to nominal scan start time)" << endl
#ifdef MARK4 // GhostBuster filter needs to be on by default for Mark4 correlator output
         << "               THIS FILTER IS ON BY DEFAULT!\n"
#endif
         << "\n"
         << "For some formats, per-buffer specific options can be given by\n"
         << "suffixing the buffer name with '%<comma separated options>'.\n"
         << "\n"
         << "For UniBoard data format there exist at least the following options:\n"
         << "      debug[=N]    set debug level to 1 (without '=N') or N; higher number\n"
         << "                   gives more debug output\n"
         << "      normalize=0  switch OFF normalization (default is ON)\n"
         << "\n"
         << "  e.g.:  j2ms2 F15L1-edit.json%normalize=0,debug=2\n"
         << "\n"
		 << "See http://www.jive.nl/~verkout/j2ms2.html for more detail.\n";
    return;
#endif
}

int main( int , char** argv ) {
    int   rv = 0;
	char* thisexe;

	if( (thisexe=::basename(argv[0]))==0 )
		thisexe = argv[0];
	cout << "\n"
		 << thisexe
#ifdef BUILDTIME
         << ": Version of " << BUILDTIME
#endif
		 << " begins"
         << "\n\n"
		 << flush;

    try
    {
		// define runmode to be able to find out what
        // mode we're actually running...
		typedef enum _runmode_t {
			unset, interactive, buflistfromfile, buflistfromcmdline, lisfile
		} runmode_t;
	
		runmode_t          runmode( unset );
			
		// Outputname -> used if non-empty
        // AND not running from lisfile, in which case it's overridden
		string           outname;
        
		// Buffers that need translation -> if this is empty -> interactively!
		vector<subjob_t> buffers2do;

		// thingy for lisfile runmode
        // backwardcompatibility mode (use -V to switch on)
		bool               compat(false); 
		job_t              job;
		
		// optional stuff that could be passed to the experiment
		// and/or the filler
        // fo: == "filler option:"
        // eo: == "experiment option"
		String             eo;
#ifdef MARK4 // GhostBuster filter needs to be on by default for Mark4 correlator output
		String             fo("filter/ghostbuster");
#else
		String             fo; 
#endif

		// In what domain do we want the data?
		Visibility::domain dstdomain( Visibility::undefined );

		// Let's count the errors...
		unsigned int       errcnt = 0;
		
		//  Before we do anything, we tell the system what to do when
		//  specific signals are raised...
		signal( SIGINT,  throwOnSignal );
		signal( SIGKILL, throwOnSignal );
		signal( SIGTERM, throwOnSignal );

		//  Ok. Do some commandline parsing. If there are entries on
		//  the cmd-line that are not options, we assume the
		//  usr. wants to run non-interactively. So what we do is to
		//  parse the cmdline and base our conclusions on the outcome
		//  of that!
		while( *(++argv) ) {
			// Do some pre-parsing... if the arg matches eo:<bla> or fo:<bla>
			// treat it somewhat different.
			if( String(*argv).matches(filleropt) ) {
				// Create a pointer that points to the first character
				// after the first ':'-sign
				char*   valptr = ::strchr(*argv, ':') + 1;

                // String multiple fo: optionsvalues together into
                // a ;-separated string
				if( fo.size() )
					fo += ';';
				fo += valptr;
				continue;
			} else if( String(*argv).matches(experopt) ) {
				// Create a pointer that points to the first character
				// after the first ':'-sign
				char*   valptr = ::strchr(*argv, ':') + 1;

                // Same as with the filler options
				if( eo.size() )
					eo += ';';
				eo += valptr;

				continue;
			}
            // Tried the fo: and eo: style argumenttypes
            // Go on and try '-[a-zA-Z]' style argumenttypes
			switch( *(*argv) ) {
				case '-':
					switch( *((*argv)+1) ) {
						case 'd':
							// Expect: -d <domain>
							if( !(*(argv+1)) || (**(argv+1)=='-') ) {
								cout << "Missing argument to -d option!"
                                     << endl;
								errcnt++;
								break;
							}
			    
							++argv;
							if( dstdomain!=Visibility::undefined ) {
								cout << "Duplicate -d option. Discarding "
                                     << "this value: " << *argv << endl;
								errcnt++;
								break;
							}
							if( !strcmp(*argv, "time") )
								dstdomain = Visibility::time;
							else if( !strcmp(*argv, "frequency") )
								dstdomain = Visibility::frequency;
							else {
								cout << "Unrecognized domain `"
                                     << *argv << "'" << endl;
								errcnt++;
							}
							break;
			    
						case 'o':
							// Expect: -o <outputfile>
							// Not acceptable when running in lis-file mode
							// (in that case, the output filename is determined
							// from the contents of the lis-file)
							if( runmode==lisfile ) {
								if( (*(argv+1)) && (**(argv+1)!='-') )
									argv++;
								cout << "Cannot use -o option in conjunction "
                                     << "with lis-file based operation!"
									 << endl;
								errcnt++;
								break;
							}

							if( !(*(argv+1)) || (**(argv+1)=='-') ) {
								cout << "Missing argument to -o option!"
                                     << endl;
								errcnt++;
								break;
							}
							++argv;

							if( outname.size()==0 )
								outname = *argv;
							else {
								cout << "Duplicate -o option! "
                                     << "Discarding this value: "
									 << *argv << endl;
								errcnt++;
							}
							break;

						case 'f':
							// Expect: -f <file with buffer-ids>
							// Only acceptable if runmode==unset...
							if( !(*(argv+1)) || (**(argv+1)=='-') ) {
								cout << "Missing argument to -f option!"
                                     << endl;
								errcnt++;
								break;
							}
							++argv;
							if( runmode!=unset ) {
								// if necessary, skip over nxt argument..
								cout << "Hum... it seems as if you're trying to blend "
									 << "different runmodes. The -f option is mutually "
									 << "exclusive to any other runmode";
								if( runmode==buflistfromfile ) {
									cout << ", even with itself; the option may only "
										 << "appear once on the commandline...";
								}
								cout << "." << endl
									 << "As a result, this one, " << *argv
									 << ", will be ignored" << endl;
								errcnt++;
								break;
							}

							if( readListFromFile(*argv, buffers2do) )
								runmode = buflistfromfile;
							else {
								cout << "Drat! Failed to read buf-list from file "
									 << *argv << endl;
								errcnt++;
							}
							break;
			  
						case 'V':
							compat = true;
						case 'v':
							// Expect: -v <lisfilename>
							// Only acceptable if runmode==unset...
							if( !(*(argv+1)) || (**(argv+1)=='-') ) {
								cout << "Missing argument to -v option!" << endl;
								errcnt++;
								break;
							}
							++argv;
							if( runmode!=unset ) {
								// if necessary, skip over nxt argument..
								cout << "You ok in there?"
								     << " It seems as if you're trying to blend "
									 << "different runmodes. The -[vV] option is mutually "
									 << "exclusive to any other runmode";
								if( runmode==lisfile ) {
									cout << ", even with itself; the option may only "
										 << "appear once on the commandline...";
								}
								cout << "." << endl
									 << "As a result, this one, " << *argv
									 << ", will be ignored" << endl;
								errcnt++;
								break;
							}

							if( readLisFile(*argv, job, compat) ) {
								runmode = lisfile;

								if( outname.size()!=0 ) {
									cout << endl
										 << "WARN: Apparently you wanted to set the "
										 << "output MSfilename via the commandline; "
										 << "this is not acceptable when using the "
										 << "lis-file runmode. Your setting of "
										 << outname << " will be overridden with the "
										 << "value found in the lis-file, "
										 << job.outputMSName
										 << endl;
									outname = "";
									errcnt++;
								}
							} else {
								cout << "Drat! Failed to read lis-file "
									 << *argv << endl;
								errcnt++;
							}
							break;

                        case 'h':
                            Usage();
                            return 0;
                            
						default:
							cout << "Unrecognized option '"
                                 << *argv << "'" << endl;
							errcnt++;
							break;
					}
					break;
		    
				default:
					static Bool  errorshown = False;
		    
					// If runmode not unset or buflistfromcmdline
					// we do not accept this!
					if( runmode!=unset && runmode!=buflistfromcmdline ) {
						if( !errorshown ) {
							cout << "You cannot mix styles of runmode,"
								 << " you\n"
							 	 << "have already chosen a different runmode; hence the\n"
								 << "buffers on the cmd-line will be ignored." << endl;
						    errorshown = True;
							errcnt++;
						}
						cout << "Discarding cmd-line buffer specification " << *argv 
							 << endl;
						break;
					}
		   
					//  Assume it is the name of a buffer....
				   	runmode = buflistfromcmdline;
					buffers2do.push_back( subjob_t(*argv) );					
					break;
			}
		}

		if( errcnt )
			THROW_JEXCEPT("There " << ((errcnt>1)?("were "):("was "))
			      << errcnt << " error" << ((errcnt>1)?("s"):(""))
                  << " detected. Giving up.");
		
		//  Now we can really start...
		String            scanid;
		Experiment*       theExperiment;
		const Regex       freqdomain( "^[ \t]*frequency[ \t]*$" );
		const Regex       timedomain( "^[ \t]*time[ \t]*$" );
		JIVEMSFiller      theFiller( fo );


		theExperiment = 0;
	
		try {
			theExperiment = new VEXperiment( eo, runmode==lisfile ? job.vexfileName : String() );
		}
		catch( const std::exception& y ) {
			cerr << y.what() << endl;
            return -1;
		}

		//  Now decide if we are running interactively or not.  If
		//  usr. did not specify any filenames or lisfile or just a
        //  file with visbufids on the cmdline, we ask
		//  for it; i.e. we run interactively!
		if( runmode==unset )
			runmode = interactive;
	
		//  See if usr. specified some other options on the
		//  commandline!

        // If dstdomain was explicitly set:
		if( dstdomain!=Visibility::undefined )
			theExperiment->setDestinationDomain( dstdomain );

        // outputfilename overridden?
		if( outname.size() ) 
			theExperiment->setMSname( outname.c_str() );

		// If running in lisfile mode, copy some stuff across
        // Note: the -o option is discarded here
		if( runmode==lisfile ) {
            if( outname.size() ) {
                cerr << "\n"
                     << "WARN: Discarding -o option [" << outname << "]; "
                     << "because running from lisfile!!!!" << endl;
            }
			theExperiment->setMSname( job.outputMSName.c_str() );
			buffers2do = job.jobList;
		}

        //  Show exp. summary to usr.
		cout << *theExperiment << endl;

		//  Either run interactively or not!
		if( runmode==interactive ) {
			cin.tie( &cout );
			cout << "DataBuffer (\"quit\" to quit): ";
			while( cin >> scanid ) {
				if( scanid=="quit" ) 
					break;
		
				if( String(scanid.c_str()).matches(freqdomain) )
					theExperiment->setDestinationDomain( Visibility::frequency );
				else if( String(scanid.c_str()).matches(timedomain) )
					theExperiment->setDestinationDomain( Visibility::time );
				else if( !theFiller.fillDataToExperimentsMS(*theExperiment,scanid,true) )
					cerr << "Failed to write data to MS" << endl;
			    cout << "DataBuffer (\"quit\" to quit): ";
			}
		} else {
			//  Do not run interactively...
			vector<subjob_t>::iterator   cur = buffers2do.begin();

			while( cur!=buffers2do.end() ) {
                // if the current subjob added experiment options now is the
                // time to process those
                theExperiment->resetOptions( cur->eo );
				theFiller.fillDataToExperimentsMS( *theExperiment,
												   *cur,
												   false );
				cur++;
			}
		}
	
		delete theExperiment;
    }
    catch( const std::exception& x ) {
		cout << "Rats! " << x.what() << endl;
        rv = 1;
    }
    
    return rv;
}
