//  Implementation of the experiment class
//
//  Author:  Harro Verkouter, 8-12-1998
//
//
#include <jive/j2ms/Experiment.h>
#include <jive/Utilities/stringutil.h>
#include <jive/Utilities/jexcept.h>


#include <iostream>
#include <fstream>
#include <sstream>

#include <unistd.h>

using namespace std;
using namespace casacore;

ostream& operator<<( ostream& os, const Experiment& expref ) {
    os << "************ Experiment " << expref.getCode() << " ************" << endl;
    //os << "Antennas: " << expref.getAntennas() << endl;
    //os << "Sources: " << expref.getSources() << endl;
    os << "Output is placed in: " << expref.getMSname() << endl;
    os << "Data is written in the MS in the " << expref.getDestinationDomain() << " domain" << endl;
    return os;
}


Experiment::Experiment( const String& opt, const String& exprootdir ) :
	myOptions( opt ),
    myRootDir( exprootdir ),
    myLogStreamptr( new ostream* )
{
    (*myLogStreamptr) = (ostream *)0;
    
    //  If the exp. root dir. not set, determine the cwd
    if( myRootDir=="" ) {
		char*   cwd( getcwd((char*)0, 1024) );
	
		if( cwd ) {
			myRootDir = (const char *)cwd;
			::free( cwd );
		}
    }
    // Parse options into map
    myOptionValues = ::mk_optionmap( myOptions );
}

// Look up the requested option key and return value or empty string
String Experiment::optionValue( const String& opt ) const {
	optionmap_type::const_iterator optptr = myOptionValues.find( opt );

    return (optptr==myOptionValues.end() ? String() : optptr->second);
}

String const& Experiment::options( void ) const {
    return myOptions;
}

// In order to see if the options changed we must parse them 
// such that order and white space &cet does not count towards change
void Experiment::resetOptions( String const& opts ) {
    // Parse the options into a new optionmapping
    optionmap_type  newOpts = ::mk_optionmap( opts );

    // Merge the current options in with the new options
    // this means that updated values for existing keys will be kept
    // and old values that did not get a new value in the new options
    // are retained
    newOpts.insert( myOptionValues.begin(), myOptionValues.end() );

    // If the new ones STILL equal to the old ones, nothing changed!
    if( newOpts==myOptionValues )
        return;
    // OK something changed ...
    // compile the new option string + parsed key[value] map
    myOptions = string();
    for(optionmap_type::const_iterator curopt=newOpts.begin(); curopt!=newOpts.end(); curopt++)
        myOptions += (curopt->first + "=" + curopt->second + ';');
    myOptionValues = newOpts;

    // And inform upstream classes that something may have changed
    this->optionsReset();
}

const String& Experiment::getArrayCode( void ) const {
    static String   defaultArrayCode( "EVN" );
    
    return defaultArrayCode;
}


const String& Experiment::getRootDir( void ) const {
    return myRootDir;
}


const j2ms::scaninfolist_t& Experiment::getScanlist( void ) const {
    static const j2ms::scaninfolist_t  empty_scan_list;
    return empty_scan_list;
}

const j2ms::stationscanlist_t& Experiment::getStationScanlist( int ) const {
    static const j2ms::stationscanlist_t   empty;

    return empty;
}


void Experiment::logLogMessage( const String& msg, const LogOrigin& org ) const {
    if( !(*myLogStreamptr) ) {
        //  Create a suitable logfile....
        ostringstream strm;
	
        strm << myRootDir << "/" << this->getMSname() << ".log";

        (*myLogStreamptr) = new fstream(strm.str().c_str(), ios::app);
    }
  
    *(*myLogStreamptr) << LogMessage(msg, org) << flush;
    return;
}


Experiment::~Experiment() {
    // close the log-file
    delete (*myLogStreamptr);
    delete myLogStreamptr;
}

