//
//  FrequencyDB
//
//  A database interface to locate frequency setups
//
//  Author:   Harro Verkouter,   3-12-1998
//
//  $Id: FrequencyDB.h,v 1.6.2.1 2018/06/07 15:27:12 verkout Exp $
//
#ifndef FREQUENCYDB_H
#define FREQUENCYDB_H

#include <casacore/casa/aips.h>
#include <casacore/casa/BasicSL/String.h>
#include <jive/j2ms/FrequencyConfig.h>
#include <jive/labels/polarization.h>


class FrequencyDB
{
public:
    //  Create a database object...
    FrequencyDB();

    //  Do a lookup based on code
    virtual FrequencyConfig operator[]( const casacore::String& key ) const = 0;
    virtual casacore::Bool            searchFrequencyConfig( FrequencyConfig& fcref,
						                                 const casacore::String& key ) const = 0;
   
    // Do a lookup based on freqgroupnumber
    virtual casacore::Bool            searchFrequencyConfig( FrequencyConfig& fcref,
						                                 casacore::Int fqgrp ) const = 0;

    // HV: Apr 2018 - Maybe it's time to start thinking of pointing at
    //                something!
    virtual FrequencyConfig const*    findFrequencyConfigPtr( casacore::String const& key   ) const = 0; 
    virtual FrequencyConfig const*    findFrequencyConfigPtr( casacore::Int           fqgrp ) const = 0; 

    //  Allow looping over the FrequencyConfigs:
    //
    //  Look up by index (ie, get the n-th frequency config.
    //  If you want the FrequencyConfig with a given freqgroupnr, use 
    //  "searchFrequencyConfig()".
    //  Note: frequency-group-number for entry #n may or may not be equal to 'n'.
    //  Don't count on it.
    virtual casacore::uInt      nEntries( void ) const = 0;
    virtual FrequencyConfig operator[]( casacore::uInt index ) const = 0;

    // Get the recorded polarizations
    virtual const j2ms::polarizations_t&  recordedPolarizations( void ) const = 0; 

    //  Delete the resources this database used....
    virtual ~FrequencyDB();

private:
    //  Let's prohibit these, it's not wise to copy databases, assign
    //  to them
    FrequencyDB( const FrequencyDB& );
    const FrequencyDB& operator=( const FrequencyDB& );
};

#endif
