// filters out data produced from one or more station(s) without scheduled scan
//
// $Id: GhostBusterFilter.h,v 1.2.2.1 2018/08/21 12:26:29 verkout Exp $
//
// $Log: GhostBusterFilter.h,v $
// Revision 1.2.2.1  2018/08/21 12:26:29  verkout
// HV: * Added 'option_map' implementation so it can be reused to
//       support "key=value,key2=value2..." passing of settings
//     * On multi-phase-center (700-ish) j2ms2 become /SUPER/ slow:
//       the ghostbuster-filter (enabled by default) was effectively
//       disabled (on purpose) by SFXCVisBuffer implementation but did
//       that in the most time-consuming way ...
//     * GhostBusterFilter changed:
//         - now takes options (see j2ms2 help) [can honour station based
//           data_good from VEX file if desired]
//         - responsibility for implementing the filter moved out of
//           *VisBuffer classes to better suited logical level
//     * More descriptive SFXCVisBuffer error messages
//     * UNBVisBuffer: overlooked the fact that subbands now have to be
//       sorted by frequency AND sideband in stead of just freq.
//
// Revision 1.2  2016/06/23 13:02:25  jive_cc
// HV: * moved codebase to casacore 2.x compatibility - the namespace is now
//       casacore::  in stead of casa::. A lot of improvement ......
//     * now link against vex parser which understands $BITSTREAMS and $THREADS
//
// Revision 1.1  2006-02-14 13:30:49  verkout
// HV: Implemented a filter which removes ghostdata based on individual scan-lists (scanlist per station). If any of the participating stations in a baseline has no scan defined the visibility is discarded
//
//
#ifndef J2MS_STATIONSCANFILTER_H
#define J2MS_STATIONSCANFILTER_H

#include <jive/j2ms/Filter.h>
#include <jive/j2ms/option_map.h>


class GhostBusterFilter:
    public Filter
{
    public:

        // this string is neglected -
        //  this filter supports no options
        // HV: July 2018 - Nope, this filter will support options!
        //                 "exact": no scans at all for station => don't let
        //                          data through. Default: off ('inexact' -
        //                          no scans at all => let all data through)
        //                 "data_good": if set, honours data_good from
        //                          VEX file to raze station based data.
        //                          Default: off.
        GhostBusterFilter( const casacore::String& opts );

        // and because this filter doesn't support
        // options, we might as well support a default
        // c'tor...
        // HV: July 2018: Not true anymore, see above
        GhostBusterFilter();

        virtual casacore::Bool    letPass( const Visibility& vis );

        virtual ~GhostBusterFilter();

    private:
        // July 2018: Default action is now changed.
        //            In the past: station scan list empty => visibility dropped
        //            Now: station scan list empty => visibility pass
        //
        //            Use "filter/ghostbuster=strict" to get back old
        //            default usage
        optionmap_type __m_options;
        casacore::Bool __m_strict;
        casacore::Bool __m_datagood;

        // apart from protecting these...
        GhostBusterFilter( const GhostBusterFilter& );
        const GhostBusterFilter& operator=( const GhostBusterFilter& );
};


#endif
