//
//  Implementation of the Scan-class
//
//  Author:  Harro Verkouter,  7-12-1998
//
//  $Id: Scan.cc,v 1.8.2.1 2018/06/07 15:27:12 verkout Exp $
//
#include <jive/j2ms/Scan.h>
#include <jive/j2ms/Experiment.h>
#include <jive/j2ms/VisibilityBuffer.h>
#include <jive/Utilities/jexcept.h>
#include <casacore/measures/Measures/MEpoch.h>
#include <casacore/casa/Quanta/MVTime.h>
#include <iostream>

using namespace std;
using namespace casacore;

namespace j2ms
{

ostream& operator<<( ostream& os, const Scan& sref ) {
   	MVTime   tim( sref.getStartTime().getValue().getTime() );
 
    os << "Scan " << sref.getScanId() << " (#" << sref.getScanNo() << "): "
       << sref.getSourceCode() << "/" << sref.getFrequencyCode()
       << " ** " << MVTime::setFormat(MVTime::DMY) << tim << flush;
    return os;
}


Scan::Scan( const VisibilityBuffer* visbufptr ) :
    itsVisibilityBufferptr( visbufptr ), itsFrequencyConfigptr( 0 )
{
    //  If we don't have a parent-visibilitybuffer: we throw up!
    if( !itsVisibilityBufferptr )
    	THROW_JEXCEPT("Scan::Scan(): No VisibilityBuffer given!");
}

FrequencyConfig const* Scan::getFrequencyConfig( void ) const {
    return (itsFrequencyConfigptr ? itsFrequencyConfigptr :
            (itsFrequencyConfigptr = itsVisibilityBufferptr->getExperiment().getFrequencyDB().findFrequencyConfigPtr( this->getFrequencyCode() )));
}

//  De-reference the Experiment-ptr -> usr has RO-access to the
//  experiment properties
const Experiment& Scan::getExperiment( void ) const {
    return itsVisibilityBufferptr->getExperiment();
}

const VisibilityBuffer& Scan::getVisibilityBuffer( void ) const {
    return *itsVisibilityBufferptr;
}


// D'tor doesn't do anything [we do not 0wn the visbufptr so
// do not delete it]
Scan::~Scan()
{}

// End of j2ms namespace
};
