// implementation of the optionmap_type support
#include <jive/j2ms/option_map.h>
#include <jive/Utilities/stringutil.h>
#include <vector>
#include <utility>
#include <cstring>

using std::string;
using std::vector;
using std::cout;
using std::endl;
using std::pair;


casacore::String identity_xform(casacore::String x) {
    return x;
}
casacore::String lower_case(casacore::String x) {
    casacore::String  copy(x);
    copy.downcase();
    return copy;
}
casacore::String upper_case(casacore::String x) {
    casacore::String  copy(x);
    copy.upcase();
    return copy;
}
casacore::String capitalize(casacore::String x) {
    casacore::String  copy(x);
    copy.capitalize();
    return copy;
}

optionmap_type mk_optionmap(string const& opts, char sep, xform_function key_xform, xform_function value_xform) {
	// Parse the option string, splitting it at ';'s, getting <key>=<value> pairs
	// First, remove all the whitespace
	string                 optcopy;
    optionmap_type         retval;
	string::const_iterator curchar = opts.begin();
	
	while( curchar!=opts.end() ) {
		if( !::strchr(" \t\n\f", *curchar) ) {
			optcopy.append( 1, *curchar );
		}
		curchar++;
	}

	// Now split the 'optcopy' at ';'s...
	const vector<string>  optv = ::split(optcopy, sep);

	for( vector<string>::const_iterator curopt=optv.begin(); curopt!=optv.end(); curopt++ ) {
		// if the string has no length, skip it
		if( curopt->size()==0 )
			continue;

		// split at =
		const vector<string> kv = ::split( *curopt, '=' );
		
		// if we do not have exactly 1 or 2 entries, the option is malformed?
		if( kv.size()!=1 && kv.size()!=2  ) {
			cout << "Malformed option " << *curopt << endl
				 << "   Not exactly none or one '='-sign. Expect "
				 << "<key>[=<value>]" << endl;
			continue;
		}

		if( kv[0].size()==0 || (kv.size()==2 && kv[1].size()==0) ) {
			cout << "Malformed option " << *curopt << endl
			     << "   Either <key> or <value> is zero-length!"
			     << endl;
			continue;
		}

		// Add the <key>=<value> entry to the map (if not already defined!)
        pair<optionmap_type::iterator, bool> const insres = retval.insert( make_pair(key_xform(kv[0]), kv.size()==1 ? string() : value_xform(kv[1])) );

        if( insres.second==false ) {
			cout << "AArgh: duplicate definition for key="
				 << kv[0] << endl;
            if( kv.size()==2 )
				 cout << "    Discarding value " << kv[1] << endl;
		}
	}
    return retval;
}
