// function to read all scans from a vexfile
//
// $Id: scanlist.h,v 1.3.2.1 2018/08/21 12:26:29 verkout Exp $
//
// Author: H Verkouter
//
#ifndef J2MS_SCANLIST_H
#define J2MS_SCANLIST_H

#include <list>
#include <map>
#include <string>
#include <jive/j2ms/Scan.h>
//#include <jive/j2ms/CorrelatorSetup.h>
#include <casacore/casa/Quanta/MVEpoch.h>
#include <casacore/casa/Quanta/Quantum.h>

// stick the typedef in a namespace
namespace j2ms {
    typedef std::list<ScanInfo>  scaninfolist_t;

    // (small) struct to keep track of station-based
    // scaninfo. For now we only need the station-based
    // start/stop times such that we can easily check
    // if a station was scheduled at some instance of time
    // or not
    struct stationscaninfo_t {
        casacore::MVEpoch   scanStart;
        casacore::MVEpoch   scanEnd;
        casacore::MVEpoch   dataStart;

        // sets scanStart/scanEnd to '0'
        stationscaninfo_t();

        // create from 'start' and 'end'
        stationscaninfo_t(const casacore::MVEpoch& s, const casacore::MVEpoch& e, casacore::Quantity const& ds);
        // create from 'start' + 'duration'
        stationscaninfo_t(const casacore::MVEpoch& s, const casacore::Quantity& d, casacore::Quantity const& ds);
    };

    // define a stationscanlist
    typedef std::list<stationscaninfo_t>      stationscanlist_t;

    // and all the scans per station grouped in a map
    // Key = station number, Value = stationscanlist_t
    typedef std::map<int, stationscanlist_t>  station2listmap_t;
}
// Parse VEX file once - get the global list of scans into 'sil' and the
//                       per-station scan lists into a map of station => scanlist (s2lm)
// throws on wonky.
void parseVEXScans(j2ms::scaninfolist_t& sil, j2ms::station2listmap_t& s2lm, std::string const& vexfilename, bool verbose);

#endif
