// convert a MS to IDI FITS
//
// The old tConvert/ms2uvf converter was broken for cases where not all
// baselines contain data for all the IFs - which has been a .TRUE.
// statement for the EVN for quite a while now ...
// It took way too long for us to recognize this.
//
// The old converter assumed that basically all data is there. But part of
// the EVN started observing wider bands than the remaining part. JIVE's
// sfxc correlator does not output data for subbands on baselines where
// either end of the baseline did not have any data.
//
// This throws off the expectation of the splitters + uvtable generators
// about how many visibilities present in a (sub)MeasurementSet.
//
// Clearly a new approach is needed.
//
//    Harro Verkouter, 24-May-2018
//
//
//
//  $Id: ms2idifitsConverter.h,v 1.1.2.1 2018/06/08 11:53:58 verkout Exp $
//
#ifndef MS2IDIFITSCONVERTER_H_INC
#define MS2IDIFITSCONVERTER_H_INC


//  Aips++ includes
#include <casacore/casa/aips.h>
#include <casacore/casa/Arrays/Vector.h>
#include <casacore/ms/MeasurementSets/MeasurementSet.h>

//  Need this for the base-class declaration
#include <jive/ms2uvfitsimpl/Converters/FileFormatConverter.h>


#include <jive/ms2uvfitsimpl/Generators/hduGenerator.h>
#include <jive/ms2uvfitsimpl/MSConversionInfo.h>
#include <casacore/casa/Containers/SimOrdMap.h>

#include <iostream>
#include <list>


class ms2idifitsConverter :
    public FileFormatConverter
{

public:
   
    //  Default ctor: this is the only thing needed.
    ms2idifitsConverter();

    // construct with specific hdulist_t and optional fragmentfn
    // (built-in default is 'defSplit', used if "fptr==0")
    ms2idifitsConverter( const generatorlist_t& hdus);
    
    //  Do convert the MS into a UVFITS-file
    bool  convert( const casacore::String& fitsfilename,
                   const casacore::String& MSfilename ) const;

    //  Replace the current generators by those listed in the
    //  block. The conversion class will set up the engine in such a
    //  way that when the convert method is run, the (eventually)
    //  produced FITS-file will contain those HDU's U specified in the
    //  'hdus' parameter (and in the same order).
    casacore::Bool   setConversion( const generatorlist_t& hdus );

    //  Destruct the conversionengine
    virtual ~ms2idifitsConverter();
    
private:
    //  The objects private parts

    // remember list of HDU's to generator for a conversion.
    // can be replaced by call to "setConversion"
    generatorlist_t       hdus2do;
    
    //*****************************************************************
    //              The object's private methods...
    //*****************************************************************
    //  Prevent copying and assignment
    ms2idifitsConverter( const ms2idifitsConverter& );
    const ms2idifitsConverter& operator=( const ms2idifitsConverter& );
};


#endif
