//
//  Class: 
//
//       FITSAsciiColumn
//
//  Purpose:
//
//       Object describes a column in a FITSBinaryTable Object. Does
//       correct formatting on get/put optins.
//
//
//
//  Author:
//
//       Harrro Verkoutrer,    18-05-1998
//
//
//  $Id: FITSAsciiColumn.h,v 1.5 2016/06/23 13:02:28 jive_cc Exp $
//
#ifndef FITSASCIICOLUMN_H_INC
#define FITSASCIICOLUMN_H_INC

#include <casacore/casa/aips.h>
#include <casacore/fits/FITS.h>
#include <jive/ms2uvfitsimpl/FITSUtils/FITSColumn.h>

//
//  Forward declarations
//
class FITSAsciiTableHDU;


class FITSAsciiColumn : 
    public FITSColumn
{
public:
    //
    // The public interface of the FITSAsciiColumn. For explanation
    // of the three different creators, see FITSColumn.h
    //
    FITSAsciiColumn( const casacore::String& type, const casacore::String& comment, const casacore::String& form, const casacore::String& unit );
    
    FITSAsciiColumn( casacore::Int number, casacore::uLong offset, casacore::FitsKeywordList& kwsource, FITSAsciiTableHDU* ptr2parent );
    
    FITSAsciiColumn( casacore::Int number, casacore::uLong offset, const FITSAsciiColumn& other, FITSAsciiTableHDU* ptr2parent );
    
    //
    //  Via recordfieldid! Default to the first field in the record
    //
    virtual casacore::Bool            get( casacore::uInt rownr, casacore::RecordInterface& recordref, 
                                 const casacore::RecordFieldId& fieldidx=0 ) const;
    virtual casacore::Bool            put( casacore::uInt rownr, const casacore::RecordInterface& recordref, 
                                 const casacore::RecordFieldId& fieldidx=0 );

    //
    //  We have our own method of translating this into FITSkeywords!
    //
    virtual casacore::FitsKeywordList asFITSKeys( void ) const;

    //
    //  Width of this column
    //
    virtual casacore::uLong           getWidthInBytes( void ) const;

    //
    //  Destruct the column object!
    //
    ~FITSAsciiColumn();

protected:
    //
    // Any protected functions go here.
    //


private:
    //
    //  The BinaryColumn's specific data
    //

    //
    //  We keep track of the width of this column ourselves
    //
    casacore::uLong                 itsTotalWidth;

    //
    //  The width of the field
    //
    casacore::uInt                  itsFieldWidth;
    
    //
    //  And the precision (nr of digits after the decimal point)
    //
    casacore::uInt                  itsPrecision;
    
    //
    //  Number of digits in exponent
    //
    casacore::uInt                  itsExponentPrecision;
    
    //
    //  AsciiColumn private methods
    //

    //
    //  This function checks the 'form' string and from there tries
    //  to determine the datatype of the column and the total column
    //  width. If a nonsupported datatype or invalid form-description
    //  was entered, an exception will be thrown!
    //
    void           CheckFormatAndInit( void );

    //  No default ascii columns possible
    FITSAsciiColumn( );
    FITSAsciiColumn( const FITSAsciiColumn& );
    FITSAsciiColumn& operator=( const FITSAsciiColumn& );
};



#endif
