//
//  Some global functions to test if DataTypes match etc. See
//  descriptions that go with the functions for detals.
//
//  Also conversion from DataType -> BITPIX and vice-versa
//
//  And test for equality of two FitsKeywords (match on name+(index))
//
//
//
//  Author:      
//
//               Harro Verkouter,     20-04-1998
//
//   $Id: freefunctions.h,v 1.4 2016/06/23 13:02:30 jive_cc Exp $
//
#ifndef FREEFUNCTIONS_H_INC
#define FREEFUNCTIONS_H_INC


#include <casacore/casa/aips.h>
#include <casacore/casa/Utilities/DataType.h>
#include <casacore/casa/Arrays/IPosition.h>
#include <casacore/fits/FITS.h>

#include <iostream>


//
//  This is a local free function, specially suited for the
//  FITSParameter class. it takes two DataTypes and decides whether or
//  not the source type can be changed in the destination type, by
//  either casting or indexing (especially for arrays, indexed by
//  IPositions) or taking the real/imaginary part or any combination
//  of the actions.
//
//  It basically checks if the BASIC types are equal/compatible, and
//  only for those basic types that are supported in FITS!
//
casacore::Bool dataTypesMatch( casacore::DataType destinationtype,
						   casacore::DataType sourcetype );


//
//  Test if the given datatype is a basic fits-compatible datatype
//
casacore::Bool isBasicFitsType( casacore::DataType sometype );

//
// Make an array type of a basic type (if sometype is a valid basic
// fitstype!)
//
casacore::DataType MakeArrayType( casacore::DataType sometype );

//
//  Convert DataTypes to bitpix value (and the reverse)
//

//
//  Return value is TpOther if the 'bitpixvalue' is not recognized as
//  a valid value for the 'BITPIX' keyword.
//
casacore::DataType  BitpixToDataType( casacore::Int bitpixvalue );

//
//  Return value is 0 if 'sometype' is not a basic fitstype!
//
casacore::Int       DataTypeToBitpix( casacore::DataType sometype );

//
//  Return True if the names of the two keywords (and the indices, if
//  applicable) match!
//
casacore::Bool operator==( const casacore::FitsKeyword& lval,
		               const casacore::FitsKeyword& rval );

//
//  Read an IPosition from a stream, in ASCII format; the inverse from
//  operator<<(IPosition)
//
std::istream& operator>>( std::istream& is, casacore::IPosition& rval );

#endif
