//
//  Froma a MSConversionInfo, generate a uv-binary table!
//
//  Author: Harro Verkouter,  13-09-1999
//
//
//  $Id: uvtableGenerator.h,v 1.4.2.1 2018/06/08 11:44:07 verkout Exp $
//
//  $Log: uvtableGenerator.h,v $
//  Revision 1.4.2.1  2018/06/08 11:44:07  verkout
//  HV: * start fixing tConvert missing visibilities, use iterator
//        discuss droppoing of UVFITS support
//      * FITSDataSegment now resizeable
//      * output of tConvert shortened whilst retaining information content
//
//  Revision 1.4  2006/03/02 14:21:44  verkout
//  HV: * Removed remaining instances of strstream, replaced with strstream * Got rid of (home grown) Container class!!!! YES! Finally replaced with STLs vector/list, whichever is appropriate * Added pure virtual method to hduGenerator - class writers MUST return an ID which (uniquely) identifies their specific Generator * mapping of generator-ID to actual Generator is now built automagically rather than by ms2uvfitsConverter, class writers should look in Registrar.h for details
//
//  Revision 1.3  2006/01/13 11:35:48  verkout
//  HV: CASAfied version of the implement
//
//  Revision 1.2  2001/05/30 11:54:08  verkout
//  HV: Made changes to accomodate MSv2.0 and moved all code from the 'vlbi' branch to the 'jive' branch.
//
//  Revision 1.1.1.1  2000/03/20 15:16:47  verkout
//  HV: Imported aips++ implement-stuff
//
//
#ifndef UVTABLEGENERATOR_H
#define UVTABLEGENERATOR_H

#include <casacore/casa/aipstype.h>
#include <casacore/tables/Tables/TableIter.h>
#include <jive/ms2uvfitsimpl/Generators/hduGenerator.h>
#include <jive/ms2uvfitsimpl/Generators/Registrar.h>


class uvtableGenerator :
    public hduGenerator
{
public:
    // do generate the uvtable
    virtual hdulist_t generate( const MSConversionInfo& inforef ) const;
    hdulist_t generate( const MSConversionInfo& inforef, casacore::TableIterator& tableiter, casacore::uLong nVis ) const;

    virtual idtype_t  getID( void ) const {
        return "uvtable";
    }

    //  delete allocated resources
    virtual ~uvtableGenerator();

    // You have to be aware that someone else initializes/updates those
    // behind your back
    mutable casacore::uLong   visCount;
    mutable casacore::uLong   pieceCount;
};

// See Registrar.h
DECLARE_GENERATOR(uvtableGenerator);




#endif
