//
//  Translate a raw correlator dump into a MS
//
//
//  $Id: j2ms2.cc,v 1.4 2000/11/17 00:15:36 verkout Exp $
//
//  $Log: j2ms2.cc,v $
//  Revision 1.4  2000/11/17 00:15:36  verkout
//  HV: Added build-time info; compiled into exe. Displayed on startup
//
//  Revision 1.3  2000/03/28 07:39:13  verkout
//  HV: By popular demand: added the cmd-line opetion -f to j2ms2 = read buffer specifications from a file.
//
//  
//
#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef DTDVISIBILITYBUFFER_H
#include <vlbi/j2ms/DTDVisibilityBuffer.h>
#endif

#ifndef AIPSEXCEPTIONOBJECT_H
#include <vlbi/Utilities/AIPSExceptionObject.h>
#endif

#ifndef EXCEPTIONOBJECT_H
#include <ExceptionObject/ExceptionObject.h>
#endif

#ifndef VISIBILITY_H
#include <vlbi/j2ms/Visibility.h>
#endif

#ifndef INIANTENNADB_H
#include <vlbi/j2ms/IniAntennaDB.h>
#endif

#ifndef INISOURCEDB_H
#include <vlbi/j2ms/IniSourceDB.h>
#endif

#ifndef INIFREQUENCYDB_H
#include <vlbi/j2ms/IniFrequencyDB.h>
#endif

#ifndef CORRELATIONCODE_H
#include <vlbi/DtdLUT/CorrelationCode.h>
#endif

#ifndef SCAN_H
#include <vlbi/j2ms/Scan.h>
#endif

#ifndef INIEXPERIMENT_H
#include <vlbi/j2ms/IniExperiment.h>
#endif

#ifndef VEXPERIMENT_H
#include <vlbi/j2ms/VEXperiment.h>
#endif

#ifndef JIVEMSFILLER_H
#include <vlbi/j2ms/JIVEMSFiller.h>
#endif

#ifndef REGULAREXPRESSIONS_H
#include <iniutils/RegularExpressions.h>
#endif

#include <iostream.h>
#include <signal.h>
#include <stdlib.h>
#include <libgen.h>


#ifndef BUILDTIME
#define BUILDTIME "<no buildtime info present>"
#endif


//
// We want to have an exception thrown whenever signal's are
// generated...
//
void throwOnSignal( int signr )
{
    char        buffer[256];
    ostrstream  strm( buffer, 256 );
    
    strm << "Signal #" << signr << " raised. Stopping!" << ends;
    throw( AipsError(strm.str()) );
    return;
}


//
//  Read a list of buffers from file.
//
//  Assume:
//
//  1) one buffer/line
//  2) all characters following # (inclusive the #-sign itself) are discarded on the same line
//  3) empty lines are discarded
//
Bool  readListFromFile( const char* fname, char**& buflist, unsigned int& nbuf )
{
    char     linebuf[1024];
    char     outbuf[1024];
    Bool     retval;
    char*    ptr;
    char*    optr;
    ifstream strm( fname );
   
 
    //
    //  Init return values to default
    //
    buflist = 0;
    nbuf    = 0;
    retval  = False;
    
    if( !strm || strm.bad() || strm.fail() || !strm.good() )
    {
	cout << "readListFromFile: Drat! Failed to open file "
	     << ((fname==0)?("<NULL>"):(fname)) << "!" << endl;
	return retval;
    }
    
    //
    //  Ok. Read the file!
    //
    retval = True;
    while( !strm.eof() )
    {
	strm.getline( linebuf, sizeof(linebuf) );

	//
	//  Rub out comments
	//
	if( (ptr=strchr(linebuf, '#'))!=0 )
	{
	    *ptr = '\0';
	}
	
	//
	//  Filter out whitespace....
	//
	ptr  = linebuf;
	optr = outbuf;
	while( *ptr )
	{
	    if( !strchr(" \t\f\r", *ptr) )
	    {
		*optr = *ptr;
		optr++;
	    }
	    ptr++;
	}
	*optr = '\0';

	//
	//  If there still are characters left, it must be a buffername!
	//
	if( strlen(outbuf) )
	{
	    if( (buflist=(char**)realloc(buflist, (nbuf+1)*sizeof(char*)))!=NULL )
	    {
		buflist[ nbuf ] = strdup( outbuf );
		nbuf++;
	    }
	    else
	    {
		buflist = 0;
		nbuf    = 0;
		retval  = False;
		
		cout << "readListFromFile: Aargh! Not enough mem?! Ask Huib for more memory..."
		     << endl;
		break;
	    }
	}
    }
    strm.close();
    return retval;
}



int main( int argc, char** argv )
{

#ifdef BUILDTIME
	char*                 thisexe;

	if( (thisexe=::basename(argv[0]))==0 )
	{
		thisexe = argv[0];
	}
	cout << "\n"
		 << thisexe << ": Version of " << BUILDTIME << " begins..."
		 << "\n\n" 
		 << flush;
#endif

    try
    {
	// Should we run interactively?
	Bool               interactive( True );  

	// Did we get a file with buffers to translate?
	// NB: need to keepo track of this since it is mutually exclusive to any other
	//     method of entering buffers!
	Bool               listfromfile( False );

	// Optional connection to server for debugging (unlisted cmd-line option!)
	char*              connection( 0 );

	// Outputname -> used if non-NULL
	char*              outname( 0 ); 
        
	// Buffers that need translation -> if this is NULL -> interactively!
	char**             buffers2do( 0 );
	unsigned int       nrbuffers2do( 0 );

	// In what domain do we want the data?
	Visibility::domain dstdomain( Visibility::undefined );

	//
	//  Before we do anything, we tell the system what to do when
	//  specific signals are raised...
	//
	signal( SIGINT,  throwOnSignal );
	signal( SIGKILL, throwOnSignal );
	signal( SIGTERM, throwOnSignal );
	signal( SIGSEGV, throwOnSignal );

	//
	//  Ok. Do some commandline parsing. If there are entries on
	//  the cmd-line that are not options, we assume the
	//  usr. wants to run non-interactively. So what we do is to
	//  parse the cmdline and base our conclusions on the outcome
	//  of that!
	//
	while( *(++argv) )
	{
	    switch( *(*argv) )
	    {
		case '-':
		    switch( *((*argv)+1) )
		    {
			case 'c':
			    //
			    //  Expect: -c <connection>
			    //
			    if( !(*(argv+1)) || (**(argv+1)=='-') )
			    {
				cout << "Missing argument to -c option!" << endl;
				break;
			    }

			    ++argv;
			    connection = *argv;
			    break;

			case 'd':
			    //
			    // Expect: -d <domain>
			    //
			    if( !(*(argv+1)) || (**(argv+1)=='-') )
			    {
				cout << "Missing argument to -d option!" << endl;
				break;
			    }
			    
			    ++argv;
			    if( !strcmp(*argv, "time") )
			    {
				dstdomain = Visibility::time;
			    }
			    else if( !strcmp(*argv, "frequency") )
			    {
				dstdomain = Visibility::frequency;
			    }
			    else
			    {
				cout << "Unrecognized domain `" << *argv << "'" << endl;
			    }
			    break;
			    
			case 'o':
			    //
			    // Expect: -o <outputfile>
			    //
			    if( !(*(argv+1)) || (**(argv+1)=='-') )
			    {
				cout << "Missing argument to -o option!" << endl;
				break;
			    }
			    ++argv;
			    outname = *argv;
			    break;

			case 'f':
			    //
			    // Expect: -f <file with buffer-ids>
			    //
			    if( !(*(argv+1)) || (**(argv+1)=='-') )
			    {
				cout << "Missing argument to -f option!" << endl;
				break;
			    }
			    ++argv;

			    if( listfromfile )
			    {
				cout << "You can use the file-option only once!\n"
				     << "Ignoring this one " << *argv << endl;
				break;
			    }
			    if( buffers2do )
			    {
				cout << "You cannot mix two styles of buffer-specification, you\n"
				     << "have already specified buffers on the cmd-line hence the\n"
				     << "buffers in file " << *argv << " will be ignored." << endl;
				break;
			    }

			    if( readListFromFile(*argv, buffers2do, nrbuffers2do) )
			    {
				listfromfile = True;
			    }
			    break;
			    
			    
			default:
			    cout << "Unrecognized option '" << *argv << "'" << endl;
			    break;
		    }
		    break;
		    
		default:
		    static Bool  errorshown = False;
		    
		    //
		    //  If listfromfile==True -> stoopid usr has mixed two type of specification!
		    //
		    if( listfromfile )
		    {
			if( !errorshown )
			{
			    cout << "You cannot mix two styles of buffer-specification, you\n"
				 << "have already read buffers from a file; hence the\n"
				 << "buffers on the cmd-line will be ignored." << endl;
			    errorshown = True;
			}
			cout << "Discarding cmd-line buffer specification " << *argv << endl;
			break;
		    }
		    
		    //
		    //  Assume it is the name of a buffer....
		    //
		    if( (buffers2do=(char**)realloc(buffers2do, (nrbuffers2do+1)*sizeof(char*)))!=NULL )
		    {
			buffers2do[ nrbuffers2do ] = strdup( *argv );
			nrbuffers2do++;
		    }
		    break;
	    }
	}
	
	//
	//  Now we can really start...
	//
	String            scanid;
	Experiment*       theExperiment;
	JIVEMSFiller      theFiller( connection );
	RegularExpression freqdomain( "^[ \t]*frequency[ \t]*$" );
	RegularExpression timedomain( "^[ \t]*time[ \t]*$" );


	theExperiment = 0;
	
	try
	{
	    theExperiment = new VEXperiment();
	    cout << "\n\n"
		 << "**** Detected VEX-periment (assumes COF-datafiles)..."
		 << "\n\n" << flush;
	}
	catch( AIPSExceptionObject y )
	{
	    y.getMesg();
	    theExperiment = 0;
	}
	catch( const ExceptionObject& )
	{
	    theExperiment = 0;
	}
	

	if( !theExperiment )
	{
	    try
	    {
		theExperiment = new IniExperiment();
		cout << "\n\n"
		     << "**** Detected ini-file experiment (assumes Alberts format datafiles)..."
		     << "\n\n" << flush;
	    }
	    catch( AIPSExceptionObject x )
	    {
		x.getMesg();
		theExperiment = 0;
	    }
	    catch( const ExceptionObject& )
	    {
		theExperiment = 0;
	    }
	}

	if( !theExperiment )
	{
	    throw( EXCEPTION("Experiment is neither COF nor Alberts format") );
	}

	//
	//  Now decide if we are running interactively or not.  If
	//  usr. did not specify any filenames on the cmdline, we ask
	//  for it; i.e. we run interactively!
	//
	interactive = ToBool(nrbuffers2do==0);
	
	//
	//  See if usr. specified some other options on the
	//  commandline!
	//
	if( dstdomain!=Visibility::undefined )
	{
	    //
	    // It was set by the usr!
	    //
	    theExperiment->setDestinationDomain( dstdomain );
	}

	if( outname )
	{
	    //
	    // It was set by the usr!
	    //
	    theExperiment->setMSname( (const char*)outname );
	}
	
	//
        //  Show exp. summary to usr.
        //
	cout << *theExperiment << endl;

	//
	//  Either run interactively or not!
	//
	if( interactive )
	{
	    while( 1 )
	    {
		cin.tie( &cout );
		cout << "DataBuffer: ";
		cin >> scanid;
		cin.tie( 0 );

		if( scanid=="quit" )
		{
		    break;
		}
		
		if( scanid==freqdomain )
		{
		    theExperiment->setDestinationDomain( Visibility::frequency );
		}
		else if( scanid==timedomain )
		{
		    theExperiment->setDestinationDomain( Visibility::time );
		}
		else if( !theFiller.fillDataToExperimentsMS( *theExperiment, scanid, true) )
		{
		    cout << "Failed to write data to MS" << endl;
		}
	    }
	    //
	    // Done!
	    //
	}
	else
	{
	    //
	    //  Do not run interactively...
	    //
	    for( unsigned int i=0; i<nrbuffers2do; ++i )
	    {
		buffers2do[i] && theFiller.fillDataToExperimentsMS( *theExperiment, buffers2do[i], false);
	    }
	}
	
	//
	//  Free memory!
	//
	for( unsigned int j=0; j<nrbuffers2do; ++j )
	{
	    if( buffers2do[j] )
	    {
		free( buffers2do[j] );
	    }
	}
	free( buffers2do );
	buffers2do = 0;
	
	//
	//  Don't need the experiment anymore...
	//
	delete theExperiment;
    }
    catch( AIPSExceptionObject x )
    {
	cout << "Rats! " << x.getMesg() << endl << " ("
	     << x.thrownFile() << ":" << x.thrownLine() << ")" << endl;
    }
    catch( const ExceptionObject& y )
    {
	cout << "Aargh - ..." << endl << y << endl;
    }
    
    return 0;
}
