//
//
//
#include <iostream.h>

#include <aips/Utilities/String.h>
#include <vlbi/tmplsrc/ConverterBase.h>
#include <vlbi/ms2uvfitsimpl/Converters/ms2uvfitsConverter.h>
#include <vlbi/ms2uvfitsimpl/Converters/uvfits2msConverter.h>

#ifndef LINEARCONTAINER_H_INC
#include <vlbi/tmplsrc/LinearContainer.h>
#endif

//
//  Splitting by rownrs
//
#include <vlbi/hvplugin/rowSplit.h>

#include <signal.h>
#include <libgen.h>

#ifndef BUILDTIME
#define BUILDTIME "<no buildtime info present>"
#endif


void Usage( const char* prgname )
{
    cout << "Usage: " << prgname << " [option] <source> <destination>" << endl;
    cout << "       where option can be:" << endl;
    cout << "         -m2f  -> conversion from MS (source) to FITSfile (UVFITS for AIPS)" << endl;
    cout << "         -m2i  -> conversion from MS (source) to IDI FITSfile (VLBA format)" << endl;
    cout << "         -wm2f -> conversion from Westerbork MS (source) to FITSfile (destination)"
	 << endl;
    cout << "         -f2m  -> conversion from FITSfile (source) to MS (destination)" << endl;
    cout << "           default is m2f" << endl;
    //cout << "         -v    -> show lots of info (if apropriate)" << endl;
    return;
}

void throwonsignal( int signalnr )
{
    ostrstream   buf;
    
    buf << endl << "**** Caught a signal #" << signalnr << "..." << endl;
    
    throw( AipsError(buf) );
}



int main( int, char** argv )
{
    int                   retval( 1 );
    int                   errorcount( 0 );
    String                thisprogname;
    String                infile;
    String                outfile;
    String                method( "m2f" );
    String                splitter( "default" );
    FileFormatConverter*  ptr2converter( 0 );
    
#ifdef BUILDTIME
	char*                 thisexe;

	if( (thisexe=::basename(argv[0]))==0 )
	{
		thisexe = argv[0];
	}
	cout << "\n"
		 << thisexe << ": Version of " << BUILDTIME << " begins..."
		 << "\n\n" 
		 << flush;
#endif



    //
    //  Set the handler for various signals!
    //
    sigset( SIGSEGV, throwonsignal );
    sigset( SIGBUS, throwonsignal );
    sigset( SIGINT, throwonsignal );
    sigset( SIGKILL, throwonsignal );
    

    //
    //  Get the name of this program
    //
    thisprogname=*argv;
    
    if( !*(++argv) )
    {
        Usage( thisprogname );
        return retval;
    }

    while( *argv )
    {
        String    curarg( *argv );
        
        if( curarg[0]!='-' )
        {
            //
            //  Must be a filename?
            //
            if( !(infile.length()) )
            {
                infile=curarg;
            }
            else if( !(outfile.length()) )
            {
                outfile=curarg;
            }
            else
            {
                cout << "You supplied too much arguments. This one - " << curarg 
                     << " - will be discarded." << endl;
                errorcount++;
            }
        }
        else
        {
            //
            //  Check if valid option:
            //
            String     opt( curarg.after(0) );
            
            if( opt=="m2f" || opt=="f2m" || opt=="wm2f" || opt=="m2i" )
            {
                method=opt;
            }
	    else if( opt=="row" )
	    {
		splitter = opt;
	    }
            else
            {
                cout << "Unrecognized option: " << opt << ". Ignoring." << endl;
            }
        }

        //
        //  It might be an idea to increase the arg-counter....
        //
        argv++;
    }
    
    if( !infile.length() )
    {
        cout << "Hmm... you specified no inputfile!" << endl;
        errorcount++;
    }
    
    if( !outfile.length() )
    {
        cout << "What? No outputfile?!" << endl;
        errorcount++;
    }
    

    if( errorcount!=0 )
    {
        cout << endl << "------------ Summary ------------" << endl;
        cout << errorcount << " error" << ((errorcount==1)?(" was"):("s were")) 
             << " detected. Now showing what you should have done:" << endl;
        Usage( thisprogname );
        return errorcount;
    }
    
    //
    // Depending on what the user entered, create the appropriate
    // transformation!
    //
    LinearContainer<MS2UVF::generator_t>  hdustodo;

    //
    //  These hdu's need to be written anyway....
    //
    hdustodo.add( MS2UVF::mainprimarygroup );
    hdustodo.add( MS2UVF::antennabinary );
    hdustodo.add( MS2UVF::freqbinary );
    hdustodo.add( MS2UVF::sourcebinary );
    
    
    if( method=="m2f" )
    {
	MS2UVF*                              newconverter( new MS2UVF );
	
	//
	//  Set up the conversion....(standard conversion)
	//
	newconverter->setConversion( hdustodo );
	
        ptr2converter= newconverter;
    }
    else if( method=="m2i" )
    {
	MS2UVF*                              newconverter( new MS2UVF );
	
	hdustodo.resize( 0 );
	hdustodo.add( MS2UVF::idiheader );
	hdustodo.add( MS2UVF::arraygeometry );
	hdustodo.add( MS2UVF::idifrequency );
	hdustodo.add( MS2UVF::idisource );
	hdustodo.add( MS2UVF::idiantenna );
	hdustodo.add( MS2UVF::uvtable );
	//hdustodo.add( MS2UVF::antennabinary );
	//hdustodo.add( MS2UVF::freqbinary );
	//hdustodo.add( MS2UVF::sourcebinary );	


	//
	//  Set up the conversion....(standard conversion)
	//
	newconverter->setConversion( hdustodo );

	if( splitter=="row" )
	{
	    newconverter->setFragmenter( rowSplit );
	}

        ptr2converter= newconverter;

    }    
    else if( method=="wm2f" )
    {
	MS2UVF*                              newconverter( new MS2UVF );
	
	//
	//  Set up the conversion.... add some stuff for the
	//  westerbork MSs
	//
	hdustodo.add( MS2UVF::tsysbinary );
	hdustodo.add( MS2UVF::gaincurvebinary );

	newconverter->setConversion( hdustodo );

        ptr2converter= newconverter;
    }
    else if( method=="f2m" )
    {
        ptr2converter= new uvfits2msConverter;
    }
    else
    {
        cout << "AArrgghh - invalid conversion format (" << method << ") encountered." << endl;
        return 1;
    }
    

    //
    //  Do the transformation....
    //
    if( !(ptr2converter->convert(outfile, infile)) )
    {
        cout << "Rats. Tranformation not succeeded" << endl;
    }
    else
    {
        retval=0;
    }
    
    //
    //  Make sure the transformer is deleted correctly
    //
    delete ptr2converter;
    

    return retval;
}
