//
//  Implementation of the BaseLine class.....
//
//  Author: Harro Verkouter,   20-11-1998
//
//  $Id: BaseLine.cc,v 1.1.1.1 2000/03/20 15:05:34 verkout Exp $
//
#ifndef BASELINE_H
#include <vlbi/DtdLUT/BaseLine.h>
#endif


ostream& operator<<( ostream& os, const BaseLine& blref )
{
    os << blref.blcode << " -> correlations: ";
    for( uint16 i=0; i<4; i++ )
    {
	if( blref.correlationpresent[i] )
	{
	    os << CorrelationCode::ordinal2code(i) << " (" << blref.correlations(i) << ")  ";
	}
    }
    return os;
}

BaseLine::BaseLine() :
    correlationpresent( Container<bool>(4, false) ),
    correlations( Container<InterferometerDescr>( 4 ) )
{
}

BaseLine::BaseLine( const BaseLine& other ) :
    correlationpresent( other.correlationpresent ),
    blcode( other.blcode ),
    correlations( other.correlations )
{
}

BaseLine& BaseLine::operator=( const BaseLine& other )
{
    if( this!=&other )
    {
	correlationpresent = other.correlationpresent;
	blcode = other.blcode;
	correlations = other.correlations;
    }
    return *this;
}

BaseLine& BaseLine::operator=( const BaseLineIdx& newidx )
{
    blcode=newidx;
    return *this;
}

const BaseLineIdx& BaseLine::getBaseLineIdx( void ) const
{
    return blcode;
}

void BaseLine::addCorrelation( const InterferometerDescr& ifref, const CorrelationCode& ccref )
{
    uint16            ordinalidx( CorrelationCode::code2ordinal(ccref) );
    uint16            transposedidx;
    CorrelationCode   transposed( ccref.idx2(), ccref.idx1() );
    
    transposedidx = CorrelationCode::code2ordinal( transposed );
    
    if( correlationpresent[ordinalidx] )
    {
	cout << "*** " << blcode << " " << ccref << " (" << ifref << ") is duplicated, previous "
	     << "interferometer was " << correlations(ordinalidx)  << endl;
	return;
    }
    
    if( !correlationpresent[ordinalidx] )
    {
	correlationpresent[ordinalidx]=true;
	correlations[ordinalidx]=ifref;
    }
    
    
    if( ccref.idx1()!=ccref.idx2() )
    {
	//
	//  cross polarization
	//
	if( blcode.idx1()==blcode.idx2() )
	{
	    // auto-correlation; add the transposed idx!
	    if( !correlationpresent[transposedidx] )
	    {
		correlationpresent[transposedidx]=true;
		correlations[transposedidx]=ifref;
	    }
	}
    }
}


Container<IFinfo> BaseLine::getConfiguredCorrelations( void ) const    
{
    IFinfo            tmp;
    Container<IFinfo> retval;
    
    for( int i=0; i<4; i++ )
    {
	if( correlationpresent(i) )
	{
	    tmp.itsIF = correlations(i);
	    tmp.itsCode = CorrelationCode::ordinal2code(i);
	    
	    retval.add( tmp );
	}
    }
    return retval;
}



BaseLine::~BaseLine()
{
}
