//
// Class BaseLine:
//
//   Holds all possible info on a baseline; like what the baselinecode
//   is and what correlations are calculated on this baseline
//
#ifndef BASELINE_H
#define BASELINE_H

#ifndef HVTYPES_H
#include <vlbi/DtdLUT/hvtypes.h>
#endif

#ifndef INTERFEROMETERDESCR_H
#include <vlbi/DtdLUT/InterferometerDescr.h>
#endif

#ifndef CORRELATIONCODE_H
#include <vlbi/DtdLUT/CorrelationCode.h>
#endif

#ifndef CONTAINER_H
#include <vlbi/tmplsrc/Container.h>
#endif

#ifndef BASELINEIDX_H
#include <vlbi/DtdLUT/BaseLineIdx.h>
#endif

#include <iostream.h>


typedef struct _IFinfo
{
    CorrelationCode        itsCode;
    InterferometerDescr    itsIF;
} IFinfo;
    


//
//  A baseline has a code and max 4 correlations!
//
class BaseLine
{
    friend ostream& operator<<( ostream& os, const BaseLine& blref );
    
            
public:
    //
    //  Default c'tor
    //
    BaseLine();

    BaseLine( const BaseLine& other );
    BaseLine& operator=( const BaseLine& other );
    
    //
    //  Set the baselineidx member of this BaseLine
    //
    BaseLine& operator=( const BaseLineIdx& newidx );

    const BaseLineIdx& getBaseLineIdx( void ) const;

    void addCorrelation( const InterferometerDescr& ifref, const CorrelationCode& ccref );

    //
    // Return the interferometernr of the  indexed correlation
    //
    inline uint32 operator()( const CorrelationCode& ccode ) const
    {
	//uint16     linearindex( (uint16)ccode );
	uint16     linearindex( CorrelationCode::code2ordinal(ccode) );
	uint32     retval( (uint32)-1 );
	
	if( correlationpresent[linearindex] )
	{
	    retval= (uint32)( correlations[linearindex] );
	}
	return retval;
    }

    //
    //  Return which correlations are present
    //
    Container<IFinfo>  getConfiguredCorrelations( void ) const;
    
    //
    //  Desctruct the object!
    //
    ~BaseLine();

private:
    //
    //  The BaseLine's private parts
    //
    Container<bool>                correlationpresent;
    BaseLineIdx                    blcode;
    Container<InterferometerDescr> correlations;

    //
    // I think it's a wise thing to prohobit these
    //
};




#endif
