//
// BaseLineIdx, class derived from DualIndex, holding two
// antennanumbers
//
//
#ifndef BASELINEIDX_H
#define BASELINEIDX_H

//
//  My typedefs
//
#ifndef HVTYPES_H
#include <vlbi/DtdLUT/hvtypes.h>
#endif

//
//  Alberts Types
//
#ifndef ALBERTTYPEDEFS_H
#include <vlbi/DtdLUT/AlbertTypedefs.h>
#endif

//
//  We need this, it's our baseclass
//
#ifndef DUALINDEX_H
#include <vlbi/DtdLUT/DualIndex.h>
#endif



class BaseLineIdx :
    public DualIndex
{
public:
    //
    //  zero-BaseLineIdx with maximum nr of stations
    //
    BaseLineIdx( uint16 nrstations=32 ) :
	DualIndex( nrstations )
    {
    }
    
    //
    //  full BaseLineIdx
    //
    BaseLineIdx( uint16 st1, uint16 st2, uint16 nrstations=32 ) :
	DualIndex( nrstations )
    {
	idxval1=st1;
	idxval2=st2;
    }
    
    //
    // Create a BaseLineIdx from an INTERFEROMETER DESCR
    //
    //BaseLineIdx( uint16 nrstations, const ifdescr& ifref ) : 
    //	DualIndex( nrstations )
    //{
    //     idxval1=ifref.XANTNR;
    //   idxval2=ifref.YANTNR;
    //}

    //
    //  Attach the BaseLineIdx to a interferometer
    //
    inline BaseLineIdx& operator=( const ifdescr& ifref )
    {
        idxval1=ifref.XANTNR;
        idxval2=ifref.YANTNR;

	//itsXStationUnit = ifref.XSIGOUTP/8;
	//itsYStationUnit = ifref.YSIGOUTP/8;
	
        return *this;
    }
    
    //
    //  Assign one to the other
    //
    inline BaseLineIdx& operator=( const BaseLineIdx& other )
    {
        if( this!=&other )
        {
            idxval1=other.idxval1;
            idxval2=other.idxval2;
            maxidx=other.maxidx;
        }
        return *this;
    }

    //
    //  Comparison
    //
    inline bool     operator==( const BaseLineIdx& rval ) const
    {
	return ( this->DualIndex::idxval1==rval.DualIndex::idxval1 &&
		 this->DualIndex::idxval2==rval.DualIndex::idxval2 &&
		 this->DualIndex::maxidx==rval.DualIndex::maxidx );
    }

    inline bool     operator!=( const BaseLineIdx& rval ) const
    {
	return !(this->operator==(rval));
    }
    

    //
    //  Empty:
    //
    ~BaseLineIdx()
    {
    }
private:
};



#endif
