//
//  CorrelationCode - clas to help in UNIQUELY map correlationnr to
//  correlation code and back
//
//
//
#ifndef CORRELATIONCODE_H
#define CORRELATIONCODE_H

//
//  My typedefs
//
#ifndef HVTYPES_H
#include <vlbi/DtdLUT/hvtypes.h>
#endif

//
//  Alberts Types
//
#ifndef ALBERTTYPEDEFS_H
#include <vlbi/DtdLUT/AlbertTypedefs.h>
#endif

//
//  We need this, it's our baseclass
//
#ifndef DUALINDEX_H
#include <vlbi/DtdLUT/DualIndex.h>
#endif

class CorrelationCode :
    public DualIndex
{
public:
    //
    //  a default correlation code; there are only two possibilities
    //  on any axis!
    //
    CorrelationCode( );
    
    //
    //  Create a defined correlationcode
    //
    CorrelationCode( uint16 pol1, uint16 pol2 ) :
	DualIndex( 2 )
    {
        idxval1=pol1;
        idxval2=pol2;
    }


    //
    //  Create it from an interferometer
    //
    CorrelationCode( const ifdescr& ifref ) : DualIndex( 2 )
    {
        idxval1=ifref.XPOLNR;
        idxval2=ifref.YPOLNR;
    }

    inline CorrelationCode& operator=( const ifdescr& ifref )
    {
        idxval1=ifref.XPOLNR;
        idxval2=ifref.YPOLNR;
        
        return *this;
    }

    //
    //  The correlation needs to map from code to ordinal number and
    //  back. Why? Usually, the correlation codes are labelled like
    //  this (for. e.g R/L polarization combinations):
    //
    //  ordinal number    correlation code     polarization code
    //       0              (0,0)                 (R,R)
    //       1              (1,1)                 (L,L)
    //       2              (1,0)                 (R,L)
    //       3              (0,1)                 (L,R)
    //
    static inline uint16 code2ordinal( const CorrelationCode& ccref )
    {
        if( !CorrelationCode::mappingsok )
        {
            CorrelationCode::initMappings();
        }

        return maplinear2ordinal[ ((uint16)ccref) ];
    }
    
    static inline CorrelationCode ordinal2code( uint16 ordnr )
    {
        if( !CorrelationCode::mappingsok )
        {
            CorrelationCode::initMappings();
        }

        return mapordinal2code[ordnr];
    }

    //
    //  Destruct
    //
    virtual ~CorrelationCode();
    
private:
    //
    //  The mapping function from correlation-code linear index to
    //  ordinal number!
    //
    static uint16           maplinear2ordinal[];
    static bool             mappingsok;
    static CorrelationCode  mapordinal2code[];

    //
    //  Private member function: initialize the mappings
    //
    static void              initMappings( void )
    {
        mapordinal2code[0]=CorrelationCode( 0, 0 );
        mapordinal2code[1]=CorrelationCode( 1, 1 );
        mapordinal2code[2]=CorrelationCode( 1, 0 );
        mapordinal2code[3]=CorrelationCode( 0, 1 );

        maplinear2ordinal[ ((uint16)CorrelationCode(0,0)) ]=0;
        maplinear2ordinal[ ((uint16)CorrelationCode(1,1)) ]=1;
        maplinear2ordinal[ ((uint16)CorrelationCode(1,0)) ]=2;
        maplinear2ordinal[ ((uint16)CorrelationCode(0,1)) ]=3;

        mappingsok=true;
    }
};

//
//  Declare statics
//
#ifdef CORRELATIONCODE_CC

uint16           CorrelationCode::maplinear2ordinal[4];
bool             CorrelationCode::mappingsok=false;
CorrelationCode  CorrelationCode::mapordinal2code[4];

#endif



#endif
