//
//  Tell 'm we're compiling DtdLUT.cc
//
#define DTDLUT_CC

#ifndef DTDLUT_H
#include <vlbi/DtdLUT/DtdLUT.h>
#endif

#ifndef VISLABEL_H
#include <vlbi/DtdLUT/VisLabel.h>
#endif

#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

ostream& operator<<( ostream& os, const DtdLUT& dref )
{
    os << "LookupTable summary, configured " << dref.jobs.elements() << " jobs..." << endl;
    os << dref.jobs << endl;
    return os;
}


//
//  Nothing....
//
DtdLUT::DtdLUT()
{
}

DtdLUT::DtdLUT( const dtd& dtdref )
{
    attach( dtdref );
}

bool DtdLUT::attach( const dtd& dtdref )
{
    //
    //  Parse the dtd and fill in ourselves!
    //
    uint16                njob( dtdref.NJOB );
    uint16                nsubarray;
    uint16                nintfr;
    uint16                subidxstart;
    uint16                ifidxstart;
    uint16                intfrcnt;
    uint16                subarcnt;
    uint16                ifzerobasednr;
    BaseLineIdx         bl;
    CorrelationCode     ccode;
    InterferometerDescr curif;



    jobs.resize( njob );
    
    for( uint16 jobcnt=0; jobcnt<njob; jobcnt++ )
    {
        const jobdescr&  curjobref( dtdref.JOB_DATADESCR_ARRAY[jobcnt] );
        
        nsubarray=curjobref.NSUBARRAY;
        subidxstart=curjobref.SA_DINDEX;
        
        //jobs(jobcnt).subarrays.resize( nsubarray );
    
        for( subarcnt=0; subarcnt<nsubarray; subarcnt++ )
        {
            const sadescr& saref( dtdref.SA_DATADESCR_ARRAY[ subidxstart+subarcnt ] );
            SubArrayDescr  cursa;
            
            nintfr=saref.NINTFR;
            ifidxstart=saref.IF_DINDEX;

            for( intfrcnt=0,ifzerobasednr=ifidxstart; intfrcnt<nintfr; intfrcnt++,ifzerobasednr++ )
            {
		Bool            swapit( False );
                const ifdescr&  ifref( dtdref.INTFR_DATADESCR_ARRAY[ ifzerobasednr ] );

		//
		//  Set the interferometer's zero-based nr
		//
                curif=ifzerobasednr;

		//
		//  Set the baseline code
		//
                bl=ifref;

		if( bl.idx1()>bl.idx2() )
		{
		    bl = BaseLineIdx( bl.idx2(), bl.idx1() );
		    swapit = True;
		}
		
		//
		//  Get the correlation code
		//
                ccode=ifref;

		if( swapit )
		{
		    ccode = CorrelationCode( ccode.idx2(), ccode.idx1() );
		}
		
                cursa.addBaseLine( bl, curif, ccode, ifref.SBANDNR );
                
            }

            //
            //  This subarray has been processed, now add it to the job!
            //
            jobs(jobcnt).addSubArray( cursa );
        }
    }
    return true;
}

uint16 DtdLUT::lookup( const VisLabel& idx ) const
{
    //
    //  Try to lookup as fast as possible the if the userrequests!
    //
    BaseLineIdx     blidx( idx.baseline() );
    CorrelationCode ccode;
    
    ccode = CorrelationCode::ordinal2code( idx.correlationnr() );
    
    return (((jobs(idx.jobnr()))(idx.subarraynr()))(blidx,idx.subbandnr()))(ccode);
}

//
//  Delete all resources....
//
DtdLUT::~DtdLUT()
{
}

Container<JobDescr>& DtdLUT::jobarray( void )
{
    return jobs;
}

const Container<JobDescr>& DtdLUT::jobarray( void ) const
{
    return jobs;
}



//
//  The DtdHack-class!
//
DtdHack::DtdHack() :
    DtdLUT()
{
    
}

DtdHack::DtdHack( const dtd& dtdref ) :
    DtdLUT( dtdref )
{
}

bool DtdHack::attach( const dtd& dtdref )
{
    //
    //  Parse the dtd and fill in ourselves!
    //
    uint16                njob( dtdref.NJOB );
    uint16                nsubarray;
    uint16                nintfr;
    uint16                subidxstart;
    uint16                ifidxstart;
    uint16                intfrcnt;
    uint16                subarcnt;
    uint16                ifzerobasednr;
    uint16                xsubbandnr;
    uint16                ysubbandnr;
    BaseLineIdx         bl;
    CorrelationCode     ccode;
    InterferometerDescr curif;


    jobarray().resize( njob );
    
    for( uint16 jobcnt=0; jobcnt<njob; jobcnt++ )
    {
        const jobdescr&  curjobref( dtdref.JOB_DATADESCR_ARRAY[jobcnt] );
        
        nsubarray=curjobref.NSUBARRAY;
        subidxstart=curjobref.SA_DINDEX;
        
        //jobs(jobcnt).subarrays.resize( nsubarray );
    
        for( subarcnt=0; subarcnt<nsubarray; subarcnt++ )
        {
            const sadescr& saref( dtdref.SA_DATADESCR_ARRAY[ subidxstart+subarcnt ] );
            SubArrayDescr  cursa;
            
            nintfr=saref.NINTFR;
            if( nintfr>288 )
            {
                nintfr=288;
            }
            
            ifidxstart=saref.IF_DINDEX;

            for( intfrcnt=0,ifzerobasednr=ifidxstart; intfrcnt<nintfr; intfrcnt++,ifzerobasednr++ )
            {
                const ifdescr&  ifref( dtdref.INTFR_DATADESCR_ARRAY[ ifzerobasednr ] );

                //
                //  This is the hack: from the XSIGSRC/YSIGSRC members
                //  we derive the 'antenna' (in real life, the
                //  SU-number), and the subbandnr
                //
                curif=ifzerobasednr;

                bl.idx1()=ifref.XSIGOUTP/8;
                bl.idx2()=ifref.YSIGOUTP/8;
                xsubbandnr=ifref.XSIGOUTP%8;
                ysubbandnr=ifref.YSIGOUTP%8;
                
                ccode=CorrelationCode( 0, 0 );

                if( xsubbandnr==ysubbandnr )
                {
                    cursa.addBaseLine( bl, curif, ccode, xsubbandnr );
                }
            }

            //
            //  This subarray has been processed, now add it to the job!
            //
            jobarray()(jobcnt).addSubArray( cursa );
        }
    }
    return true;
}

uint16 DtdHack::lookup( const VisLabel& idx ) const
{
    //
    //  Try to lookup as fast as possible the if the userrequests!
    //
    BaseLineIdx     blidx( idx.baseline() );
    CorrelationCode ccode;
    
    ccode = CorrelationCode::ordinal2code( idx.correlationnr() );
    
    return ((((jobarray()(idx.jobnr())))(idx.subarraynr()))(blidx,idx.subbandnr()))(ccode);
}



DtdHack::~DtdHack()
{
    //
    //  Nothing....
    //
}



ExposedDtd::ExposedDtd() :
    DtdLUT()
{
}

ExposedDtd::ExposedDtd( const dtd& dtdref ) :
    DtdLUT( dtdref )
{
}

Container<JobDescr>& ExposedDtd::jobs( void )
{
    return jobarray();
}

const Container<JobDescr>& ExposedDtd::jobs( void ) const
{
    return jobarray();
}


ExposedDtd::~ExposedDtd()
{
}
