//
//  Class JobDescr - keeps all job related stuff
//
//
//
#ifndef JOBDESCR_H
#define JOBDESCR_H

#ifndef HVTYPES_H
#include <vlbi/DtdLUT/hvtypes.h>
#endif

#ifndef CONTAINER_H
#include <vlbi/DtdLUT/Container.h>
#endif

#ifndef BASELINEIDX_H
#include <vlbi/DtdLUT/BaseLineIdx.h>
#endif

#ifndef SUBARRAYDESCR_H
#include <vlbi/DtdLUT/SubArrayDescr.h>
#endif

#include <iostream.h>


class JobDescr
{
    friend ostream& operator<<( ostream& os, const JobDescr& jref )
    {
	os << "JOB summary: " << endl;
	os << "Configured " << jref.subarrays.elements() << " subarrays." << endl;
	os << jref.subarrays << endl;
	return os;
    }
    

public:

    //
    //  Return a list of baselines for the requested
    //  sub-array/subband. Container is empty when sa does not exist
    //
    Container<BaseLineIdx>  getBaseLines( uint16 sanr, uint16 sbnr ) const
    {
        if( sanr>=subarrays.elements() )
        {
            return Container<BaseLineIdx>();
        }
        
        return subarrays(sanr).getBaseLines( sbnr );
    }
    
    uint16  getNrSubarrays( void ) const
    {
	return subarrays.elements();
    }

    const SubArrayDescr& getSubarray( uint16 which ) const
    {
	static SubArrayDescr   empty;
	
	if( which>=subarrays.elements() )
	{
	    return empty;
	}
	return subarrays(which);
    }
    

    //
    //  Return a list of all stations in this experiment
    //
    Container<int16>          getStations( void ) const
    {
        Container<int16>   retval;
        
        for( uint16 i=0; i<stationflags.elements(); i++ )
        {
            if( stationflags(i) )
            {
                retval.add( (int16)i );
            }
        }
        return retval;
    }

    //
    //  Add a subarray to this job
    //
    void addSubArray( const SubArrayDescr& newsa )
    {
        //
        //  Check the subarray and filter out the stations that are
        //  present!
        //
        Container<BaseLineIdx>   baselines=newsa.getBaseLines( 0 );
        
        for( uint16 i=0; i<baselines.elements(); i++ )
        {
            uint16     st1( baselines(i).idx1() );
            uint16     st2( baselines(i).idx2() );
            uint16     maxst( st1 );
            
            if( st2>maxst )
            {
                maxst=st2;
            }

            if( stationflags.elements()<=maxst )
            {
                stationflags.resize( maxst+1, false );
            }
            
            if( !stationflags(st1) )
            {
                stationflags(st1)= true;
            }
            if( !stationflags(st2) )
            {
                stationflags(st2)= true;
            }
        }
        subarrays.add( newsa );
    }

    //
    //  return a subarray...
    //
    inline const SubArrayDescr& operator()( uint16 nr ) const
    {
        return subarrays( nr );
    }

private:
    //
    //  A job consist of a number of subarrays
    //
    Container<SubArrayDescr>     subarrays;

    //
    //  Every time a subarray is added, we update these lists, to keep
    //  a record of which stations are present!
    //
    Container<bool>              stationflags;
};



#endif
