//
//  Class SubArrayDescr - describes a complete subarray (including per
//  subband!)
//
#ifndef SUBARRAYDESCR_H
#define SUBARRAYDESCR_H

#ifndef HVTYPES_H
#include <vlbi/DtdLUT/hvtypes.h>
#endif

#ifndef CONTAINER_H
#include <vlbi/DtdLUT/Container.h>
#endif

#ifndef BASELINE_H
#include <vlbi/DtdLUT/BaseLine.h>
#endif

#include <iostream.h>



class SubArrayDescr
{
    friend ostream& operator<<( ostream& os, const SubArrayDescr& saref )
    {
	os << "Subarray summary..." << endl;
	
	for( uint16 i=0; i<saref.blflags.elements(); i++ )
	{
	    os << endl << "*** Subband #" << i << endl;
	    
	    for( uint16 j=0; j<saref.blflags(i).elements(); j++ )
	    {
		if( saref.blflags(i)(j) )
		{
		    os << "*** " << saref.baselines(i)(j) << endl;
		}
	    }
	}
	return os;
    }
    

public:

    //
    //  return a list of configured baselines for this subarray/subband
    //
    Container<BaseLineIdx>   getBaseLines( uint16 sbnr ) const
    {
        Container<BaseLineIdx>   retval;
        
        //
        // Check if adressed subband is Ok.
        //
        if( sbnr>=blflags.elements() )
        {
            return retval;
        }
        
        //
        //  Okiedokie, add every flagged baseline
        //
        for( uint16 i=0; i<blflags(sbnr).elements(); i++ )
        {
            if( (blflags(sbnr))(i) )
            {
                retval.add( (baselines(sbnr))(i).getBaseLineIdx() );
            }
        }
        return retval;
    }
    
    //
    //  Add a baseline to this subarray
    //
    void addBaseLine( const BaseLineIdx& idx,
		      const InterferometerDescr& ifref,
		      const CorrelationCode& ccref,
                      uint16 subbandnr=0 )
    {
        uint16            linearindex1( (uint16)idx );
        uint16            maxlinear;
        
	//
	//  We might need to resize
	//
        if( blflags.elements()<=subbandnr )
        {
            blflags.resize( subbandnr+1 );
            baselines.resize( subbandnr+1 );
        }
        
        //
        //  find out if the baseline is already present!
        //
        maxlinear=linearindex1;
        
	//
	//  Maybe resize?
	//
        if( blflags(subbandnr).elements()<=maxlinear )
        {
            blflags(subbandnr).resize( maxlinear+1, false );
            baselines(subbandnr).resize( maxlinear+1 );
        }

	//
	//  Check if baseline is set
	//
        if( !(blflags(subbandnr))(linearindex1) )
        {
            (baselines(subbandnr))( linearindex1 )=idx;
            (blflags(subbandnr))( linearindex1 )=true;
        }

	//
	//  Add th index correlation
	//
        (baselines(subbandnr))( linearindex1 ).addCorrelation( ifref, ccref );
    }

    //
    //  How many subbands are configured?
    //
    uint16  getNrSubbands( void ) const
    {
	return blflags.elements();
    }

    //
    //  How many baselines for the indicated subband?
    //
    uint16  getNrBaselines( uint16 subbandnr ) const
    {
	if( subbandnr>=blflags.elements() )
	{
	    return 0;
	}
	
	return getBaseLines(subbandnr).elements();
    }

    //
    //  Return a baseline.... now this is tricky since we don't do any
    //  checking in here!
    //
    inline const BaseLine& operator()( const BaseLineIdx& nr, uint16 sbnr=0 ) const
    {
        return baselines(sbnr)( (uint16)nr );
    }

private:

    //
    //  A subarray has an array of flags and an array of baselines for
    //  every subband!
    //
    Container<Container<bool> >     blflags;
    Container<Container<BaseLine> > baselines;
};


#endif
