//
//  Implementation of the VisLabel-class
//
//
#ifndef VISLABEL_H
#include <vlbi/DtdLUT/VisLabel.h>
#endif

#ifndef CORRELATION_CODE_H
#include <vlbi/DtdLUT/CorrelationCode.h>
#endif

#include <iomanip.h>


ostream& operator<<( ostream& os, const VisLabel& vlref )
{
    os.form("%2d: SA=%2d SB=%3d CR=%2d FR=%4d BL=", vlref.itsJobnr,
            vlref.itsSubarraynr, vlref.itsSubbandnr, vlref.itsCorrelationnr,
            vlref.itsFramenr )
                << vlref.itsBaseLine;
    return os;
}

ostream& operator<<( ostream& os, const ExtVisLabel& visref )
{
    os << visref.itsCorrelationnrs.elements() << " correlation products on ";
    os << (const VisLabel&)visref << endl;

    for( uint16 i=0; i<visref.itsCorrelationnrs.elements(); i++)
    {
	os << CorrelationCode::ordinal2code( visref.itsCorrelationnrs(i) ) << " ";
    }
    os << endl;
    return os;
}



VisLabel::VisLabel( uint16 jobnr, uint16 sanr, uint16 sbnr, uint16 crnr, uint32 frnr,
                      uint16 ifnr, const BaseLineIdx& bl ) :
    itsJobnr( jobnr ),
    itsSubarraynr( sanr ),
    itsSubbandnr( sbnr ),
    itsCorrelationnr( crnr ),
    itsFramenr( frnr ),
    itsInterferometernr( ifnr ),
    itsBaseLine( bl )
{
}

VisLabel::VisLabel( const VisLabel& other ) :
    itsJobnr( other.itsJobnr ),
    itsSubarraynr( other.itsSubarraynr ),
    itsSubbandnr( other.itsSubbandnr ),
    itsCorrelationnr( other.itsCorrelationnr ),
    itsFramenr( other.itsFramenr ),
    itsInterferometernr( other.itsInterferometernr ),
    itsBaseLine( other.itsBaseLine )
{
}

VisLabel& VisLabel::operator=( const VisLabel& rval )
{
    if( this!=&rval )
    {
        itsJobnr=rval.itsJobnr;
        itsSubarraynr=rval.itsSubarraynr;
        itsSubbandnr=rval.itsSubbandnr;
        itsCorrelationnr=rval.itsCorrelationnr;
        itsFramenr=rval.itsFramenr;
        itsInterferometernr=rval.itsInterferometernr;
        itsBaseLine=rval.itsBaseLine;
    }
    return *this;
}


VisLabel::~VisLabel()
{
}



ExtVisLabel::ExtVisLabel() :
    VisLabel()
{
}

ExtVisLabel::ExtVisLabel( const ExtVisLabel& other ) :
    VisLabel( other ),
    itsCorrelationnrs( other.itsCorrelationnrs )
{
}

ExtVisLabel& ExtVisLabel::operator=( const ExtVisLabel& other )
{
    if( this!=&other )
    {
	*((VisLabel*)this)=(const VisLabel&)other;
	itsCorrelationnrs = other.itsCorrelationnrs;
    }
    return *this;
}

Container<uint16>&  ExtVisLabel::getCorrelations( void )
{
    return itsCorrelationnrs;
}

const Container<uint16>& ExtVisLabel::getCorrelations( void ) const
{
    return itsCorrelationnrs;
}

ExtVisLabel::~ExtVisLabel()
{
}


//
//
//  The JIVE-specific label....
//
//
JIVEVisLabel::JIVEVisLabel() :
    ExtVisLabel(),
    itsStartBOCF( 0xFFFFFFFF ),
    itsEndBOCF( 0xFFFFFFFF ),
    itsXStationUnit( 0xFFFF ),
    itsYStationUnit( 0xFFFF )
{
}


JIVEVisLabel::JIVEVisLabel( const JIVEVisLabel& other ) :
    ExtVisLabel( (const ExtVisLabel&)other ),
    itsStartBOCF( other.itsStartBOCF ),
    itsEndBOCF( other.itsEndBOCF ),
    itsXStationUnit( other.itsXStationUnit ),
    itsYStationUnit( other.itsYStationUnit )
{
}

JIVEVisLabel& JIVEVisLabel::operator=( const JIVEVisLabel& other )
{
    if( this!=&other )
    {
	(ExtVisLabel&)(*this) = (const ExtVisLabel&)other;
	
	itsStartBOCF = other.itsStartBOCF;
	itsEndBOCF = other.itsEndBOCF;
	itsXStationUnit = other.itsXStationUnit;
	itsYStationUnit = other.itsYStationUnit;
    }
    return *this;
}


uint32& JIVEVisLabel::getStartBOCF( void )
{
    return itsStartBOCF;
}

const uint32& JIVEVisLabel::getStartBOCF( void ) const
{
    return itsStartBOCF;
}

uint32& JIVEVisLabel::getEndBOCF( void )
{
    return itsEndBOCF;
}

const uint32& JIVEVisLabel::getEndBOCF( void ) const
{
    return itsEndBOCF;
}

uint16& JIVEVisLabel::getXStationUnit( void )
{
    return itsXStationUnit;
}

const uint16& JIVEVisLabel::getXStationUnit( void ) const
{
    return itsXStationUnit;
}

uint16& JIVEVisLabel::getYStationUnit( void )
{
    return itsYStationUnit;
}

const uint16& JIVEVisLabel::getYStationUnit( void ) const
{
    return itsYStationUnit;
}

JIVEVisLabel::~JIVEVisLabel()
{
}


