
/**/
/*********************************************************** EVNFRA_TRANSF */
/**/
/*
*  Description: Global declarations with file scope common to all the
*               EVN and DZB data transfer routines.
*
*
*  Author : A. Bos NFRA
*  Date   : 25-11-97
*  History: AB10-10-98: virtual inputs added in intfr data descr
*                       hardware resource descriptors added
*                       tot+job+sa data offsets added in intfr data descr
*           AB27-10-98: virtual and physical serial link masks added 
*           AB13-05-99: subblock and correlation product entries removed     
*
*
*    $Id: evnfra_transf.h,v 1.2 2000/11/21 14:16:11 verkout Exp $
*
*    $Log: evnfra_transf.h,v $
*    Revision 1.2  2000/11/21 14:16:11  verkout
*    HV: Albert's got a new DTD...
*
*    Revision 1.1.1.1  2000/03/20 15:05:34  verkout
*    HV: Imported aips++ implement-stuff
*
*    Revision 1.2  1999/06/23 10:25:04  verkout
*    Albert's changed something...
*
*/

/* size definitions */

#ifndef EVNFRA_TRANSFER_STRUCT
#define EVNFRA_TRANSFER_STRUCT

#ifdef EVNFRA_CD3
  const int MAX_JOB= 32;               /* maximum number of jobs */
  const int MAX_SIGNAL_MAP= 512;
#endif  
  const int MAX_SUBARRAY= 32;          /* maximum number of subarrays */
  const int MAX_CORPR= 8192;           /* maximum nr of correlation products */
  const int MAX_CORFN= MAX_CORPR*3/4;  /* maximum nr of corr functions */
  const int MAX_INTFR= MAX_CORFN;      /* maximum nr of interferometers */
  const int MAX_DATABLOCK= 32;         /* maximum number of data blocks */
  const int MAX_CORPR_per_BOARD= MAX_CORPR/32;
  const int MAX_INTFR_per_BOARD= MAX_INTFR/32;   
  const int MAX_CORFN_per_INTFR= 256; /* max nr corr functs per intfr */
  
/*  job data descriptor */

typedef struct JOBDATADESCR
{
  int JOB;            /* job number = JOB_DATADESCR_ARRAY index */  
  int JOBID;          /* the job identification code */
  int NSUBARRAY;      /* the number of subarrays for this job */
  int SA_DINDEX;      /* start index in SA_DATADESCR_ARRAY */
  int NINTFR;         /* number of interferometers in this job */
  int IF_DINDEX;      /* start entry in the INTFR_DATADESCR_ARRAY */
                      /* = start entry in SYS_CONFIG.INTFR_DESCR_TABLE*/    
  int NCORFN;         /* number of correlation functions in this job */
  int CF_DINDEX;      /* start entry in the  CORFN_DATADESCR_ARRAY */
                      /* = start entry in the SYS_CONFIG.CORFN_DESCR_TABLE */
  int NDATABLOCK;     /* the number of data blocks for this job */
  int DB_DINDEX;      /* start index in DB_DATADESCR_ARRAY */           
  int JOB_DATASIZE;   /* data volume of all subarrays in this job in words */
  int MSA_DATASIZE;   /* maximum value of subarray data sizes in this job */
  int MIF_DATASIZE;   /* max value of interferometer data sizes in this job */
  int MCF_DATASIZE;   /* max value of corr fn data sizes in this job */      
  int MDB_DATASIZE;   /* the max value of data block sizes in this job */
  int JOB_HARDW_MASK; /* job hardware resource mask */
  int JOB_COR_SIG_RSRC[MAX_SIGNAL_MAP];   /* job cor signal resource map */
  int JOB_DDU_SIG_RSRC[MAX_SIGNAL_MAP];   /* job ddu signal resource map */
  int JOB_ADC_SIG_RSRC[MAX_SIGNAL_MAP*4]; /* job adc signal resource map */
} JOB_DATADESCR_TYPE;

/* subarray data descriptor */

typedef struct SADATADESCR_TYPE
{
  int SUBARRAY;     /* subarray number = SA_DATADESCR_ARRAY index */
  int SUBARRAYID;   /* subarray identification code */
  int NINTFR;       /* number of interferometers in this subarray */
  int IF_DINDEX;    /* start entry in the INTFR_DATADESCR_ARRAY */
                    /* = start entry in SYS_CONFIG.INTFR_DESCR_TABLE*/    
  int NCORFN;       /* number of correlation functions in this subarray */
  int CF_DINDEX;    /* start entry in the  CORFN_DATADESCR_ARRAY */
                    /* = start entry in the SYS_CONFIG.CORFN_DESCR_TABLE */  
  int NDATABLOCK;   /* number of data blocks in this subarray */   
  int DB_DINDEX;    /* start index in DB_DATADESCR_ARRAY */       
  int SA_DATASIZE;  /* data volume of all data blocks in this subarray */
  int MIF_DATASIZE; /* max value of interfr data sizes in this subarray */
  int MCF_DATASIZE; /* max value of corrfn data sizes in this subarray */
  int MDB_DATASIZE; /* the max value of data block sizes in this subarray */
  int SA_HARDW_MASK;/* subarray hardware resource mask */
  int SA_COR_SIG_RSRC[MAX_SIGNAL_MAP];   /* subarray cor signal resource map */
  int SA_DDU_SIG_RSRC[MAX_SIGNAL_MAP];   /* subarray ddu signal resource map */
  int SA_ADC_SIG_RSRC[MAX_SIGNAL_MAP*4]; /* subarray adc signal resource map */
  int VMODE;        /* validity mode */
} SA_DATADESCR_TYPE;

/* interferometer data descriptor definition */

typedef struct INTFRDATADESCR
{
  int INTFR;          /* interferometer number = INTFR_DATADESCR_ARRAY index */
                      /* = entry in SYS_CONFIG.INTFR_DESCR_TABLE */
  int SBANDNR;        /* subband nr or frequency channel nr of X- and Y-input */
  int XANTNR;         /* antenna nr or station nr of X-input (see 6.7.1+2) */
  int YANTNR;         /* antenna nr or station nr of Y-input (see 6.7.1+2) */
  int XPOLNR;         /* polarization nr of X-input (see 6.7.1+2) */
  int YPOLNR;         /* polarization nr of Y-input (see 6.7.1+2) */
  int XVIRTINP;       /* X-virtual input code */
  int YVIRTINP;       /* Y-virtual input code */
  int XSEGINP;        /* X-segment input code = inp+unit*128+segment*64 */
  int YSEGINP;        /* Y-segment input code = inp+unit*128+segment*64 */
  int XDDUOUTP;       /* X-output code = unit*maxoutput+output (see 6.7.1+2) */
  int YDDUOUTP;       /* Y-output code = unit*maxoutput+output (see 6.7.1+2) */  
  int XDDUINP;        /* X-ddu input code = presel board *64 + signal */
  int YDDUINP;        /* Y-ddu input code = presel board *64 + signal */      
  int XSIGOUTP;       /* X-output code = unit*maxoutput+output (see 6.7.1+2) */
  int YSIGOUTP;       /* Y-output code = unit*maxoutput+output (see 6.7.1+2) */
  int NLAG;           /* the number of lags for this interferometer */
  int NLAGOFFS;       /* number of lag offsets for time multiplexing (0= off) */
  int NDATASET;       /* number of data sets for time multiplexing  (0= off) */
  int NCORFN;         /* number of correlation functions for this intfr */
  int CF_DINDEX;      /* start entry in the  CORFN_DATADESCR_ARRAY */
                      /* = start entry in the SYS_CONFIG.CORFN_DESCR_TABLE */
  int NDBLOCKpINTFR;  /* number of data blocks for this interferometer */
  int DB_DINDEX;      /* start entry in DB_DATADESCR_ARRAY */    
  int IF_DATATYPE;    /* interferometer data type code (see 9.4.4) */
  int IF_DATASIZE;    /* interferometer data size */
  int IF_DATAOFFS;    /* interferometer data offset in local data block */
  int SA_DATAOFFS;    /* data offset of local block in subarray data */
  int JOB_DATAOFFS;   /* data offset of local block in job data */ 
  int TOT_DATAOFFS;   /* data offset of local block in all data */
  int IF_MONDSIZE;    /* interferometer monitor data size */
  int IF_MONDOFFS;    /* interferometer monitor data offset in data block */  
  int NSIGSRC;        /* size signal source information block */  
  int XSIGSRC;        /* local pointer to X-signal source info block */
  int YSIGSRC;        /* local pointer to Y-signal source info block */
  int VALCNTR;        /* local pointer to the validity counter information */ 
  int CORCHIP;        /* correlator chip coordinate */
} INTFR_DATADESCR_TYPE;

/* correlation function data descriptor definition */

typedef struct CORFNDATADESCR
{
  int CORFN;          /* correlation funct nr = CORFN_DATADESCR_ARRAY index */
                      /* = entry in SYS_CONFIG.CORFN_DESCR_TABLE*/
  int LAGOFFS;        /* lag offset for time multiplexing */
  int DATASET;        /* data set number for time multiplexing */
  int NCORPR;         /* number of correlation products for this corrfn */
  int NDBLOCKpCORFN;  /* number of data blocks for this correlation function */
  int DB_DINDEX;      /* start entry in DB_DATADESCR_ARRAY */     
  int CF_DATATYPE;    /* correlation function data type code (see 9.4.4) */
  int CF_DATASIZE;    /* correlation function data size */
  int CF_DATAOFFS;    /* correlation function data offset in data block */  
  int CF_MONDSIZE;    /* correlation function monitor data size */
  int CF_MONDOFFS;    /* correlation funct monitor data offset in data block */
  int VALCNTR;        /* local pointer to the validity counter information */ 
  int CORCHIP;        /* correlator chip coordinate */
} CORFN_DATADESCR_TYPE;

/* data block transfer descriptor struct */

typedef struct DBDATADESCR
{
  int DBLOCK;         /* data block number  = DB_DATADESCR_ARRAY index */
  int LOGDBLOCKID;    /* logical data block identifier */
  int NINTFR;         /* number of interferometers in this data block */
  int IF_DINDEX;      /* start entry in the INTFR_DATADESCR_ARRAY */
                      /* = start entry in SYS_CONFIG.INTFR_DESCR_TABLE */   
  int NCORFN;         /* number of correlation functions in this data block */
  int NLAGOFFS;       /* lag offset for time multiplexing */
  int NDATASET;       /* data set number for time multiplexing */
  int CF_DINDEX;      /* start entry in the CORFN_DATADESCR_ARRAY */
                      /* = start entry in SYS_CONFIG.CORFN_DESCR_TABLE*/  
  int DB_TSIZE;       /* size of the whole data block in words */        
  int DB_DSIZE;       /* size of the (lag etc) data in of data block in words */
  int DB_DOFFS;       /* word offset of data block in local data buffer */
  int DB_ASIZE;       /* the size of block auxiliary data in words */
  int DB_AOFFS;       /* the start location of block aux data in words */
  int NINTG_STAT;     /* number of integration status information words */  
  int INTG_STAT;      /* start location integration status information */
  int NTRANSF_STAT;   /* number of transfer status information words */  
  int TRANSF_STAT;    /* start location transfer status information */
  int NHARDW_STAT;    /* number of hardware status information words */  
  int HARDW_STAT;     /* start location hardware status information */
  int INTG_ACTCNWD;   /* integr action control word for this data block */
  int SORT_ACTCNWD;   /* sort action control word for this data block */
  int PROC_ACTCNWD;   /* processing action control word for this data block */
  int BLOCK_SORTTAB[32];   /* block sort table */
  int CELL_SORTTAB[512];   /* cell sort table */
} DB_DATADESCR_TYPE;

/* data transfer descriptor  struct */

typedef struct DATATRANSFDESCR
{
  int NJOB;               /* number of active jobs */
  int NSUBARRAY;          /* total number of subarrays */
  int NINTFR;             /* total number of interferometers */ 
  int NCORFN;             /* total number of correlation functions */    
  int NDATABLOCK;         /* total number of data blocks */
  int TOTAL_DATASIZE;     /* data volume for all jobs */
  int MJOB_DATASIZE;      /* maximum value of job data sizes */
  int MSA_DATASIZE;       /* maximum value of subarray data sizes */
  int MDB_DATASIZE;       /* the max value of data block sizes */
  int TOT_HARDW_MASK;     /* total hardware resource mask */
  int TOT_COR_SIG_RSRC[MAX_SIGNAL_MAP];   /* total cor signal resource map */
  int TOT_DDU_SIG_RSRC[MAX_SIGNAL_MAP];   /* total ddu signal resource map */
  int TOT_ADC_SIG_RSRC[MAX_SIGNAL_MAP*4]; /* total adc signal resource map */
  JOB_DATADESCR_TYPE JOB_DATADESCR_ARRAY[MAX_JOB];
  SA_DATADESCR_TYPE SA_DATADESCR_ARRAY[MAX_SUBARRAY];
  INTFR_DATADESCR_TYPE INTFR_DATADESCR_ARRAY[MAX_INTFR];  
  CORFN_DATADESCR_TYPE CORFN_DATADESCR_ARRAY[MAX_CORFN];  
  DB_DATADESCR_TYPE DB_DATADESCR_ARRAY[MAX_DATABLOCK];  
} DATA_TRANSFDESCR_TYPE;

static int DATA_TRANSFDESCR_RD= 0;   /* data transfer descr flag (1= read) */
const int DATA_TRANSFDESCR_SIZE= sizeof(DATA_TRANSFDESCR_TYPE)/4; 

#endif /* EVNFRA_TRANSFER_STRUCT */

/**/
/* end of EVNFRA_TRANSF */
/**/
