//
//  Implementation of the FreqCounter class 
//
//  Author:  Harro Verkouter,   23-03-1999
//
//  $Id: FreqCounter.cc,v 1.1.1.1 2000/03/20 15:15:40 verkout Exp $
//
#ifndef FREQCOUNTER_H
#include <vlbi/Utilities/FreqCounter.h>
#endif

#include <iostream.h>


ostream& operator<<( ostream& os, const _CounterEntry& ceref )
{
    os << ceref.itsValue << " occurred " << ceref.itsNrofOccurrences << " times";
    return os;
}


ostream& operator<<( ostream& os, const FreqCounter& fcref )
{
    CounterEntry*  ptr( fcref.itsFirstCounterptr );
    
    while( ptr )
    {
	os << *ptr << endl;
	ptr = ptr->itsPtr2next;
    }
    
    return os;
}




_CounterEntry::_CounterEntry( int value ) :
    itsValue( value ),
    itsNrofOccurrences( 0 ),
    itsPtr2next( 0 )
{
}

void _CounterEntry::operator++()
{
    itsNrofOccurrences++;
}

void _CounterEntry::operator++(int)
{
    itsNrofOccurrences++;
}

bool _CounterEntry::operator==( int rval ) const
{
    return itsValue==rval;
}

bool _CounterEntry::operator!=( int rval ) const
{
    return itsValue!=rval;
}


//
//  If we allocated something, delete it...
//
_CounterEntry::~_CounterEntry()
{
    //
    //  Remove link to next element...
    //
    itsPtr2next = 0;
}




FreqCounter::FreqCounter() :
    itsFirstCounterptr( 0 )
{
}

//
// Return a reference to the appropriate counterelement
//
CounterEntry& FreqCounter::operator[]( int value )
{
    CounterEntry*  ptr( itsFirstCounterptr );
    CounterEntry*  lastptr( 0 );

    //
    //  Locate entry
    //
    while( ptr )
    {
	lastptr = ptr;
	
	if( ptr->itsValue==value )
	{
	    break;
	}
	
	ptr = ptr->itsPtr2next;
    }
    
    if( !ptr )
    {
	ptr = new CounterEntry( value );
	
	if( !lastptr )
	{
	    itsFirstCounterptr = ptr;
	}
	else
	{
	    lastptr->itsPtr2next = ptr;
	}
    }

    return *ptr;
}

const CounterEntry& FreqCounter::operator[]( int value ) const
{
    CounterEntry   dummy( 0 );
    CounterEntry*  ptr( itsFirstCounterptr );


    //
    //  Locate entry
    //
    while( ptr )
    {
	if( ptr->itsValue==value )
	{
	    break;
	}	
	ptr = ptr->itsPtr2next;
    }
    
    if( !ptr )
    {
	ptr = &dummy;
    }
    return *ptr;
}


//
//  Delete the list!
//
FreqCounter::~FreqCounter()
{
    CounterEntry*   ptr;
    
    while( itsFirstCounterptr )
    {
	ptr = itsFirstCounterptr->itsPtr2next;
	
	delete itsFirstCounterptr;
	
	itsFirstCounterptr = ptr;
    }

    //
    //  Done!
    //
}
