//
//  FreqCounter - a class that allows one to keep track of the number
//  of occurences of numbers
//
//  Author:   Harro Verkouter,  23-03-1999
//
//  $Id: FreqCounter.h,v 1.1.1.1 2000/03/20 15:15:40 verkout Exp $
//
#ifndef FREQCOUNTER_H
#define FREQCOUNTER_H


//
//  Forward declaration
//
class ostream;
class FreqCounter;


//
//  The FreqCounter holds a linked list of structs of this type
//
typedef struct _CounterEntry
{
    //
    //  Needs to be able to access nextPtr
    //
    friend class FreqCounter;
    friend ostream& operator<<( ostream& os, const FreqCounter& fcref );
    friend ostream& operator<<( ostream& os, const _CounterEntry& ceref );
    
    
public:
    //
    //  We must at least have a value. Nr of occurrences is set to 0
    //
    _CounterEntry( int value );
    
    //
    //  Overload the ++ operator -> increments nr of occurrences
    //
    void operator++();
    void operator++(int);

    //
    //  Compare against integers
    //
    bool operator==( int rval ) const;
    bool operator!=( int rval ) const;

    //
    //  Delete it
    //
    ~_CounterEntry();
    
private:
    //
    //  Private parts
    //
    int            itsValue;
    unsigned int   itsNrofOccurrences;

    //
    //  Pointer to next element
    //
    _CounterEntry* itsPtr2next;

    //
    //  Disallow these
    //
    _CounterEntry();
    _CounterEntry( const _CounterEntry& );
    _CounterEntry& operator=( const _CounterEntry& );
} CounterEntry;


class FreqCounter
{
    friend ostream& operator<<( ostream& os, const FreqCounter& fcref );
    
public:
    //
    //  Default -> no entries etc.
    //
    FreqCounter();

    //
    // operator[] -> return the CounterEntry for the value 'value'
    //
    CounterEntry&       operator[]( int value );
    const CounterEntry& operator[]( int value ) const;

    //
    //  De-allocate allocated stuff!
    //
    ~FreqCounter();

private:
    //
    //  Our private parts
    //
    CounterEntry*    itsFirstCounterptr;
    
    //
    //  Prohibit these
    //
    FreqCounter( const FreqCounter& );
    FreqCounter& operator=( const FreqCounter& );
};


#endif
