//
//  Implementation of the range class
//
//  Author:   Harro Verkouter,   19-10-1998
//
//
#ifndef RANGE_H
#include <vlbi/Utilities/Range.h>
#endif

#ifndef AIPS_EXCEPTIONS_H
#include <aips/Exceptions.h>
#endif


//
// First, let's do the iterator stuff...
//
Range::Iterator::Iterator() :
    itsIndex( 0 ),
    itsParent( 0 )
{
}

Range::Iterator::Iterator( const Range::Iterator& other ) :
    itsIndex( other.itsIndex ),
    itsParent( other.itsParent )
{
}

Range::Iterator::Iterator( uInt val, Range* parent ) :
    itsIndex( val ),
    itsParent( parent )
{
}


Range::Iterator& Range::Iterator::operator=( const Range::Iterator& rval )
{
    if( this!=&rval )
    {
	itsIndex=rval.itsIndex;
	itsParent=rval.itsParent;
    }
    return *this;
}

void Range::Iterator::reset( void )
{
    if( itsParent )
    {
	itsIndex=0;
    }
}

void Range::Iterator::next( void )
{
    if( itsParent )
    {
	itsIndex++;
    }
}

Bool Range::Iterator::pastEnd( void ) const
{
    Bool   retval( False );
    
    if( itsParent )
    {
	retval =ToBool( itsIndex>=itsParent->itsValues.elements() );
    }
    return retval;
}

Int& Range::Iterator::operator*()
{
    if( !itsParent )
    {
	throw( AipsError("Attempt to de-reference NULL iterator") );
    }

    //
    // If past end, nothing to do!
    //
    if( this->pastEnd() )
    {
	throw( AipsError("Iterator past end") );
    }
    
    return itsParent->itsValues(itsIndex);
}

const Int& Range::Iterator::operator*() const
{
    if( !itsParent )
    {
	throw( AipsError("Attempt to de-reference NULL iterator") );
    }

    //
    // If past end, nothing to do!
    //
    if( this->pastEnd() )
    {
	throw( AipsError("Iterator past end") );
    }
    
    return itsParent->itsValues(itsIndex);
}

const Int* Range::Iterator::operator->() const
{
    if( !itsParent )
    {
	throw( AipsError("Attempt to reference from NULL iterator") );
    }

    //
    // If past end, nothing to do!
    //
    if( this->pastEnd() )
    {
	throw( AipsError("Iterator past end") );
    }
    
    return &itsParent->itsValues(itsIndex);
}

Range::Iterator::~Iterator()
{
    itsParent=0;
    itsIndex=0;
}




//
// Now the range class itself.....
//
Range::Range()
{
}

Range::Range( const String& str )
{
    int      nrcommas;
    int      nrsubstrings;
    Regex    rangetype1( "^[+-]?[0-9]*$" );
    Regex    rangetype2( "^[+-]?[0-9]*:[+-]?[0-9]*$" );
    String*  substring;

    //
    //  Do create from string. use the number of commas as an
    //  indicator of how many substrings there could be.
    //
    nrcommas=str.freq( ',' );
    
    //
    //  Allocate c-array of String-objects
    //
    substring = new String[ nrcommas+2 ];
    
    //
    //  Split the str into substrings
    //
    nrsubstrings = split( str, substring, nrcommas+2, "," );
    
    for( int i=0; i<nrsubstrings; i++ )
    {
	if( substring[i].matches(rangetype1) )
	{
	    //
	    // It's a single value...
	    //
	    itsValues.add( atoi(substring[i].chars()) );
	}
	else if( substring[i].matches(rangetype2) )
	{
	    int     from,to;
	    Bool    reverse;
	    String  fromstr,tostr;
	    
	    fromstr=substring[i].before( ':' );
	    tostr=substring[i].after( ':' );
	    
	    from=atoi( fromstr.chars() );
	    to=atoi( tostr.chars() );
	    
	    reverse = ToBool( from<0 || to<0 );
	    
	    if( ((reverse)?(from<to):(from>to)) )
	    {
		cout << "Range `" << substring[i] << "' improperly formatted; first>last!" << endl;
	    }
	    else
	    {
		for( int k=from; ((reverse)?(k>=to):(k<=to)); ((reverse)?(k--):(k++)) )
		{
		    itsValues.add( k );
		}
	    }
	}
	else
	{
	    cout << "Range `" << substring[i] << "' not recognized as valid rangestring." << endl;
	}
    }
    

    //
    //  We don't need the substrings anymore
    //
    delete [] substring;
}

Range::Iterator Range::first( void ) const
{
    return Range::Iterator( 0, (Range*)this );
}

Range::Iterator Range::last( void ) const
{
    uInt     lastval( ((itsValues.elements()>0)?(itsValues.elements()-1):(0)) );
    
    return Range::Iterator( lastval, (Range*)this );
}


Range::~Range()
{
    //
    //  Nothing to delete...
    //
}

