//
//  Implementation of the AntennaInfo class...
//
//
//  Author:   Harro Verkouter, 1-12-1998
//
//  $Id: AntennaInfo.cc,v 1.2 2000/03/29 09:57:43 verkout Exp $
//
#ifndef ANTENNAINFO_H
#include <vlbi/j2ms/AntennaInfo.h>
#endif

#include <iomanip.h>


ostream& operator<<( ostream& os, const AntennaInfo& airef )
{
    os << airef.itsTLC << ": " << airef.itsName << " (part of " 
       << airef.itsStation << ") at " << airef.itsPosition
       << " has a diameter of " << airef.itsDiameter << "m"
       << ", mount=" << airef.itsMount << ", offset=" << airef.itsOffset;

    return os;
}

//
// Default Construct a complete object
//
AntennaInfo::AntennaInfo( ) :
    itsName( "" ),
    itsTLC( "" ),
    itsStation( "" ),
    itsMount( "none" ),
    itsDiameter( 0.0 ),
    itsOffset( Vector<Double>(3, 0.0) )
{
}

//
//  The copyconstructor:
//
AntennaInfo::AntennaInfo( const AntennaInfo& other ) :
    itsPosition( other.itsPosition ),
    itsName( other.itsName ),
    itsTLC( other.itsTLC ),
    itsStation( other.itsStation ),
    itsMount( other.itsMount ),
    itsDiameter( other.itsDiameter ),
    itsOffset( other.itsOffset )
{

}

//
//  Construct a complete object
//
AntennaInfo::AntennaInfo( const String& name, const String& tlc, const String& station,
			  const MPosition& pos, Double diameter,
			  const String& mount,
			  const Vector<Double>& offset ) :
    itsPosition( pos ),
    itsName( name ),
    itsTLC( tlc ),
    itsStation( station ),
    itsMount( mount ),
    itsDiameter( diameter )
    //    itsOffset( offset )
{
    Int   len;
    
    offset.shape( len );
    itsOffset.resize( len );
    
    itsOffset = offset;
    
    //
    //  Make sure offset has length of three
    //
    itsOffset.shape( len );
    if( len!=3 )
    {
	itsOffset.resize( 3, True );	
    }
}

//
//  Assignment...
//
AntennaInfo& AntennaInfo::operator=( const AntennaInfo& other )
{
    if( this!=&other )
    {
        itsName     = other.itsName;
	itsTLC      = other.itsTLC;
        itsStation  = other.itsStation;
        itsPosition = other.itsPosition;
	itsMount    = other.itsMount;
        itsDiameter = other.itsDiameter;
	itsOffset   = other.itsOffset;
    }
    return *this;
}


const String& AntennaInfo::getName() const
{
    return itsName;
}

const String& AntennaInfo::getTLC() const
{
    return itsTLC;
}


const String& AntennaInfo::getStation() const
{
    return itsStation;
}

const String& AntennaInfo::getMount() const
{
    return itsMount;
}

const MPosition& AntennaInfo::getPosition() const
{
    return itsPosition;
}

Double AntennaInfo::getDiameter() const
{
    return itsDiameter;
}

const Vector<Double>& AntennaInfo::getOffset() const
{
    return itsOffset;
}


AntennaInfo::~AntennaInfo()
{
}

