//
//  AntennaInfo:   a class describing the properties of an antenna
//
#ifndef ANTENNAINFO_H
#define ANTENNAINFO_H

#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef AIPS_MPOSITION_H
#include <aips/Measures/MPosition.h>
#endif

class AntennaInfo
{
    //
    // Show this station's info neatly formatted on an outputstream
    //
    friend ostream& operator<<( ostream& os, const AntennaInfo& airef );

public:
    //
    //  Create an empty AntennaInfo
    //
    AntennaInfo();
    
    //
    //  The copy ctor
    //
    AntennaInfo( const AntennaInfo& other);

    //
    //  Create a 'fully' defined antenna info object. NOTE: 'tlc' =
    //  t(wo) l(etter) c(ode)!
    //
    AntennaInfo( const String& name, const String& tlc, const String& station="<NONE>", 
                 const MPosition& pos=MPosition(), Double diameter=0.0,
		 const String& mount="none",
		 const Vector<Double>& offset=Vector<Double>(3, 0.0) );

    //
    //  Assignment
    //
    AntennaInfo& operator=( const AntennaInfo& other );

    //
    //  Get access to the private parts of this object:
    //
    Double                getDiameter() const;
    const String&         getName() const;
    const String&         getTLC() const;
    const String&         getStation() const;
    const String&         getMount() const;
    const MPosition&      getPosition( void ) const;

    const Vector<Double>& getOffset() const;
    //
    //  The destructor!
    //
    ~AntennaInfo();

private:
    //
    // The position of the station....
    //
    MPosition         itsPosition;

    //
    //  The name of this particular antenna, like WSRT8
    //
    String            itsName;
    
    //
    //  The official two-lettercode of this antenna
    //
    String            itsTLC;
    
    //
    //  The station name, like VLA or WSRT, or Jodrell Bank
    //
    String            itsStation;

    //
    //  The mount. Basically any string could be used...
    //
    String            itsMount;

    //
    //  The diameter of the dish in meters
    //
    Double            itsDiameter;

    //
    //  The offset. Thrtee-dimensional (so it will fit neatly in the
    //  Antenna-subtable of a MS)
    //
    Vector<Double>    itsOffset;
};

#endif
