//
//  Implementation of the baseband class
//
//  Author:    Harro Verkouter, 2-12-1998
//
//  $Id: BaseBand.cc,v 1.1.1.1 2000/03/20 15:14:35 verkout Exp $
//
#ifndef BASEBAND_H
#include <vlbi/j2ms/BaseBand.h>
#endif

#ifndef CORRELATIONCODE_H
#include <vlbi/DtdLUT/CorrelationCode.h>
#endif

#ifndef CONTAINER_H
#include <vlbi/tmplsrc/Container.h>
#endif

#ifndef AIPS_EXCEPTIONS_H
#include <aips/Exceptions.h>
#endif

#ifndef FREQUENCYCONFIG_H
#include <vlbi/j2ms/FrequencyConfig.h>
#endif

#include <iostream.h>


ostream& operator<<( ostream& os, const BaseBand& bbref )
{
    os << bbref.getBandwidth() << "Hz, from " << bbref.getStartfreq() << "Hz, " 
       << bbref.getSideBand();
    return os;
}

ostream& operator<<( ostream& os, const BaseBand::SideBand& sbref )
{
    switch( sbref )
    {
	case BaseBand::upper:
	    os << "USB";
	    break;
	    
	case BaseBand::lower:
	    os << "LSB";
	    break;
	    
	case BaseBand::none:
	    os << "<not set>";
	    break;
	    
	default:
	    os << "<INVALID SIDEBAND>";
	    break;
    }
    return os;
}


BaseBand::SideBand BaseBand::stringToSideBand( const String& sbtxt )
{
    BaseBand::SideBand   retval( BaseBand::none );
    
    if( sbtxt=="USB" || sbtxt=="U" || sbtxt=="u" )
    {
	retval = BaseBand::upper;
    }
    else if( sbtxt=="LSB" || sbtxt=="L" || sbtxt=="l" )
    {
	retval = BaseBand::lower;
    }

    return retval;
}



BaseBand::BaseBand() :
    itsStartfreq( 0.0 ),
    itsBandwidth( 0.0 ),
    itsSideBand( BaseBand::none ),
    itsBaseBandnr( (uInt)-1 ),
    itsParentConfigurationptr( 0 )
{
}

BaseBand::BaseBand( const BaseBand& other ) :
    itsStartfreq( other.itsStartfreq ),
    itsBandwidth( other.itsBandwidth ),
    itsSideBand( other.itsSideBand ),
    itsBaseBandnr( other.itsBaseBandnr ),
    itsParentConfigurationptr( other.itsParentConfigurationptr )
{
}

BaseBand::BaseBand( Double startfreq, Double bandwidth, BaseBand::SideBand sideband ) :
    itsStartfreq( startfreq ),
    itsBandwidth( bandwidth ),
    itsSideBand( sideband ),
    itsBaseBandnr( (uInt)-1 ),
    itsParentConfigurationptr( 0 )
{
}

//
//  The protected c'tor -> Can only be used by FrequencyConfig-class
//  because that class has all the info needed to fill in these
//  details!
//
BaseBand::BaseBand( const FrequencyConfig* ptr2parent, uInt basebandnr, const BaseBand& bbdata ) :
    itsStartfreq( bbdata.itsStartfreq ),
    itsBandwidth( bbdata.itsBandwidth ),
    itsSideBand( bbdata.itsSideBand ),
    itsBaseBandnr( basebandnr ),
    itsParentConfigurationptr( ptr2parent )
{
}

BaseBand& BaseBand::operator=( const BaseBand& other )
{
    if( this!=&other )
    {
	itsStartfreq=other.itsStartfreq;
	itsBandwidth=other.itsBandwidth;
	itsSideBand=other.itsSideBand;
	itsBaseBandnr=other.itsBaseBandnr;
	itsParentConfigurationptr=other.itsParentConfigurationptr;
    }
    return *this;
}


Double BaseBand::getStartfreq( void ) const
{
    return itsStartfreq;
}

Double BaseBand::getBandwidth( void ) const
{
    return itsBandwidth;
}

BaseBand::SideBand BaseBand::getSideBand( void ) const
{
    return itsSideBand;
}

Bool BaseBand::hasConfiguration( void ) const
{
    return ToBool( itsParentConfigurationptr!=0 );
}

const FrequencyConfig& BaseBand::getConfiguration( void ) const
{
    if( !itsParentConfigurationptr )
    {
	throw( AipsError("BaseBand - attempt to get parent whilst no parent configured!") );
    }
    return *itsParentConfigurationptr;
}

bool BaseBand::operator==( const BaseBand& other )
{
    bool     freqsIdentical( (itsStartfreq==other.itsStartfreq) );
    bool     bwIdentical( (itsBandwidth==other.itsBandwidth) );
    bool     sbIdentical( (itsSideBand==other.itsSideBand) );
    
    return (freqsIdentical && bwIdentical && sbIdentical);
}



BaseBand::~BaseBand()
{
}

