//
//  implementation of the COFScan class
//
//
//  Author:   Harro Verkouter,  2-6-1999
//
//   $Id: COFScan.cc,v 1.2 2001/06/22 14:05:00 verkout Exp $
//
//   $Log: COFScan.cc,v $
//   Revision 1.2  2001/06/22 14:05:00  verkout
//   HV: A number of changes: * fixed conflict between class names in j2ms and JCCS (class 'Scan' was defined in both) * fixed bug in j2ms2, Lorant uncovered it: when only RR product in data AND baseline needed swapping => j2ms2 coredumped * made some cosmetic changes in output of j2ms2 * removed the necessity to have 'J2MSDBPATH' environment var set
//
//   Revision 1.1.1.1  2000/03/20 15:14:35  verkout
//   HV: imported aips++ implement stuff
//
//
#ifndef COFSCAN_H
#include <vlbi/j2ms/COFScan.h>
#endif

#ifndef COFVISIBILITYBUFFER_H
#include <vlbi/j2ms/COFVisibilityBuffer.h>
#endif

#ifndef EXPERIMENT_H
#include <vlbi/j2ms/Experiment.h>
#endif

//
//  The following two files are contained in libExceptionObject...
//
#ifndef EXCEPTIONOBJECT_H
#include <ExceptionObject/ExceptionObject.h>
#endif

#ifndef REGULAREXPRESSIONS_H
#include <iniutils/RegularExpressions.h>
#endif


#ifndef VEXPLUS_H
#include <prep_job/vexplus.h>
#endif

#ifndef VISIBILITY_H
#include <vlbi/j2ms2/Visibility.h>
#endif

#ifndef AIPS_MEPOCH_H
#include <aips/Measures/MEpoch.h>
#endif

#ifndef AIPS_MVTIME_H
#include <aips/Quanta/MVTime.h>
#endif

#ifndef DAYCONVERSION_H
#include <vlbi/j2ms/DayConversion.h>
#endif

#ifndef CORRELATORSETUP_H
#include <vlbi/j2ms/CorrelatorSetup.h>
#endif

#ifndef AIPS_QUANTA_H
#include <aips/Quanta.h>
#endif

#include <iostream.h>
#include <strstream.h>
#include <fstream.h>
#include <iomanip.h>
#include <sys/types.h>
#include <sys/stat.h>



ostream& operator<<( ostream& os, const j2ms::COFScan& csref )
{
    os << "COFScan  :" << csref.getScanId() << "\n"
       << "SRC/FREQ :" << csref.getSourceCode() << "/" << csref.getFrequencyCode() << "\n"
       << flush;
    return os;
}

namespace j2ms
{

COFScan::COFScan( const COFVisibilityBuffer* visbufptr, const ScanInfo& si ) :
    Scan( visbufptr ),
    myScanID( si.scanId ),
    mySourceCode( si.sourceCode ),
    myFrequencyCode( si.frequencyCode ),
    myStartTime( MEpoch(si.scanStart, MEpoch::UTC) ),
    myCorrelatorSetup( si.correlatorSetup )
{
}

//
// RO-access
//
const String& COFScan::getScanId( void ) const
{
    return myScanID;
}

const String& COFScan::getSourceCode( void ) const
{
    return mySourceCode;
}

const String& COFScan::getFrequencyCode( void ) const
{
    return myFrequencyCode;
}

const MEpoch& COFScan::getStartTime( void ) const
{
    return myStartTime;
}

const CorrelatorSetup& COFScan::getCorrelatorSetup( void ) const
{
    return myCorrelatorSetup;
}

//
//  delete allocated resources
//
COFScan::~COFScan()
{
}

// end of namespace
};

