//
//  COFScan - A specialization of Scan, properly describing a scan
//  saved in the Correlator Output Format.
//
//
//  Author:   Harro Verkouter,  2-6-1999
//
//  $Id: COFScan.h,v 1.2 2001/06/22 14:05:01 verkout Exp $
//
//  $Log: COFScan.h,v $
//  Revision 1.2  2001/06/22 14:05:01  verkout
//  HV: A number of changes: * fixed conflict between class names in j2ms and JCCS (class 'Scan' was defined in both) * fixed bug in j2ms2, Lorant uncovered it: when only RR product in data AND baseline needed swapping => j2ms2 coredumped * made some cosmetic changes in output of j2ms2 * removed the necessity to have 'J2MSDBPATH' environment var set
//
//  Revision 1.1.1.1  2000/03/20 15:14:35  verkout
//  HV: imported aips++ implement stuff
//
//
#ifndef COFSCAN_H
#define COFSCAN_H


//
//  This is our base-clas...
//
#ifndef SCAN_H
#include <vlbi/j2ms/Scan.h>
#endif

//
//  We keep the JB definition of int32 etc. ('cuz that's what the COF
//  library's compiled with)
//
#ifndef STD_TYPES_H
#include <fundamental/std_types.h>
#endif

#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef AIPS_MEPOCH_H
#include <aips/Measures/MEpoch.h>
#endif

#ifndef CORRELATORSETUP_H
#include <vlbi/j2ms/CorrelatorSetup.h>
#endif


//
//  Forward declarations
//
class ostream;
class COFVisibilityBuffer;

namespace j2ms
{

class COFScan :
    public j2ms::Scan
{
    //
    //  Only the experiment class can create a valid COFScan
    //
    friend class COFVisibilityBuffer;
    
public:
    //
    //  These methods are inherited from the base-class; we MUST implement them!!
    //
    virtual const String&     getScanId( void ) const;
    virtual const String&     getSourceCode( void ) const;
    virtual const String&     getFrequencyCode( void ) const;
    virtual const MEpoch&     getStartTime( void ) const;

    virtual const CorrelatorSetup& getCorrelatorSetup( void ) const;

    //
    //  Delete all the resources that were allocated by this object
    //
    virtual ~COFScan();
    
protected:
    //
    //  This c'tor is typically called by an experiment -> the
    //  experiment 'requests' to open this scan. An ExceptionObject
    //  will be thrown if something fails that shouldn't have..
    //
    COFScan( const COFVisibilityBuffer* visbufptr, const ScanInfo& si );

private:
    //
    // The COFScan's private parts
    //
    String          myScanID;
    String          mySourceCode;
    String          myFrequencyCode;
    MEpoch          myStartTime;
    CorrelatorSetup myCorrelatorSetup;
    
    //
    //  Private methods
    //

    //
    //  These are undefined and therefore we prohibit use of them
    //
    COFScan();
    COFScan( const COFScan& );
    COFScan& operator=( const COFScan& );
};

// end of namespace
};

//
//  Show on a stream in HRF
//
ostream& operator<<( ostream& os, const j2ms::COFScan& csref );


#endif
