//
//  Implementation of the COF visibilitybuffer class
//
//  Author:  Harro Verkouter,  31-05-1999
//
//  $Id: COFVisibilityBuffer.cc,v 1.5 2001/06/22 14:05:03 verkout Exp $
//
//  $Log: COFVisibilityBuffer.cc,v $
//  Revision 1.5  2001/06/22 14:05:03  verkout
//  HV: A number of changes: * fixed conflict between class names in j2ms and JCCS (class 'Scan' was defined in both) * fixed bug in j2ms2, Lorant uncovered it: when only RR product in data AND baseline needed swapping => j2ms2 coredumped * made some cosmetic changes in output of j2ms2 * removed the necessity to have 'J2MSDBPATH' environment var set
//
//  Revision 1.4  2001/05/10 06:44:45  verkout
//  HV: Added a little printout in a certain error condition (the program would stop quietly without telling why)
//
//  Revision 1.3  2001/05/10 06:28:58  verkout
//  HV: Added support to j2ms2 to make decisions based on Albert's different software versions (ABMajor/ABMinor keywords in output VEXfile). Also a 'bypass' mode is present now (12345/67890)=>data is left unprocessed.
//
//  Revision 1.2  2000/06/07 15:12:07  verkout
//  HV: Drat! A nasty bug in getting the baseline data out of the files....But! It\'s fixed
//
//  Revision 1.1.1.1  2000/03/20 15:14:35  verkout
//  HV: imported aips++ implement stuff
//
//
#ifndef COFVISIBILITYBUFFER_H
#include <vlbi/j2ms/COFVisibilityBuffer.h>
#endif

#ifndef MAPPINGSFILE_H
#include <data_handler/MappingsFile.h>
#endif

#ifndef MAPPINGENTRY_H
#include <data_handler/MappingEntry.h>
#endif

#ifndef CORRELATEDDATAFILE_H
#include <data_handler/CorrelatedDataFile.h>
#endif

#ifndef VISIBILITY_H
#include <vlbi/j2ms/Visibility.h>
#endif

#ifndef DATAFILERECORD_H
#include <data_handler/DataFileRecord.h>
#endif

#ifndef DATAFILE_H
#include <data_handler/DataFile.h>
#endif

#ifndef CORRELATEDDATARECORD_H
#include <data_handler/CorrelatedDataRecord.h>
#endif

#ifndef DATADESCRIPTOR_H
#include <data_handler/DataDescriptor.h>
#endif

#ifndef MAPPINGENTRY_H
#include <data_handler/MappingEntry.h>
#endif

#ifndef AIPS_MATRIX_H
#include <aips/Arrays/Matrix.h>
#endif

#ifndef AIPS_VECTOR_H
#include <aips/Arrays/Vector.h>
#endif

#ifndef AIPS_IPOSITION_H
#include <aips/Lattices/IPosition.h>
#endif

#ifndef DATAFILE_H
#include <data_handler/DataFile.h>
#endif

#ifndef AIPS_MVTIME_H
#include <aips/Quanta/MVTime.h>
#endif

#ifndef DAYCONVERSION_H
#include <vlbi/j2ms/DayConversion.h>
#endif

#ifndef COFSCAN_H
#include <vlbi/j2ms/COFScan.h>
#endif

#ifndef CORRELATORSETUP_H
#include <vlbi/j2ms/CorrelatorSetup.h>
#endif

#ifndef AIPS_MEPOCH_H
#include <aips/Measures/MEpoch.h>
#endif

#ifndef INTERFEROMETERDESCRIPTOR_H
#include <data_handler/InterferometerDescriptor.h>
#endif

#ifndef EXCEPTIONOBJECT_H
#include <ExceptionObject/ExceptionObject.h>
#endif

#ifndef VEXPERIMENT_H
#include <vlbi/j2ms/VEXperiment.h>
#endif

#ifndef REGULAREXPRESSIONS_H
#include <iniutils/RegularExpressions.h>
#endif

#ifndef CORRELATIONCODE_H
#include <vlbi/DtdLUT/CorrelationCode.h>
#endif

#ifndef FREQUENCYCONFIG_H
#include <vlbi/j2ms/FrequencyConfig.h>
#endif

#ifndef FREQUENCYDB_H
#include <vlbi/j2ms/FrequencyDB.h>
#endif

#ifndef CLIENTCONNECTION_H
#include <Connections/ClientConnection.h>
#endif

#ifndef EXCEPTIONOBJECT_H
#include <ExceptionObject/ExceptionObject.h>
#endif


//
//  Need this for the SYSCLK_FREQ - define
//
#include <fundamental/ccc_constants.h>

//
//  Vexplus?
//
#include <prep_job/vexplus.h>

#include <strstream.h>
#include <fstream.h>
#include <iomanip.h>

#include <string.h>    
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <stdlib.h>

//
//
//  The implementation of the COFVisibilityBuffer class
//
//
COFVisibilityBuffer::COFVisibilityBuffer( uint32 jobid, uint32 subjobid,
										  const VEXperiment* experimentptr ) :
    VisibilityBuffer( experimentptr ),
    myJobID( jobid ),
    mySubJobID( subjobid ),
    myMappingsFileptr( 0 ),
    myCorrelatedDataFileptr( 0 ),
    myCurrentCorrelatedDataRecordptr( 0 ),
    myCurrentDataDescriptorptr( 0 ),
    myCurrentMappingptr( 0 )
{
    int             trycounter;
    int             lastvalid;
    int             result;
    int             nscans;
    bool            done;
    char            strmbuffer[ 1024 ];
    char*           firststation;
    String          vexfilename;
    MVTime          scanmvtime;
    VexPlus*        vexfileptr;
    strstream       strm( strmbuffer, sizeof(strmbuffer) );
    struct stat     statbuffer;
    CorrelatorSetup correlatorsetup;
    //
    //  Form the directoryname where we should look for data in the first
    //  place...
    //
    strm.seekp( 0 );
    strm << this->getExperiment().getRootDir() << "/" << myJobID << "/" << mySubJobID << ends;
    
    myRootDir = strm.str();
    
    //
    //  Ok. 'myRootDir' contains something like:
    //
    //  <experiment-root-dir>/<jobid>/<subjobid>
    //
    //  This is only the beginning, there may be multiple correlations
    //  present for this subjob. If more than one correlation is
    //  present, the data is found in directories named as follows:
    //
    //  <myRootDir>.<x>
    //
    //  where <x> is an integer >0. Attempt to find the highest number
    //  and we'll use that.
    //
    done = false;
    trycounter = 0;
    lastvalid = -1;
    while( !done )
    {
		strm.seekp( 0 );
		strm << myRootDir;
		if( trycounter )
		{
			strm << "." << trycounter;
		}
		strm << ends;
	
		//
		//  Try to stat the thing. If it's a directory, we'll try the
		//  next one!
		//
		if( stat(strm.str(), &statbuffer)<0 )
		{
			//
			//  Failed to stat -> we're done?!
			//
			done = true;
			continue;
		}

		//
		//  Ok. Stat succeeded. Now... if it's not a directory, we'll
		//  skip it (i.e. leave the 'lastvalid' variable alone)
		//
		if( (statbuffer.st_mode&S_IFDIR)==S_IFDIR )
		{
			lastvalid = trycounter;
		}
		trycounter++;
    }

    //
    //  Ok. 'lastvalid' holds the value of the last 'valid'
    //  directory. If it's less than zero -> nothing valid was found.
    //
    if( lastvalid<0 )
    {
		strm.seekp( 0 );
		strm << "COFVisibilityBuffer: " << myJobID << "/" << mySubJobID
			 << " not found?!" << ends;
	
		throw( EXCEPTION(strm.str()) );
    }
    
    //
    //  Really form the full directory path
    //
    strm.seekp( 0 );
    strm << myRootDir;
    if( lastvalid )
    {
		strm << "." << lastvalid;
    }
    strm << ends;
    
    //
    //  replace rootdir with finslized result...
    //
    myRootDir = strm.str();

    //
    //  Attempt to create files. They throw exceptions when they fail!
    //
    myMappingsFileptr = new MappingsFile( myRootDir+"/MAP" );
    myCorrelatedDataFileptr = new CorrelatedDataFile( myRootDir+"/CDF" );

    //
    //  Build a list of scan start/stop times so we can easily check
    //  against the visibility-time if a new scan was detected...
    //
    strm.seekp( 0 );
    strm << this->getExperiment().getRootDir() << "/" << myJobID << "/"
	     << this->getExperiment().getCode() << "_" << myJobID << ".vex" << ends;
    
    vexfilename = strm.str();
    
    //
    //  Read the correlator setup from the comment in the vexfile???
    //
    if( COFVisibilityBuffer::readCorrelatorSetup(correlatorsetup, vexfilename)==false )
    {
		throw( EXCEPTION("COFVisibilityBuffer - failed to read correlatorsetup!") );    
    }

	//
	// Save the correlator setup for future reference
	//
	mySetup = correlatorsetup;

    //
    //  Ok. Now's the time to read the vexfile with VEX-stuff
    //
    vexfileptr = new VexPlus( vexfilename );

    if( (result=vexfileptr->parseVex())!=0 )
    {
		strm.seekp( 0 );
	
		strm << "COFVisibilityBuffer - ";
	
		switch( result )
		{
			case -1:
				strm << "could not open file " << vexfilename;
				break;
		
			case -2:
				strm << "error parsing vex-file " << vexfilename << " to memory";
				break;
		
			default:
				strm << "Unknown error?! Check VexPlus::parseVex()"
					 << " for error return codes!!";
				break;
		}
		strm << ends;
	
		throw( EXCEPTION(strm.str()) );
    }

    //
    //  Find the maximum nr. of scans and let's hope that it is the
    //  number of scans (in order for this to be true, there must be
    //  at least one station that is partaking in all the scans!)
    //
    firststation = 0;
    nscans = 0;
    for( int stationcnt=0; stationcnt<vexfileptr->N_Stations(); stationcnt++ )
    {
		char*    tmpstation;
	
		tmpstation = vexfileptr->Station( stationcnt );
		if( tmpstation && vexfileptr->N_Scans(tmpstation)>nscans )
		{
			firststation = tmpstation;
			nscans = vexfileptr->N_Scans(tmpstation);
		}
    }
    
    //
    //  Check if we found a station or, what's more interesting, a
    //  positive number of scans!
    //
    if( !firststation )
    {
		strm.seekp( 0 );
	
		strm << "COFVisibilityBuffer - no scans found in VEX-file!\n";
		strm << ">> No station with positive number of scans was found.\n";
		
		strm << ends;
	
		throw( EXCEPTION(strm.str()) );	
    }
    

    //
    //  Loop over all scans and add them
    //
    for( int i=0; i<nscans; i++ )
    {
		int      year, daynr, hour, minute, second;
		char*    scanstart( vexfileptr->ScanStart(firststation, i) );
		uint32   month, day;
		double   dayfraction;
		double   scanduration( vexfileptr->ScanDuration(firststation, i) );
		ScanInfo si;

		if( !scanstart )
		{
			continue;
		}
		
		//
		//  Ok. Transform the scanstart into a mvepoch
		//
		if( sscanf(scanstart, "%dy%dd%dh%dm%ds", &year, &daynr,
				   &hour, &minute, &second)!=5 )
		{
			continue;
		}

		DayConversion::dayNrToMonthDay( month, day, daynr, year );

		dayfraction=((hour*3600.0)+(minute*60.0) +
					(double)second)/(double)DayConversion::secondsPerDay;
	
		scanmvtime = MVTime( year, month+1, day, dayfraction );
	

		strm.seekp( 0 );
		strm << i+1 << ends;

		//
		//  Fill in the struct
		//
		si.scanStart       = MVEpoch( scanmvtime.day() );
		si.scanEnd         = si.scanStart + Quantity(scanduration, "s");
		si.scanId          = strm.str();
		si.frequencyCode   = vexfileptr->ScanMode(firststation, i);
		si.sourceCode      = vexfileptr->ScanSource(firststation, i);
		si.correlatorSetup = correlatorsetup;
		
		//
		//  Add the scan to our list...
		//
		myScanList.add( si );
		myScanPtrs.add( new j2ms::COFScan(this, si) );
    }

    //
    //  Don't need the vexfile any more
    //
    delete vexfileptr;
    
    //
    //  Let the system find the first
    //  datadescriptor/correlated-data-record combination!
    //
    myMappingsFileptr->seek( 0, ios::beg );
    myCorrelatedDataFileptr->seek( 0, ios::beg );
    
    while( !myCorrelatedDataFileptr->eof() )
    {
		if( this->next() )
		{
			break;
		}
    }
}

Bool COFVisibilityBuffer::getCurrentVisibility( Visibility& vref )
{
    //
    //  Static stuff - prevents from being created over and over again
    //
    static Bool            flag;
    static Muvw            uvw;
    static MEpoch          visibilitytime;
    static MVEpoch         mvepoch;
    static ExtVisLabel     lab;
    static Vector<Double>  weight;
    static Vector<Double>  blweight;
    static Matrix<Complex> data;
    static Matrix<Complex> bldata;

	//
	// Clear the retval; just to make sure that even if we return false
	// and the caller does not respond to that and still wants to use
	// the 'vref' they get squat!
	//
	vref = Visibility();

    //
    //  If no iterator, we must be out of range...
    //
    if( !myBaselineIterator || !myCurrentCorrelatedDataRecordptr
		|| !myCurrentDataDescriptorptr )
    {
		//cout << "getCurrentVisibility - Error\n";
		//if( !myBaselineIterator )
		//{
		//    cout << ">> No baselineiterator!\n";
		//}
		//if( !myCurrentCorrelatedDataRecordptr )
		//{
		//    cout << ">> No correlated data record!\n";
		//}
		//if( !myCurrentDataDescriptorptr )
		//{
		//    cout << ">> No datadescriptor!\n";
		//}
		//cout << flush;
		return False;
    }

    //
    //  Need to fill in the correlations!
    //
    lab.getCorrelations() = (*myBaselineIterator).getCorrelations();

    //
    //  First things first: let's find out in which scan we are!
    //
    //  so: calculate the visibility time
    //
    visibilitytime = 
	COFVisibilityBuffer::calcMidPointAsMEpoch( 
						  myCurrentDataDescriptorptr->getStartRot(),
						  myCurrentCorrelatedDataRecordptr->getStartSFRot(),
						  myCurrentCorrelatedDataRecordptr->getEndSFRot() );
    //
    //  Depending on the time, find out in which scan we are...
    //
    uint32     i;
    
    mvepoch = visibilitytime.getValue();

    for( i=0; i<myScanList.elements(); i++ )
    {
		const ScanInfo&    siref( myScanList(i) );
	
		if( (mvepoch.get()>siref.scanStart.get()) && (mvepoch.get()<siref.scanEnd.get()) )
		{
			break;
		}
    }

    if( i<myScanPtrs.elements() )
    {
		this->setCurrentScanptr( myScanPtrs(i) );
    }
    else
    {
		this->setCurrentScanptr( 0 );
    }

    //
    //  Great! 
    //
    //  now read the data
    //
    data.resize( 0, 0 );
	bldata.resize( 0, 0 );

    weight.resize( 0 );
	blweight.resize( 0 );   

	//
	// Changed this one a bit: this now reads *all* the data from
	// a given baseline. Now we may have to reorder stuff such
	// that the order in the matrix/vector is conform the order 
	// in the correlator setup!
	// As a bonus, we may have to swap stuff?
	//
    bldata = this->getBaselineData( blweight, *myBaselineIterator );

    //
    //  If datamatrix empty, preset flag to true....
    //
    flag = False;
    
    if( !bldata.shape().product() )
    {
		flag = True;
		cout << "COFVisibilityBuffer - No data read for baseline " << endl
			 << *myBaselineIterator << endl;
		return False;
    }
   

	//
	// Ok. Let's reorder the stuff!
	//
	// dataorder = order of the correlations in the datamatrix (or: correlations
	//             present in the data)
	//             (getCorrelations() return an array of 'correlation-indices',
	//               e.g. [2,3] which would indicate that only the cross products
	//               were calculated (i.e. [0,1] and [1,0]; see CorrelationCode.h)
	//
	// reqorder  = array of correlations as configured by the correlator.
	//             our job now is to make sure the order in the resulting
	//             datamatrix is equal to that in the correlatorsetup.
	//
	// Note: if the the two arrays are equal we do not have to do anything!
	// Note: this is only true if the arrays are equal element-by-element;
	//       if they aren't we still need to do some re-ordering!
	//
	bool                reorder;
	bool                lengthequal;
	unsigned int        corcnt;
	Container<uint16>   dataorder = myBaselineIterator->getCorrelations();
	Container<uint16>   reqorder  = mySetup.getCorrelations();

	//
	// Resize the resultant datamatrix
	//
	data.resize( reqorder.elements(), bldata.ncolumn() );
	weight.resize( reqorder.elements() );

	//
	// First test for equality
	//
	reorder     = false;
	lengthequal = (dataorder.elements()==reqorder.elements());

	for( corcnt=0; lengthequal && corcnt<dataorder.elements(); corcnt++ )
	{
		if( dataorder(corcnt)!=reqorder(corcnt) )
		{
			break;
		}
	}
	reorder = !lengthequal || (corcnt<reqorder.elements());

	for( corcnt=0; reorder && corcnt<reqorder.elements(); corcnt++ )
	{
		uint16    lookingfor = reqorder(corcnt);
		uint32    location;

		//
		// Check if we can find the desired correlation somewhere in the
		// data....
		//
		for( location=0; location<dataorder.elements(); location++ )
		{
			if( dataorder(location)==lookingfor )
			{
				break;
			}
		}
		if( location>=dataorder.elements() )
		{
			// Hey! Requested correlation not present??
			cout << "COFVisibilityBuffer: Unable to satisfy request for " << endl
				 << "    correlation " << CorrelationCode::ordinal2code(lookingfor)
				 << " (it's not present in the data):" << endl
				 << "    in data   = " << dataorder << endl
				 << "    requested = " << reqorder << endl;
			break;
		}

		//
		// Now copy data across
		//
		data.row( corcnt ) = bldata.row(location).copy();
		weight( corcnt )   = blweight(location);
	}

 	if( reorder && corcnt<reqorder.elements() )
	{
		cout << "Error occured; giving up..." << endl;
		return False;
	}

	if( !reorder )
	{
		// nothing has happened with 'data' yet since apparently
		// the order in which the stuff is in the data is the 
		// same as we want it in the resulting matrix. 
		// Only thing left is to copy the data across from
		// 'bldata' to 'data'....
		data   = bldata.copy();
		weight = blweight.copy();
	}


    //
    //  Transfer the baseline indices
    //
    lab.baseline().idx1() = 0xFFFF;
    lab.baseline().idx2() = 0xFFFF;    
    lab.subbandnr()       = 0xFFFF;

    //
    //  Now then.. fill in the indices from the mapping!? (that is, if
    //  we can find them...
    //
    //  <<TODO>>
    //
    if( myCurrentMappingptr )
    {
		COFVisibilityBuffer::MapInfo  mi;
	
		mi = COFVisibilityBuffer::decodeMapping( *myCurrentMappingptr,
								   				 *myBaselineIterator );

		lab.baseline().idx1() = (uint16)mi.XTelescopeNr;
		lab.baseline().idx2() = (uint16)mi.YTelescopeNr;
    	lab.subbandnr()       = (uint16)mi.XLogicalChannel;
    }

    //
    //  If, after mapping, the labels are valid, we may have to change
    //  things around a bit!
    //
    Bool              antsvalid;
    uint16            idx1( lab.baseline().idx1() );
    uint16            idx2( lab.baseline().idx2() );
    
    
    antsvalid = ( (idx1!=0xFFFF) && (idx2!=0xFFFF) );
   
	try
	{ 
    if( antsvalid && idx1>idx2 )
    {
		//
		//  Ok. Baseline was reversed.... reverse order and take
		//  complex conjugate....
		//
		//  We do this row by row since the polarization labels are to
		//  be exchanged and hence rows may end up in a different row
		//  in the final matrix....
		//
		uInt               centralchannel;
		uInt               nchannel;
		uint16             tmplabel;
		uint16             notvalid( 0xFFFF );
		uint32             dstrow;
		Vector<Double>     newweights;
		Matrix<Complex>    newdata( data.nrow(), data.ncolumn() );
		CorrelationCode    tmpcc;
		Container<uint16>  newcors( reqorder.elements(), notvalid );

		//
		//  Copy the old weights to the neweeights (now newweights has
		//  the correct shape); later on we will re-assign the weights
		//  to newweights in such an order as is wanted
		//
		newweights = weight.copy();

		//
		//  This is easy; swap the antenna labels...
		//  (idx1 and idx2 are inferred from the baseline *before* we enter this
		//  block
		//
		lab.baseline().idx1() = idx2;
		lab.baseline().idx2() = idx1;

		//
		//  Loop over all configured polarization combinations in
		//  order to change the datamatrix!
		//
		//  We loop over all entries in 'reqorder' since that is the
		//  order the stuff is in the 'data' matrix (after reading the baseline
		//  data already one re-ordering has taken place: from bldata -> data)
		//
		for( uint32 srcrow=0; 
			        srcrow<reqorder.elements();
			        srcrow++ )
		{
			uInt       colcnt;
			uint16     srccorindex;
			uint16     dstcorindex;
			uint32     dstrow;

			//
			// Ok. We are looking at row 'srcrow' in the 'data'-matrix.
			// The correlationindex represented by this row is:
			//
			//    reqorder[srcrow]
			//
			// From that we can work out the full CorrelationCode:
			//
			//    cc = CorrelationCode::ordinal2code( reqorder[srcrow] )
			//
			// Now we can swap the labels of the correlation code 'cc'
			// and work out the 'destination correlation code'.
			// Then we go through 'reqorder' and locate the desired correlation index.
			// The location of that index tells us the destination row!
			//
	   		srccorindex = reqorder[ srcrow ];

			tmpcc = CorrelationCode::ordinal2code( srccorindex );

			tmplabel     = tmpcc.idx1();
			tmpcc.idx1() = tmpcc.idx2();
			tmpcc.idx2() = tmplabel;
 
			dstcorindex  = CorrelationCode::code2ordinal( tmpcc );
				   
 			//
			// Find that index in the reqorder!
			//
			for( dstrow=0; dstrow<reqorder.elements(); dstrow++ )
			{
				if( reqorder(dstrow)==dstcorindex )
				{
					break;
				}
			}
			if( dstrow>=reqorder.elements() )
			{
				cout << "COFVisibilityBuffer: Aargh! Swapping baselinedata error:"
					 << endl
					 << "Looking for correlation " 
					 << CorrelationCode::ordinal2code( dstcorindex ) << " but that "
					 << "one is not present in " << flush;

				for( uint32 k=0; k<reqorder.elements(); k++ )
				{
					cout << CorrelationCode::ordinal2code(reqorder[k]) << " ";
				}
				//
				// We don't have to break here to signal an error;
				// if we continue with the loop from here, newcors[dstrow] will
				// not be set and hence later on, a check on wether all
				// entries in the 'newcors' array have been filled in will fail
				// and the error will still be detected!
				//
				continue;
			}
			

			//
			// We can now update the 'newcors' array. This array
			// represents the correlation-order in the new data matrix
			//	
			newcors[ dstrow ] = dstcorindex;
	    
			//
			//  Do change the data (i.e. flip it about the center
			//  channel....)
			//
			centralchannel = data.ncolumn()/2;
			nchannel       = (data.ncolumn()/2)-1;

			for( colcnt=0; colcnt<=nchannel; colcnt++ )
			{
				newdata( dstrow, centralchannel+colcnt ) = 
						conj( data(srcrow, centralchannel-colcnt) );
				newdata( dstrow, centralchannel-colcnt ) = 
						conj( data(srcrow, centralchannel+colcnt) );
			}

			//
			//  Set all lags that have not been rotated to Complex(0,0)...
			//
			newdata( dstrow, 0 ) = Complex(0.0f,0.0f);

			//
			//  The weight
			//
			newweights( dstrow ) = weight( srcrow );
		}

		//
		// If not all 'newcors' have been filled in, we're stuffed!
		//
		if( frequencyOfArg(newcors, notvalid)>0 )
		{
			cout << "COFVisibilityBuffer: Failed to swap baseline. Skipping this "
				 << "visibility!" << endl;
			return False;
		}

		//
		//  Do not forget to set the new ordinal numbers!
		//
		lab.getCorrelations() = newcors;

		//
		//  And do not forget to COPY the data across....
		//
		data = newdata.copy();

		//
		//  Now copy the re-ordered weights
		//
		weight = newweights.copy();

	}
    else if( !antsvalid )
    {
		flag = True;
    }
	}
	catch( AIPSExceptionObject x )
	{
		cout << "COFVisibilityBuffer - Caught exception whilst swapping baseline"
			 << endl;
		cout << x.getMesg() << "(" << x.thrownFile() << "," << x.thrownLine() << ")"
			 << endl;
		flag = True;
		return False;
	}

    //
    //  If the time between the start/endpoint is not the
    //  integrationtime, we flag the data as invalid?
    //
    if( this->getScanptr() )
    {
		const SFROT&           startROT( myCurrentCorrelatedDataRecordptr->getStartSFRot() );
		const SFROT&           endROT( myCurrentCorrelatedDataRecordptr->getEndSFRot() );	
		const j2ms::Scan*      scanptr( this->getScanptr() );
		const CorrelatorSetup& cs( scanptr->getCorrelatorSetup() );
	
		//
		//  Any data with start/end times that differ more than 1.2
		//  times the integrationtime are discarded
		//
		if( fabs(endROT.get_sysclks()-startROT.get_sysclks()) > 
			(1.2*cs.getIntegrationTime()*SYSCLK_FREQ) )
		{
			flag = True;
		}

		//
		//  Now we can try to find out what the FrequencyConfig was
		//  (so we can find out how many polarizations there were)
		//
		FrequencyConfig        fc;
		const FrequencyDB&     freqdb( this->getExperiment().getFrequencyDB() );
		
		if( freqdb.searchFrequencyConfig(fc, this->getScanptr()->getFrequencyCode()) )
		{
			if( fc.getNrPolarizations() )
			{
				lab.subbandnr()/=fc.getNrPolarizations();
			}
		}
    }

    //cout << "Creating visibility with weight=" << weight << ", flag=" << flag
    //	 << ", datashape=" << data.shape() << "..." << flush;

    vref = Visibility( lab, data, Visibility::time, flag,
					   weight, visibilitytime, uvw, this );

    return True;
}


Bool COFVisibilityBuffer::next( void )
{
    Bool             retval( False );
    Bool             done;
    DataFileRecord*  tmprecordptr;


    //
    //  Attempt to advance iterator
    //
    myBaselineIterator++;
    
    //
    //  If no more baselines left, check for a new integration (if we
    //  find a new datadescriptor in the mean time, we take our
    //  baselines from this one!
    //
    if( !myBaselineIterator )
    {

		//
		// Remove previous CorrelatedDataRecord
		//
		delete myCurrentCorrelatedDataRecordptr;
		myCurrentCorrelatedDataRecordptr = 0;
	
		//
		//  Read record until we find a correlated data record!
		//
		done = False;
		while( !done )
		{
			if( myCorrelatedDataFileptr->eof() )
			{
				done = True;
				continue;
			}
	    
	    
			(*myCorrelatedDataFileptr) >> tmprecordptr;	

			if( !tmprecordptr )
			{
				continue;
			}
	
			switch( tmprecordptr->recordType() )
			{
				case DataFileRecord::correlateddata:
					//
					//  If the baselineset is empty (i.e. no begin) it
					//  has not been set yet, i.e. no data descriptor
					//  record has been found. Skip this integration!
					//
					if( !(myBaselineSet.begin()) )
					{
						cout << "Skipping CorrelatedDatarecord for"
							 << " there's no DataDescriptor (yet)"
							 << endl;
						delete tmprecordptr;

						break;
					}
		    
					//
					//  Keep pointer to record
					//
					myCurrentCorrelatedDataRecordptr=(CorrelatedDataRecord*)tmprecordptr;
		    
					//
					//  reset iterator
					//
					myBaselineIterator = myBaselineSet.begin();
		
					//
					// set return values/loop control vars
					//
					done = True;
					retval = True;
					break;
		    
				case DataFileRecord::datadescriptor:
					//
					//  Delete previous datadecsriptor
					//
					delete myCurrentDataDescriptorptr;
					myCurrentDataDescriptorptr = (DataDescriptor*)tmprecordptr;


					//
					//  We attach to the new data descriptor
					//
					myBaselineSet = (*myCurrentDataDescriptorptr);

					//
					//  We found a new data-descriptor. However, we
					//  have no data (yet) so the iterator should be
					//  set to an empty value. Only if we have data we
					//  can safely iterate over the baselines!
					//
					myBaselineIterator = COFBaselineIterator();
				
					//
					//  Last thing to do, is to read the accompanying
					//  MappingEntry!
					//
					if( myCurrentMappingptr )
					{
						//
						//  Only remove the mapping if it's different
						//  from the one we need
						//
						if( myCurrentMappingptr->getOffsetInDataFile() !=
							myCurrentDataDescriptorptr->getMappingEntryRef() )
						{
							delete myCurrentMappingptr;
							myCurrentMappingptr = 0;
						}
					}

		    
					if( !myCurrentMappingptr )
					{
						myMappingsFileptr->seek( myCurrentDataDescriptorptr->getMappingEntryRef(),
						 ios::beg );

						(*myMappingsFileptr) >> tmprecordptr;

						if( tmprecordptr &&
							tmprecordptr->recordType()==DataFileRecord::mappingentry )
						{
							myCurrentMappingptr = (MappingEntry*)tmprecordptr;
						}
						else
						{
							cout << "AARGH? failed to read mapping @" 
								 << myCurrentDataDescriptorptr->getMappingEntryRef()
								 << endl;
						}
					}
					break;
		    
				default:
					delete tmprecordptr;
					break;
			}
		}
    }
    else
    {
		retval = True;
    }

    return retval;
}

const String& COFVisibilityBuffer::getRootDir( void ) const
{
    return myRootDir;
}


uint32 COFVisibilityBuffer::getJobID( void ) const
{
    return myJobID;
}

uint32 COFVisibilityBuffer::getSubJobID( void ) const
{
    return mySubJobID;
}



//
//  Delete allocated resources
//
COFVisibilityBuffer::~COFVisibilityBuffer()
{
    delete myMappingsFileptr;
    delete myCorrelatedDataFileptr;
    delete myCurrentCorrelatedDataRecordptr;
    delete myCurrentDataDescriptorptr;
    delete myCurrentMappingptr;

    for( uint32 i=0; i<myScanPtrs.elements(); i++ )
    {
	delete myScanPtrs(i);
    }
}



/******************************************************************************/
//
//             This is where the private section starts
//
/******************************************************************************/

//
//  The MapInfo - struct c'tor
//
COFVisibilityBuffer::_MapInfo::_MapInfo( ) :
    XTelescopeNr( 0xFFFFFFFF ),
    YTelescopeNr( 0xFFFFFFFF ),
    XDataSourceId( 0xFFFFFFFF ),
    YDataSourceId( 0xFFFFFFFF ),
    XLogicalChannel( 0xFFFFFFFF ),
    YLogicalChannel( 0xFFFFFFFF )
{
}


//
//  Get all the data for a given baseline
//
const Matrix<Complex>& COFVisibilityBuffer::getBaselineData( Vector<Double>& weight, 
							  							const COFBaseline& cbref ) const
{
    //
    //  Prevent this one from being created over and over again...
    //
    static Matrix<Complex>              retval;
    
    //
    //  These are the automatic vars
    //
    Int                      			shape0;
    Bool                     			vernier( this->getVernierDelay() );
    bool*                    			present( cbref.getConfigured() );
    float*                   			ifdataptr;
    Vector<Complex>*         			datarow[ 4 ];
    Container<uint32>         			ifstodo;
	CorrelatorSetup::CorSWVersion 		swver = mySetup.getSoftwareVersion();
	const CorrelatorSetup::CorSWVersion version1( 1, 0 );
	const CorrelatorSetup::CorSWVersion bypassversion( 12345, 67890 );


    //
    // Clear the return value
    //
    retval.resize( 0, 0 );
    weight.resize( 0 );

    //
    //  Check if we have anything to do at all
    //
    if( !myCurrentCorrelatedDataRecordptr )
    {
		return retval;
    }

	//
	// Find out all the configured correlations from the baseline
	// and clear the storage as well
	//
	for( uint32 i=0; i<4; i++ )
	{
		datarow[ i ] = (Vector<Complex>*)0;
		if( present[i] )
		{
			ifstodo.add( i );
		}
	}

    if( !ifstodo.elements() )
    {
		//
		//  Apparently nothing to do....
		//
		return retval;
    }
    
    //
    //  Now we know how many Interferometers need to be read, we can
    //  resize the weight vector (and theoretically the data
    //  matrix). We do not resize the data matrix since we will check
    //  later on that all the interferometers have the same amount of
    //  data! For the weights this is not so important; if we detect
    //  an error, we resize it back to zero-length!
    //
    weight.resize( ifstodo.elements() );
    weight = (Double)0.0;

    //
    //  Read all interferometers
    //
    for( uint32 row=0; row<ifstodo.elements(); row++ )
    {
		Bool                             swapit( False );
		Bool                             autocorrelation;
		Bool                             crosspolarization;
		Bool                             expand;
		Double                           ifweight;
		Complex*                         cmplxreordered;
		IPosition                        ipos;
		const InterferometerDescriptor&  ifref( cbref[ifstodo[row]] );
	
		//
		//  Cache this value
		//
		uint32                           nlag( ifref.getNumberOfLags() );
		uint32                           nrfloats( ifref.getNumberOfLags() );
		uint32                           nrweights;

		autocorrelation   = ToBool( ifref.getXAntenna()==ifref.getYAntenna() );
		crosspolarization = ToBool( ifref.getXPolarization()!=ifref.getYPolarization() );
	    
		expand = ToBool( autocorrelation && !crosspolarization );

		//
		//   HV: 22-03-2001 
		//
		//          **** NOTE NOTE NOTE ****
		//
		//   In order to be able to have some sort of a 'bypass' mode,
		//   I introduced a 'magic' correlatorsoftware version:
		//
		//   ABMajor = 12345
		//   ABMinor = 67890
		//
		//   If this version number is used, the data does not get swapped
		//   (i.e. the fourth polarization will not be rotated) NOR does
		//   the data get shifted (i.e. no vernier delay corrections applied,
		//   no re-definition of delay 0 applied...)
		//

		//
		//   NOTE: If we're in bypass mode, swapit will *always* be False!
		//
		if( (ifref.getXAntenna()>ifref.getYAntenna()) &&
			(swver!=bypassversion) )
		{
			swapit = True;
		}
	    

		if( ifref.getDataType()==InterferometerDescriptor::complexdata )
		{
			nrfloats*=2;
		}

		switch( ifref.getValidity() )
		{
			case InterferometerDescriptor::local:
				//nrweights = nlag;
				// HV: 13-03-2000 -> Chris told me that Albert told him that in 
				//                   local validity also only ONE weight is given.....
				nrweights = 1;
				break;
		
			case InterferometerDescriptor::global:
				nrweights = 1;
				break;
		    
			default:
				nrweights = 0;
				break;
		}
	    
		//
		//  Get the data...
		//
		//  Detail: we receive a COPY of the data so we can mess
		//  around with it if we like!
		//
		ifdataptr = myCurrentCorrelatedDataRecordptr->getInterferometerData( ifref );
	
		//
		//  Do the weight-thing
		//
		ifweight = 0.0;
		for( uint32 j=0; j<nrweights; j++ )
		{
			ifweight += ifdataptr[ nrfloats+j ];
		}

		if( nrweights>1 )
		{
			ifweight/=nrweights;
		}
		weight( row ) = ifweight;
	    

		//  
		//  HV: 30-11-2000 - Decided to first work out the total amount
		//                   of datashift (since Albert's messing around
		//                   with the lag-spectrum....). After we that,
		//                   we can guarantee that the central-lag (=delay 0)
		//                   is at lag nr (NLAG/2). This had certain 
		//                   advantages :) (upstream operations like 
		//                   e.g. FFT, rotation become easier)
		//
		//
		//      22-03-2001 - NOTE! Built in support for bypass mode!
		//                   Only shows up at the very last stage, just before
		//                   starting to do the shift. Don't forget that the
		//                   the value of datashift may be overridden at the
		//                   last moment..... (no matter of other decisions 
		//                   and/or calculations made before that statement);
		//                   in short: don't let the code fool you!!!!!
		//
		int         datashift;

		datashift = 0;

		//
		//  If vernier-delay was applied, it means the data has been shifted
		//  one lag to the right so we need a shift of -1 to get the central lag
		//  back at (NLAG/2)...
		//
		//if( vernier && !autocorrelation )
		if( vernier && (!autocorrelation || (autocorrelation && crosspolarization)) )
		{
			datashift -= 1;
		}
			
		//
		//  As of software version 1.0 Albert shifted *all* lagspectra one lag
		//  to the left! In this case we need a shift of +1 to get the central lag
		//  back at (NLAG/2)
		//
		//if( swver>=version1 && !autocorrelation )
		if( swver>=version1 && (!autocorrelation || (autocorrelation&&crosspolarization)))
		{
			datashift += 1;
		}

		//
		//  If we detect the bypassversion, we completely discard all the results
		//  of all stuff before! Force datashift to 0 (so no datashift is performed)
		//
		//  **************************** NOTE **************************************
		//
		//  You MUST leave this statement as the last before actually starting to do
		//  the datashift (i.e. *just* before the 'if(datashift)...' statement!!
		//
		//  ************************************************************************ 
		//
		if( swver==bypassversion )
		{
			datashift = 0;
		}		

		if( datashift )
		{
			//
			//  Ok! Data needs to be shifted!
			//
			int       bsrc, bdst;
			int       ssrc, sdst;
			int       numshift( ::abs(datashift) );
			float     buffer[ numshift ];

			if( datashift<0 )
			{
				// we need to buffer lags 0..abs(datashift)
				// so we can stick them at the back later on!
				bsrc = 0;
				bdst = nlag-numshift;
				
				// sstart = shiftstart = index from where the data needs 
				//      needs to be shifted
				ssrc = numshift;
				sdst = 0;
			}
			else
			{
				//  need to buffer the last values so we can stick them in
				//  front later on...
				bsrc = nlag-numshift;
				bdst = 0;

				// compute start and dest. indices of shift operation
				ssrc = 0;
				sdst = numshift;
			}

			//
			//  Buffer values for real part (imag will be conditional...)
			//
			::memcpy( (void*)buffer, (void*)(&ifdataptr[bsrc]), numshift*sizeof(float) );

			//
			//  Move all the real-parts
			//
			::memmove( (void*)(&ifdataptr[sdst]), (void*)(&ifdataptr[ssrc]), 
						   (nlag-numshift)*sizeof(float) );
	
			//
			// Put back buffer at correct location
			//	
			::memcpy( (void*)(&ifdataptr[bdst]), (void*)buffer, numshift*sizeof(float) );
			
			//
			//  Interferometers satisfying the following conditions have
			//  imaginary parts... All indices are increased by NLAG
			//
			if( !autocorrelation || (autocorrelation && crosspolarization) )
			{
				bsrc += nlag;
				bdst += nlag;
				ssrc += nlag;
				sdst += nlag;

				//
				//  Buffer values for imaginary part
				//
				::memcpy( (void*)buffer, (void*)(&ifdataptr[bsrc]), numshift*sizeof(float));

				//
				//  Move all the real-parts
				//
				::memmove( (void*)(&ifdataptr[sdst]), (void*)(&ifdataptr[ssrc]), 
					 		(nlag-numshift)*sizeof(float) );
	
				//
				// Put back buffer at correct location
				//	
				::memcpy( (void*)(&ifdataptr[bdst]), (void*)buffer, numshift*sizeof(float));
			}
		}

		//
		//  Having dealt with that, we can now create the complex numbers!
		//

		//
		//  Create a buffer for the re-ordered complex numbers
		//
		if( expand )
		{
			nlag *= 2;
		}
	
		cmplxreordered = new Complex[ nlag ];
	   
		//
		//  The destination index in the result matrix
		// 
		ipos = IPosition( 1, nlag );

		
		//
		//  The only exeptional case is 'expand' -> we deal with that occasion
		//  differently....
		//
		//  Note: the following code features some duplication but it enables
		//  us to form all dataarrays with one loop (only one for-loop will
		//  be executed per case!) Otherwise there might have been occasions
		//  where we have to execute two for-loops
		//
		if( expand )
		{
			uint32    centrallag( nlag/2 );
			uint32    lags2do( (nlag/2)-1 );
	
			for( uint32 i=0; i<=lags2do; i++ )
			{
				cmplxreordered[ centrallag+i ] = Complex( ifdataptr[i], 0.0f );
				cmplxreordered[ centrallag-i ] = Complex( ifdataptr[i], 0.0f );
			}
			cmplxreordered[ 0 ] = Complex( 0.0f, 0.0f );
		}
		else
		{
			//   Remeber: the structure of the data in memory is this -
			//
			//   <real><real>.....<real><imag><imag>....<imag>
			//     |                 |     |               |
			//     ------ nlag ------      ----- nlag ------
			//
			//
			//  Note: ifdataptr = ptr to float!
			//
			//  Ok. Vernier-delay has been dealt with. Now either copy
			//  the data or fill it in swapped...
			//
			//  If we need to swap it then we must mirror the lags
			//  about the central lag (which is different from just
			//  reversing the lag order (guess how we found *that*
			//  out... by experience) and take the complex
			//  conjugate. Reversing would have done the trick if this
			//  were an odd-sized array of samples but alas, the
			//  number of lags is always even....
			//
			if( swapit )
			{
				int32    lagcnt;
				int32    zerolag( nlag/2 );
				int32    lags2do( (nlag/2)-1 );
	
	
				for( lagcnt=0; lagcnt<=lags2do; lagcnt++ )
				{
					cmplxreordered[ zerolag+lagcnt ] = Complex( ifdataptr[zerolag-lagcnt],
													   -1.0*ifdataptr[nlag+zerolag-lagcnt]);
					cmplxreordered[ zerolag-lagcnt ] = Complex( ifdataptr[zerolag+lagcnt],
													   -1.0*ifdataptr[nlag+zerolag+lagcnt]);
				}
				//
				//  Only the zeroth point in the data has not been filled in....
				//
				//  HV: 22-03-2001 - After dicussion with Chris and Bob, decided
				//                   to take this statement out. The first lag
				//                   *is* filled in! So no need to force it to
				//                   zero! NOOOOOH! It must be filled in with data
				//                   from the buffer.....
				//
				//cmplxreordered[ 0 ] = Complex( 0.0f, 0.0f );
				cmplxreordered[ 0 ] = Complex( ifdataptr[0], ifdataptr[nlag] );
			}
			else
			{
				//
				//  Ultimately simple!
				//
				for( int32 lagcnt=0; lagcnt<nlag; lagcnt++ )
				{
					cmplxreordered[ lagcnt ] = Complex( ifdataptr[lagcnt],
													ifdataptr[nlag+lagcnt] );
				}
			}
		}

		//
		// Done!
		//
		datarow[ row ] = new Vector<Complex>( ipos, cmplxreordered, COPY );	    

		//
		//  Clean up, update loop stuff and go on
		//
		delete [] ifdataptr;
		delete [] cmplxreordered;
    }


    //
    //  Check shapes
    //
    uint32     rowcnt;

    datarow[0]->shape( shape0 );

    if( shape0 )
    {
		//
		//  Ok... Now we can resize the returnvalue
		//
		retval.resize( ifstodo.elements(), shape0 );

		for( rowcnt=0; rowcnt<ifstodo.elements(); rowcnt++ )
		{
			retval.row( rowcnt ) = (*datarow[rowcnt]);
		}
    }
    
    //
    //  Clean up stuff
    //
    delete [] present;

    for( uint32 tmp=0; tmp<4; tmp++ )
    {
		delete datarow[tmp];
    }

    return retval;
}






/**************************************************************/
//
//
//              Static method(s)
//
//
/**************************************************************/


//
//  Transform the FF/SFROT into a midpoint time as MEpoch
//
const MEpoch& COFVisibilityBuffer::calcMidPointAsMEpoch( const FFROT& ffrot,
							 const SFROT& startrot,
							 const SFROT& endrot ) 
{
    //
    //  Prevent this one from being created over and over again...
    //
    static MEpoch        retval;
    static MVTime        mvtime;
    static MVEpoch       mvepoch;
    static DayConversion dayConversion;

    //
    // Okiedokie, do the dirty work...
    //
    int32       year( 0 );
    SFROT       duration;
    uint32      month, day;       // month 0..11, day=1..31
    uint32      nrwholedays;
    Double      remaininsysclks;
    Double      sysclksperday( (Double)SYSCLK_FREQ*(Double)DayConversion::secondsPerDay );
    Double      daywithfraction;  // daywithfraction = <day in month>.<fractional day>
    double      halfduration;
    
    //
    //  First determine the duration of the period
    //
    duration = endrot - startrot;
    
    halfduration = (duration.get_sysclks()) / 2;
    
    //
    //  (mis)use the integrationDuration, and transform it such that
    //  it is the midpoint of the integration... (still in sysclks)
    //
    duration = (Short_Time_Value&)startrot + Short_Time_Value(halfduration);
    
    //
    //  Ok. The short-form rot is the time in SYSCLKS since the start
    //  of the YEAR.... so work out the daynr from here...
    //
    nrwholedays=(uint32)((duration.get_sysclks())/sysclksperday);

    remaininsysclks = (duration.get_sysclks()) - (Double)nrwholedays*sysclksperday;

    //
    //  Base date is the year???
    //
    if( ffrot.get_base_date()!=0xFFFFFFFF )
    {
	year = ffrot.get_base_date();
    }

    if( DayConversion::dayNrToMonthDay(month, day, nrwholedays+1, year) )
    {
	daywithfraction = (Double)day + remaininsysclks/sysclksperday;
    }
    else
    {
	month = 0;
	daywithfraction = 0.0;
    }

    mvtime = MVTime( year, month+1, daywithfraction );
    mvepoch = MVEpoch( mvtime.day() );
    
    retval = MEpoch( mvepoch, MEpoch::UTC );

    return retval;
}


//
//  Decode a MappingEntry and locate the properties for the baseline
//
COFVisibilityBuffer::MapInfo COFVisibilityBuffer::decodeMapping( const MappingEntry& mapping,
								  const COFBaseline& baseline )
{
    Bool                         xfound[ 4 ];
    Bool                         yfound[ 4 ];
    Bool                         bothfound;
    bool*                        present( baseline.getConfigured() );
    uint32                       interferometercnt;
    uint32                       nrmaps( mapping.nrMappings() );
    static Bool                  errorShown( False );
    COFVisibilityBuffer::MapInfo retval;

    //
    //  NOTE: I leave all the x/y properties uninitialized because
    //  I set a flag when they are found in the connectionmap
    //  table. So if e.g. (xfound==True) holds, one may rely on
    //  the fact that xntenna, xpolnr and xsubband are filled in.
    //
    uint32                        xantenna[ 4 ];
    uint32                        yantenna[ 4 ];
    uint32                        xlogchan[ 4 ];
    uint32                        ylogchan[ 4 ];
    uint32                        xdatasource[ 4 ];
    uint32                        ydatasource[ 4 ];
    

    //
    //  Initialize the found flags
    //
    for( uint32 tmpcnt=0; tmpcnt<4; tmpcnt++ )
    {
	xfound[tmpcnt] = yfound[tmpcnt] = False;
    }
    
    //
    //  Loop over max. 4 interferometers
    //
    interferometercnt = 0;
    
    for( uint32 ifcnt=0; ifcnt<4; ifcnt++ )
    {
	if( !present[ifcnt] )
	{
	    continue;
	}

	bothfound = False;
	
	//
	//  Get a reference to the indexed intefrerometer
	//
	const InterferometerDescriptor&  curIF( baseline[ifcnt] );
	
	//
	//  Loop over all physical inputs until we find a match!
	//
	for( uint32 i=0; i<nrmaps && !bothfound; i++ )
	{
	    const Mapping& mapref( mapping[i] );
	    
	    for( uint32 inpcnt=0; inpcnt<16 && !bothfound; inpcnt++ )
	    {
		if( !xfound[interferometercnt] 
		    && mapref.itsPhysicalInput[inpcnt]==curIF.getXInput() )
		{
		    xfound[ interferometercnt ]      = True;
		    xlogchan[ interferometercnt ]    = inpcnt;
		    xantenna[ interferometercnt ]    = mapref.itsTelescopeNr;
		    xdatasource[ interferometercnt ] = mapref.itsDataSourceId;
		}
		if( !yfound[interferometercnt] 
		    && mapref.itsPhysicalInput[inpcnt]==curIF.getYInput() )
		{
		    yfound[ interferometercnt ]      = True;
		    ylogchan[ interferometercnt ]    = inpcnt;
		    yantenna[ interferometercnt ]    = mapref.itsTelescopeNr;
		    ydatasource[ interferometercnt ] = mapref.itsDataSourceId;
		}
		
		//
		//  If both x/y were found we can quit...
		//
		bothfound = ToBool( xfound[interferometercnt] && yfound[interferometercnt] );
	    }	    
	}
	//
	//  Processed another interferometer
	//
	interferometercnt++;
    }
    
    //
    //  Don't need the flags anymore
    //
    delete [] present;
    
    //
    //  Now's the time to analyze our results.....
    //
    uint32             errorcnt( 0 );
    const BaseLineIdx& baselineidx( baseline.getIndex() );
    
    for( uint32 cnt=0; cnt<interferometercnt; cnt++ )
    {
	if( !xfound[cnt] || !yfound[cnt] )
	{
	    if( !errorShown )
	    {
			cout << "Error in MAPPING:\n";
	    }
	    
	    if( !xfound[cnt] )
	    {
		if( !errorShown )
		{
		    cout << ">> Failed to find mapping information for IF#"
			 << setw(2) << cnt << "/ ANT=X in baseline "
			 << baselineidx << "\n";
		}
		errorcnt++;
	    }
	    if( !yfound[cnt] )
	    {
		if( !errorShown )
		{
		    cout << ">> Failed to find mapping information for IF#"
			 << setw(2) << cnt << "/ ANT=Y in baseline "
			 << baselineidx << "\n";
		}
		errorcnt++;
	    }
	    if( !errorShown )
	    {
		cout << flush;
	    }
	    errorShown = True;
	    continue;
	}
	
	//
	//  If the x/y antenna labels do not agree with those from
	//  the baseline index, we have a problem
	//	
	/*
	if( xantenna[cnt]!=baselineidx.idx1() )
	{
	    cout << ">> X-antenna does not equal XANTNR from DTD for IF#" << setw(2) 
		 << cnt << "\n";
	    errorcnt++;
	}
	if( yantenna[cnt]!=baselineidx.idx2() )
	{
	    cout << ">> Y-antenna does not equal YANTNR from DTD for IF#" << setw(2)
		 << cnt << "\n";
	    errorcnt++;
	}

	if( xlogchan[cnt]!=ylogchan[cnt] )
	{
	    if( !errorShown )
	    {
		cout << ">> X/Y Subband numbers differ for IF#" << setw(2) << cnt << "\n";
	    }
	    errorcnt++;
	}

	if( xlogchan[cnt]!=subbandidx )
	{
	cout << ">> X-subband does not equal SUBBAND from DTD for IF#"
	<< setw(2) << cnt << "\n";
	errorcnt++;
	}
	if( ylogchan[cnt]!=subbandidx )
	{
	cout << ">> Y-subband does not equal SUBBAND from DTD for IF#"
	<< setw(2) << cnt << "\n";
	errorcnt++;
	}
	*/
	if( !errorShown )
	{
	    cout << flush;
	}
	
	if( errorcnt )
	{
	    errorShown = True;
	}

    }
    
    //
    //  If no errors encountered, go on!
    //
    if( !errorcnt && interferometercnt )
    {
	retval.XTelescopeNr    = xantenna[ 0 ];
	retval.YTelescopeNr    = yantenna[ 0 ];

	retval.XDataSourceId   = xdatasource[ 0 ];
	retval.YDataSourceId   = ydatasource[ 0 ];    

	retval.XLogicalChannel = xlogchan[ 0 ];
	retval.YLogicalChannel = ylogchan[ 0 ];
    }

    return retval;
}


bool COFVisibilityBuffer::readCorrelatorSetup( CorrelatorSetup& result,
					       const char* vexfilename )
{
    bool     retval = false;
    ifstream f;

    //
    //  Init result to empty correlatorsetup
    //
    result = CorrelatorSetup();

    if( !vexfilename || (::strlen(vexfilename)==0) )
    {
	cout << "COFVisibilityBuffer::readCorrelatorSetup - No vexfilename specified!" << endl;
	return retval;
    }

    //
    //  Now attempt to open the file and see what's it got in store
    //  for us 
    //
    f.open( (const char *)vexfilename );
    
    if( !f.good() )
    {
	cout << "COFVisibilityBuffer::readCorrelatorSetup - failed to open "
	     << vexfilename << " as VEX-file?!" << endl;
	return retval;
    }

    //
    //  Need to be able to read old and new format...
    //
    typedef enum _format {
		unknown, oldformat, newformat
    } format;

    typedef enum _crosscorrelations {
		notset, present, notpresent
    } crosscorrelations;
    
    //
    //  Variable that will be filled in from the vex-file
    //  comments/prep_job command line..
    //
    int               cormode;
    int               nlag;
    int               nrecpol;
    char              linebuf[ 1024 ];
    char              recpol[3] = { "\0" };
    bool              done;
    double            integrationtime;
    format            fileformat = unknown;
	unsigned int      major=(unsigned int)-1,minor=(unsigned int)-1;
    crosscorrelations products = notset;
    
    //
    //  Setup regex's for the things we want to look for
    //
    RegularExpression pjcmdline( "^\\*[ \t]*prep_job" );
    RegularExpression newformatprefix( "^\\*[ \t]*prep_job::" );

    //
    //  Init correlator-setup values to ludicrous values so we can
    //  easily detect if they've been filled in
    //
    done = false;
    cormode = -1;
    nlag = -1;
    nrecpol = -1;
    integrationtime = -1.0;
    
    while( !done )
    {
		if( f.eof() )
		{
			done = true;
			continue;
		}

		f.getline( linebuf, sizeof(linebuf) );
	
		if( linebuf==newformatprefix )
		{
			char    keyword[256];
			char    value[256];
			bool    alreadyset;
			char*   ptr;
			char*   equal;

			if( fileformat!=unknown && fileformat!=newformat )
			{
				//
				//  Old/new format mixed...
				//
				done = true;
				fileformat = unknown;
				cout << "readCorrelatorSetup: parse error - old and new formats mixed\n"
					 << "readCorrelatorsetup: in file " << vexfilename
					 << endl;
				continue;
			}

			//
			//  Passed that test. Now check which value is
			//  mentioned...
			//
			fileformat = newformat;
	    
			//
			//  We can do this safely since the RE matched and hence
			//  we MUST have at least ONE occurrence of `::' in
			//  linebuf, so no check for NULL should have to be
			//  done....
			//
			ptr   = ::strstr( linebuf, "::" );
			equal = ::strchr( ptr, '=' );
	
			if( !equal )
			{
				cout << "readCorrelatorSetup: parse error - failed to get key/value\n"
					<< "readCorrelatorsetup: in line " << linebuf << "\n"
					<< "readCorrelatorsetup: in file " << vexfilename << "\n"
					<< endl;
				continue;
			}

			//
			//  Get key/value
			//
			*equal = '\0';
			::strncpy( keyword, ptr+2, sizeof(keyword) );
			::strncpy( value, equal+1, sizeof(value) );
	    
			//
			//  filter out whitespace...
			//
			//  Note: we don't need ptr anymore so we can (re)use that
			//  var as a tmp...
			//
			ptr = &keyword[ ::strlen(keyword)-1 ];
			while( (ptr>=keyword) && (*ptr==' ' || *ptr=='\t') )
			{
				ptr--;
			}
			//
			//  ptr poinnts now *at* the last non-whitespace character
			//  in keyword. So let's put a '\0' in after that one!
			//
			*(ptr+1) = '\0';
	    
			//
			//  Filter out value
			//
			ptr = value;
			while( *ptr==' ' || *ptr=='\t' )
			{
				ptr++;
			}
			//
			//  ptr now points at the first non-whitespace char in
			//  value...
			//
			::memmove( (void *)value, ptr, ::strlen(value)-(ptr-value)+1 );
	    
			//
			//  Now remove trailing whitespace
			//
			ptr = &value[ ::strlen(value)-1 ];
			while( (ptr>=value) && (*ptr==' ' || *ptr=='\t') )
			{
				ptr--;
			}
			*(ptr+1) = '\0';

			//
			//  Now see what we really got...
			//
			//cout << "Decoded: keyword='" << keyword << "', value='" << value << "'"
			//	   << endl;

			if( ::strcmp(keyword, "integration_time")==0 &&
				(alreadyset=(integrationtime!=-1.0))==false )
			{
				integrationtime = ::atof( value );
			}
			else if( ::strcmp(keyword, "number_of_lags")==0 && 
					(alreadyset=(nlag!=-1))==false )
			{
				nlag = ::atoi( value );
			}
			else if( ::strcmp(keyword, "recorded_polarizations")==0 &&
					(alreadyset=(nrecpol!=-1))==false )
			{
				int      cnt;
				bool     valid;
		
				//
				//  Find out which polarizations were recorded...
				//
				nrecpol = ::strlen(value);
		
				valid = (nrecpol==1 || nrecpol==2);
		
				for( cnt=0; valid && cnt<nrecpol; cnt++ )
				{
					recpol[ cnt ] = ::toupper(value[cnt]);
		    
					if( !::strchr("RL", recpol[cnt]) )
					{
						break;
					}
				}
		
				if( valid && cnt<nrecpol )
				{
					cout << "readCorrelatorSetup: parse error - Value of " 
						 << keyword << " out\n"
						 << "readCorrelatorSetup: of range. Valid values are [L|R|LR|RL],\n"
						 << "readCorrelatorSetup: you specified " << value
						 << endl;

					recpol[0] = '\0';
					nrecpol   = -1;
				}
			}
			else if( ::strcmp(keyword, "cross_polarizations")==0 &&
					(alreadyset=(products!=notset))==false )
			{
				if( ::strcmp(value, "true")==0 )
				{
					products = present;
				}
				else if( ::strcmp(value, "false")==0 )
				{
					products = notpresent;
				}
				else
				{
					cout << "readCorrelatorSetup: parse error - Value of " 
						 << keyword << " out\n"
						 << "readCorrelatorSetup: of range. Valid values are [true|false],\n"
		    			 << "readCorrelatorSetup: you specified " << value << "\n"
		   				 << endl;
					products = notset;
				}
			}
			else if( ::strcmp(keyword, "ABMajor")==0 &&
					(alreadyset=(major!=(unsigned int)-1))==false )
			{
				major = (unsigned int)::atoi( value );
			}
			else if( ::strcmp(keyword, "ABMinor")==0 &&
					(alreadyset=(minor!=(unsigned int)-1))==false )
			{
				minor = (unsigned int)::atoi( value );
			}
			else
			{
				if( alreadyset )
				{
					cout << "readCorrelatorSetup: parse error - value for " << keyword 
						 << "\n"
						 << "readCorrelatorSetup: was alreay set. Discarding value " 
						 << value << "\n"
						 << "readCorrelatorSetup: in file " << vexfilename
						 << endl;
				}
				else
				{
					cout << "readCorrelatorSetup: parse error - unrecognized keyword " 
						 << keyword << "\n"
			   			 << "readCorrelatorSetup: Discarding it in file " << vexfilename
		   				 << endl;
				}
			}	    
		}
		else if( linebuf==pjcmdline )
		{
			//
			//  We allow only one prep_job commandline!
			//
			switch( fileformat )
			{
				case newformat:
					cout << "readCorrelatorSetup: parse error - old and new formats mixed\n"
						<< "readCorrelatorsetup: in file " << vexfilename
						<< endl;
					fileformat = unknown;
					break;

				case oldformat:
					cout << "readCorrelatorSetup: parse error - old format allows only\n"
						<< "readCorrelatorsetup: one prep_job cmd-line. Discarding this\n"
						<< "readCorrelatorSetup: one in file " << vexfilename
						<< endl;
					fileformat = unknown;
					break;
		
				case unknown:
				default:
					fileformat = oldformat;
					break;
			}
	    
			//
			//  If fileformat==unknown at this stage, an error has
			//  occurred and we're done!
			//
			if( fileformat==unknown )
			{
				done = true;
				continue;
			}

			//
			//  Now find out the correlation parameters
			//
			char*  tmp;
			char*  cormodeptr( ::strstr(linebuf, "-c") );
			char*  inttimptr( ::strstr(linebuf, "-i") );
	    
			if( cormodeptr )
			{
				tmp = cormodeptr + 2;
		
				while( ::strchr(" \t", *tmp) )
				{
					tmp++;
				}
				cormodeptr = tmp;
				while( ::strchr("0123456789", *tmp) )
				{
					tmp++;
				}
				*tmp ='\0';
		
				cormode = ::atoi( cormodeptr );
			}
			if( inttimptr )
			{
				tmp = inttimptr + 2;
			
				while( ::strchr(" \t", *tmp) )
				{
					tmp++;
				}
				inttimptr = tmp;
				while( ::strchr("0123456789.eE-", *tmp) )
				{
					tmp++;
				}
				*tmp ='\0';
		
				integrationtime = ::atof( inttimptr );
			}
			done = true;
		}
	}
    
    f.close();

    //
    //  Phew... now see what we found!
    //
    if( fileformat==unknown )
    {
		//
		// Error occurred. Bummer.  We don't mention it since whilst
		// parsing, the user should have found at that something's
		// wrong...
		//
		return retval;
    }
    
    if( fileformat==newformat )
	{
		//
		//  test if we got all the info we required from the new fileformat:
		//
		//   nlag            should not be equal to -1
		//   nrecpol         should not be equal to -1
		//   integrationtime should be not equal to -1.0
		//   products        should not be equal to 'notset'
		//   major/minor     well.. this one is different...
		//                   older versions of prep_job did not write
		//                   these values so we must be able to allow
		//                   these to be not present. However, if either
		//                   of the one is unset (i.e. equal to -1) it means
		//                   not both nu,bers were correctly decode and hence 
		//                   we should give an error. If both are -1 or both
		//                   are not equal to -1, we accept the values...
		//
		retval = ( (nlag!=-1) && (nrecpol!=-1) && 
		 		(integrationtime!=-1.0)  && (products!=notset) && 
			      !((major==(unsigned int)-1 && minor!=(unsigned int)-1) ||
				  (major!=(unsigned int)-1 && minor==(unsigned int)-1))
				);
	
		if( retval )
		{
			Container<uint16>   correlations;
	    
			//
			//  nrecpol is either 1,2 or -1...
			//  if it == -1, we don't arrive here...
			//
			if( nrecpol==1 )
			{
				if( recpol[0]=='R' )
				{
					correlations.add( CorrelationCode::code2ordinal(CorrelationCode(1,1)) );
				}
				else
				{
					correlations.add( CorrelationCode::code2ordinal(CorrelationCode(0,0)) );
				}
			}
			else
			{
				//
				//  We have at least these combi's
				//
				correlations.add( CorrelationCode::code2ordinal(CorrelationCode(0,0)) );
				correlations.add( CorrelationCode::code2ordinal(CorrelationCode(1,1)) );

				//
				//  If cross-polarizations were recorded, add those to the
				//  list!
				//
				if( products==present )
				{
					correlations.add( CorrelationCode::code2ordinal(CorrelationCode(0,1)) );
					correlations.add( CorrelationCode::code2ordinal(CorrelationCode(1,0)) );
				}
			}
	   
			//
			//  For backward compatibility we translate a software version
			//  that was not set to vs. 0.0 (older versions of prep_job
			//  did not write the ABMajor/ABMinor keywords in the vex-file).
			//  So if we did not find them, major/minor should be -1.
			//  It suffices to test if only one of them equals -1 since the
			//  test of whether they're both unequal to -1 or both equal to
			//  -1 was already performed.
			//
			if( major==(unsigned int)-1 )
			{
				major = 0;
				minor = 0;
			}
			//
			//  Now we can fully create the setup!
			//
			result = CorrelatorSetup( nlag/2, correlations, integrationtime,
									  CorrelatorSetup::CorSWVersion(major, minor) );
		}
		else
		{
			cout << "readCorrelatorSetup: Not all required information found in new format"
				<< endl;

			if( nlag==-1 )
			{
				cout << "readCorrelatorSetup: missing number_of_lags keyword\n" << endl;
			}
			if( nrecpol==-1 )
			{
				cout << "readCorrelatorSetup: missing recorded_polarizations keyword\n" 
					 << endl;
			}
			if( integrationtime==-1.0 )
			{
				cout << "readCorrelatorSetup: missing integration_time keyword\n" 
					 << endl;
			}
			if( products==notset )
			{
				cout << "readCorrelatorSetup: missing cross_polarizations keyword\n" 
					 << endl;
			}
			if( major==(unsigned int)-1 )
			{
				cout << "readCorrelatorSetup: missing ABMajor keyword\n" 
					 << endl;
			}
			if( minor==(unsigned int)-1 )
			{
				cout << "readCorrelatorSetup: missing ABMinor keyword\n" 
					 << endl;
			}
		}
    }
    else
    {
		//
		//  Old format, consisting of correlatormode + integration
		//  time
		//
		retval = ( (cormode!=-1) && (integrationtime!=-1.0) );
	
		if( retval )
		{
			result = CorrelatorSetup::correlatorModeToSetup( cormode, integrationtime );

			if( result.nrOfSpectralPoints()==0xFFFFFFFF )
			{
				cout << "readCorrelatorSetup: no CorrelatorSetup found for correlator mode #"
					<< cormode << endl;

				result = CorrelatorSetup();
				retval = false;
			}
		}
		else
		{
			cout << "readCorrelatorSetup: Not all required information found in new format"
				<< endl;
		}
	}
    return retval;
}



