//
//  Implementation of the CorrelatorSetup - class
//
//  Author:   Harro Verkouter,  13-07-1999
//
//
//    $Id: CorrelatorSetup.cc,v 1.2 2001/05/10 06:29:00 verkout Exp $
//
//
//    $Log: CorrelatorSetup.cc,v $
//    Revision 1.2  2001/05/10 06:29:00  verkout
//    HV: Added support to j2ms2 to make decisions based on Albert's different software versions (ABMajor/ABMinor keywords in output VEXfile). Also a 'bypass' mode is present now (12345/67890)=>data is left unprocessed.
//
//    Revision 1.1.1.1  2000/03/20 15:14:36  verkout
//    HV: imported aips++ implement stuff
//
//
#ifndef CORRELATORSETUP_H
#include <vlbi/j2ms/CorrelatorSetup.h>
#endif

#ifndef CORRELATIONCODE_H
#include <vlbi/DtdLUT/CorrelationCode.h>
#endif


#include <iostream.h>


//
//  Alloc space and initalize static datamembers of CorrelatorSetup-class
//
const CorrelatorSetup::CorSWVersion CorrelatorSetup::bypassVersion = 
				CorrelatorSetup::CorSWVersion( 12345, 67890 );


static const char* verswarn = "**** WARNING: SYSTEM IS IN BYPASS MODE!\n"
							  "     DATA WILL BE IN RAW FORMAT; NO CORRECTIONS APPLIED!";

ostream& operator<<( ostream& os, const CorrelatorSetup& csref )
{
	const CorrelatorSetup::CorSWVersion  swver( csref.getSoftwareVersion() );

    os << "CorrelatorSetup:\n"
       << "configured             : " << csref.nrOfSpectralPoints() << " spectral points\n"
       << "integration time       : " << csref.getIntegrationTime() << "s\n"
	   << "albertsoftware v.      : " << swver.major() << '.' << swver.minor() << "\n"
	   << ((swver==CorrelatorSetup::bypassVersion)?(verswarn):(""))	
       << "correlations formed are: ";
    for( uint32 i=0, n=csref.getCorrelations().elements(); i<n; i++ )
    {
		os << CorrelationCode::ordinal2code( csref.getCorrelations()(i) );
		if( i<(n-1) )
		{
			os << ", ";
		}
    }
    
    os << flush;
    return os;
}


//
//  Comparison of versions
//
Bool CorrelatorSetup::_CorSWVersion::operator<( const CorrelatorSetup::_CorSWVersion& rhs ) const
{
	return ToBool((myMajor<rhs.myMajor) || (myMajor==rhs.myMajor && myMinor<rhs.myMinor));
}

Bool CorrelatorSetup::_CorSWVersion::operator<=( const CorrelatorSetup::_CorSWVersion& rhs ) const
{
	return ToBool((myMajor<rhs.myMajor) || (myMajor==rhs.myMajor && myMinor<=rhs.myMinor));
}

Bool CorrelatorSetup::_CorSWVersion::operator>( const CorrelatorSetup::_CorSWVersion& rhs ) const
{
	return ToBool((myMajor>rhs.myMajor) || (myMajor==rhs.myMajor && myMinor>rhs.myMinor));
}

Bool CorrelatorSetup::_CorSWVersion::operator>=( const CorrelatorSetup::_CorSWVersion& rhs ) const
{
	return ToBool((myMajor>rhs.myMajor) || (myMajor==rhs.myMajor && myMinor>=rhs.myMinor));
}



//
//  Comparison
//
Bool operator==( const CorrelatorSetup& lhside, const CorrelatorSetup& rhside )
{
    Bool                     cond1 = False;
    Bool                     cond2 = False;
    Bool                     cond3 = False;
	Bool                     cond4 = False;
    Container<Bool>          alreadydone;
    const Container<uint16>& lh( lhside.myCorrelations );
    const Container<uint16>& rh( rhside.myCorrelations );

    cond1 = ToBool( lhside.myNumberOfSpectralPoints==rhside.myNumberOfSpectralPoints );
    cond2 = ToBool( lhside.myIntegrationTime==rhside.myIntegrationTime );

    if( (cond3=ToBool(lh.elements()==rh.elements()))==True )
    {
		alreadydone = Container<Bool>( lh.elements(), False );
    }
    
    
    //
    //  If cond3 evaluates to False, we don't even have to go through
    //  the loop....
    //
    for( uint32 i=0; cond3 && i<lh.elements(); i++ )
    {
		uint32     j;

		for( j=0; j<rh.elements(); j++ )
		{
			if( rh(j)==lh(i) && !alreadydone(j) )
			{
				alreadydone(j) = True;
				break;
			}
		}
		//
		//  The loop finishes early if the i-th element of lh was
		//  found somewhere in rh. So if j==rh.elements() the i-th
		//  element was not found, indicating that lh and rh are NOT
		//  equal
		//
		cond3 = ToBool( j!=rh.elements() );
    }

	cond4 = ToBool( lhside.mySoftwareVersion==rhside.mySoftwareVersion );
    return ToBool( cond1 && cond2 && cond3 && cond4 );
}


Bool operator!=( const CorrelatorSetup& lhside, const CorrelatorSetup& rhside )
{
    return ToBool( !(lhside==rhside) );
}




CorrelatorSetup::CorrelatorSetup() :
    myNumberOfSpectralPoints( 0xFFFFFFFF ),
    myIntegrationTime( 0.0 )
{
}

CorrelatorSetup::CorrelatorSetup( uint32 nrofspectralpoints,
								  const Container<uint16>& correlations,
								  double integrationtime,
								  const CorrelatorSetup::CorSWVersion& swver ) :
    myNumberOfSpectralPoints( nrofspectralpoints ),
    myCorrelations( correlations ),
    myIntegrationTime( integrationtime ),
	mySoftwareVersion( swver )
{
}

CorrelatorSetup::CorrelatorSetup( const CorrelatorSetup& other ) :
    myNumberOfSpectralPoints( other.myNumberOfSpectralPoints ),
    myCorrelations( other.myCorrelations ),
    myIntegrationTime( other.myIntegrationTime ),
	mySoftwareVersion( other.mySoftwareVersion )
{
}

CorrelatorSetup& CorrelatorSetup::operator=( const CorrelatorSetup& other )
{
    if( this!=&other )
    {
		myNumberOfSpectralPoints = other.myNumberOfSpectralPoints;
		myCorrelations           = other.myCorrelations;
		myIntegrationTime        = other.myIntegrationTime;
		mySoftwareVersion        = other.mySoftwareVersion;
    }
    return *this;
}


uint32 CorrelatorSetup::nrOfSpectralPoints( void ) const
{
    return myNumberOfSpectralPoints;
}

const Container<uint16>& CorrelatorSetup::getCorrelations( void ) const
{
    return myCorrelations;
}

double CorrelatorSetup::getIntegrationTime( void ) const
{
    return myIntegrationTime;
}

const CorrelatorSetup::CorSWVersion& CorrelatorSetup::getSoftwareVersion( void ) const
{
	return mySoftwareVersion;
}

//
//  Delete any allocated resources...
//
CorrelatorSetup::~CorrelatorSetup()
{
}


/***************************************/
//
//    STATIC METHOD(s)
//
/***************************************/
CorrelatorSetup CorrelatorSetup::correlatorModeToSetup( int correlatormode, double integrationtime )
{
    uint32                        nrcombinations( 0 );
    uint32                        nrfreqpoints( 0 );
    CorrelatorSetup               retval;
    Container<uint16>             correlations;
	CorrelatorSetup::CorSWVersion swver;   
 
    switch( correlatormode )
    {
	case 201:
	case 203:
	    nrfreqpoints = 32;
	    nrcombinations = 1;
	    break;
	    
	case 207:
	case 209:
	    nrfreqpoints = 32;
	    nrcombinations = 2;
	    break;

	case 213:
	case 215:
	case 217:
	case 219:
	    nrfreqpoints = 32;
	    nrcombinations = 4;
	    break;
	    
	case 221:
	case 223:
	case 225:
	    nrfreqpoints = 64;
	    nrcombinations = 1;
	    break;

	case 229:
	case 231:
	case 233:
	    nrfreqpoints = 64;
	    nrcombinations = 2;
	    break;

	case 237:
	case 239:
	case 241:
	    nrfreqpoints = 64;
	    nrcombinations = 4;
	    break;
	    
	case 243:
	case 245:
	case 247:
	case 249:
	    nrfreqpoints = 128;
	    nrcombinations = 1;
	    break;

	case 251:
	case 253:
	case 255:
	    nrfreqpoints = 128;
	    nrcombinations = 2;
	    break;

	case 257:
	case 259:
	    nrfreqpoints = 128;
	    nrcombinations = 4;
	    break;
	    
	case 261:
	case 263:
	case 265:
	    nrfreqpoints = 256;
	    nrcombinations = 1;
	    break;

	case 267:
	case 269:
	    nrfreqpoints = 256;
	    nrcombinations = 2;
	    break;

	case 271:
	    nrfreqpoints = 256;
	    nrcombinations = 4;
	    break;
	    
	case 273:
	case 275:
	    nrfreqpoints = 512;
	    nrcombinations = 1;
	    break;
	    
	case 277:
	    nrfreqpoints = 512;
	    nrcombinations = 2;
	    break;
	    
	case 279:
	    nrfreqpoints = 1024;
	    nrcombinations = 1;
	    break;
	    
	default:
	    break;
    }
    
    if( nrfreqpoints!=0 )
    {
		if( nrcombinations>=1 )
		{
			correlations.add( CorrelationCode::code2ordinal(CorrelationCode(0, 0)) );
		}
		if( nrcombinations>=2 )
		{
			correlations.add( CorrelationCode::code2ordinal(CorrelationCode(1, 1)) );
		}
		if( nrcombinations==4 )
		{
			correlations.add( CorrelationCode::code2ordinal(CorrelationCode(0, 1)) );
			correlations.add( CorrelationCode::code2ordinal(CorrelationCode(1, 0)) );
		}
		retval = CorrelatorSetup( nrfreqpoints, correlations, integrationtime, swver );
    }
    return retval;
}
