//
//  Implementation of the DTDScan-class
//
//  Author:  Harro Verkouter,  2-6-1998
//
//  $Id: DTDScan.cc,v 1.3 2001/06/22 14:05:06 verkout Exp $
//
//  $Log: DTDScan.cc,v $
//  Revision 1.3  2001/06/22 14:05:06  verkout
//  HV: A number of changes: * fixed conflict between class names in j2ms and JCCS (class 'Scan' was defined in both) * fixed bug in j2ms2, Lorant uncovered it: when only RR product in data AND baseline needed swapping => j2ms2 coredumped * made some cosmetic changes in output of j2ms2 * removed the necessity to have 'J2MSDBPATH' environment var set
//
//  Revision 1.2  2001/05/10 06:29:03  verkout
//  HV: Added support to j2ms2 to make decisions based on Albert's different software versions (ABMajor/ABMinor keywords in output VEXfile). Also a 'bypass' mode is present now (12345/67890)=>data is left unprocessed.
//
//  Revision 1.1.1.1  2000/03/20 15:14:35  verkout
//  HV: imported aips++ implement stuff
//
//
#ifndef DTDSCAN_H
#include <vlbi/j2ms/DTDScan.h>
#endif

#include <vlbi/tmplsrc/Container.h>

#ifndef INIFILE_H
#include <inifile/IniFile.h>
#endif

#ifndef AIPSEXCEPTIONOBJECT_H
#include <vlbi/Utilities/AIPSExceptionObject.h>
#endif

//#ifndef EXCEPTIONOBJECT_H
//#include <data_handler/ExceptionObject.h>
//#endif
#ifndef EXCEPTIONOBJECT_H
#include <ExceptionObject/ExceptionObject.h>
#endif

#ifndef REGULAREXPRESSIONS_H
#include <iniutils/RegularExpressions.h>
#endif

#ifndef AIPS_MVTIME_H
#include <aips/Quanta/MVTime.h>
#endif

#ifndef AIPS_REGEX_H
#include <aips/Utilities/Regex.h>
#endif

#ifndef EXPERIMENT_H
#include <vlbi/j2ms/Experiment.h>
#endif

#ifndef DTDVISIBILITYBUFFER_H
#include <vlbi/j2ms/DTDVisibilityBuffer.h>
#endif

#ifndef CORRELATORSETUP_H
#include <vlbi/j2ms/CorrelatorSetup.h>
#endif

#include <strstream.h>
#include <iostream.h>


ostream& operator<<( ostream& os, const j2ms::DTDScan& sref )
{
    os << (const j2ms::Scan&)sref << endl;
    return os;
}

namespace j2ms
{
uint32 frequencyOfArg( const Container<unsigned short int>& container,
					   const unsigned short int& arg )
{
   uint32    retval( 0 );
    
    for( uint32 cnt=0; cnt<container.elements(); cnt++ )
    {
        if( container(cnt)==arg )
        {
            retval++;
        }
    }
    return retval;
}




DTDScan::DTDScan( const String& scanid, const DTDVisibilityBuffer* visbufptr ) :
    Scan( visbufptr ),
    itsScanid( scanid ),
    itsBOCFRate( 0.0 )
{
    //
    //  Derive the scan-directory
    //
    char         buffer[512];
    String       scanDirectory;
    ostrstream   strm( buffer, 512 );

    strm << this->getExperiment().getRootDir() << '/' << itsScanid << ends;
    scanDirectory = strm.str();

    //
    //  Read ini-file....
    //
    IniFile    theInifile( scanDirectory+"/"+itsScanid+".ini" );
    
    /*
    cout << "Opened inifile " << itsScanDirectory << "/" << itsScanid << ".ini\n" 
	 << "Start reading source code..." << flush;
	 */

    //
    //  Read the pointinglabel from the inifile
    //
    theInifile.getValue( "Scan", "source", itsSourcecode, "<DEF>" );
    
    if( itsSourcecode=="<DEF>" )
    {
	throw( AipsError("Scan "+itsScanid+": no source specified!") );
    }
    
    /*
    cout << itsSourcecode << '\n'
	 << "Start reading frequencysetup..." << flush;
	 */

    //
    //  We need the frequencysetup code
    //
    theInifile.getValue( "Scan", "frequencysetup", itsFrequencycode, "<DEF>" );
    
    if( itsFrequencycode=="<DEF>" )
    {
	throw( AipsError("Scan "+itsScanid+": no frequencysetup specified!") );
    }

    /*
    cout << itsFrequencycode << '\n'
	 << "Start decoding scan start time..." << flush;
	 */

    //
    //  Basically, we need to read the start-time
    //
    String        dateobsstring;
    String        timesysstring;
    Quantity      dateobs;
    MEpoch::Types timesys( MEpoch::DEFAULT );


    theInifile.getValue( "Scan", "dateobs", dateobsstring, "<DEF>" );
    theInifile.getValue( "Scan", "timesystem", timesysstring, "<DEF>" );
    
    if( dateobsstring=="<DEF>" )
    {
	throw( AipsError("DTDScan "+itsScanid+": no starttime (=dateobs) specified!") );
    }
    
    if( timesysstring!="<DEF>" )
    {
	if( !MEpoch::getType(timesys, timesysstring) )
	{
	    throw( AipsError("DTDScan "+itsScanid+": invalid timesystem specified!") );
	}
    }

    /*
    cout << "values read from file..." << flush;
    */

    //
    //  Parse the string...
    //
    if( !MVTime::read(dateobs, dateobsstring) )
    {
	throw( AipsError("DTDScan "+itsScanid+": invalid dateobs!") );
    }
    
    //
    //  Make the start-time
    //
    itsStarttime = MEpoch( dateobs, timesys );

    /*
    cout << itsStarttime << '\n'
	 << "Start finding bocfrate..." << flush;
	 */

    //
    //  Get the bocfrate (if any given)
    //
    Double    bocfrate;
    
    theInifile.getValue( "Scan", "bocfrate", bocfrate, -1.0 );
    
    if( bocfrate>0.0 )
    {
	itsBOCFRate = bocfrate;
    }

    //
    //  Read the correlator setup
    //
    int               nrfrequencypoints;
    int               maxcombis( 4 );
    int               nrcombisfound;
    double            integrationtime;
    //Regex             validcombi( "[01]\\{2\\}" );
    String            polcombis;
    String            result[ maxcombis ];
	unsigned int      major;
	unsigned int      minor;
    CorrelationCode   tmpcode;
    Container<uint16> correlations;
    RegularExpression validcombi( "^[01][01]$" );

    theInifile.getValue( "CorrelatorSetup", "nrfrequencypoints", nrfrequencypoints, -1 );
    theInifile.getValue( "CorrelatorSetup", "polarizationcombinations", polcombis, "<DEF>" );
    theInifile.getValue( "CorrelatorSetup", "integrationtime", integrationtime, -1.0 );
    theInifile.getValue( "CorrelatorSetup", "ABMajor", major, 0 );
    theInifile.getValue( "CorrelatorSetup", "ABMinor", minor, 0 );

    if( nrfrequencypoints<0 )
    {
	throw( AipsError("DTDScan "+itsScanid+": missing nr of frequencypoints in CorrelatorSetup") );
    }
    
    if( integrationtime<0.0 )
    {
	throw( AipsError("DTDScan "+itsScanid+": missing integrationtime in CorrelatorSetup") );
    }
    
    if( polcombis=="<DEF>" )
    {
	throw( AipsError("DTDScan "+itsScanid+": no configured polarization combinations entry in "
			 +"CorrelatorSetup") );
    }
    //
    // Parse the polarization combinations....
    //
    //  Allowed/expected format:
    //
    //  {[01]\{2\}:}*
    //
    nrcombisfound = split( polcombis, result, maxcombis, ":" );
    
    if( !nrcombisfound )
    {
	throw( AipsError("DTDScan "+itsScanid+": error in polarization combinations entry in "
			 +"CorrelatorSetup") );
    }
    
    for( int i=0; i<nrcombisfound; i++ )
    {
		if( (const char *)(result[i])==validcombi )
		{
			uint16   arg;

			tmpcode = CorrelationCode( (int)((result[i][0])-'0'),
									   (int)((result[i][1])-'0') );

			arg = (uint16)tmpcode;	    
			if( ::frequencyOfArg(correlations, arg)==0 )
			{
				correlations.add( (uint16)tmpcode );
			}
			else
			{
				cout << "DTDScan " << itsScanid << ": correlation code "
		   			 << tmpcode << " is duplicated in CorrelatorSetup." << endl;
			}
		}
    }

    if( !correlations.elements() )
    {
	throw( AipsError("DTDScan "+itsScanid+": no valid polarization combinations found in "
			 +"CorrelatorSetup entry") );
    }
    
    itsCorrelatorSetup = CorrelatorSetup( (uint32)nrfrequencypoints,
										  correlations, integrationtime,
										  CorrelatorSetup::CorSWVersion(major, minor) );

    /*
    cout << bocfrate << endl;
    */

    //
    //  Send a logmessage to tell that we're in business....
    //
    String     hello( "DTDScan "+itsScanid+" created" );
    
    this->getExperiment().logLogMessage( hello, LogOrigin("DTDScan", "DTDScan", WHERE) );
}



Double DTDScan::getBOCFRate() const
{
    return itsBOCFRate;
}

const String& DTDScan::getSourceCode( void ) const
{
    return itsSourcecode;
}

const String& DTDScan::getScanId( void ) const
{
    return itsScanid;
}


const String& DTDScan::getFrequencyCode( void ) const
{
    return itsFrequencycode;
}

const MEpoch& DTDScan::getStartTime( void ) const
{
    return itsStarttime;
}

const CorrelatorSetup& DTDScan::getCorrelatorSetup( void ) const
{
    return itsCorrelatorSetup;
}


DTDScan::~DTDScan()
{
    //
    //  NOTE: don't delete the experiment-ptr, we only keep that as a
    //  'reference' to our parent, we don't own it!
    //
}

// end of namespace
};

