//
//  DTDScan - holding the properties of a scan saved as a binary
//  correlator dump(aka Alberts format)!
//
//  Author:   Harro Verkouter,  2-6-1999
//
//  $Id: DTDScan.h,v 1.2 2001/06/22 14:05:07 verkout Exp $
//
//  $Log: DTDScan.h,v $
//  Revision 1.2  2001/06/22 14:05:07  verkout
//  HV: A number of changes: * fixed conflict between class names in j2ms and JCCS (class 'Scan' was defined in both) * fixed bug in j2ms2, Lorant uncovered it: when only RR product in data AND baseline needed swapping => j2ms2 coredumped * made some cosmetic changes in output of j2ms2 * removed the necessity to have 'J2MSDBPATH' environment var set
//
//  Revision 1.1.1.1  2000/03/20 15:14:35  verkout
//  HV: imported aips++ implement stuff
//
//
#ifndef DTDSCAN_H
#define DTDSCAN_H

//
//  This is our base-class; we need it..
//
#ifndef SCAN_H
#include <vlbi/j2ms/Scan.h>
#endif

#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef AIPS_MEASURES_H
#include <aips/Measures.h>
#endif

#ifndef AIPS_MEPOCH_H
#include <aips/Measures/MEpoch.h>
#endif

#ifndef CORRELATORSETUP_H
#include <vlbi/j2ms/CorrelatorSetup.h>
#endif


//
// Forward declaration
//
class ostream;
class DTDVisibilityBuffer;

namespace j2ms
{

//
//  The scan-class itself
//
class DTDScan :
    public j2ms::Scan
{
    //
    // Only an Experiment can create a valid DTDScan-object...
    //
    friend class DTDVisibilityBuffer;

public:
    //
    //  (RO)Access to properties of this scan
    //

    //
    //  This one is DTD-Scan specific
    //
    Double                    getBOCFRate() const;

    //
    //  These come from the baseclass
    //
    virtual const String&     getScanId( void ) const;
    virtual const String&     getSourceCode( void ) const;
    virtual const String&     getFrequencyCode( void ) const;
    virtual const MEpoch&     getStartTime( void ) const;

    virtual const CorrelatorSetup& getCorrelatorSetup( void ) const;
    
    //
    //  Delete the scan-resources
    //
    virtual ~DTDScan();

protected:
    //
    //  Create a DTDScan, properties are read from <exp.rootdir>/<scanid>/<scanid>.ini
    //
    //  NOTE: If an error occurred, an AipsError-exception will be
    //  thrown.
    //
    DTDScan( const String& scanid, const DTDVisibilityBuffer* visbufptr );
    
private:
    //
    //  The scan's private parts
    //

    //
    //  The start-time of this scan
    //
    MEpoch               itsStarttime;

    //
    //  During this scan, the (sub)array has been looking at the
    //  source indicated by this code
    //
    String               itsSourcecode;

    //
    //  And observing with frequency setting indicated by this code
    //
    String               itsFrequencycode;

    //
    //  The 'id' of the scan
    //
    String               itsScanid;

    //
    //  The number of BOCF/s for this scan...
    //
    Double               itsBOCFRate;

    //
    // The correlator setup in use when correlating this scan!
    //
    CorrelatorSetup      itsCorrelatorSetup;

    //
    //  Private method:
    //
    

    //
    //  Empty scans don't exist?
    //
    DTDScan();
    DTDScan( const DTDScan& );
    DTDScan& operator=( const DTDScan& );
};

// end of namespace
};

//
//  Show summary of the scan on a stream
//
ostream& operator<<( ostream& os, const j2ms::DTDScan& sref );


#endif
