//
//  Implementation of the DTDVisibilityBuffer class
//
//  Author:    Harro Verkouter,  18-11-1998
//
// $Id: DTDVisibilityBuffer.cc,v 1.4 2001/06/22 14:05:09 verkout Exp $
//
#ifndef DTDVISIBILITYBUFFER_H
#include <vlbi/j2ms/DTDVisibilityBuffer.h>
#endif

#ifndef VISIBILITY_H
#include <vlbi/j2ms/Visibility.h>
#endif

#ifndef AIPS_EXCEPTIONS_H
#include <aips/Exceptions.h>
#endif

#ifndef INIFILE_H
#include <inifile/IniFile.h>
#endif

#ifndef CORRELATION_CODE_H
#include <vlbi/DtdLUT/CorrelationCode.h>
#endif

#ifndef EVNFRAFLAGS_H
#include <vlbi/evnfraflags.h>
#endif

#ifndef INIEXPERIMENT_H
#include <vlbi/j2ms/IniExperiment.h>
#endif

#ifndef DTDSCAN_H
#include <vlbi/j2ms/DTDScan.h>
#endif

#ifndef BASELINE_H
#include <vlbi/DtdLUT/BaseLine.h>
#endif

#ifndef BASELINEIDX_H
#include <vlbi/DtdLUT/BaseLineIdx.h>
#endif

#ifndef AIPS_MEPOCH_H
#include <aips/Measures/MEpoch.h>
#endif

#ifndef AIPS_MVEPOCH_H
#include <aips/Quanta/MVEpoch.h>
#endif

#ifndef FREQUENCYCONFIG_H
#include <vlbi/j2ms/FrequencyConfig.h>
#endif

#ifndef FREQUENCYDB_H
#include <vlbi/j2ms/FrequencyDB.h>
#endif

#ifndef AIPS_MVTIME_H
#include <aips/Quanta/MVTime.h>
#endif

#ifndef VISLABEL_H
#include <vlbi/DtdLUT/VisLabel.h>
#endif

#ifndef FREQCOUNTER_H
#include <vlbi/Utilities/FreqCounter.h>
#endif

#ifndef CORRELATORSETUP_H
#include <vlbi/j2ms/CorrelatorSetup.h>
#endif

#include <iostream.h>
#include <stdlib.h>


/************************************************************/
//
//
//    This is support stuff to decode the 
//    SIGNAL SOURCE stuff
//
//  Cut from Albert's code,
//
//  calchb_unpack_hdr.c 
//  evnfra_calchx.c
//  dzbsp_lstsx.c
//  lstg_signal_source.c
//
//
//  We have (more or less) verified that the SigSrc decoding works,
//  it worked for one integration, in file 'RAW_S_.DAT'
//
//
/************************************************************/

const int KCFGFACT[5]=  /* to convert KCFG codes to division factors */
{  
    1,2,4,8,16
};  
const Double    two_32( 4294967296.0 );


typedef struct EVNHEADER
{
    short int DATA_IDENT;            /* data identification word */
    short int FRAME_SER_NR;          /* frame serial number */
    int DELAY_OFFS;                  /* delay offset */
    int DELAY_RATE;                  /* delay rate */
    int PHASE_OFFS;                  /* phase offset */
    int PHASE_RATE;                  /* phase rate */
    int PHASE_ACCEL;                 /* phase acelleration */
    int PHASE_PER;                   /* phase increment periods */
    short CHECK_WORD;                /* check word */          
} EVN_HEADER_TYPE;

typedef struct _SigSrc
{
    //
    //  Basic stuff, 
    //  
    //  EVN_HEADER_TYPE is Albert's struct, 
    //  
    //  ARR32 is the array that Albert parses to fill in the
    //  EVN_HEADER_TYPE, whereas we first parse the array read from
    //  the datafile to fill in the ARR32-array....
    //
    EVN_HEADER_TYPE   EVNHDR;
    unsigned int      ARR32[12];

    //
    //  These are the derived values that we'd like to access to....
    //
    Long   channelid;
    Long   suid;
    uint16 frsernr1;
    uint16 frsernr2;
    Float  deloff;
    Float  delrat;
    Float  phaoff;
    Float  pharat;
    Float  phaacc;
    Long   pharatper;
    int    kcfg;
    int    ovsamp;


    //
    //  Default c'tor
    //
    _SigSrc( ) :
	channelid( 0 ),
	suid( 0 ),
	frsernr1( 0 ),
	frsernr2( 0 ),
	deloff( 0.0 ),
	delrat( 0.0 ),
	phaoff( 0.0 ),
	pharat( 0.0 ),
	phaacc( 0.0 ),
	pharatper( 0 ),
	kcfg( 0 ),
	ovsamp( 0 )
    {
    }

    //
    //    Create from array of unsigned int, supposedly just read from
    //    file....
    //
    _SigSrc( unsigned int* ARR24 ) :
	channelid( 0 ),
	suid( 0 ),
	frsernr1( 0 ),
	frsernr2( 0 ),
	deloff( 0.0 ),
	delrat( 0.0 ),
	phaoff( 0.0 ),
	pharat( 0.0 ),
	phaacc( 0.0 ),
	pharatper( 0 ),
	kcfg( 0 ),
	ovsamp( 0 )
    {
	if( ARR24 )
	{
	    this->attach( ARR24 );
	}
    }


    //
    //  The attach - method:
    //
    //  parses ARR24 to fill in member EVNHDR, after which
    //  EVNHDR is parsed to fill in ARR32... after which we can safely
    //  fill in our derived values (in floats etc.).....
    //
    void attach( unsigned int* ARR24 )
    {
	//
	//  Some predefined bitmasks
	//
	const unsigned int    MASK0_7( 0x000000FF );
	const unsigned int    MASK0_15( 0x0000FFFF );
	const unsigned int    MASK0_23( 0x00FFFFFF );

	//
	//  Decode header
	//
	EVNHDR.DATA_IDENT= (ARR24[0] >> 8) & MASK0_15;    

	EVNHDR.FRAME_SER_NR= ((ARR24[0] & MASK0_7) << 8) |
	    ((ARR24[1] >> 16) & MASK0_7);
	    
	EVNHDR.DELAY_OFFS= ((ARR24[1] & MASK0_15) << 16) |
	    ((ARR24[2] >> 8) & MASK0_15);
	
	EVNHDR.DELAY_RATE= ((ARR24[2] & MASK0_7) << 24) |
	    (ARR24[3] & MASK0_23);
	
	EVNHDR.PHASE_OFFS= ((ARR24[4] & MASK0_23) << 8) |
	    ((ARR24[5] >> 16) & MASK0_7);
	
	
	EVNHDR.PHASE_RATE= ((ARR24[5] & MASK0_15) << 16) |
	    ((ARR24[6] >> 8) & MASK0_15);
	
	EVNHDR.PHASE_ACCEL= ((ARR24[6] & MASK0_7) << 24) |
		(ARR24[7] & MASK0_23);
	
	EVNHDR.PHASE_PER= ((ARR24[8] & MASK0_23) << 8) |
	    ((ARR24[9] >> 16) & MASK0_7);
	
	EVNHDR.CHECK_WORD= (ARR24[9] & MASK0_15);

	//
	//  Transfer values to ARR32
	//
	ARR32[0]= (unsigned int) EVNHDR.DATA_IDENT & 63;
	ARR32[1]= (unsigned int) ((EVNHDR.DATA_IDENT >> 6) & 31);
	ARR32[2]= (unsigned int) ((EVNHDR.DATA_IDENT >> 11) & 31);
	ARR32[3]= ((unsigned int) EVNHDR.FRAME_SER_NR) & MASK0_15;
	ARR32[4]= (unsigned int) EVNHDR.DELAY_OFFS;     
	ARR32[5]= (unsigned int) EVNHDR.DELAY_RATE;       
	ARR32[6]= (unsigned int) EVNHDR.PHASE_OFFS;
	ARR32[7]= (unsigned int)((EVNHDR.PHASE_PER >> 24) & 255);
	ARR32[8]= (unsigned int) EVNHDR.PHASE_RATE;
	ARR32[9]= (unsigned int) (EVNHDR.PHASE_PER & 0x00FFFFFF);
	ARR32[10]= (unsigned int) EVNHDR.PHASE_ACCEL;
	ARR32[11]= ((unsigned int) EVNHDR.CHECK_WORD) & MASK0_15;                

	//
	//  Now transfer from ARR32 into local variables....
	//
	Double   pclkhz( 32000000.0 );
	Double   pzerorot;
	Double   pincrot;
	Double   paccelrot;
	Double   pcnthz;
	Double   pratehz;
	Double   paccelhz;
	Double   pclkns;
	Double   dzerosa;
	Double   dincrsa;
	Double   dzerons;
	Double   dratens;
	int      t;
	
	    
	kcfg = ARR32[7]&7;
	ovsamp = (ARR32[7]>>3)&7;
	pclkhz = pclkhz/((double)KCFGFACT[kcfg]);
	pzerorot = ((double)(ARR32[6]))/two_32;
	
	t = (int)ARR32[8];
	pincrot = ((double)t)/two_32;
	
	t = (int)ARR32[10];
	paccelrot = ((double)t)/two_32;
	
	pcnthz = 32000000.0;
	
	if( ARR32[9]!=0 )
	{
	    pcnthz /= ((double)ARR32[9]);
	}
	
	pratehz = pincrot*pclkhz;
	paccelhz = paccelrot*pcnthz*pclkhz;
	
	pclkns = 1000000000.0/pclkhz;
	pclkhz = 32000000.0*pclkns;
	dzerosa = ((double)ARR32[4])/two_32;
	
	t = (int)ARR32[5];
	dincrsa = ((double)t)/two_32;
	
	dzerons = dzerosa*pclkns;
	dratens = dincrsa*pclkhz;
	
	//
	//  Finally, we can derive the struct members from the tmp. variables
	//
	channelid = ARR32[1];
	suid = ARR32[2];
	frsernr1 = ARR32[3];
	deloff = dzerons;
	delrat = dratens;
	phaoff = 0.0;
	pharat = pratehz;
	phaacc = paccelhz;
	
	pharatper= ARR32[9];
	frsernr2 = ARR32[11];
    }
} SigSrc;


ostream& operator<<( ostream& os, const SigSrc& ssref )
{
    os.form( "%4d ", ssref.frsernr1 );
    os.form( "%9.1f ", ssref.deloff );
    os.form( "%9.1f ", ssref.delrat );
    os.form( "%9.1f ", ssref.phaoff );
    os.form( "%9.1f ", ssref.pharat );
    os.form( "%9.1f ", ssref.phaacc );
    os.form( "%4d ", ssref.frsernr2 );
    os.form( "%2d %2d ", ssref.suid, ssref.channelid );
    os.form( "%2d %2d", ssref.kcfg, ssref.ovsamp );
    
    return os;
}







/************************************************************/
//
//
//    Here follows the DTDVisibilityBuffer class itself...
//
//
//
/************************************************************/
DTDVisibilityBuffer::DTDVisibilityBuffer( const String& filename, const IniExperiment* experimentptr ) :
    VisibilityBuffer( experimentptr ),
    itsBaseFilename( filename ),
    itsRootDir( this->getExperiment().getRootDir()+"/"+itsBaseFilename ),
    itsDTDptr( new dtd ),
    itsScanptr( 0 ),
    itsDatafileptr( 0 ),
    itsTimeCvt( new BOCFToTime() ),
    itsCurrentjob( 0 ),
    itsCurrentsubarray( 0 ),
    itsCurrentsubband( 0 ),
    itsCurrentbaseline( 0 ),
    itsCurrentintegration( 0 ),
    itsNrintegrations( 0L ),
    itsSUErrorStatsptr( new FreqCounter() )
{
    Bool      dtdread( True );
    uLong     dtdoffset( 0L );
    uLong     dtdfilesize;
    uLong     filesize;
    uLong     jobdatasize;
    IniFile   theInifile( itsRootDir+"/"+filename+".ini" );
    ifstream  theDTDfile( itsRootDir+"/"+filename+".dtd", ios::in | ios::nocreate );
    
    /*
    cout << "Start creating DTDVisibilyBuffer\n" 
	 << "Checking if BOCFToTime ok..." << flush;
	 */

    //
    //  Check if call to new succeeded
    //
    if( !itsTimeCvt )
    {
	throw( AipsError("Sorry, failed to create BOCF -> time converter...") );
    }

    /*
    cout << "yes\n" 
	 << "Checking existence of files..." << flush;
	 */

    //
    //  Check if DTD file was found
    //
    if( !theDTDfile.good() )
    {
	throw( AipsError("Sorry, DTD file `"+filename+".dtd' does not exist!") );
    }
    

    //
    //  Ok Set up the buffer, check for existance of both .cor and
    //  .dtd files....
    //
    itsDatafileptr = new ifstream( filename+".cor", ios::in | ios::nocreate );
    
    if( !itsDatafileptr->good() )
    {
	throw( AipsError("Sorry, binary datafile `"+filename+".cor' does not exist!") );
    }

    //cout << "all found\n" << flush;

    //
    //  Ok. Having verified that, we now try to read the DTD from the
    //  DTDfile...
    //
    theDTDfile.seekg( 0L, ios::end );
    dtdfilesize = theDTDfile.tellg();

    //
    //  Check if an offset was specified in the ini-file
    //
    String   tmp;
    
    theInifile.getValue( "DTD", "offset", tmp, "-1" );
    if( atol((const char*)tmp)>=0 )
    {
	dtdoffset = atol( (const char*)tmp );
    }
    
    //cout << "Attempt to read DTD..." << flush;

    //
    //  Do some checking....
    //
    if( (dtdoffset+(uLong)sizeof(dtd))>(uLong)dtdfilesize )
    {
	float   fraction( ((float)dtdfilesize-(float)dtdoffset)/(float)sizeof(dtd) );

	cout << "### Warning! DTD file `" << filename << ".dtd' is too small." << endl;
	cout << "### Cannot read complete DTD!" << endl;
	
	if( fraction>=0.9 )
	{
	    cout << "*** The system will read " << fraction*100.0 << "% of DTD." << endl;
	}
	else
	{
	    dtdread = False;
	    cout << "*** Less than 90% of datatransferdescriptor could be recovered!" << endl;
	    cout << "*** Stop!" << endl;
	}
    }

    //cout << ((dtdread==True)?("Yep"):("Nope")) << endl;

    if( !dtdread )
    {
	throw( AipsError("Blast. No DTD read.") );
    }
    
    theDTDfile.seekg( dtdoffset, ios::beg );
    theDTDfile.read( itsDTDptr, sizeof(dtd) );
    
    //
    //  Attacht the dtd to the lookuptable
    //
    //cout << "Attaching to lookuptable..." << flush;
    
    itsLookupTable.attach( *itsDTDptr );

    //cout << "done" << endl;
    
    
    //
    //  Deduce the number of integrations?
    //

    //
    //  Get the total datafilesize
    //
    itsDatafileptr->seekg( 0L, ios::end );
    filesize = itsDatafileptr->tellg();
    
    itsDatafileptr->seekg( 0L, ios::beg );

    //
    //  The job datasize
    //
    //cout << "Determining total datasize...." << flush;

    jobdatasize=( itsDTDptr->TOTAL_DATASIZE*4L );
    
    //cout << jobdatasize << endl;
    
    if( !jobdatasize )
    {
	throw( AipsError("Something wrong with DataTransferDescriptor; TOTAL_DATASIZE=0!!") );
    }
    
    //cout << "Calculate nr. of integrations..." << flush;    
    itsNrintegrations = filesize/jobdatasize;

    //cout << itsNrintegrations << endl;
    
    //
    //  Check if we need 2 overwrite nrintegrations from the inifile
    //
    theInifile.getValue( "DTD", "nrintegrations", tmp, "<DEF>" );
    
    if( tmp!="<DEF>" )
    {
	itsNrintegrations = atol( (const char*)tmp );
    }
    //cout << "Found " << itsNrintegrations << " integrations" << endl;

    //
    //  Maybe we want to start at another integration than the
    //  first...
    //
    theInifile.getValue( "DTD", "firstintegration", tmp, "<DEF>" );
    
    if( tmp!="<DEF>" )
    {
      itsCurrentintegration = atoi( (const char *)tmp );
    }

    //
    //  Attempt to create a scan-object...
    //
    try
    {
	itsScanptr = new j2ms::DTDScan( filename, this );
    }
    catch( AipsError& )
    {
	itsScanptr = 0;
    }
    
    //
    // Set this as the current scan!
    //
    this->setCurrentScanptr( itsScanptr );
    
    //
    //  Initialise the scan start-time
    //
    MEpoch      tmpScanstart;

    if( !itsScanptr )
    {
	//
	//  We've got no scan! Tell'm that times will be fake
	//
	cout << "DTDVisibilityBuffer: no scan attached -> visibilty times will be faked!" << endl;
    }
    else
    {
	tmpScanstart = itsScanptr->getStartTime();
    }
    
    //cout << "Finding zero-point of BOCFToTime translation..." << flush;
    
    if( itsCurrentintegration!=0 && itsScanptr )
    {
	//
	//  Somebody told us to skip to integration nr blahblah. Need
	//  to take this into account when setting scan-start-time..
	//
	FrequencyConfig     fc( "def" );
	const FrequencyDB&  freqdb( itsScanptr->getExperiment().getFrequencyDB() );
	
	if( freqdb.searchFrequencyConfig(fc, itsScanptr->getFrequencyCode()) )
	{
	    //
	    //  Ok. advance scan start time by nr integrations times
	    //  integrationtime
	    //
	    double      integrationtime( itsScanptr->getCorrelatorSetup().getIntegrationTime() );
	    Quantity    headstart( itsCurrentintegration*integrationtime, "s" );
	    MVEpoch     newStarttime( tmpScanstart.getValue()+headstart );

	    tmpScanstart = MEpoch( newStarttime, tmpScanstart.getRef() );
	}
	else
	{
	    cout << "DTDVisibilityBuffer: cannot find frequency configuration needed to skip to" << endl;
	    cout << " integrationnr " << itsCurrentintegration << endl;
	}
    }

    /*
    cout << tmpScanstart << '\n'
	 << "And make it so..." << flush;
	 */
    //
    //  Initialise the BOCF to time translator!
    //
    if( itsScanptr )
    {
	(*itsTimeCvt) = BOCFToTime( tmpScanstart, itsScanptr->getBOCFRate() );
    }
    //cout << "done." << endl;
}

//Bool DTDVisibilityBuffer::getCurrentVisibility( Visibility& vis ) const
Bool DTDVisibilityBuffer::getCurrentVisibility( Visibility& vis )
{
    Bool               flag;
    JIVEVisLabel       lab;
    Vector<Double>     weight;
    Matrix<Complex>    t;
    const j2ms::Scan*  scanptr;
    Visibility::domain d;

    //
    //  Cache these for time-calculations....
    //
    static Bool        firstpointvalidated( False );
    static uint32      lastintegration( 0xFFFFFFFF );
    static uint32      lastendbocf( 0 );
    
    //
    //  Init vis to be an empty visibility....
    //
    vis = Visibility();
    
    //
    //  Check adressing
    //
    if( itsCurrentjob>=itsLookupTable.jobs().elements() )
    {
	return False;
    }
    
    const JobDescr&    curjobref( itsLookupTable.jobs()(itsCurrentjob) );
    
    if( itsCurrentsubarray>=curjobref.getNrSubarrays() )
    {
	return False;
    }
    
    const SubArrayDescr&  cursaref( curjobref.getSubarray(itsCurrentsubarray) );
    
    if( itsCurrentsubband>=cursaref.getNrSubbands() )
    {
	return False;
    }
    
    if( itsCurrentbaseline>=cursaref.getNrBaselines(itsCurrentsubband) )
    {
	return False;
    }
    
    const BaseLine&  blref( cursaref(cursaref.getBaseLines(itsCurrentsubband)(itsCurrentbaseline),
				     itsCurrentsubband) );
    
    lab.jobnr() = itsCurrentjob;
    lab.subarraynr() = itsCurrentsubarray;
    lab.subbandnr() = itsCurrentsubband;
    lab.framenr() = itsCurrentintegration;
    lab.baseline() = blref.getBaseLineIdx();


    //
    //  All that's left is to read the matrix!
    //
    Container<IFinfo>  configured( blref.getConfiguredCorrelations() );
    
    lab.getCorrelations().resize( configured.elements() );
    
    for( uint16 i=0; i<configured.elements(); i++ )
    {
	lab.getCorrelations()( i ) = CorrelationCode::code2ordinal( configured(i).itsCode );
    }

    //
    //  Re-order the data in such a way that the order in the matrix
    //  is the same order as in the correlator setup correlations
    //
    if( (scanptr=this->getScanptr())!=0 )
    {
	Bool                     error( False );
	const CorrelatorSetup&   setup( scanptr->getCorrelatorSetup() );
	
	if( setup.getCorrelations().elements()==lab.getCorrelations().elements() )
	{
	    uint32            nrelements( lab.getCorrelations().elements() );
	    Container<Bool>   alreadydone( nrelements, False );
	    Container<uint16> neworder( nrelements, (uint16)0xFFFF );
	    
	    const Container<uint16>& indata( lab.getCorrelations() );
	    const Container<uint16>& req( setup.getCorrelations() );
	    
	    //
	    //  Re-order them in the baseline such that they resemble
	    //  the order in the correlator-setup
	    //
	    for( uint32 i=0; i<nrelements; i++ )
	    {
		for( uint32 j=0; j<nrelements; j++ )
		{
		    if( indata(j)==req(i) && !alreadydone(j) )
		    {
			neworder(i) = j;
			alreadydone(j) = True;
		    }
		}
	    }

	    if( frequencyOfArg(neworder, (uint16)0xFFFF) )
	    {
		error = True;
	    }
	    else
	    {
		Container<IFinfo>   saved( configured );

		cout << "\nOrder changed from " << lab.getCorrelations()
		     << "to.." << flush;
		for( uint32 tmp=0; tmp<neworder.elements(); tmp++ )
		{
		    configured( tmp ) = saved( neworder(tmp) );
		    lab.getCorrelations()( tmp ) =
			CorrelationCode::code2ordinal( configured(tmp).itsCode );
		}
		cout << lab.getCorrelations() << endl;
	    }
	    
	    //
	    // N.B.: this here statement is the easiest way
	    // out... though it relies on the fact that if there's
	    // less than ALL correlations, the requested/found
	    // correlations match up, even if they dont!  This here
	    // statement would 'ruin' the system when the requested
	    // polarization was [0,0] and in the data it would say
	    // [1,1]...(or [1,x], or [x,1] for that matter...)
	    //
	    //lab.getCorrelations() = setup.getCorrelations();
	}
	else
	{
	    error = true;
	}
	
	if( error )
	{
	    cout << "\n" 
		 << "\n"
		 << "     ----- AARrrgh ----\n"
		 << "Number of required polarization combinations not found\n"
		 << "for baseline " << lab.baseline() << "\n"
		 << "required: " << setup.getCorrelations() << "\n"
		 << "found   : " << lab.getCorrelations() << "\n"
		 << flush;
	}
    }

    //
    //  readData fills in the domain, flag and weight and returns the
    //  data matrix associated
    //
    t = this->readData( lab, configured, d, flag, weight );

    //
    //  Finally, get the begin/end correlator frame nrs
    //
    //  This function also fills in the stationunit nrs!
    //
    this->getCorrelatorFrameNr( lab, configured );

    Bool       validbocfs;
    uint32     startbocf( lab.getStartBOCF() );
    uint32     endbocf( lab.getEndBOCF() );
    uint32     midbocf( 0 );

    validbocfs = (startbocf!=0xFFFFFFFF && endbocf!=0xFFFFFFFF);

    if( validbocfs )
    {
	if( endbocf<startbocf )
	{
	    endbocf += itsTimeCvt->getMaxBOCFnr();
	}
	
	midbocf =  (startbocf+endbocf)/2;
    }
    else
    {
	flag = True;
    }
    
    
    //
    //  Time calculations... if last integrationnr is different from
    //  the current, we may have to do something!!
    //
    if( itsCurrentintegration!=lastintegration && validbocfs )
    {
	//
	//  If the BOCFcounter did wrap, move the zero point of the
	//  time conversion ahead...
	//
	if( startbocf<lastendbocf )
	{
	    cout << "++++ BOCF counter wrap detected..\n"
		 << "     moving clock 10min. ahead" << endl;
	    
	    itsTimeCvt->adjust( 1 );
	}

	//
	// Save this number for later use....
	//
	lastendbocf = endbocf;

	//
	// Ok. Check if start-point was validated
	//
	//  <<ASSUMPTION>> the next() method was not called more times
	//  than there are baselines in one integration, otherwise we
	//  will be comparing the timestamps against the wrong
	//  integration!!
	//
	if( !firstpointvalidated && this->getScanptr() )
	{
	    MEpoch      firsttime( (*itsTimeCvt)(midbocf) );
	    MVTime      ftime, stime;
	    Double      finsecs, sinsecs;
	    
	    ftime = firsttime.getValue();
	    stime = this->getScanptr()->getStartTime().getValue();

	    finsecs = ftime.get("s").getValue();
	    sinsecs = stime.get("s").getValue();
	    
	    //
	    //  If start-endtime differ by more than 250s (can be
	    //  positive or negative) we need to change or zeropoint
	    //
	    if( (finsecs-sinsecs)>250.0 )
	    {
		//
		//  First time in data > expected starttime (with
		//  250s margin), therefore the zero-point is
		//  10min ahead of current time so we go back one
		//  interval...
		//
		cout << "++++ Time of first datapoint > expected start+250s ->\n"
		     << "     zeropoint 10min ahead of what it should be!" << endl;
		cout << "     Receding zeropoint by 10min" << endl;
		itsTimeCvt->adjust( -1 );
	    }
	    else if( (finsecs-sinsecs)<-250.0 )
	    {
		//
		//  Start/first time differ by more than 250s
		//  (negative) -> zero-point is 10m behind wrt
		//  expected starttime so we must go ahead!
		//
		cout << "++++ Time of first datapoint < expected start-250s ->\n"
		     << "     zeropoint 10min behind of what it should be!" << endl;
		cout << "     Advancing zeropoint by 10min" << endl;
		itsTimeCvt->adjust( 1 );
	    }
	    else
	    {
		cout << "++++ Time of first datapoint conforms Ok. to what's expected..." << endl;
	    }

	    //
	    //  Done validating the first datapoint....
	    //
	    firstpointvalidated = True;
	}

	//
	//  Ok. Now *this* was the last integration...
	//
	lastintegration = itsCurrentintegration;
    }

    //
    //  And make the visibility for the usr.
    //
    vis=Visibility( lab, t, d, flag, weight, (*itsTimeCvt)(midbocf), Muvw(), this );
    return True;
}


Bool DTDVisibilityBuffer::next( void )
{
    Bool     retval( False );
    
    //
    //  Increase to next visibility??
    //

    //
    //  Check adressing
    //
    uInt        nrjobs( itsLookupTable.jobs().elements() );
    
    if( itsCurrentjob>=nrjobs )
    {
	return retval;
    }
    
    const JobDescr&    curjobref( itsLookupTable.jobs()(itsCurrentjob) );
    uInt               nrsubarrays( curjobref.getNrSubarrays() );
    
    if( itsCurrentsubarray>=nrsubarrays )
    {
	return retval;
    }
    
    const SubArrayDescr&  cursaref( curjobref.getSubarray(itsCurrentsubarray) );
    
    uInt                  nrsubbands( cursaref.getNrSubbands() );
    
    if( itsCurrentsubband>=nrsubbands )
    {
	return retval;
    }

    uInt     nrbasel( cursaref.getNrBaselines(itsCurrentsubband) );
    

    if( itsCurrentbaseline>=nrbasel )
    {
	return retval;
    }
    
    //
    // Attempt to increment basleline...
    //
    if( ++itsCurrentbaseline>=nrbasel )
    {
	//
	//  Ok. Start at zero...
	//
	itsCurrentbaseline=0;

	if( ++itsCurrentsubband>=nrsubbands )
	{
	    //
	    // Attempt to increment subarray
	    //
	    itsCurrentsubband=0;
	    
	    if( ++itsCurrentsubarray>=nrsubarrays )
	    {
		//
		//  Attempt to move on to next job...
		//
		itsCurrentsubarray=0;

		if( ++itsCurrentintegration>=itsNrintegrations )
		{
		    itsCurrentintegration=0;
		    
		    if( ++itsCurrentjob>=nrjobs )
		    {
			return retval;
		    }
		}
	    }
	}
    }



    return True;
}


const String& DTDVisibilityBuffer::getRootDir( void ) const
{
    return itsRootDir;
}



DTDVisibilityBuffer::~DTDVisibilityBuffer()
{
    //
    //  Show error stats
    //
    cout << "SU BOCF error stats:" << endl << endl;
    cout << " SU         errorcount" << endl;
    cout << "----------------------" << endl;
    cout << (*itsSUErrorStatsptr) << endl;

    //
    // Clean up....
    //
    delete itsDTDptr;

    delete itsScanptr;

    delete itsDatafileptr;

    delete itsTimeCvt;
    
    delete itsSUErrorStatsptr;
}



//
//
//  Private methods!
// 
//
//
Matrix<Complex> DTDVisibilityBuffer::readData( const JIVEVisLabel& idx,
					       const Container<IFinfo>& ifinfo,
					       Visibility::domain& domainref, Bool& flag,
					       Vector<Double>& weight ) const
{
    uInt               nrpolcombi( idx.getCorrelations().elements() );
    Bool               IFflag;
    Double             IFweights[4];
    Vector<Complex>*   polcombidata[4];
    Visibility::domain datadomain( Visibility::undefined );
    
    weight.resize( 0 );

    if( nrpolcombi!=ifinfo.elements() || !nrpolcombi )
    {
	return Matrix<Complex>();
    }

    weight.resize( nrpolcombi );
    weight = (Double)0.0;
    
    //
    //  Ok! Go for it!
    //
    for( uInt i=0; i<nrpolcombi; i++ )
    {
	polcombidata[i] = new Vector<Complex>( this->getIFdata(idx, ifinfo(i).itsIF,
							       datadomain, IFweights[i])
					       );
	//
	//  Transfer the weight to the dest.
	//
	weight( i ) = IFweights[i];
	
	if( i==0 )
	{
	    domainref=datadomain;
	}
	else
	{
	    if( domainref!=Visibility::undefined && domainref!=datadomain )
	    {
		cout << "Visibility data-domain varies across polarization combinations!?" << endl;
		domainref=Visibility::undefined;
	    }
	}
    }


    //
    // Now check if all vectors have the same length and (approx.)
    // weight
    //
    Int     length0;
    Int     lengthi;
    uInt    cnt;
    
    for( cnt=0; cnt<nrpolcombi; cnt++ )
    {
	if( !cnt )
	{
	    polcombidata[cnt]->shape( length0 );

	}
	
	polcombidata[cnt]->shape( lengthi );
	
	if( lengthi!=length0 )
	{
	    break;
	}
    }

    if( cnt<nrpolcombi )
    {
	for( uInt tmp=0; tmp<nrpolcombi; tmp++ )
	{
	    delete polcombidata[tmp];
	}
	
	cout << "polcombi shapes vary in visibility!" << endl;
	return Matrix<Complex>();
    }
    
    //
    //  Make the matrix
    //
    Matrix<Complex>    retval( nrpolcombi, length0 );
    
    for( uInt i=0; i<nrpolcombi; i++ )
    {
	retval.row( i ) = *polcombidata[i];
	delete polcombidata[i];
    }

    //
    //  And set flag/weight
    //
    IFflag = False;

    flag = IFflag;
    
    return retval;
}


//
//  Get the data, interferometer based: just do a jump in the
//  interferometer table, see what's there and get it!
//
const Vector<Complex>& DTDVisibilityBuffer::getIFdata( const JIVEVisLabel& idx, const InterferometerDescr& ifref,
						       Visibility::domain& domainref, Double& weight ) const
{
    //
    //  Remember buffer specifics
    //
    static Float           IFweight;
    static Float*          buffer( 0 );
    static Vector<Complex> noans;

    //
    //  The return value
    //
    noans.resize( 0 );
    IFweight = 1.0;
    
    weight = IFweight;

    //
    // Ok. Get the interferometer data
    //
    uInt               nrlags;
    uInt               nrfloats;
    Bool               vernier( this->getVernierDelay() );
    Bool               swapit( False );
    Bool               autocorrelation;
    Bool               crosspolarization;
    Float              sign( 1.0 );
    Complex*           cmplxreordered;
    //sadescr*     saptr( &itsDTDptr->SA_DATADESCR_ARRAY[idx.subarraynr()] );
    ifdescr*           ifptr( &itsDTDptr->INTFR_DATADESCR_ARRAY[(uint32)ifref] );
	dbdescr*           dbptr( &itsDTDptr->DB_DATADESCR_ARRAY[ifptr->DB_DINDEX] );
	const j2ms::Scan*  scanptr( this->getScanptr() );

	//
	//  Retrieve the software version with which this scan
	//  was correlated. Me need to make decisions, based on this
	//  infi
	//
	CorrelatorSetup::CorSWVersion    swver;
	CorrelatorSetup::CorSWVersion    version1( 1, 0 );
	
	if( scanptr )
	{
		swver = scanptr->getCorrelatorSetup().getSoftwareVersion();		
	}


    //
    // Let's fill in the domain of the data
    //
    if( ifptr->IF_DATATYPE&SPECTRUMFLAG )
    {
	domainref = Visibility::frequency;
    }
    else
    {
	domainref = Visibility::time;
    }

    autocorrelation = ToBool( ifptr->XANTNR==ifptr->YANTNR );
    crosspolarization = ToBool( ifptr->XPOLNR!=ifptr->YPOLNR );

    //
    //  If this is LAG data and not an autocorrelation, we have twice
    //  the number of channels in the lag-domain, since 'nrchannels'
    //  now holds the value in the spectral domain.
    //
    nrfloats = nrlags = ifptr->NLAG;
    
    //
    //  If we're dealing with cmplex data, nlags is sum of real+imag,
    //  therefore, we only have half the number of complex numbers.
    //
    if( ifptr->IF_DATATYPE & CMPLXFLAG )
    {
	nrlags/=2;
    }

    //
    // Check if the IF_DATASIZE is at least the nrlags...
    //
    if( (uInt)ifptr->IF_DATASIZE<nrfloats )
    {
	cout << "IF_DATASIZE=" << ifptr->IF_DATASIZE << " whilst nrfloats=" << nrfloats << endl;
	return noans;
    }

    //
    //  If tghis interferometer was located in the lower half of the
    //  matrix, we should make the data the complex conjugate
    //
    if( ifptr->XANTNR>ifptr->YANTNR )
    {
	sign = -1.0;
	swapit = True;
    }
    
    //
    // The next step is to locate the data for this interferometer in
    // Alberts datablock!
    //

    //
    //  Resize our vector
    //
    noans.resize( 0 );

    //
    //  Get the buffer
    //
    buffer = new Float[ nrfloats ];

    cmplxreordered = new Complex[ nrlags ];

    //
    //  And read the data; first determine the total fileoffset in
    //  words, then multilply by sizeof(Float) to get the offset in
    //  bytes!
    //
    Long        offset( 0 );
    Long        datablockoffset( 0 );

    //
    // 1) -> the start of the whole block of data this dtd describes:
    //
    datablockoffset = itsDTDptr->TOTAL_DATASIZE*idx.framenr();

    //
    //  2) Now then, the ifptr->DB_DINDEX tells us what the ordinal
    //  number of the datablock attached to this interferometer
    //  is. This in itself doesn't tell us much; what we want is to
    //  locate the block. Albert uses equal-sized blocks, the size
    //  being the maximum datasize found in the subarray. In the
    //  subarraydescr this value is saved. Hence we multiply the
    //  blocknr with the blocksize and TADA! we have the offset of the
    //  datablock for this interferometer.
    //
    //datablockoffset += ( ifptr->DB_DINDEX * saptr->MDB_DATASIZE );
	datablockoffset += dbptr->DB_DOFFS;
    
    //
    //  3) Now we find the INTERFEROMETERdata by adding to the
    //  block-offset the IF_DATAOFFS value.
    //
    offset = datablockoffset + ifptr->IF_DATAOFFS;

    //
    //  4) Since Albert works with units of words (=32-bit floating
    //  point values), we multiply the found value by 4 to get the
    //  offset in bytes!
    //
    offset *= 4;
    
    itsDatafileptr->seekg( offset, ios::beg );
    
    itsDatafileptr->read( buffer, ifptr->IF_DATASIZE*sizeof(Float) );

    //
    //  Now transform the buffer into complex numbers
    //  The organization in the data-blocks is:
    //
    //  _____ nrchannels _____ ____ nrchannels _______
    //  |                     ||                      |
    //  <real><real>.....<real><imag><imag>......<imag>
    //
    if( ifptr->IF_DATATYPE & ACFLAG )
    {
        for( uInt lagcnt=0; lagcnt<nrlags; lagcnt++ )
        {
	    cmplxreordered[ lagcnt ] = Complex( buffer[lagcnt], 0.0 );
            //noans( lagcnt )=Complex( buffer[lagcnt], 0.0 );
        }
    }
    else
    {
	//
	//  If label was reversed, mirror the lags around the zero-lag
	//  (which is different from reversing the order, since we
	//  have an even sized dataset rather than an odd sized one!
	//
	if( swapit )
	{
		int32    i;
	    int32    zerolag( nrlags/2 );
	    int32    lags2do( (nrlags/2)-1 );
	
		if( swver>=version1 )
		{
			//
			//  Dang! Albert shifted the compete lagspectrum one lag 
			//  'down'wards. Need to compensate for that...
			//
			zerolag = (nrlags/2)-1;
			lags2do = (nrlags/2)-2;
		}
    
	    for( i=0; i<=lags2do; i++ )
	    {
			cmplxreordered[zerolag+i] = Complex(buffer[zerolag-i],
										-1.0*buffer[zerolag+nrlags-i]);
			cmplxreordered[zerolag-i] = Complex(buffer[zerolag+i],
										-1.0*buffer[zerolag+nrlags+i]);
	    }
	    //
	    //  Fill in the zero-channel.. well..
	    //	There may be more channels not filled in
		//  fill those up with Complex(0,0)
		while( i<=(nrlags/2) )
		{
			int32  idx;

			if( (idx=(zerolag-i))>=0 )
			{
	    		cmplxreordered[ i ] = Complex( 0.0f, 0.0f );
			}
			if( (idx=(zerolag+i))<nrlags )
			{
	    		cmplxreordered[ i ] = Complex( 0.0f, 0.0f );
			}
			i++;
		}
	}
	else
	{
	    for( uInt i=0; i<nrlags; i++ )
	    {
		cmplxreordered[ i ] = Complex( buffer[i], buffer[i+nrlags] );
	    }
	}
	

	
	//
	//  Ok. lag-reversal/cmplx conjugate have been take care of
	//  (if necessary)
	//
	//  Now check vernierdelay
	//
	if( vernier && (!autocorrelation || (autocorrelation && crosspolarization)) )
	{
	    Complex   sample0( cmplxreordered[0] );
		    
	    memmove( (void *)cmplxreordered, (void *)&cmplxreordered[1],
		     (nrlags-1)*sizeof(Complex) );
	    cmplxreordered[ nrlags-1 ] = sample0;
	}
    }

    //
    //  Creat the return value!
    //
    noans = Vector<Complex>( IPosition(1,nrlags), cmplxreordered, COPY );

    //
    //  Now let offset point at the VALCNTR (validity counter) for this IF
    //
    offset = datablockoffset + ifptr->VALCNTR;

    offset *= 4;
    
    itsDatafileptr->seekg( offset, ios::beg );
    
    itsDatafileptr->read( &IFweight, sizeof(Float) );
    
    weight = IFweight;

    //
    //  Cleanup
    //
    delete [] buffer;
    buffer = 0;

    delete [] cmplxreordered;
    
    return noans;
}


//
//
//  Find the correlator frame nrs. for the indexed visibility.
//
//  Mark a couple of things:
//
//  -> The framenr in the ExtVisLabel should contain the actual integration number, zero-based.
//
//  -> Up to 16 correlator frame nrs can be found (2 start + 2 end per interferometer,
//     max. 4 interferometers per visibility). It will be verified that all start and
//     end correlator frame nrs. match
//
//  -> The labref.startbocf/endbocf members will bve filled
//  in. 0xFFFFFFFF by default, overwritten when consisten/valid values
//  are found
//
uint32 DTDVisibilityBuffer::getCorrelatorFrameNr( JIVEVisLabel& labref, const Container<IFinfo>& ifinfo ) const
{
    //
    //  Find the frame nr. for the indicated label. Assume that
    //  labref.framenr() at this moment is the integrationnr rather
    //  than the framenr....
    //
    uInt              nrpolcombi( labref.getCorrelations().elements() );	
    Bool              error;
    uint32            retval( labref.framenr() ); // retval defaults to previous integrationnr
    uint32            datablockstart;      // start of datablock (in 4-byte words)
    uint32            sigsrcoffset;
    uint32            startbocf0,endbocf0;
    uint32            startxbocf[4],endxbocf[4];
    uint32            startybocf[4],endybocf[4];
    ifdescr*          ifptr;
    sadescr*          saptr;
    jobdescr*         jobptr;

    //
    //  In order to speed up stuff we keep these variables static
    //
    static SigSrc       xsigsrc;
    static SigSrc       ysigsrc;
    static unsigned int sigsrcbuf[20]; // This buffer is bigger than
    // what it's supposed to be. Might need to make it dynamic
    
    //
    //  Start off by filling in the bocf's with 'unset' or 'invalid'
    //  marker values
    //
    labref.getStartBOCF()=0xFFFFFFFF;
    labref.getEndBOCF()=0xFFFFFFFF;

    //
    //  Basic checks
    //
    if( nrpolcombi!=ifinfo.elements() )
    {
	return retval;
    }
    
    //
    //  Locate job/subarray
    //
    jobptr = &itsDTDptr->JOB_DATADESCR_ARRAY[ labref.jobnr() ];
    saptr  = &itsDTDptr->SA_DATADESCR_ARRAY[ jobptr->SA_DINDEX+labref.subarraynr() ];

    //
    //  Loop over interferometers (making up the polarization
    //  combinations) to check that all correlator framenrs are the
    //  same.....
    //
    datablockstart = itsDTDptr->TOTAL_DATASIZE * labref.framenr();
    
    error = False;
    
    for( uInt polcnt=0; polcnt<nrpolcombi; polcnt++ )
    {
	//
	//  Jump to the appropriate INTFR_DATADESCR_TYPE
	//
	ifptr  = &itsDTDptr->INTFR_DATADESCR_ARRAY[ saptr->IF_DINDEX+ifinfo(polcnt).itsIF.getZeroBasedIFnr() ];

	//
	//  Deduce the X/Y station unit nrs....
	//
	if( !polcnt )
	{
	    labref.getXStationUnit() = ifptr->XSIGOUTP/8;
	    labref.getYStationUnit() = ifptr->YSIGOUTP/8;
	}
	

	//
	// Read X-sigsrc
	//
	sigsrcoffset = (datablockstart + ifptr->XSIGSRC)*4;
	
	itsDatafileptr->seekg( sigsrcoffset );
	itsDatafileptr->read( sigsrcbuf, ifptr->NSIGSRC*4 );
	
	//
	//  Attach this buffer to xsigsrc (will decode it and copy values to its local buffer)
	//
	xsigsrc.attach( sigsrcbuf );
	
	//
	//  Repeat for Y...
	//
	sigsrcoffset = (datablockstart + ifptr->YSIGSRC)*4;
	
	itsDatafileptr->seekg( sigsrcoffset );
	itsDatafileptr->read( sigsrcbuf, ifptr->NSIGSRC*4 );

	ysigsrc.attach( sigsrcbuf );
	
	//
	//  Now verify framenrs....
	//
	if( polcnt==0 )
	{
	    //
	    //  Init start/end bocf
	    //
	    startbocf0 = xsigsrc.frsernr1;
	    endbocf0 = xsigsrc.frsernr2;
	}
	
	startxbocf[ polcnt ] = xsigsrc.frsernr1;
	endxbocf[ polcnt ] = xsigsrc.frsernr2;
	
	startybocf[ polcnt ] = ysigsrc.frsernr1;
	endybocf[ polcnt ] = ysigsrc.frsernr2;
    }
    
    //
    //  Now check BOCFs. First of all, if statbocf0 and endbocf0 are
    //  equal, then we have a problem right from the start!
    //
    error = ToBool( startbocf0==endbocf0 );
    
    for( uInt cnt=0; cnt<nrpolcombi && !error; cnt++ )
    {
	if( startxbocf[cnt]!=startbocf0 || startybocf[cnt]!=startbocf0 || 
	    endxbocf[cnt]!=endbocf0 || endybocf[cnt]!=endbocf0 )
	{
	    error = True;
	}
    }
    

    if( !error )
    {
	labref.getStartBOCF()=startbocf0;
	labref.getEndBOCF()=endbocf0;
    }
    else
    {
	if( this->getScanptr() )
	{
	    char              buffer[1024];
	    uint16            su0;
	    uint16            su1;	    
	    ostrstream        strm( buffer, sizeof(buffer) );
	    const Experiment& expref( this->getScanptr()->getExperiment() );
	    
	    //
	    //  Ok. Show'm!
	    //
	    strm << "BOCF mismatch in visibility.. start/end BOCFs differ across interferometer";
	    strm << ((nrpolcombi!=1)?("s"):("")) << endl << ends;

	    expref.logLogMessage( strm.str() );
	    
	    su0 = labref.getXStationUnit();
	    su1 = labref.getYStationUnit();

	    //
	    //  Seek buffer to start
	    //
	    strm.seekp( 0 );
	    
	    strm << "SUs involved: SU X=" << su0 << " SU Y=" << su1 << endl;
	    strm << "BOCFs found in data: " << endl;
	    for( uInt i=0; i< nrpolcombi; i++ )
	    {
		strm << "SU X start/end: " << startxbocf[i] << "/" << endxbocf[i] << endl;
		strm << "SU Y start/end: " << startybocf[i] << "/" << endybocf[i] << endl;
	    }
	    strm << ends;
	    
	    expref.logLogMessage( strm.str() );

	    //
	    //  Update statistics
	    //
	    (*itsSUErrorStatsptr)[ su0 ]++;
	    (*itsSUErrorStatsptr)[ su1 ]++;
	}
    }
    

    /*
    dbptr  = &itsDTDptr->DB_DATADESCR_ARRAY[ saptr->DB_DINDEX ];
    
    //
    //  The framenr is found at:
    //
    //  datablockstart + intgstat-offset + 4
    //
    datablockstart = itsDTDptr->TOTAL_DATASIZE * labref.framenr();
    
    intgstatoffset = dbptr->INTG_STAT;

    framenrfileoffset = (datablockstart+intgstatoffset+framenroffset)*4;
    
    itsDatafileptr->seekg( framenrfileoffset, ios::beg );
    
    //
    //  Assume same byte ordering.... (maybe tricky?) between writing
    //  and reading system
    //
    itsDatafileptr->read( &retval, 4 );
    */


    return retval;
}
