//
// DTDVisibilityBuffer - visibility buffer that reads visibilities
// from a binary correlator dump + binary DTD dump.
//
//
//  Author:   Harro Verkouter, 18-11-1998
//
//
//  $Id: DTDVisibilityBuffer.h,v 1.2 2001/06/22 14:05:11 verkout Exp $
//
#ifndef DTDVISIBILITYBUFFER_H
#define DTDVISIBILITYBUFFER_H

#ifndef VISIBILITYBUFFER_H
#include <vlbi/j2ms/VisibilityBuffer.h>
#endif

#ifndef DTDSCAN_H
#include <vlbi/j2ms/DTDScan.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef DTDLUT_H
#include <vlbi/DtdLUT/DtdLUT.h>
#endif

#ifndef VISLABEL_H
#include <vlbi/DtdLUT/VisLabel.h>
#endif

#ifndef AIPS_VECTOR_H
#include <aips/Arrays/Vector.h>
#endif

#ifndef AIPS_MATRIX_H
#include <aips/Arrays/Matrix.h>
#endif

#ifndef AIPS_VECTOR_H
#include <aips/Arrays/Vector.h>
#endif

#ifndef VISIBILITY_H
#include <vlbi/j2ms/Visibility.h>
#endif

#ifndef BOCFTOTIME_H
#include <vlbi/j2ms/BOCFToTime.h>
#endif



#include <fstream.h>

//
//  Forward declarations...
//
class FreqCounter;
class JIVEVisLabel;
class IniExperiment;


//
//  The class itself
//
class DTDVisibilityBuffer :
    public VisibilityBuffer
{
public:

    //
    //  Create a DTDVisibilityBuffer from <filename>.cor and
    //  <filename>.dtd and <filename>.ini
    //
    //  Optionally, this vis.buffer may be tied to a particular experiment...
    //
    DTDVisibilityBuffer( const String& filename, const IniExperiment* experimentptr=0 );

    //virtual Bool          getCurrentVisibility( Visibility& vis ) const;
    virtual Bool          getCurrentVisibility( Visibility& vis );
    virtual Bool          next( void );
    virtual const String& getRootDir( void ) const;
    

    virtual ~DTDVisibilityBuffer();
    
private:
    //
    //  The private parts
    //
    String         itsBaseFilename;
    String         itsRootDir;
    dtd*           itsDTDptr;
    j2ms::DTDScan* itsScanptr;
    ifstream*      itsDatafileptr;
    ExposedDtd     itsLookupTable;
    BOCFToTime*    itsTimeCvt;

    //
    // This should be iterator stuff....
    //
    uInt           itsCurrentjob;
    uInt           itsCurrentsubarray;
    uInt           itsCurrentsubband;
    uInt           itsCurrentbaseline;
    uLong          itsCurrentintegration;
    
    uLong          itsNrintegrations;

    //
    //  Count SU-error stats
    //
    FreqCounter* itsSUErrorStatsptr;
    
    //
    //  Private methods....
    //

    //
    //  Read the full datamatrix for the labelled visibility. Note:
    //  you need to supply a container of IFinfo-struct so the
    //  function can look up the interferometers. I know this is a
    //  lousy contruct, but I didn't want to put the
    //  Interferometerdescriptor into the visibility-label: it doesn't
    //  belong there; the user of a VisLabel class should not be
    //  bothered by interferometer numbers.
    //
    Matrix<Complex>        readData( const JIVEVisLabel& idx, const Container<IFinfo>& ifinfo,
				     Visibility::domain& domainref, Bool& flag,
				     Vector<Double>& weight ) const;
    
    //
    //  This functions reads the data for one inetrferometer! (Note
    //  the difference with the previous function, which reads the
    //  data for a visibility!)
    //
    const Vector<Complex>& getIFdata( const JIVEVisLabel& idx, const InterferometerDescr& ifref,
				      Visibility::domain& domainref, Double& weight ) const;

    //
    //  Read the start/end bocf-counters for the indexed integrationnr.
    //
    //  NOTE: the ExtVisLabel is CHANGED by calling this function!!!
    //  (but that's what we want anyway...)
    //
    uint32                 getCorrelatorFrameNr( JIVEVisLabel& labref,
						 const Container<IFinfo>& ifinfo ) const;

    //
    //  Prohibit these
    //
    DTDVisibilityBuffer();
    DTDVisibilityBuffer( const DTDVisibilityBuffer& );
    DTDVisibilityBuffer& operator=( const DTDVisibilityBuffer& );
};



#endif
