//
//          Implementation of DayConversion 
//
//  Author:  Harro Verkouter, 15-06-1999
//
//  $Id: DayConversion.cc,v 1.2 2000/04/03 09:57:02 verkout Exp $
//
#define DAYCONVERSION_CC

#ifndef DAYCONVERSION_H
#include <vlbi/j2ms/DayConversion.h>
#endif


_DayConversion::_DayConversion()
{
}


bool _DayConversion::dayNrToMonthDay( uint32& month, uint32& day, uint32 daynr, int32 year )
{
    bool           retval( false );
    uint32         monthcnt;
    uint32         cumulativeday;
    const uint32*  daysPerMonthptr( _DayConversion::daysPerMonth );
    
    if( _DayConversion::isLeapYear(year) )
    {
	daysPerMonthptr = daysPerMonthLeap;
    }
    
    for( monthcnt=0, cumulativeday=0; cumulativeday<daynr && monthcnt<12;
	 (cumulativeday+=daysPerMonthptr[monthcnt++]) )
    {
	if( (cumulativeday+daysPerMonthptr[monthcnt])>=daynr )
	{
	    break;
	}
    }
    
    if( monthcnt<12 )
    {
	retval = true;
	
	month = monthcnt;
	day = (daynr-cumulativeday);
    }
    else
    {
	month = 0xFFFFFFFF;
	day = 0xFFFFFFFF;
    }
    
    return retval;
}

bool _DayConversion::dayMonthDayToNr( uint32& daynr, uint32 month, uint32 day, int32 year )
{
    bool           retval( false );
    uint32         monthcnt;
    const uint32*  daysPerMonthptr( _DayConversion::daysPerMonth );
    
    if( _DayConversion::isLeapYear(year) )
    {
	daysPerMonthptr = daysPerMonthLeap;
    }
    
    for( monthcnt=0, daynr=day; monthcnt<month && monthcnt<12 ; daynr+=daysPerMonthptr[monthcnt++] );
    
    
    if( monthcnt<12 )
    {
	retval = true;
    }
    else
    {
	daynr = 0xFFFFFFFF;
    }
    
    return retval;
}


bool _DayConversion::isLeapYear( int32 year )
{
    bool    retval( false );

    //
    //  Year is leap year if:
    //
    //  1) It's divisible by 4
    //  2) It's NOT divisible by 400
    //
    //     !(year%4) && (year%400) )
    if( ((!(year%4)&&(year%100)) || !(year%400)) )
    {
	retval = true;
    }
    return retval;
}


_DayConversion::~_DayConversion()
{
}

