//
//      Provide transformations from daynr to (day, month) and vice versa
//
//  Author:  Harro Verkouter,   15-6-1999
//
//  $Id: DayConversion.h,v 1.1.1.1 2000/03/20 15:14:36 verkout Exp $
//
#ifndef DAYCONVERSION_H
#define DAYCONVERSION_H

//
//  Need this for uint32 etc.
//
#ifndef HVTYPES_H
#include <vlbi/DtdLUT/hvtypes.h>
#endif

//
// Time conversion stuff
//
typedef struct _DayConversion
{
public:
    //
    //  Only allowed c'tor -> default -> init everything (if that's necessary)....
    //
    _DayConversion();

    //
    //  Some useful constants....
    //
    static const uint32 secondsPerDay;

    //
    //  Convert daynr to day/month and vice versa
    //
    static bool  dayNrToMonthDay( uint32& month, uint32& day, uint32 daynr, int32 year );
    static bool  dayMonthDayToNr( uint32& daynr, uint32 month, uint32 day, int32 year );
    
    //
    // Tell us if the year 'year' is a leap year or not...
    //
    static bool  isLeapYear( int32 year );

    //
    //  If We Allocated stuff, release it
    //
    ~_DayConversion();
    
private:
    //
    //  All our private parts
    //
    static const uint32       daysPerMonth[ ];
    static const uint32       daysPerMonthLeap[ ];

    //
    //  Prohibit these functions....
    //
    _DayConversion( const _DayConversion& );
    _DayConversion& operator=( const _DayConversion& );
} DayConversion;



//
//  Define (and initialize) statics in here, only if we're compiling DayConversion.cc
//
#ifdef DAYCONVERSION_CC

//
//  Note: month 0 is January!
//    
const uint32 _DayConversion::secondsPerDay          = 24*60*60;
const uint32 _DayConversion::daysPerMonth[ 12 ]     = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
const uint32 _DayConversion::daysPerMonthLeap[ 12 ] = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

#endif // DAYCONVERSION_CC


#endif // DAYCONVERSION_H
