//
//  Implementation of the experiment class
//
//  Author:  Harro Verkouter, 8-12-1998
//
//
#ifndef EXPERIMENT_H
#include <vlbi/j2ms/Experiment.h>
#endif

//
//  For JB def's of uint32 etc
//
#ifndef STD_TYPES_H
#include <fundamental/std_types.h>
#endif

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>

#include <unistd.h>


ostream& operator<<( ostream& os, const Experiment& expref )
{
    os << "************ Experiment " << expref.getCode() << " ************" << endl;
    os << "Antennas: " << expref.getAntennas().ac() << endl;
    os << "Sources: " << expref.getSources().ac() << endl;
    os << "Output is placed in: " << expref.getMSname() << endl;
    os << "Data is written in the MS in the " << expref.getDestinationDomain() << " domain" << endl;
    return os;
}




Experiment::Experiment( const String& exprootdir ) :
    myRootDir( exprootdir ),
    myLogStreamptr( new ostream* )
{
    (*myLogStreamptr) = (ostream *)0;
    
    //
    //  If the exp. root dir. not set, determine the cwd
    //
    if( myRootDir=="" )
    {
	char*   cwd;
	
	cwd = getcwd( (char *)0, 1024 );

	if( cwd )
	{
	    myRootDir = (const char *)cwd;
	
	    free( cwd );
	}
    }
}

const String& Experiment::getArrayCode( void ) const
{
    static String   defaultArrayCode( "EVN" );
    
    return defaultArrayCode;
}


const String& Experiment::getRootDir( void ) const
{
    return myRootDir;
}

void Experiment::logLogMessage( const String& msg, const LogOrigin& org ) const
{
    if( !(*myLogStreamptr) )
    {
	//
	//  Create a suitable logfile....
	//
	char*      logfilename;
	strstream  strm;
	
	strm << myRootDir << "/" << this->getMSname() << ".log" << ends;
	logfilename = strm.str();
	
	(*myLogStreamptr) = new fstream( logfilename, ios::app );
	
	free( logfilename );
    }
    
    *(*myLogStreamptr) << LogMessage( msg, org ) << flush;
    
    return;
}

Experiment::~Experiment()
{
    //
    // close the log-file
    //
    delete (*myLogStreamptr);

    delete myLogStreamptr;
}
