//
//  Struct? ExperimentCache -> used to cache experiment properties,
//  like antennatable/sourcetable
//
//
#ifndef EXPERIMENTCACHE_H
#define EXPERIMENTCACHE_H

#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef CONTAINER_H
#include <vlbi/tmplsrc/Container.h>
#endif

#ifndef ANTENNAINFO_H
#include <vlbi/j2ms/AntennaInfo.h>
#endif

#ifndef SOURCEINFO_H
#include <vlbi/j2ms/SourceInfo.h>
#endif

#ifndef BOCFTOTIME_H
#include <vlbi/j2ms/BOCFToTime.h>
#endif


//
//  Ok. We drag in al the measures stuff; we need it for the
//  uvw-calculation (on a per scan basis)
//
#ifndef AIPS_MEASURES_H
#include <aips/Measures.h>
#endif

#ifndef AIPS_MPOSITION_H
#include <aips/Measures/MPosition.h>
#endif

#ifndef AIPS_MDIRECTION_H
#include <aips/Measures/MDirection.h>
#endif

#ifndef AIPS_MBASELINE_H
#include <aips/Measures/MBaseline.h>
#endif

#ifndef AIPS_MEPOCH_H
#include <aips/Measures/MEpoch.h>
#endif


//
// Do this to get around bug(s)
//
typedef MeasConvert<MEpoch,MVEpoch,MCEpoch>             StooPidgcc_ExperimentCache_0;
typedef MeasConvert<MPosition,MVPosition,MCPosition>    StooPidgcc_ExperimentCache_1;
typedef MeasConvert<MDirection,MVDirection,MCDirection> StooPidgcc_ExperimentCache_2;

//
//  Forward declarations
//
namespace j2ms
{
	class Scan;
};
class SourceDB;
class AntennaDB;
class Experiment;
class ExperimentCache;

struct _ExperimentCacheElement;


typedef struct _ScanCacheElement
{
    friend _ExperimentCacheElement;

public:
    //
    //  Alloc/init
    //
    _ScanCacheElement();
    
    //
    //  Public datamembers
    //
    String              itsScanCode;
    SourceInfo          itsSourceInfo;

    //
    //  Machinery for UVW-calculation
    //
    MEpoch::Ref         outEpochRef;
    MEpoch::Convert     timeCvt;

    //
    //  Variables needed for BOCF to time translation
    //
    int                 prefixtimeadjust;
    int                 postfixtimeadjust;
    uint32              samplecount;
    uint32              itsLastIntegration;
    uint32              itsLastendbocf;
    BOCFToTime          bocf2time;

    //
    //  Free resources
    //
    ~_ScanCacheElement();
    
private:
    //
    //  Keep pointer to next scan
    //
    _ScanCacheElement*    nextScan;

    //
    //  cache-elements cannot be copied/assigned to
    //
    _ScanCacheElement( const _ScanCacheElement& );
    _ScanCacheElement& operator=( const _ScanCacheElement& );
} ScanCacheElement;




typedef struct _ExperimentCacheElement
{
    friend class ExperimentCache;
    
public:
    
    //
    //  Alloc/init a new element
    //
    _ExperimentCacheElement();
    
    //
    //  Struct datamembers, make them public to speed up access
    //
    String                   itsCode;
    Container<AntennaInfo>   itsAntennaTable;

    //
    //  
    //
    ScanCacheElement*        addScan( const j2ms::Scan& scanref,
									  const SourceDB* alternatedbptr );

    //
    //  Returns pointer to cached scan (if in cache).
    //  
    //  null-pointer if scan not found in cache
    //
    ScanCacheElement*        locateScan( const String& scancode );

    //
    //  De-allocate stuff
    //
    ~_ExperimentCacheElement();
    
private:
    //
    //  Pointer to the first cached scan
    //
    ScanCacheElement*               itsFirstScanptr;

    //
    //  The pointer to next
    //
    struct _ExperimentCacheElement* nextExperiment;

    //
    //  We don't like it if elements are copied/assigned!!!!
    //
    _ExperimentCacheElement( const _ExperimentCacheElement& );
    _ExperimentCacheElement& operator=( const _ExperimentCacheElement& );
} ExperimentCacheElement;



class ExperimentCache
{
public:
    //
    //  Init to empty cache
    //
    ExperimentCache();

    //
    //  Attempt to add the experiment in the cache.
    //
    //  NOTE: pass a pointer to an alternate AntennaDB (or NULL for none).
    //
    //  When finding the antennainfo, first the experiments antennadb
    //  (if any) is queried. If no success, the alternate DB is
    //  queried.
    //
    ExperimentCacheElement*  addExperiment( const Experiment& expref, const AntennaDB* alternatedbptr );

    //
    //  Do a lookup, based on experiment-code.
    //
    //  Returns null-pointer if indicated experiment not found in cache.
    //
    ExperimentCacheElement*  locateExperiment( const String& expcode ) const;

    //
    //  Delete the cache
    //
    ~ExperimentCache();
    
private:
    //
    //  We only  keep a pointer to the first element...
    //
    ExperimentCacheElement*    itsFirstElementptr;

    //
    // Prohibit copying/assigning of Experiment caches
    //
    ExperimentCache( const ExperimentCache& );
    ExperimentCache& operator=( const ExperimentCache& );
};




#endif
