//
//  Implementation of the frequency config class
//
//  Author:   Harro Verkouter,   2-12-1998
//
//  $Id: FrequencyConfig.cc,v 1.2 2001/06/22 14:05:17 verkout Exp $
//
#ifndef FREQUENCYCONFIG_H
#include <vlbi/j2ms/FrequencyConfig.h>
#endif

#include <iostream.h>
#include <ctype.h>

ostream& operator<<( ostream& os, const FrequencyConfig& fcref )
{
	FrequencyConfig::PolMap    pm;

    os << "FrequencyConfig: `" << fcref.itsName << "'" << endl;

	fcref.getPolMap( pm );

    for( uInt i=0; i<2; i++ )
    {
		if( pm[i] )
		{
			os << "Polarization #" << i << ": " << pm[i] << endl;
		}
    }
    //os << fcref.itsPolMap << endl;

    for( uInt i=0; i<fcref.itsBaseBands.elements(); i++ )
    {
		os << "[" << fcref.itsBaseBands(i) << "]" << endl;
    }
    return os;
}

/*
ostream& operator<<( ostream& os, const FrequencyConfig::PolMap& pmref )
{
    for( uInt i=0; i<2; i++ )
    {
	os << ((pmref[i]!='\0')?(pmref[i]):('*'));
    }
    return os;
}
*/



FrequencyConfig::FrequencyConfig( const String& name ) :
    itsName( name )
{
    FrequencyConfig::initPolMap( itsPolMap );
}

FrequencyConfig::FrequencyConfig( const FrequencyConfig& other ) :
    itsName( other.itsName ),
    itsBaseBands( other.itsBaseBands )
{
    FrequencyConfig::copyPolMap( itsPolMap, other.itsPolMap );
}

FrequencyConfig& FrequencyConfig::operator=( const FrequencyConfig& other )
{
    if( this!=&other )
    {
	itsName = other.itsName;
	itsBaseBands = other.itsBaseBands;
	FrequencyConfig::copyPolMap( itsPolMap, other.itsPolMap );
    }
    return *this;
}

Bool FrequencyConfig::addBaseBand( const BaseBand& newbb )
{
    BaseBand   bb2add( this, itsBaseBands.elements(), newbb );
    
    itsBaseBands.add( bb2add );
    return True;
}

Bool FrequencyConfig::setPolMap( const FrequencyConfig::PolMap& pmref )
{
    Bool     retval( False );
    
    if( !itsPolMap[0] && !itsPolMap[1] )
    {
		retval = True;
    
		FrequencyConfig::copyPolMap( itsPolMap, pmref );
    }
    return retval;
}

void FrequencyConfig::getPolMap( FrequencyConfig::PolMap& pmref ) const
{
    FrequencyConfig::copyPolMap( pmref, itsPolMap );
    return;
}


const BaseBand* FrequencyConfig::getBaseBand( uInt bbnr ) const
{
    const BaseBand*   retval( 0 );
    
    if( bbnr<itsBaseBands.elements() )
    {
	retval = &itsBaseBands(bbnr);
    }
    return retval;
}

uint32 FrequencyConfig::getNrBaseBands( void ) const
{
    return itsBaseBands.elements();
}


uint16 FrequencyConfig::getNrPolarizations( void ) const
{
    uint16    retval( 0 );
    
    for( uint16 i=0; i<2; i++ )
    {
		if( itsPolMap[i] )
		{
			retval++;
		}
    }
    return retval;
}


const String& FrequencyConfig::getCode( void ) const
{
    return itsName;
}
    

FrequencyConfig::~FrequencyConfig()
{
}

//
//  Private methods!
//
void FrequencyConfig::initPolMap( FrequencyConfig::PolMap& pmref )
{
    pmref[0]='\0';
    pmref[1]='\0';
    return;
}

void FrequencyConfig::copyPolMap( FrequencyConfig::PolMap& dst, const FrequencyConfig::PolMap& src )
{
    dst[0]= toupper( src[0] );
    dst[1]= toupper( src[1] );
    return;
}

