//
//  FrequencyConfig -> class that holds a number of basebands (and thus
//  a frequency setup)
//
//
//  Author:   Harro Verkouter, 2-12-1998
//
//  $Id: FrequencyConfig.h,v 1.1.1.1 2000/03/20 15:14:35 verkout Exp $
//
//
#ifndef FREQUENCYCONFIG_H
#define FREQUENCYCONFIG_H


#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef BASEBAND_H
#include <vlbi/j2ms/BaseBand.h>
#endif

#ifndef CONTAINER_H
#include <vlbi/tmplsrc/Cointainer.h>
#endif


//
//  Forward declarations
//
class ostream;



class FrequencyConfig
{
    //
    //  Output the Frequencysetup!
    //
    friend ostream& operator<<( ostream& os, const FrequencyConfig& fcref );


public:
    //
    //  There's only two polarizations per feed possible, make a typedef
    //  of the mapping
    //
    typedef Char  PolMap[2];
    static void   initPolMap( FrequencyConfig::PolMap& pmref );
    static void   copyPolMap( FrequencyConfig::PolMap& dst, const FrequencyConfig::PolMap& src );


    //
    //  An empty frequency config does not exist, it must have at
    //  least a name by which we can identify it! If the name is
    //  empty, an excpetion is thrown!
    //
    FrequencyConfig( const String& name="<DEF>" );
    
    //
    //  Copy the whole frequency config
    //
    FrequencyConfig( const FrequencyConfig& other );
    
    //
    //  Assignment...
    //
    FrequencyConfig& operator=( const FrequencyConfig& other );

    //
    //  Add a copy of the baseband to this frequency setup!
    //
    Bool     addBaseBand( const BaseBand& newbb );

    //
    //  Set the polmap, only if it wasn't set yet!!!
    //
    Bool     setPolMap( const FrequencyConfig::PolMap& pmref );
    void     getPolMap( FrequencyConfig::PolMap& pmref ) const;

    //
    //  return a pointer to a const baseband, the outsideworld may
    //  read what's in here but not edit it. If the indexed baseband
    //  is not present, a null-pointer value is returned.
    //
    const BaseBand*  getBaseBand( uInt bbnr ) const;
    
    uint32           getNrBaseBands( void ) const;

    //
    //  How many polarizations were configured???
    //
    //  Possible return values are: 0, 1, or 2
    //
    uint16           getNrPolarizations( void ) const;

    //
    //  Get the 'name' of this configuration
    //
    const String&    getCode( void ) const;

    //
    //  Delete all resources used by this object
    //
    ~FrequencyConfig();
    
private:
    //
    //  The private parts
    //
    String               itsName;

    //
    //  The configured basebands
    //
    Container<BaseBand>  itsBaseBands;

    //
    //  The polarization map:
    //
    PolMap               itsPolMap;

    //
    //  Private methods
    //
};

//
//  Output a PolMap...
//
//ostream& operator<<( ostream& os, const FrequencyConfig::PolMap& pmref );

#endif
