//
//  Implementation of the GlobalDBs - class
//
//
//  Author:   Harro Verkouter,   10-12-1998
//
//  $Id: GlobalDBs.cc,v 1.2 2001/06/25 07:11:52 verkout Exp $
//
#define GLOBALDBS_CC

#ifndef GLOBALDBS_H
#include <vlbi/j2ms/GlobalDBs.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef INISOURCEDB_H
#include <vlbi/j2ms/IniSourceDB.h>
#endif

#ifndef INIANTENNADB_H
#include <vlbi/j2ms/IniAntennaDB.h>
#endif

#ifndef AIPS_EXCEPTIONS_H
#include <aips/Exceptions.h>
#endif

#define DEFAULTDBPATH "/tmp"

GlobalDBs::GlobalDBs()
{
	if( !GlobalDBs::theirDBPath )
	{
	    char*      ptr2path;
    
	    //
	    // Let's test if the environment variable is set.... 
	    //
	    ptr2path = getenv( ENV_DBPATH );
    
	    if( !ptr2path )
		{
			if( !GlobalDBs::haveWarned )
			{
			    char       errbuff[512];
			    ostrstream errstrm( errbuff, sizeof(errbuff) );

				errstrm.seekp(0);
				errstrm << "GlobalDBs: environment var. " << ENV_DBPATH << " is not set!"
						<< endl
						<< "GlobalDBs: Defaulting to " << DEFAULTDBPATH << endl
					    << ends;

				cout << errstrm.str();
				GlobalDBs::haveWarned = true;
			}

			ptr2path = DEFAULTDBPATH;
		}

		GlobalDBs::theirDBPath = ::strdup( ptr2path );
	}

    //
    //  Check (and if needed) make the DBs
    //
    if( !GlobalDBs::theirSourceDBptr )
    {
	GlobalDBs::theirSourceDBptr = this->makeSourceDB();
    }

    if( !GlobalDBs::theirAntennaDBptr )
    {
	GlobalDBs::theirAntennaDBptr = this->makeAntennaDB();
    }

    //
    //  Increment the reference-count
    //
    GlobalDBs::theirReferenceCount++;
}

SourceDB& GlobalDBs::getSourceDB( void ) const
{
    if( !GlobalDBs::theirSourceDBptr )
    {
	throw( AipsError("GlobalDBs -> attempt to get at non-existant SourceDB!") );
    }
    return *GlobalDBs::theirSourceDBptr;
}

AntennaDB& GlobalDBs::getAntennaDB( void ) const
{
    if( !GlobalDBs::theirAntennaDBptr )
    {
	throw( AipsError("GlobalDBs -> attempt to get at non-existant AntennaDB!") );
    }
    return *GlobalDBs::theirAntennaDBptr;
}



GlobalDBs::~GlobalDBs()
{
    //
    //  Decrement the reference-count
    //
    GlobalDBs::theirReferenceCount--;
    
    //
    //  If count reached zero: delete the DBs!
    //
    if( !GlobalDBs::theirReferenceCount )
    {
		delete GlobalDBs::theirSourceDBptr;
		delete GlobalDBs::theirAntennaDBptr;
	
		//
		//  Reset the pointers to NULL, so they are re-created when
		//  needed!
		//
		GlobalDBs::theirSourceDBptr = 0;
		GlobalDBs::theirAntennaDBptr = 0;

		if( GlobalDBs::theirDBPath )
		{
			::free( GlobalDBs::theirDBPath );
			GlobalDBs::theirDBPath   = 0;
		}
    }
}


//
//  The private methods....
//
SourceDB* GlobalDBs::makeSourceDB( void )
{
    char       errbuff[512];
    ostrstream errstrm( errbuff, sizeof(errbuff) );
    
    //
    //  Ok. Let's attempt to make an Ini-SourceDB
    //
    errstrm.seekp(0);
    errstrm << GlobalDBs::theirDBPath << "/source.ini" << ends;
    return new IniSourceDB( errstrm.str() );
}

AntennaDB* GlobalDBs::makeAntennaDB( void )
{
    char       errbuff[512];
    ostrstream errstrm( errbuff, sizeof(errbuff) );
   
	//
	// Ok. Let's attemt to make an Ini-AntennaDB
	// 
    errstrm.seekp(0);
    errstrm << GlobalDBs::theirDBPath << "/antenna.ini" << ends;
    return new IniAntennaDB( errstrm.str() );
}
