//
//   GlobalDBs - a wrapper-class to hide the implementation details of
//   the antenna and source databases. It has static datamemembers,
//   allGlobalDB-objects share the same databases (otherwise they
//   wouldn't be global would they?!)
//
//   Author:   Harro Verkouter,   10-12-1998
//
//   $Id: GlobalDBs.h,v 1.2 2001/06/25 07:11:53 verkout Exp $
//
#ifndef GLOBALDBS_H
#define GLOBALDBS_H

#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif


#define ENV_DBPATH  "J2MSDBPATH"


//
//  Forward declarations
//
class SourceDB;
class AntennaDB;


//
//  The class itself
//
class GlobalDBs
{
public:
    //
    //  We provide only the default c'tor
    //
    GlobalDBs();
    
    //
    //  Get access to the databases...
    //
    SourceDB&      getSourceDB( void ) const;
    AntennaDB&     getAntennaDB( void ) const;

    //
    //  Delete all resources
    //
    ~GlobalDBs();
    
private:
    //
    //  The private parts
    //
    static uInt          theirReferenceCount;
	static bool          haveWarned;
	static char*         theirDBPath;
    static SourceDB*     theirSourceDBptr;
    static AntennaDB*    theirAntennaDBptr;

    //
    //  Private methods
    //
    SourceDB*            makeSourceDB( void );
    AntennaDB*           makeAntennaDB( void );

    //
    //  Prohibit these
    //
    GlobalDBs( const GlobalDBs& );
    GlobalDBs& operator=( const GlobalDBs& );
};


//
//  If we're compiling GlobalDBs.cc, we should allocate storagespace
//  for the pointers etc.
//
#ifdef GLOBALDBS_CC

uInt       GlobalDBs::theirReferenceCount = 0;
bool       GlobalDBs::haveWarned          = false;
char*      GlobalDBs::theirDBPath         = 0;
SourceDB*  GlobalDBs::theirSourceDBptr = 0;
AntennaDB* GlobalDBs::theirAntennaDBptr = 0;


#endif // GLOBALDBS_CC



#endif // GLOBALDBS_H
