//
//  Implementation of the IniAntennaDB class
//
//  Author:   Harro Verkouter,   1-12-1998
//
//  $Id: IniAntennaDB.cc,v 1.1.1.1 2000/03/20 15:14:35 verkout Exp $
//
#ifndef INIANTENNADB_H
#include <vlbi/j2ms/IniAntennaDB.h>
#endif

#ifndef INIFILE_H
#include <inifile/IniFile.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif


IniAntennaDB::IniAntennaDB( const String& inifilename ) :
    AntennaDB(),
    itsInifileName( inifilename ),
    itsInifilePtr( new IniFile(itsInifileName) )
{
    //
    //  Drain the inifiule from all the antenna info that we can find.
    //
}

AntennaInfo IniAntennaDB::operator[]( const String& key ) const
{
    //
    //  Look in the infile if we can find an entry for this telescope!
    //
    String      station;
    String      position;
    String      name;
    String      antdiameter;
    Quantity    antdiameterq;
    MPosition   antposmeasure;
    

    itsInifilePtr->getValue( key, ENTRY_ANTNAME, name, "" );
    itsInifilePtr->getValue( key, ENTRY_ANTDIAMETER, antdiameter, "" );
    itsInifilePtr->getValue( key, ENTRY_ANTSTATION, station, "" );
    itsInifilePtr->getValue( key, ENTRY_ANTPOSITION, position, "" );
    
    //
    //  Ok... parse the position string to get the position of the
    //  antenna??
    //
    //  Allowed formats:
    //
    //  <x>,<y>,<z> (implies ITRF)
    //  <lat>,<lon> (implies WGS84)
    //
    uInt         nrelements;
    String*      elements( 0 );
    
    nrelements = position.freq( ',' ) + 1;
    elements = new String[ nrelements ];
    
    split( position, elements, nrelements, "," );
    
    if( nrelements==2 )
    {
	Quantity          lat;
	Quantity          lon;
	Vector<Double>    v(2);
	
	Quantity::read( lat, elements[0] );
	Quantity::read( lon, elements[1] );
	
	v(0)=lat.get("deg").getValue();
	v(1)=lon.get("deg").getValue();

	antposmeasure = MPosition( MVPosition(v), MPosition::WGS84 );
    }
    else if( nrelements==3 )
    {
	//
	//  Pos in x,y,z
	//
	Quantity       x;
	Quantity       y;
	Quantity       z;
	Vector<Double> v( 3 );

	
	Quantity::read( x, elements[0] );
	Quantity::read( y, elements[1] );
	Quantity::read( z, elements[2] );

	v(0)=x.get("m").getValue();
	v(1)=y.get("m").getValue();
	v(2)=z.get("m").getValue();

	antposmeasure = MPosition( MVPosition(v), MPosition::ITRF );
    }
    else
    {
	return AntennaInfo();
    }
    
    
    //
    //  Get rid of the strings
    //
    delete [] elements;

    //
    //  Make sure the diameter is converted to m
    //
    Quantity::read( antdiameterq, antdiameter );

    if( name=="" )
    {
	return AntennaInfo();
    }
    
    
    return AntennaInfo( name, key, station, antposmeasure, antdiameterq.get("m").getValue() );
}

Bool IniAntennaDB::searchAntenna( AntennaInfo& airef, const String& key ) const
{
    Bool   retval( True );
    
    airef = (*this)[ key ];
    
    if( airef.getName()=="" )
    {
	retval = False;
    }
    return retval;
}


IniAntennaDB::~IniAntennaDB()
{
    //
    //  Close the infile....
    //
    delete itsInifilePtr;
}

