//
//  Implementation of the IniSourceDB class
//
//  Author:   Harro Verkouter,   2-12-1998
//
//  $Id: IniSourceDB.cc,v 1.1.1.1 2000/03/20 15:14:35 verkout Exp $
//
#ifndef INISOURCEDB_H
#include <vlbi/j2ms/IniSourceDB.h>
#endif

#ifndef INIFILE_H
#include <inifile/IniFile.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef AIPS_MVANGLE_H
#include <aips/Quanta/MVAngle.h>
#endif


IniSourceDB::IniSourceDB( const String& inifilename ) :
    SourceDB(),
    itsInifileName( inifilename ),
    itsInifilePtr( new IniFile(itsInifileName) )
{
    //
    //  Drain the inifiule from all the source info that we can find.
    //
}

SourceInfo IniSourceDB::operator[]( const String& key ) const
{
    //
    //  Look in the infile if we can find an entry for this source!
    //
    String      position;
    String      name;
    MDirection  srcdirmeasure;
    

    itsInifilePtr->getValue( key, ENTRY_SOURCENAME, name, "<DEF>" );
    itsInifilePtr->getValue( key, ENTRY_SOURCEDIRECTION, position, "<DEF>" );
    
    //
    // Check....
    //
    if( name=="<DEF>" || position=="<DEF>" )
    {
	return SourceInfo();
    }


    //
    //  Ok... parse the position string to get the direction of the
    //  source??
    //
    //  Allowed formats:
    //
    //    <ra>,<dec>,<epoch>
    //
    //  where <epoch> defaults to 2000.0
    //
    uInt         nrelements;
    String*      elements( 0 );
    
    nrelements = position.freq( ',' ) + 1;
    elements = new String[ nrelements ];
    
    split( position, elements, nrelements, "," );
    
    if( nrelements<2 || nrelements>3 )
    {
	return SourceInfo();
    }
    
    //
    //  read ra/dec
    //
    Quantity       ra;
    Quantity       dec;
    Double         epoch( 2000.0 );
    Vector<Double> v(2);
    
    Quantity::read( ra, elements[0] );
    Quantity::read( dec, elements[1] );
    
    
    v(0) = ra.get("rad").getValue();
    v(1) = dec.get("rad").getValue();

    if( nrelements==3 )
    {
	Quantity  q;
	
	Quantity::read( q, elements[2] );
	
	epoch = q.getValue();
    }

    //
    //  We don't need the strings anymore
    //
    delete [] elements;

    if( epoch!=2000.0 && epoch!=1950.0 )
    {
	return SourceInfo();
    }

    //
    //  Make the direction
    //
    if( epoch==2000.0 )
    {
	srcdirmeasure = MDirection( MVDirection(v), MDirection::J2000 );
    }
    else
    {
	srcdirmeasure = MDirection( MVDirection(v), MDirection::B1950 );
    }
    return SourceInfo( name, key, srcdirmeasure );
}

Bool IniSourceDB::searchSource( SourceInfo& airef, const String& key ) const
{
    Bool   retval( True );
    
    airef = (*this)[ key ];
    
    if( airef.getName()=="" )
    {
	retval = False;
    }
    return retval;
}


IniSourceDB::~IniSourceDB()
{
    //
    //  Close the infile....
    //
    delete itsInifilePtr;
}

