//
//  Implementation of the JIVEMSFiller-class
//
//  Author:  Harro Verkouter,   10-12-1998
//
//  $Id: JIVEMSFiller.cc,v 1.3 2001/06/22 14:05:19 verkout Exp $
//
#define JIVEMSFILLER_CC

#ifndef JIVEMSFILLER_H
#include <vlbi/j2ms/JIVEMSFiller.h>
#endif

#ifndef EXPERIMENT_H
#include <vlbi/j2ms/Experiment.h>
#endif

#ifndef SCAN_H
#include <vlbi/j2ms/Scan.h>
#endif

#ifndef AIPS_MEASUREMENTSETS_H
#include <aips/MeasurementSets.h>
#endif

#ifndef AIPS_TABLES_H
#include <aips/Tables.h>
#endif

#ifndef VISIBILITY_H
#include <vlbi/j2ms/Visibility.h>
#endif

#ifndef AIPS_TABLECOLUMN_H
#include <aips/Tables/TableColumn.h>
#endif

#ifndef AIPS_ARRAYMATH_H
#include <aips/Arrays/ArrayMath.h>
#endif

#ifndef VISIBILITYBUFFER_H
#include <vlbi/j2ms/VisibilityBuffer.h>
#endif

#ifndef VISLABEL_H
#include <vlbi/DtdLUT/VisLabel.h>
#endif

#ifndef FREQUENCYCONFIG_H
#include <vlbi/j2ms/FrequencyConfig.h>
#endif

#ifndef FREQUENCYDB_H
#include <vlbi/j2ms/FrequencyDB.h>
#endif

#ifndef ANTENNADB_H
#include <vlbi/j2ms/AntennaDB.h>
#endif

#ifndef SOURCEDB_H
#include <vlbi/j2ms/SourceDB.h>
#endif

#ifndef AIPS_STOKES_H
#include <aips/Measures/Stokes.h>
#endif

#ifndef CORRELATIONCODE_H
#include <vlbi/DtdLUT/CorrelationCode.h>
#endif

#ifndef ANTENNAINFO_H
#include <vlbi/j2ms/AntennaInfo.h>
#endif

#ifndef AIPS_MPOSITION_H
#include <aips/Measures/MPosition.h>
#endif

#ifndef AIPS_MUVW_H
#include <aips/Measures/Muvw.h>
#endif

#ifndef AIPS_QUANTA_H
#include <aips/Quanta.h>
#endif

#ifndef AIPS_FFTSERVER_H
#include <aips/Mathematics/FFTServer.h>
#endif

#ifndef AIPSEXCEPTIONOBJECT_H
#include <vlbi/Utilities/AIPSExceptionObject.h>
#endif

#ifndef AIPS_EXPRNODE_H
#include <aips/Tables/ExprNode.h>
#endif

#ifndef FREQCOUNTER_H
#include <vlbi/Utilities/FreqCounter.h>
#endif

#ifndef AIPS_QUANTA_H
#include <aips/Quanta.h>
#endif

#ifndef AIPS_ROTMATRIX_H
#include <aips/Quanta/RotMatrix.h>
#endif

#ifndef CORRELATORSETUP_H
#include <vlbi/j2ms/CorrelatorSetup.h>
#endif

#ifndef HISTORY_H
#include <vlbi/j2ms/History.h>
#endif

#ifndef TAPESTATSFILE_H
#include <data_handler/TapeStatsFile.h>
#endif

#ifndef MAPPINGSSFILE_H
#include <data_handler/MappingsFile.h>
#endif

#ifndef TAPESTATDATARECORD_H
#include <data_handler/TapeStatDataRecord.h>
#endif

#ifndef MAPPINGENTRY_H
#include <data_handler/MappingEntry.h>
#endif


//
//  Connection stuff!
//
#ifndef CLIENTCONNECTION_H
#include <Connections/ClientConnection.h>
#endif

#ifndef EXCEPTIONOBJECT_H
#include <ExceptionObject/ExceptionObject.h>
#endif


#include <aips/Quanta/MVAngle.h>
#include <aips/Quanta/Quantum.h>
#include <aips/Quanta/Unit.h>
#include <aips/Arrays/ArrayMath.h>
#include <aips/Quanta/Euler.h>
#include <aips/Quanta/QMath.h>


#include <iomanip.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <dirent.h>



//
//  Define RotMatrix*Vector (how to do it...)
//
Vector<Double> operator*( const RotMatrix& lhside, const Vector<Double>& rhside )
{
    Vector<Double>   retval(3);
    
    retval = 0.0;
    
    for( int i=0; i<3; i++ )
    {
	for( int j=0; j<3; j++ )
	{
	    retval(i) += lhside( i, j )*rhside(j);
	}
    }
    return retval;
}




JIVEMSFiller::JIVEMSFiller( const char* logconnection )
{
    try
    {
	if( logconnection && strlen(logconnection) && !JIVEMSFiller::myLC )
	{
	    JIVEMSFiller::myLC = ClientConnection::makeConnection( logconnection );
	}
    }
    catch( ExceptionObject& )
    {
	JIVEMSFiller::myLC = 0;
    }

    if( myLC )
    {
	(*myLC) << "\n\n********** Welcome *********\n"
		<< "The JIVEMSFiller starts again!\n\n";
    }
}

//
//  Do the job of actually making a new MS or concatenating it to an
//  existing one for the indicated scan in the given experiment....
//
Bool JIVEMSFiller::fillDataToExperimentsMS( Experiment& expref, const String& visbufname, Bool ask )
{
    Bool              retval( False );
    VisibilityBuffer* ptr2buffer( 0 );
    
    //
    //  Tell usr
    //
    if( myLC )
    {
	(*myLC) << "Request to fill " << (const char*)visbufname << " to the "
		<< "MS of experiment " << (const char*)expref.getCode() << "...";
    }
    

    //
    // The very first check: if the indicated visibility buffer is
    // present in the experiment
    //
    ptr2buffer = expref.getVisibilityBuffer( visbufname );
    
    if( !ptr2buffer )
    {
	if( myLC )
	{
	    (*myLC) << "Bummer! The buffer does not exist!\n";
	}
	
	expref.logLogMessage( "No datafile "+visbufname+" found in experiment!" );
	return retval;
    }

    if( myLC )
    {
	(*myLC) << (const char*)ptr2buffer->getRootDir() << "\n";
    }
    
    //
    //  If the buffer was found, display some properties to the user...
    //
    cout << "\n"
	 << "VisibilityBuffer " << ptr2buffer->getRootDir() << " found...\n"
	 << flush;

    //
    //  Ok. Locate an MS where to stuff the new data....
    //
    MS*       ptr2ms( this->makeMS(expref, ask) );
    
    if( !ptr2ms )
    {
	if( myLC )
	{
	    (*myLC) << "Darn. Failed to create the MS for te experiment.\n\n";
	}
	
	cout << "Failed to create output MS for the experiment!" << endl;
	
	expref.logLogMessage( "Failed to create output MS for the experiment!" );
	delete ptr2buffer;
	return retval;
    }
    
    //
    //  MS was created/opened succesfully, so continue!
    //
    //  Ok, really fill the MS!
    //
    expref.logLogMessage( "Start filling datafile "+visbufname+" to MS.." );
    
    retval = this->fillMSfromVisibilityBuffer( *ptr2ms, *ptr2buffer );

    //
    //  We don't need the pointer to the MS anymore....
    //
    delete ptr2ms;

    //
    //  Nor do we need the pointer to the scan anymore
    //
    delete ptr2buffer;
    
    //
    //  Tell'm what happened!
    //
    return retval;
}



JIVEMSFiller::~JIVEMSFiller()
{
    //
    //  Delete the connection (if any)
    //
    if( myLC )
    {
	(*myLC) << "\n\n"
		<< "***** JIVEMSFiller stopping..... ****\n"
		<< " C Ya later! Thanx for using it!\n"
		<< "bye";
    }
    
    delete myLC;
    myLC = 0;
}


//
//  The private methods!!!
//
MS* JIVEMSFiller::makeMS( const Experiment& expref, Bool ask )
{
    //
    //  First we follow the method of TMS/NFRA, see MSDefiner{.cc,.h}
    //
    MS*             retval( 0 );
    Table*          jivetable( 0 );
    String          key;
    TableDesc       td( MS::requiredTableDesc() );
    TableDesc       statstable( "JIVE_TAPESTATS", TableDesc::Scratch );
    SetupNewTable*  tablesetupptr( 0 );
    SetupNewTable*  jivetablesetup( 0 );


    //
    //  Create a sub-table (JIVE specific) that contains
    //  tapestatistics!
    //
    statstable.addColumn( (ScalarColumnDesc<uInt>("SUID", "Station Unit ID")) );
    statstable.addColumn( (ScalarColumnDesc<Int>("ANTENNA_ID", "Antenna ID")) );
    statstable.addColumn( (ScalarColumnDesc<Double>("TOT", "Telescope time (short form ROT)")) );
    statstable.addColumn( (ScalarColumnDesc<Double>("SERVO_ERROR", "Current servo error")) );
    statstable.addColumn( (ScalarColumnDesc<uInt>("N_HDSTK", "Number of headstacks")) );
    statstable.addColumn( (ArrayColumnDesc<uInt>("HDSTK_POS", "Headstack positions")) );
    statstable.addColumn( (ScalarColumnDesc<uInt>("FOOTAGE", "Tape footage")) );
    statstable.addColumn( (ScalarColumnDesc<uInt>("SU_STATE", "Station Unit status")) );
    statstable.addColumn( (ScalarColumnDesc<uInt>("SUIM_STATE", "SUIM Status")) );
    statstable.addColumn( (ScalarColumnDesc<uInt>("N_TRACK", "Number of tracks")) );
    statstable.addColumn( (ArrayColumnDesc<uInt>("ORG_TRACKNO", "The original tracknumber")) );
    statstable.addColumn( (ArrayColumnDesc<uInt>("NR_FRAMES", "The number of frames averaged")) );
    statstable.addColumn( (ArrayColumnDesc<uInt>("PE_COUNT", "The number of parity errors")) );
    statstable.addColumn( (ArrayColumnDesc<uInt>("US_COUNT", "Unexpected syncs count")) );
    statstable.addColumn( (ArrayColumnDesc<uInt>("MS_COUNT", "Missing syncs count")) );
    statstable.addColumn( (ArrayColumnDesc<uInt>("CRC_ERR", "Number of CRC errors")) );
    statstable.addColumn( (ArrayColumnDesc<uInt>("DBF_COUNT", "Discarded Bad Frame count")) );

    //
    // Ok, since this is the JIVE-filler maybe it's safe to assume
    // that we ALWAYS have complex data?! And we're filling
    // Visibilities, who always have a matrix-shaped block of data,
    // therefore we add a complex array column of dimensionality 2.
    //
    td.addColumn( ArrayColumnDesc<Complex>(MS::columnName(MS::DATA), 2) );
    td.addColumn( ArrayColumnDesc<Double>( "POL_WEIGHT", 1) );

    //
    //  Our data, when it comes from the Correlators, is UNCALIBrated
    //
    key = "UNIT";
    td.rwColumnDesc(MS::columnName(MS::DATA)).rwKeywordSet().define( key, "UNCALIB" );
    
    //
    //  Replace the existing flagcolumn by a two-dimensional array-column
    //
    //td.removeColumn( MS::columnName(MS::FLAG_ROW) );
    //td.removeColumn( MS::columnName(MS::FLAG) );

    //td.addColumn( ScalarColumnDesc<Bool>(MS::columnName(MS::FLAG_ROW)) );
    //MS::addColumnToDesc( td, MS::FLAG, 2 );

    //
    //  Same goes for the SIGMA-column
    //
    td.removeColumn( MS::columnName(MS::SIGMA) );

    MS::addColumnToDesc( td, MS::SIGMA, 1 );

    //
    //  Weight
    //
    //td.removeColumn( MS::columnName(MS::WEIGHT) );
    //td.addColumn( ScalarColumnDesc<Double>(MS::columnName(MS::WEIGHT)) );

    //
    // Add measure reference keyword to time column
    // descriptor. Theoretically, this should be gotten from the
    // scan-starttime.... For now, we default to UTC. If the
    // scan-starttime deviates, we'll have to convert the start-time
    //
    key = "MEASURE_REFERENCE";

    td.rwColumnDesc(MS::columnName(MS::TIME)).rwKeywordSet().define( key, "UTC" );

    //
    //  Ok table descriptor set up. Now we first try to create the
    //  table as NewNoReplace, which will throw an exception if the
    //  table already exists. So if we catch an exception, we prompt the user
    //
    try
    {
	tablesetupptr = new SetupNewTable( expref.getMSname(), td, Table::NewNoReplace );
	
	jivetablesetup = new SetupNewTable( expref.getMSname()+"/"+"JIVE_TAPESTATS",
					    statstable, Table::NewNoReplace );
	

	//
	//  If we get to here, we are creating the table!
	//  Continue on, make the storage managers!
	//
	StManAipsIO      aipsStMan;
	IncrementalStMan incrStMan( "ISMData" );
	
	//
	//  First, bind all columns to the default StMan, then, bind the
	//  exceptional cases to specific StMan's
	//
	tablesetupptr->bindAll( incrStMan );
	tablesetupptr->bindColumn( MS::columnName(MS::DATA), aipsStMan );
	tablesetupptr->bindColumn( MS::columnName(MS::ANTENNA1), aipsStMan );
	tablesetupptr->bindColumn( MS::columnName(MS::ANTENNA2), aipsStMan );
	
	//
	//  Ok! Create the MS.
	//
	retval = new MeasurementSet( *tablesetupptr );

	jivetable = new Table( *jivetablesetup );
	
	//
	//  Create default tables
	//
	retval->createDefaultSubtables( Table::New );
	
	//
	//  Add array-reference-position column to the array table
	//
	retval->array().addColumn( ArrayColumnDesc<Double>( MSArray::columnName(MSArray::POSITION),
							    "Array reference position",
							    IPosition(1,3),
							    ColumnDesc::Direct )
				   );

	//
	//  Attach our own table!
	//
	retval->rwKeywordSet().defineTable( "JIVE_TAPESTATS", *jivetable );

	//
	//  Let's set some reference-keywords in the various columns
	//
	MSColumns     mscol( *retval );
	
	mscol.spectralWindow().chanFreq().rwKeywordSet().define( "MEASURE_REFERENCE", "TOPO");
	mscol.spectralWindow().refFrequency().rwKeywordSet().define( "MEASURE_REFERENCE", "TOPO");
	mscol.spectralWindow().restFrequency().rwKeywordSet().define( "MEASURE_REFERENCE", "REST");
	
	//
	//  And we're done!
	//
    }
    catch( AIPSExceptionObject x )
    {
	//
	//  This call is to make sure that a) the compiler does not
	//  complain, and b) to make sure the compiler does not
	//  optimize it out?!
	//
	cout << "*** " << x.getMesg() << endl;
	//x.getMesg();
	
	//
	//  Make really sure that tablesetupptr = 0, I don't trust it
	//  to leave the value intact in case of an exception...
	//
	tablesetupptr = 0;
    } end_try;
    
    //
    //  If tablesetupptr = 0, the table already existed. Ask the user
    //  for confirmation to concatenate
    //
    if( !tablesetupptr )
    {
	//
	//  Table existed!
	//
	Bool       concatenate( ToBool(ask==False) );
	
	if( ask )
	{
	    String     ans;
	
	    cin.tie( &cout );
	    
	    cout << "MS `" << expref.getMSname() << "' already exists. Concatenate data? [yn] :";
	    cin >> ans;
	    
	    cin.tie( 0 );
	
	    if( ans=="y" || ans=="Y" )
	    {
		concatenate = True;
	    }
	}
	
	if( concatenate )
	{
	    //
	    // Ok... do concatenation!
	    //
	    retval = new MeasurementSet( expref.getMSname(), Table::Update );
	}
    }

    //
    //  Clean up...
    //
    delete tablesetupptr;
    delete jivetablesetup;
    delete jivetable;
    
    //
    //  And.... we're finished!
    //
    return retval;
}


Bool JIVEMSFiller::fillMSfromVisibilityBuffer( MeasurementSet& msref, VisibilityBuffer& visbuf )
{
    uInt              currentrownr;
    double            integrationtime;
    String            lastScanId;
    FreqCounter       sourceidcounter;
    Visibility        currentvisibility;
    FrequencyConfig   frequencyconfig;
    const j2ms::Scan* curScanptr;
    const Experiment& expref( visbuf.getExperiment() );


    //
    // Make a number of column object, to speed up access
    //
    ScalarColumn<Int>     ant1col( msref, MS::columnName(MS::ANTENNA1) );
    ScalarColumn<Int>     ant2col( msref, MS::columnName(MS::ANTENNA2) );
    ScalarColumn<Int>     arrayidcol( msref, MS::columnName(MS::ARRAY_ID) );
    ScalarColumn<Int>     pulsaridcol( msref, MS::columnName(MS::PULSAR_ID) );
    ScalarColumn<Int>     feed1col( msref, MS::columnName(MS::FEED1) );
    ScalarColumn<Int>     feed2col( msref, MS::columnName(MS::FEED2) );
    ScalarColumn<Int>     spwinidcol( msref, MS::columnName(MS::SPECTRAL_WINDOW_ID) );
    ScalarColumn<Int>     fieldidcol( msref, MS::columnName(MS::FIELD_ID) );
    ScalarColumn<Int>     correlatoridcol( msref, MS::columnName(MS::CORRELATOR_ID) );
    ScalarColumn<Int>     observationidcol( msref, MS::columnName(MS::OBSERVATION_ID) );
    ScalarColumn<Int>     pulsarbincol( msref, MS::columnName(MS::PULSAR_BIN) );
    ScalarColumn<Int>     scannumbercol( msref, MS::columnName(MS::SCAN_NUMBER) );
    ScalarColumn<Bool>    flagcol( msref, MS::columnName(MS::FLAG_ROW) );
    ArrayColumn<Double>   uvwcol( msref, MS::columnName(MS::UVW) );
    ArrayColumn<Double>   weightpolcol( msref, "POL_WEIGHT" );
    ScalarColumn<Float>   weightcol( msref, MS::columnName(MS::WEIGHT) );
    ScalarColumn<Double>  integrationtimecol( msref, MS::columnName(MS::EXPOSURE) );
    ScalarColumn<Double>  intervalcol( msref, MS::columnName(MS::INTERVAL) );
    ScalarColumn<Double>  timecol( msref, MS::columnName(MS::TIME) );
    ArrayColumn<Complex>  datacol( msref, MS::columnName(MS::DATA) );


    //
    //  In order to check if the indexed baseband is already written
    //  in the spectralwindow table, it might be handy to use these:
    //
    const MSSpectralWindow&   spwintab( msref.spectralWindow() );
    ArrayColumn<Double>       chanfreqcol( spwintab, MSSpectralWindow::columnName(MSSpectralWindow::CHAN_FREQ) );

    //
    //  Checks for the antenna table....
    //
    const MSAntenna&          antab( msref.antenna() );
    ArrayColumn<Double>       positioncol( antab, MSAntenna::columnName(MSAntenna::POSITION) );
    const Vector<String>&     antennacodelist( expref.getAntennas() );
    Int                       maxantennacode;
    
    antennacodelist.shape( maxantennacode );
    
    //
    //  These vars are needed for checks on field/source table... We
    //  can do this once since a scan has only one source....
    //
    const MSField&            fieldtab( msref.field() );
    ScalarColumn<Int>         fieldfieldidcol( fieldtab, MSField::columnName(MSField::FIELD_ID) );
    ArrayColumn<Double>       phasedircol( fieldtab, MSField::columnName(MSField::PHASE_DIR) );
    ScalarColumn<String>      fieldnamecol( fieldtab, MSField::columnName(MSField::NAME) );
    const Vector<String>&     sourcecodelist( expref.getSources() );
    uInt                      currsourceindex;

    
    //
    //  Some aux. data objects
    //
    Matrix<Bool>          flagmatrix;

    //
    //  Okiedokie: read visibilities from the scan-buffer until there
    //  are no more left!
    //
    currentrownr = msref.nrow();
    
    //
    //  Take some variables out of the loop to prevent them being put
    //  on the stack over and over again
    //
    Int              maxant;
    uInt             ant1;
    uInt             ant2;
    Bool             available;
    
    //
    // Track translation speed
    //
    uLong            nrbytes( 0 );
    hrtime_t         starttime,endtime;
    
    //
    //  tell the system to disp. DMY...
    //
    MVTime::setFormat(MVTime::DMY);

    available = True;
    lastScanId = "No scan yet";

    expref.getAntennas().shape( maxant );
    
    if( myLC )
    {
		(*myLC) << "Start reading visibilities from the buffer!\n\n";
    }
    

    starttime = gethrtime();
    while( available )
    {
		available = visbuf.getCurrentVisibility( currentvisibility );
	
		if( !available )
		{
			if( myLC )
			{
				(*myLC) << "No more visibilities available\n";
			}
	    
			continue;
		}

		curScanptr = visbuf.getScanptr();
	
		if( curScanptr && (curScanptr->getScanId()!=lastScanId) )
		{
			lastScanId = curScanptr->getScanId();
	
			//
			// Show usr. we detected a new scan...
			//
			cout << endl
				 << *curScanptr 
				 << endl;
	    
			if( myLC )
			{
				(*myLC) << "Detected new Scan=" << (const char*)lastScanId << "\n";
			}
			
			//
			//  Look up the source of this scan in the experiment's source-list
			//
			for( currsourceindex=0;
				 currsourceindex<sourcecodelist.nelements();
				 currsourceindex++ )
			{
				if( sourcecodelist(currsourceindex)==curScanptr->getSourceCode() )
				{
					break;
				}
			}
	    
			if( currsourceindex>=sourcecodelist.nelements() )
			{
				cout << "Source `" << curScanptr->getSourceCode()
					 << "' not found in experiment "
					 << expref.getCode()
					 << "; will set index to invalid value" << endl;
				currsourceindex = (uInt)-1;
			}

			//
			// Check the sourceid's that occur in the field-table
			//
			this->updateFieldTable( msref, *curScanptr );

			//
			//  Try to get the frequencysetup for this scan....
			//
			if( !expref.getFrequencyDB().searchFrequencyConfig(frequencyconfig,
				curScanptr->getFrequencyCode()) )
			{
				cout << "Failed to locate frequency setup `"
					 << curScanptr->getFrequencyCode() << "'"
					 << " in Experiment `" << expref.getCode() << "'" << endl;
			}
			cout << endl << "Scan uses:\n" << frequencyconfig << endl << endl;

			//
			//  And write the sepctral setup...
			//
			if( !this->updateSpectralWindowTable(msref, *curScanptr, frequencyconfig) )
			{
				cout << "Failed to write frequency setup " << frequencyconfig.getCode()
					 << " for scan " << curScanptr->getScanId() << " in experiment "
					 << expref.getCode() << endl;
			}
	    
			integrationtime = curScanptr->getCorrelatorSetup().getIntegrationTime();
		}
		else if( !curScanptr )
		{
			if( lastScanId!="" )
			{
				lastScanId = "";
				cout << endl
					 << "*************** Error *************\n"
					 << "Finding visibilities without a Scan\n"
					 << "will set freq/src labels invalid\n";
			
				integrationtime = 0.0;
				currsourceindex = (uInt)-1;
				frequencyconfig = FrequencyConfig();
			}
		}

		if( !curScanptr )
		{
			if( myLC )
			{
				(*myLC) << "Skipping vis. (no Scan)\n";
			}
			visbuf.next();
			continue;
		}
	
	
		//
		//  Cache a reference to the visibilitylabel
		//
		const ExtVisLabel&  label( currentvisibility.getLabel() );

		//
		// Check if the current BaseBand can be found....
		//
		if( curScanptr && !frequencyconfig.getBaseBand(label.subbandnr()) )
		{
			if( myLC )
			{
				(*myLC) << "**** Warning: subband #" 
						<< (unsigned int&)label.subbandnr() << " not found in setup " 
						<< (const char *)frequencyconfig.getCode() << " in scan " 
						<< (const char*)curScanptr->getScanId() << "\n";
			}
	    
			visbuf.next();
			continue;
		}
	
		ant1 = label.baseline().idx1();
		ant2 = label.baseline().idx2();
		
		if( ant1>=maxant || ant2>=maxant )
		{
			if( myLC )
			{
				(*myLC) << "Skipping (invalid antenna)\n"
						<< "Error:\n";
				if( ant1>=maxant )
				{
					(*myLC) << ">> ANT1=" << ant1 << " out of range\n";
				}
				if( ant2>=maxant )
				{
					(*myLC) << ">> ANT2=" << ant2 << " out of range\n";
				}
				(*myLC) << "MAXANT=" << maxant << "\n";
			}
			visbuf.next();
			continue;
		}
	

		//
		//  Ok. Dump the data in the MS!
		//
		msref.addRow();

		//
		//  First write some default stuff...
		//
		pulsaridcol.put( currentrownr, -1 );
		pulsarbincol.put( currentrownr, -1 );
		arrayidcol.put( currentrownr, 0 );	
		feed1col.put( currentrownr, -1 );
		feed2col.put( currentrownr, -1 );
		correlatoridcol.put( currentrownr, 0 );
		fieldidcol.put( currentrownr, currsourceindex );
		scannumbercol.put( currentrownr, -1 );
		observationidcol.put( currentrownr, -1 );
		weightpolcol.put( currentrownr, currentvisibility.getWeight() );
		weightcol.put( currentrownr, mean(currentvisibility.getWeight()) );
	
		//
		//  Now the real thing...
		//
		ant1col.put( currentrownr, ant1 );
		ant2col.put( currentrownr, ant2 );
	
		spwinidcol.put( currentrownr, label.subbandnr() );
	
		//
		//  Check if we need to add a row in the spectral-window
		//  table....
		//
		//if( label.subbandnr()>=spwintab.nrow() || !chanfreqcol.isDefined(label.subbandnr()) )
		//{
		//    if( !this->updateSpectralWindowTable(msref, label.subbandnr(), scanref) )
		//    {
		//	cout << "Failed to write Subband: " << *currentbbptr << endl;
		//    }
		//}
		
		//
		//  Check if we need to write entry(ies) in the
		//  antennatable...
		//
		if( !antab.nrow() )
		{
			this->writeAntennaTable( msref, expref );
		}
	
		//
		//  Time related stuff
		//
		timecol.put( currentrownr, (currentvisibility.getTime().get("s")).getValue() );
				
		integrationtimecol.put( currentrownr, integrationtime );
		intervalcol.put( currentrownr, integrationtime );
	
		//**********************************************************/
		//
		//              Put data and flags...
		//
		//**********************************************************/
	
		//
		//  The makeData() function makes sure that the visibilitydata
		//  is conform the requested domain as indicated in the
		//  experiment
		//
		const Matrix<Complex>& dataref( this->makeData(expref, currentvisibility) );
	
		//cout << "makeData made datashape=" << dataref.shape() << endl;
		
		//
		//  Write the data
		//
		datacol.put( currentrownr, dataref );
		
		//flagmatrix.resize( dataref.shape() );
		//flagmatrix = False;
		//flagcol.put( currentrownr, flagmatrix );
		flagcol.put( currentrownr, currentvisibility.getFlag() );
		
		//
		// Increment the counter that keeps track of how many bytes
		// were translated
		//
		nrbytes+= dataref.shape().product()*sizeof(Complex);
	
		//
		//  We need to calculate UVW....
		//
		uvwcol.put( currentrownr, this->calcUVW(currentvisibility) );
	
		if( !(currentrownr%100) )
		{
			cout << "Row: " << currentrownr << "\r" << flush;
		}
	
		//
		//  Move on to next visibility
		//
		currentrownr++;
		available = visbuf.next();
	}
	//
    //  get time!
    //
    endtime = gethrtime();
    
    cout << endl << "Translated @ "
		 << ((long double)nrbytes)/((endtime-starttime)/1.0E+9) 
	 	 << " bytes/s" << endl;

    if( myLC )
    {
		(*myLC) << "Done translating the buffer!\n\n"
				<< "Now start writing the statistics...\n";
    }

    JIVEMSFiller::writeStatistics( msref, visbuf );
    
    return True;
}


Bool JIVEMSFiller::updateSpectralWindowTable( MS& msref, const j2ms::Scan& scanref,
					      const FrequencyConfig& fcref )
{
    //
    //  Keep track of what we've written...
    //
    static Container<History>   history;

    //
    //  Variable needed during execution of this function
    //
    Bool                    retval( False );
    History                 tmpHistory;
    const BaseBand*         bbptr;
    const Experiment&       expref( scanref.getExperiment() );
    MSSpectralWindow&       spwintable( msref.spectralWindow() );
    MSSpWindowColumns       spwincolumns( spwintable );
    const CorrelatorSetup&  correlatorsetup( scanref.getCorrelatorSetup() );
    
    //
    //  Test if we happened to write this one already....
    //
    tmpHistory.msName         = msref.tableName();
    tmpHistory.frequencySetup = fcref.getCode();
    tmpHistory.corSetup       = correlatorsetup;
    
    for( uint32 tst=0; tst<history.elements(); tst++ )
    {
	if( history(tst)==tmpHistory )
	{
	    return True;
	}
    }
    
    //
    //  Check if we need to add rows...
    //
    if( spwintable.nrow()==fcref.getNrBaseBands() )
    {
	return True;
    }
    
    if( spwintable.nrow() )
    {
	return False;
    }
    
    //
    //  cache some values
    //
    uInt                     numchannels( (uInt)correlatorsetup.nrOfSpectralPoints() );
    const Container<uint16>& correlations( correlatorsetup.getCorrelations() );

    //
    //  Prepare the values that should be written into the spectral
    //  window table
    //
    Array<Int>               corrtypes( IPosition(1,correlations.elements()) );
    Array<Int>               corrprod( IPosition(2,2,correlations.elements()) );
    Array<Double>            centerfreqs( IPosition(1,numchannels) );
    Array<Double>            resolution( IPosition(1,numchannels) );

    //
    //  Figure out what the (aips++)-polarizations are
    //
    char                    buffer[32];
    ostrstream              strm( buffer, sizeof(buffer) );
    CorrelationCode         jivetype;
    Stokes::StokesTypes     aipstype;
    FrequencyConfig::PolMap polmap;
    
    //
    //  Get the polarization map from the frequency-setup...
    //
    fcref.getPolMap( polmap );
    
    //
    //  Translate all correlations from the user-supplied JIVE
    //  internal format to:
    //
    //  a) aips++ internal enum, indicating a complete correlation (see Stokes.h)
    //  b) polarization numbers (0's and 1's)
    //
    for( uInt i=0; i<correlations.elements(); i++ )
    {
	//
	//  First map the JIVE internal CorrelationCode into a string
	//  (e.g. 'XY' or 'RR')
	//
	jivetype = CorrelationCode::ordinal2code( correlations(i) );

	strm.seekp( 0 );
	strm << polmap[jivetype.idx1()] << polmap[jivetype.idx2()] << ends;
	
	//
	//  Transform the string to aips++ internal enum
	//
	aipstype = Stokes::type( strm.str() );
	
	//
	//  And make the output variables
	//
	corrtypes( IPosition(1,i) ) = (Int)aipstype;
	corrprod( IPosition(2,i,0) ) = jivetype.idx1();
	corrprod( IPosition(2,i,1) ) = jivetype.idx2();
    }


    cout << "Writing SpectralWindowTable in MS=" << tmpHistory.msName
		 << " for frequency setup "
	 	 << tmpHistory.frequencySetup << " of scan "
		 << scanref.getScanId() << endl;
    
    //
    //  Ok. Not written yet...
    //
    for( uint32 bb2write=0, rownr=spwintable.nrow();
		 bb2write<fcref.getNrBaseBands(); bb2write++ )
    {
		spwintable.addRow();
	
		if( (bbptr=fcref.getBaseBand(bb2write))==0 )
		{
			cout << "Update spectralWindowTable failed; cannot lookup subband #" << bb2write
				<< " in mode " << scanref.getFrequencyCode() << " in experiment "
				<< expref.getCode() << endl;
			continue;
		}

	//
	//  Subband specific values
	//
	Double                   totalbandwidth( bbptr->getBandwidth() );
	Double                   startfreq( bbptr->getStartfreq() );
	Double                   channelwidth( totalbandwidth/(Double)numchannels );

	//
	//  Compute the centerfrequencies
	//
	if( bbptr->getSideBand()==BaseBand::upper )
	{
	    for( uInt i=0; i<numchannels; i++ )
	    {
		centerfreqs( IPosition(1,i) )= startfreq + ((Double)i)*channelwidth;
	    }
	}
	else if( bbptr->getSideBand()==BaseBand::lower )
	{
	    for( Int j=(numchannels-1); j>=0; j-- )
	    {
		centerfreqs( IPosition(1,(uInt)j) )= startfreq + ((Double)j+1)*channelwidth;
	    }
	}
	else
	{
	    cout << "updateSpectralWindow: sideband information not set for subband #"
		 << setw(2) << bb2write << " in mode " << scanref.getFrequencyCode()
		 << " in experiment " << expref.getCode() << endl;
	    continue;
	}
    
    
	resolution = channelwidth;
    

	//
	//  Write the entries to the table....
	//
	spwincolumns.chanFreq().put( rownr+bb2write, centerfreqs );
	spwincolumns.totalBandwidth().put( rownr+bb2write, totalbandwidth );
	spwincolumns.refFrequency().put( rownr+bb2write, startfreq );
	spwincolumns.numChan().put( rownr+bb2write, numchannels );
	spwincolumns.numCorr().put( rownr+bb2write, correlations.elements() );
	spwincolumns.corrType().put( rownr+bb2write, corrtypes );
	spwincolumns.corrProduct().put( rownr+bb2write, corrprod );
	spwincolumns.restFrequency().put( rownr+bb2write, 0.0 );
	spwincolumns.resolution().put( rownr+bb2write, resolution );
    }
    
    spwintable.flush();

    //
    //  Clean up...
    //
    retval = True;

    history.add( tmpHistory );


    return retval;
}


Bool JIVEMSFiller::writeAntennaTable( MeasurementSet& msref, const Experiment& expref )
{
    Bool                    retval( False );
    MSAntenna&              antabref( msref.antenna() );
    AntennaInfo             ai;
    MSAntennaColumns        antabcols( antabref );
    ExperimentCacheElement* ptr2element( theirCache.locateExperiment(expref.getCode()) );

    //
    //  [Assumption: if antenna table has nr of rows != to zero, it has already been filled in...]
    //
    if( antabref.nrow() )
    {
	return True;
    }
    
    //
    //  Ok. Get data from the cache
    //
    if( !ptr2element )
    {
	ptr2element = theirCache.addExperiment( expref, &theirGlobalDBs.getAntennaDB() );
    }

    if( !ptr2element )
    {
	return False;
    }
    
    //
    //  Now, write all the data for all the antennas in the
    //  experiment....
    //
    antabref.addRow( ptr2element->itsAntennaTable.elements() );
    
    for( uInt cnt=0; cnt<ptr2element->itsAntennaTable.elements(); cnt++ )
    {
	ai = ptr2element->itsAntennaTable(cnt);

	antabcols.name().put( cnt, ai.getName() );
	antabcols.antennaId().put( cnt, cnt );
	antabcols.dishDiameter().put( cnt, ai.getDiameter() );
	antabcols.mount().put( cnt, ai.getMount() );
	antabcols.arrayId().put( cnt, 0 );
	antabcols.offset().put( cnt, ai.getOffset() );
	antabcols.position().put( cnt, (ai.getPosition().get("m")).getValue() );
	antabcols.orbitId().put( cnt, -1 );
	antabcols.phasedArrayId().put( cnt, -1 );
	antabcols.station().put( cnt, ai.getStation() );
    }

    //
    //  Flush the table
    //
    antabref.flush();
    
    retval = True;

    //
    //  Need to verify if the ARRAY-table is present and has one row...
    //
    MSArray&     arraytab( msref.array() );
    
    if( arraytab.nrow()==0 )
    {
	Vector<Double>    pos( 3 );
	MSArrayColumns    arraycols( arraytab );
	
	pos = 0.0;
	
	arraytab.addRow();
	
	arraycols.name().put( 0, expref.getArrayCode() );
	arraycols.position().put( 0, pos );

	arraytab.flush();
    }
    return retval;
}




const Matrix<Complex>& JIVEMSFiller::makeData( const Experiment& expref, const Visibility& vref )
{
    //
    //  These are always there....
    //
    static Matrix<Complex>          fftdata;
    static Matrix<Complex>          resdata;
    static FFTServer<float,Complex> transformator;

    //
    //  
    //


    //
    //  Cache the visibility datadomain
    //
    Visibility::domain              srcdomain( vref.getDatadomain() );
    Visibility::domain              dstdomain( expref.getDestinationDomain() );

    //
    //  Perform sanity checks
    //
    if( srcdomain==Visibility::undefined || dstdomain==Visibility::undefined )
    {
	if( myLC )
	{
	    (*myLC) << "makeData: either src or dst data domain is undefined!\n";
	}
	    
	resdata.resize( 0, 0 );
	return resdata;
    }
    
    //
    //  The simplest case is when the visibility data-domain and the
    //  desired output domain match...
    //
    if( dstdomain==srcdomain )
    {
	return vref.getData();
    }
    
    //
    //  If no visibilitybuffer and/or scan known return just the
    //  data....
    //
    if( !vref.hasVisibilityBuffer() || 
	(vref.hasVisibilityBuffer() && !vref.getVisibilityBuffer().getScanptr()) )
    {
	if( myLC )
	{
	    (*myLC) << "makeData: either no vis.buffer and/or no scan in visbuffer!\n";
	}
	resdata.resize( 0, 0 );
	return resdata;
    }


    //
    //  Locate the baseband we're looking at....
    //
    const BaseBand*          bbptr;
    FrequencyConfig          fc;
    const j2ms::Scan&        scanref( *(vref.getVisibilityBuffer().getScanptr()) );
    const FrequencyDB&       freqdb( expref.getFrequencyDB() );
    
    if( !freqdb.searchFrequencyConfig(fc, scanref.getFrequencyCode()) )
    {
	if( myLC )
	{
	    (*myLC) << "makeData: failed to locate frequency setup "
		    << (const char *)scanref.getFrequencyCode()
		    << " in experiment " << (const char *)expref.getCode() << "\n";
	}
	resdata.resize( 0, 0 );
	return resdata;
    }
    
    if( (bbptr=fc.getBaseBand(vref.getLabel().subbandnr()))==0 )
    {
	if( myLC )
	{
	    (*myLC) << "makeData: failed to locate baseband #"
		    << (unsigned int&)vref.getLabel().subbandnr()
		    << " in frequency setup " << (const char *)scanref.getFrequencyCode()
		    << " in experiment " << (const char *)expref.getCode() << "\n";
	}
	
	resdata.resize( 0, 0 );
	return resdata;
    }
    
    if( bbptr->getSideBand()==BaseBand::none )
    {
	if( myLC )
	{
	    (*myLC) << "makeData: sideband for baseband #" 
		    << (unsigned int&)vref.getLabel().subbandnr()
		    << " in frequency setup " << (const char *)scanref.getFrequencyCode()
		    << " in experiment " << (const char *)expref.getCode() << " unknown\n";
	}
	
	resdata.resize( 0, 0 );
	return resdata;
    }
    

    //
    //  When we arrive at this point in the program we've established
    //  a couple of facts:
    //
    //  1) source and destination domain for the data are NOT the
    //  same, hence we *need* to do a FFT
    //
    //  2) neither source nor destination domain is unknown, so no
    //  surprises are to be expected from those
    //
    //  3) we have located the baseband this data belongs to so we
    //  know exactly which part of the data we need to cut out (*if*
    //  we have to cut, that is)
    //
    uInt                     fftsize;
    uInt                     nrffts;
    Bool                     expand;
    Bool                     fftflag;
    const Matrix<Complex>*   retvalptr;
    const Matrix<Complex>&   srcdataref( vref.getData() );
    IPosition                srcshape( srcdataref.shape() );
    

    //
    //  What we do is: check if we need to expand the src data-matrix,
    //  i.e. if the source dat is an autocorrelation and the data is
    //  in time-domain, we need to expand the data (because we have
    //  already verified that the destination domain and the source
    //  domain are not equal and since the source domain (in this
    //  case) is time, the other must be frequency).
    //
    nrffts  = srcdataref.nrow();
    fftsize = srcdataref.ncolumn();
    
    //
    //  We need to expand the data if either of the following
    //  conditions holds:
    //
    //  1) we're going from time->frequency AND this is an
    //  autocorrelation (Albert gives us only half the number of lags
    //  in this case)
    //
    //  2) we're going from frequency->time domain
    //
    //  NOTE: the following two boolean variables will tell which (if
    //  any) of the two aforementioned conditions holds, they can
    //  *never* hold both (we make use of this property later on)
    //
    Bool       expandcondition1;
    Bool       expandcondition2;
    
    //
    //  30-09-1999: HV, explicitly set expandcondition 1 to False
    //  since this is (from this date on) already covered by the
    //  COFVisibilityBuffer class. Remains to be done for the
    //  DTDVisibilityBuffer!!! We can expect problems if people start
    //  reading data from this format! Changes to that are slim so
    //  forget about it for now!
    //
    //
    //expandcondition1 = ToBool( vref.isAutoCorrelation() && srcdomain==Visibility::time );
    expandcondition1 = False;
    
    expandcondition2 = ToBool( srcdomain==Visibility::frequency );
    
    if( (expand=ToBool(expandcondition1 || expandcondition2))==True )
    {
	fftsize *= 2;
    }
    
    //
    //  Now we can properly resize and initialise the 'fftdata' matrix
    //
    fftdata.resize( IPosition(2,nrffts,fftsize) );
    fftdata = Complex( 0.0f, 0.0f );
    
    //
    //  If we need to expand, do it now!
    //
    if( expand )
    {
	uInt     nr2copy( srcdataref.ncolumn() );
	
	//
	//  Depending on which expandcondition holds we take action
	//
	if( expandcondition1 )
	{
	    //
	    //  Autocorrelation && time->frequency
	    //
	    for( uInt i=0; i<nrffts; i++ )
	    {
		//
		//  Copy all channels but channel0
		//
		for( uInt lagpoint=1; lagpoint<nr2copy; lagpoint++ )
		{
		    fftdata( i, nr2copy+lagpoint ) = 
			fftdata( i, nr2copy-lagpoint ) = srcdataref( i, lagpoint );
		}
		//
		//  Copy channel 0 into the middle of the fftarray
		//
		fftdata( i, nr2copy ) = srcdataref( i, 0 );
	    }
	}
	else
	{
	    //
	    //  frequency->time, need to check sideband to know where
	    //  to copy the data to
	    //
	    uInt    startpoint;
	    
	    if( bbptr->getSideBand()==BaseBand::upper )
	    {
		startpoint = nr2copy;
	    }
	    else
	    {
		startpoint = 1;
	    }
	    //
	    //  Ok. Copy stuff
	    //
	    for( uInt i=0; i<nrffts; i++ )
	    {
		(fftdata.row(i))( Slice(startpoint, nr2copy) ) = srcdataref.row(i);
	    }
	}
    }
    else
    {
	fftdata = vref.getData();
    }
    
    //
    //  Ok. fftdata is now properly sized and filled in, now we
    //  determine the FFT-flag and do the FFT
    //
    //  fftflag (=toFrequency argument)
    //  
    //  True  implies forward transform == time -> frequency
    //  False implies backward transform == frequency -> time
    //
    fftflag = ToBool( srcdomain==Visibility::time );
    
    //
    //  Do the fft
    //
    for( uInt i=0; i<nrffts; i++ )
    {
	transformator.fft( fftdata.row(i).ac(), fftflag );
    }
    
    //
    //  Now we have to get out the data. If we were doing
    //  timetofrequency, we must keep only half of the resultant
    //  values. datastart tells us where to start in the fft'ed
    //  data. The value of datastart is determined by the sideband of
    //  the data: USB -> we take the upper half of the array, LSB ->
    //  the lower half.
    //
    //
    uInt     datastart( 0 );
    uInt     datasize( fftsize );
    
    if( fftflag )
    {
	datasize /= 2;
	
	//
	//  Which half to take depends on the sideband
	//
	if( bbptr->getSideBand()==BaseBand::upper )
	{
	    datastart = datasize;
	}
	else
	{
	    datastart = 1;
	}
    }
    
    //
    // datasize!=0 && datasize unequal to fftsize this means that we
    // have to <snip> the array...
    //
    if( datasize && datasize!=fftsize )
    {
	//
	//  Now we can resize and fill the 'resdata'-matrix
	//
	resdata.resize( nrffts, datasize );

	for( uInt i=0; i<nrffts; i++ )
	{
	    //resdata.row(i) = (fftdata.row(i))( Slice(datastart, datasize) );
	    for( uInt j=0; j<datasize; j++ )
	    {
		resdata( i, j ) = fftdata( i, datastart+j );
	    }
	}
	retvalptr = &resdata;
    }
    else if( datasize )
    {
	retvalptr = &fftdata;
    }
    else
    {
	//
	//  Error condition....
	//
	resdata.resize( 0, 0 );
	retvalptr = &resdata;
    }
    
    
    //
    //  Return the result....
    //
    return *retvalptr;
}






const Matrix<Complex>& JIVEMSFiller::makeDataOld( const Experiment& expref, const Visibility& vref )
{
    //
    //  These are always there....
    //
    static Matrix<Complex>          fftdata;
    static Matrix<Complex>          resdata;
    static FFTServer<float,Complex> transformator;

    //
    //  The simplest case is when the visibility data-domain and the
    //  desired output domain match...
    //
    if( expref.getDestinationDomain()==vref.getDatadomain() )
    {
	return vref.getData();
    }
    
    //
    //  If no visibilitybuffer and/or scan known return just the
    //  data....
    //
    if( !vref.hasVisibilityBuffer() || 
	(vref.hasVisibilityBuffer() && !vref.getVisibilityBuffer().getScanptr()) )
    {
	//cout << "makeData: No visibilitybuffer and/or no current scan\n"
	//     << "          in experiment " << expref.getCode() << endl;
	return vref.getData();
    }
    

    //
    //  Now do a transformation....
    //
    //  We know that:
    //
    //  a)  We need to do a 1-D FFT anyway
    //  b)  The only thing that matters is the buffersize
    //
    uInt                     fftsize;
    uInt                     nrffts;
    Bool                     expand( False );
    Bool                     fftflag;
    const BaseBand*          bbptr;
    const j2ms::Scan&        scanref( *(vref.getVisibilityBuffer().getScanptr()) );
    const Matrix<Complex>*   retvalptr;
    const Matrix<Complex>&   srcdataref( vref.getData() );
    IPosition                srcshape( srcdataref.shape() );

    //
    //  Locate the baseband we're looking at....
    //
    const FrequencyDB&       freqdb( expref.getFrequencyDB() );
    FrequencyConfig          fc( "n/a" );
    
    if( !freqdb.searchFrequencyConfig(fc, scanref.getFrequencyCode()) )
    {
	//cout << "makeData: Failed to locate frequency setup `" << scanref.getFrequencyCode() << "'"
	//     << " in Experiment `" << expref.getCode() << "'" << endl;
	return vref.getData();
    }
    
    if( (bbptr=fc.getBaseBand(vref.getLabel().subbandnr()))==0 )
    {
	//cout << "makeData: Failed to locate BaseBand nr " << vref.getLabel().subbandnr() << " in frequencysetup `"
	//     << scanref.getFrequencyCode() << "'" << endl;
	return vref.getData();
    }
    

    //
    //  What we do is: check if we need to expand the src data-matrix,
    //  i.e. if the source dat is an autocorrelation and the data is
    //  in time-domain, we need to expand the data (because we have
    //  already verified that the destination domain and the source
    //  domain are not equal and since the source domain (in this
    //  case) is time, the other must be frequency).
    //
    nrffts = srcshape(0);
    fftsize = srcshape(1);
    
    if( vref.isAutoCorrelation() && vref.getDatadomain()==Visibility::time )
    {
	expand = True;
	fftsize *= 2;
    }
    
    //
    //  Now we can properly resize the 'fftdata' matrix
    //
    fftdata.resize( IPosition(2,nrffts,fftsize) );
    
    //
    //  Copy the data...
    //
    if( expand )
    {
	uInt     nr2copy( srcshape(1) );
	
	for( uInt i=0; i<nrffts; i++ )
	{
	    //
	    //  Copy all channels but channel0
	    //
	    for( uInt lagpoint=1; lagpoint<nr2copy; lagpoint++ )
	    {
		fftdata( i, nr2copy+lagpoint ) = fftdata( i, nr2copy-lagpoint ) = srcdataref( i, lagpoint );
	    }
	    //
	    //  Copy channel 0 and set the zero-th element in the
	    //  fftdata matrix
	    //
	    fftdata( i, 0       ) = Complex( 0.0, 0.0 );
	    fftdata( i, nr2copy ) = srcdataref( i, 0 );
	}
    }
    else
    {
	fftdata = vref.getData();
    }
    
    //
    //  Ok. fftdata is now properly sized and filled in, now we
    //  determine the FFT-flag and do the FFT
    //
    //  fftflag (=toFrequency argument)
    //  
    //  True  implies forward transform == time -> frequency
    //  False implies backward transform == frequency -> time
    //
    if( vref.getDatadomain()==Visibility::time )
    {
	fftflag = True;
    }
    else
    {
	fftflag = False;
    }
    
    
    for( uInt i=0; i<nrffts; i++ )
    {
	transformator.fft( fftdata.row(i).ac(), fftflag );
    }
    
    //
    //  Now we have to get out the data. If we were doing
    //  timetofrequency, we must keep only half of the resultant
    //  values. datastart tells us where to start in the fft'ed data.
    //
    //  NOTE: If we're going from time -> frequency, ALbert tells us
    //  that the data can be found in the USB, i.e. in the upper half
    //  of the array!!
    //
    uInt     datastart( 0 );
    uInt     datasize( fftsize );
    
    if( fftflag )
    {
	datasize /= 2;
	
	//
	//  Which half to take depends on the sideband
	//
	switch( bbptr->getSideBand() )
	{
	    case BaseBand::upper:
		datastart = datasize;
		break;
		
	    case BaseBand::lower:
		datastart = 1;
		break;
		
	    default:
		cout << "makeData: sideband not defined!!" << endl;
		datastart = 0;
		datasize = 0;
		break;
	}
    }

    if( datasize && datasize!=fftsize )
    {
	//
	//  Now we can resize and fill the 'resdata'-matrix
	//
	resdata.resize( nrffts, datasize );

	for( uInt i=0; i<nrffts; i++ )
	{
	    resdata.row(i) = fftdata.row(i)( Slice(datastart,datasize) );
	}
	retvalptr = &resdata;
    }
    else if( datasize )
    {
	retvalptr = &fftdata;
    }
    else
    {
	//
	//  Error condition....
	//
	resdata.resize( 0, 0 );
	retvalptr = &resdata;
    }
    
    
    //
    //  Return the result....
    //
    return *retvalptr;
}

const Vector<Double>& JIVEMSFiller::calcUVW( const Visibility& vref  )
{
    //
    //  These statics are here to speed up UVW-calculation; we don't need to 
    //
    static Double         gastInRadians;
    static Double         currentHA;
    static MEpoch         currentGAST;
    static MEpoch         currenttime;
    static RotMatrix      currentRotMatrix;
    static Vector<Double> sourceRADEC;
    static Vector<Double> uvw( 3 );
    static Vector<Double> ant1xyz( 3 );
    static Vector<Double> ant2xyz( 3 );
    static Vector<Double> baseline( 3 );    

    //
    //  Init retval to a neat zero-value
    // 
    uvw = 0.0;

    if( !vref.hasVisibilityBuffer() ||
	(vref.hasVisibilityBuffer() && !vref.getVisibilityBuffer().getScanptr()) )
    {
	return uvw;
    }
    
    //
    //  Local vars that only live during execution of the function
    //
    const j2ms::Scan&       scanref( *(vref.getVisibilityBuffer().getScanptr()) );
    const Experiment&       itsexpref( scanref.getExperiment() );
    ScanCacheElement*       scancacheptr;
    const ExtVisLabel&      labelref( vref.getLabel() );
    ExperimentCacheElement* expcacheptr( theirCache.addExperiment(itsexpref,
								  &theirGlobalDBs.getAntennaDB()) );
    

    if( !expcacheptr )
    {
	return uvw;
    }

    //
    //  Locate the Scan in the Scan-cache, since that entry contains
    //  all the conversion stuff needed for the coordinate
    //  transformations.
    //
    scancacheptr = expcacheptr->addScan( scanref, &theirGlobalDBs.getSourceDB() );

    if( !scancacheptr )
    {
	return uvw;
    }

    //
    //  Verify that we can find the antennainfo....
    //
    uint32    tmpidx1( labelref.baseline().idx1() );
    uint32    tmpidx2( labelref.baseline().idx2() );
    uint32    maxantnr( expcacheptr->itsAntennaTable.elements() );
    
    if( tmpidx1>=maxantnr || tmpidx2>=maxantnr )
    {
	//cout << "calcUVW: Error!\n";
	//if( tmpidx1>=maxantnr )
	//{
	//    cout << ">> ANT1=" << tmpidx1 << " out of range!\n";
	//}
	//if( tmpidx2>=maxantnr )
	//{
	//    cout << ">> ANT2=" << tmpidx2 << " out of range!\n";
	//}
	//cout << "MAX=" << maxantnr-1 << endl;
	return uvw;
    }
    

    //
    //  Retrieve source/antennainfo
    //
    const SourceInfo&     sourceinforef( scancacheptr->itsSourceInfo );
    const AntennaInfo&    ant1inforef( expcacheptr->itsAntennaTable(labelref.baseline().idx1()) );
    const AntennaInfo&    ant2inforef( expcacheptr->itsAntennaTable(labelref.baseline().idx2()) );

    //
    //  Ok. Convert the current visibility time to GAST (Greenwich Sidereal Time)
    //
    //currenttime = this->calcVisibilityTime(scanref, vref, fcref);
    currenttime = vref.getTime();

    currentGAST = scancacheptr->timeCvt( currenttime );

    //
    //  Get the source's RA in radians.
    //
    //  NOTE: the source position should be precessed to the current
    //  visibility epoch!! Leave that for later (we're using 2000
    //  coordinates, mostly, so the difference should be very
    //  little...
    //
    sourceRADEC = ((sourceinforef.getDirection()).getValue()).get();

    //
    //  Convert GAST to radians as well.... (in order to be able to
    //  derive the HA as seen from Greenwich)
    //
    //  1) start off with the fractional day in GAST
    //  2) multiply by 2pi to get at an angle in radians
    //
    gastInRadians = (currentGAST.getValue()).getDayFraction();
    gastInRadians *= ( 2*C::pi );

    //
    //  Now we can form the HA
    //
    currentHA = gastInRadians - sourceRADEC(0);
    
    //
    // Ok. Now attempt to form the baseline
    //
    // 1) get the antenna XYZ-positions
    // 2) subtract them to get the baseline vector
    //
    ant1xyz = ((ant1inforef.getPosition()).getValue()).getValue();
    ant2xyz = ((ant2inforef.getPosition()).getValue()).getValue();
    
    baseline.ac() = ant2xyz.ac() - ant1xyz.ac();
    
    //
    //  Ok. All that's left is a) to set up the rotation matrix and b)
    //  do the rotation!
    //
    currentRotMatrix = RotMatrix( Euler(-(C::pi_2-sourceRADEC(1)), 1, (currentHA - C::pi_2), 3) );

    //
    //  Finally: do the rotation!
    //
    uvw = currentRotMatrix*baseline;

    return uvw;
    //
    //  Step one: make sure that the time (whatever frame it was in)
    //  gets converted to UTC and make that the time-reference for all
    //  future (in this function) calculations.
    //
    //
    //  Get the time of this visibility and convert to UTC
    //
    //currenttime = this->calcVisibilityTime(scanref, vref, fcref);
    //currentUTCtime = scancacheptr->timeCvt( currenttime );

    //
    //  Set as referenceframe
    //
    //scancacheptr->referenceFrame.set( currentUTCtime );

    //
    //  Convert antennapositions to ITRF
    //
    //ant1ITRFpos = scancacheptr->positionCvt( ant1inforef.getPosition() );
    //ant2ITRFpos = scancacheptr->positionCvt( ant2inforef.getPosition() );
    
    //
    //  Form the baseline in ITRF
    //
    //curbaseline = MBaseline( MVBaseline(ant2ITRFpos.getValue(), ant1ITRFpos.getValue()),
    //			     scancacheptr->baselineRef );
    
    //
    // Convert the phase-center (is source direction) to our current
    // reference frame
    //
    //scancacheptr->directionFrame.set( currentUTCtime );
    //scancacheptr->directionRef.set( scancacheptr->directionFrame );
    //phasecentercvt = scancacheptr->directionCvt( sourceinforef.getDirection() );
    
    //
    //  Now we can make the UVW...
    //
    //currentuvw = Muvw( MVuvw(curbaseline.getValue(), phasecentercvt.getValue()), Muvw::ITRF );

    //return (currentuvw.getValue()).getValue();
}


Bool JIVEMSFiller::updateFieldTable( MeasurementSet& msref, const j2ms::Scan& scanref )
{
    uInt                  sourceindex;
    uInt                  rownr;
    Bool                  sourceinfofound;
    MSField&              fieldtab( msref.field() );
    MSSource&             sourcetab( msref.source() );
    SourceInfo            si;
    MSFieldColumns        fieldcolumns( fieldtab );
    MSSourceColumns       sourcecolumns( sourcetab );
    const Experiment&     expref( scanref.getExperiment() );
    const Vector<String>& sourcecodes( expref.getSources() );

    //
    //  Find sourceindex....
    //
    for( sourceindex=0; sourceindex<sourcecodes.nelements(); sourceindex++ )
    {
	if( sourcecodes(sourceindex)==scanref.getSourceCode() )
	{
	    break;
	}
    }

    if( sourceindex>=sourcecodes.nelements() )
    {
	return False;
    }
    
    //
    //  Ok. Check if the source-id already exists
    //
    Table      idtab( sourcetab(sourcetab.col(MSSource::columnName(MSSource::SOURCE_ID))==(Int)sourceindex) );
    
    if( idtab.nrow() )
    {
	return True;
    }

    /*
    if( fieldtab.nrow()<sourceindex && fieldcolumns.phaseDir().isDefined(sourceindex) )
    {
	return True;
    }
    */

    //
    //  Ok. Attempt to locate the source-info
    //
    sourceinfofound = False;
    
    if( expref.hasSourceDB() )
    {
	sourceinfofound = expref.getSourceDB().searchSource( si, scanref.getSourceCode() );
    }
    
    if( !sourceinfofound )
    {
	sourceinfofound = theirGlobalDBs.getSourceDB().searchSource( si, scanref.getSourceCode() );
    }
    
    if( !sourceinfofound )
    {
	cout << "SourceInfo for `" << scanref.getSourceCode() << "' not found." << endl;
	return False;
    }

    //
    //  Make sure the field/source tables have enough rows...
    //
    rownr = sourcetab.nrow();
    
    sourcetab.addRow();
    fieldtab.addRow();
    /*
    if( fieldtab.nrow()<=sourceindex )
    {
	fieldtab.addRow( sourceindex-fieldtab.nrow()+1 );
    }
    if( sourcetab.nrow()<=sourceindex )
    {
	sourcetab.addRow( sourceindex-sourcetab.nrow()+1 );
    }
    */

    //
    //  Ok. Fill in the columns.....
    //
    Vector<Double>   coordinates( 2 );
    Vector<Double>   propermotion( 2 );
    Vector<Double>   position( 3 );
    
    coordinates = 0.0;
    position = 0.0;
    propermotion = 0.0;
    
    //
    // First write all those who are zeroes
    //
    fieldcolumns.delayDirRate().put( rownr, coordinates );
    fieldcolumns.phaseDirRate().put( rownr, coordinates );
    fieldcolumns.pointingDirRate().put( rownr, coordinates );
    fieldcolumns.referenceDirRate().put( rownr,coordinates );
    
    //
    //  Fill in the values from the sourceinfo
    //
    coordinates = (si.getDirection().getValue()).get();
    
    fieldcolumns.delayDir().put( rownr, coordinates );
    fieldcolumns.phaseDir().put( rownr, coordinates );
    fieldcolumns.pointingDir().put( rownr, coordinates );
    fieldcolumns.referenceDir().put( rownr, coordinates );

    fieldcolumns.name().put( rownr, si.getName() );
    fieldcolumns.sourceId().put( rownr, sourceindex );
    fieldcolumns.fieldId().put( rownr, sourceindex );
    fieldcolumns.time().put( rownr, (scanref.getStartTime().get("s")).getValue() );
    
    //
    //  We need to fill in the sourcetable as well....
    //
    sourcecolumns.sourceId().put( rownr, sourceindex );
    sourcecolumns.interval().put( rownr, 0.0 );
    sourcecolumns.time().put( rownr, 0.0 );
    
    sourcecolumns.code().put( rownr, "" );
    sourcecolumns.calibrationGroup().put( rownr, 0 );
    sourcecolumns.direction().put( rownr, coordinates );
    sourcecolumns.name().put( rownr, si.getName() );
    sourcecolumns.position().put( rownr, position );
    sourcecolumns.properMotion().put( rownr, propermotion );
    sourcecolumns.sysvel().put( rownr, 0.0 );


    //
    //  Flush the tables
    //
    sourcetab.flush();
    fieldtab.flush();
    
    return True;
}


void JIVEMSFiller::writeStatistics( MeasurementSet& ms, const VisibilityBuffer& visbuf )
{
    try
    {
	//
	//  Variables needed for compiling the list of tapestats files
	//  that need to be processed...
	//
	DIR*                 dirptr;
	char                 strmbuf[ 1024 ];
	uInt                 nfiles( 0 );
	Regex                statsfile( "^TPSTAT[0-9]*$" );
	char**               files( 0 );
	ostrstream           strm( strmbuf, sizeof(strmbuf) );
	struct dirent*       directoryentry;
	
	//
	//  Form list of tapestatsfiles
	//
	if( (dirptr=opendir(visbuf.getRootDir()))==0 )
	{
	    cerr << "writeStatistics: failed to open dir " << visbuf.getRootDir() << endl;
	    return;
	}
	
	while( (directoryentry=readdir(dirptr))!=0 )
	{
	    if( String(directoryentry->d_name).matches( statsfile ) )
	    {
		files = (char **)realloc( files, (nfiles+1)*sizeof(char*) );
		
		if( !files )
		{
		    cerr << "writeStatistics: no more mem available for new files..." << endl;
		    break;
		}
		files[ nfiles ] = strdup(directoryentry->d_name);
		nfiles++;
	    }
	}
	
	if( !files )
	{
	    cerr << "writeStatistics: no TapeStatistics-data-files found in dir="
		 << visbuf.getRootDir() << endl;
	    return;
	}
	

	//
	//  Map datasource 
	//
	//  <<NOTE>> If we eventually reach more than 32 SU's we migh
	//  get into trouble! I just made this contruction to do a
	//  fats and *very* simple way of mapping the datasource id to
	//  an antenna number!
	//
	uInt         maxdatasource( 32 );
	Int          datasource2ant[ maxdatasource ];
	
	//
	//  Initialize default mapping
	//
	for( uInt tmpi=0; tmpi<maxdatasource; datasource2ant[tmpi++]=-1 );
	
	//
	//  Look for the mapping...
	//
	try
	{
	    strm.seekp( 0 );
	    strm << visbuf.getRootDir() << "/" << "MAP" << ends;
	    
	    MappingsFile     mf( strm.str() );
	    DataFileRecord*  dfrptr;
	    
	    //
	    // <<Assume>> the MappingEntry is the first record in the
	    // MappingFile
	    //
	    mf >> dfrptr;
	    
	    if( dfrptr && dfrptr->recordType()==DataFileRecord::mappingentry )
	    {

		MappingEntry*   meptr( (MappingEntry*)dfrptr );

		for( uInt ti=0; ti<meptr->nrMappings(); ti++ )
		{
		    const Mapping&   map( (*meptr)[ti] );

		    if( map.itsDataSourceId<maxdatasource )
		    {
			datasource2ant[ map.itsDataSourceId ] = map.itsTelescopeNr;
		    }
		}		
	    }
	    
	    //
	    //  Don't need the record anymore!
	    //
	    delete dfrptr;
	}
	catch( const ExceptionObject& y )
	{
	    //
	    //  No mappingsfile available... continue with default
	    //  mapping...
	    //
	    y.getText();
	}
	

	//
	//  Variables needed for filling the table..
	//
	uInt                 currownr;
	Table                jivetable( ms.rwKeywordSet().asTable("JIVE_TAPESTATS") );
	ScalarColumn<uInt>   suid( jivetable, "SUID" );
	ScalarColumn<Int>    antennaid( jivetable, "ANTENNA_ID" );
	ScalarColumn<Double> tot( jivetable, "TOT" );
	ScalarColumn<Double> servoerror( jivetable, "SERVO_ERROR" );
	ScalarColumn<uInt>   n_hdstk( jivetable, "N_HDSTK" );
	ArrayColumn<uInt>    hdstk_pos( jivetable, "HDSTK_POS" );
	ScalarColumn<uInt>   footage( jivetable, "FOOTAGE" );
	ScalarColumn<uInt>   su_state( jivetable, "SU_STATE" );
	ScalarColumn<uInt>   suim_state( jivetable, "SUIM_STATE" );
	ScalarColumn<uInt>   n_track( jivetable, "N_TRACK" );	
	ArrayColumn<uInt>    org_trackno( jivetable, "ORG_TRACKNO" );
	ArrayColumn<uInt>    nr_frames( jivetable, "NR_FRAMES" );
	ArrayColumn<uInt>    pe_count( jivetable, "PE_COUNT" );
	ArrayColumn<uInt>    us_count( jivetable, "US_COUNT" );
	ArrayColumn<uInt>    ms_count( jivetable, "MS_COUNT" );
	ArrayColumn<uInt>    crc_err( jivetable, "CRC_ERR" );
	ArrayColumn<uInt>    dbf_count( jivetable, "DBF_COUNT" );

	//
	//  Now then: loop over all files that matched!
	//
	for( uInt filecount=0; filecount<nfiles; filecount++ )
	{
	    //jivetable.addRow();
	    //cout << "added row to statistics table..." << endl;
	    //continue;
	    
	    if( !files[filecount] )
	    {
		cerr << "writeStatistics: NULL-filename pointer for file #" << filecount << endl;
		continue;
	    }
	    
	    //
	    //  Ok. Attempt to open the file and read all the records
	    //  in it!
	    //
	    try
	    {
		strm.seekp( 0 );
		strm << visbuf.getRootDir() << "/" << files[filecount] << ends;
		
		Int             curant( -1 );
		TapeStatsFile   tpfile( strm.str() );
		DataFileRecord* currecord;

		//
		//  Look up the antenna!
		//
		if( tpfile.getDataSourceId()<maxdatasource )
		{
		    curant = datasource2ant[ tpfile.getDataSourceId() ];
		}
		
		while( !tpfile.eof() )
		{
		    tpfile >> currecord;
		    
		    if( !currecord  || 
			(currecord && currecord->recordType()!=DataFileRecord::tapestats) )
		    {
			cerr << "writeStatistics: either no record or not TapeStatType..." << endl;
			delete currecord;
			continue;
		    }

		    uInt                  nrtracks;
		    uInt                  curtrack;
		    Vector<uInt>          hdstkposns( 4 );
		    Vector<uInt>          orgtracknums;
		    Vector<uInt>          nrframes;
		    Vector<uInt>          pecounts;
		    Vector<uInt>          uscounts;		    
		    Vector<uInt>          mscounts;
		    Vector<uInt>          crcerrors;
		    Vector<uInt>          dbfcounts;
		    TrackErrorCount       trackstats;
		    TapeStatDataRecord*   ptr( (TapeStatDataRecord*)currecord );

		    //
		    // Count the active tracks...
		    //
		    nrtracks = 0;
		    
		    for( uInt i=0; i<64; i++ )
		    {
			if( ptr->getActiveTrack(i) )
			{
			    orgtracknums.resize( nrtracks+1, True );
			    orgtracknums( nrtracks ) = i;
			    nrtracks++;
			}
		    }
		    
		    //
		    //  Now we can safely resize all the arrays
		    //
		    nrframes.resize( nrtracks );
		    pecounts.resize( nrtracks );
		    uscounts.resize( nrtracks );
		    mscounts.resize( nrtracks );
		    crcerrors.resize( nrtracks );
		    dbfcounts.resize( nrtracks );
		    
		    //
		    //  Fill the arrays
		    //
		    for( uInt cnt=0; cnt<nrtracks; cnt++ )
		    {
			curtrack = orgtracknums( cnt );
			
			trackstats = ptr->getTrackErrors(curtrack);
			
			nrframes(cnt) = trackstats.getNrOfFrames();
			pecounts(cnt) = trackstats.getPECount();
			uscounts(cnt) = trackstats.getUnexpectedSyncsCount();
			mscounts(cnt) = trackstats.getMissedSyncsCount();
			crcerrors(cnt) = trackstats.getCRCErrorCount();
			dbfcounts(cnt) = trackstats.getDiscardedBadFrameCount();
		    }
		    
		    //
		    //  The headstack positions
		    //
		    for( uInt hd=0; hd<4; hd++ )
		    {
			hdstkposns( hd ) = ptr->getHeadstackPosition( hd );
		    }
		    
		    //
		    //  Add a row and fill in the stuff
		    //
		    currownr = jivetable.nrow();
		    
		    jivetable.addRow();
		    
		    suid.put( currownr, tpfile.getDataSourceId() );
		    antennaid.put( currownr, curant );
		    tot.put( currownr, ptr->getTOT().get_sysclks() );
		    servoerror.put( currownr, ptr->getServoError() );
		    n_hdstk.put( currownr, ptr->getNrOfHeadstacks() );
		    hdstk_pos.put( currownr, hdstkposns );
		    footage.put( currownr, ptr->getFootage() );
		    su_state.put( currownr, ptr->getSUState() );
		    suim_state.put( currownr, ptr->getSUIMState() );
		    n_track.put( currownr, nrtracks );
		    org_trackno.put( currownr, orgtracknums );
		    nr_frames.put( currownr, nrframes );
		    pe_count.put( currownr, pecounts );
		    us_count.put( currownr, uscounts );
		    ms_count.put( currownr, mscounts );
		    crc_err.put( currownr, crcerrors );
		    dbf_count.put( currownr, dbfcounts );
		
		    jivetable.flush();	

		    //
		    //  Delete the record
		    //
		    delete currecord;
		}
	    }
	    catch( const ExceptionObject& y )
	    {
		cerr << y << endl;
		cerr << "Failed to process file " << files[filecount] << endl;
	    }
	}
	//
	//  Ok. Free up al the memory!
	//
	for( uInt j=0; j<nfiles; j++ )
	{
	    if( files[j] )
	    {
		free( files[j] );
	    }
	}
	free( files );
    }
    catch( AIPSExceptionObject x )
    {
	cerr << "AArrggh? " << x.getMesg() << endl;
    }
    
    return;
}
