//
//  Implementation of the Scan-class
//
//  Author:  Harro Verkouter,  7-12-1998
//
//  $Id: Scan.cc,v 1.2 2001/06/22 14:05:22 verkout Exp $
//
#ifndef SCAN_H
#include <vlbi/j2ms/Scan.h>
#endif


#ifndef AIPSEXCEPTIONOBJECT_H
#include <vlbi/Utilities/AIPSExceptionObject.h>
#endif

#ifndef VISIBILITYBUFFER_H
#include <vlbi/j2ms/VisibilityBuffer.h>
#endif

#ifndef AIPS_MEPOCH_H
#include <aips/Measures/MEpoch.h>
#endif

#ifndef AIPS_MVTIME_H
#include <aips/Quanta/MVTime.h>
#endif

#ifndef CORRELATORSETUP_H
#include <vlbi/j2ms/CorrelatorSetup.h>
#endif

#include <iostream.h>


ostream& operator<<( ostream& os, const j2ms::Scan& sref )
{
	MVTime   tim( sref.getStartTime().getValue().getTime() );

    os << "\n\n";
    os << "Scan " << sref.getScanId() << "\n";
    os << "Source   : " << sref.getSourceCode() << "\n";
    os << "Mode     : " << sref.getFrequencyCode() << "\n";
    os << "Starts @ : " << MVTime::Format(MVTime::YMD) << tim << "\n";
//	   << sref.getStartTime().getValue() << "\n";
    os << sref.getCorrelatorSetup() << "\n";
    os << flush;
    return os;
}

namespace j2ms
{

//
//  Create the Scan; only check for 
//
Scan::Scan( const VisibilityBuffer* visbufptr ) :
    itsVisibilityBufferptr( visbufptr )
{
    //
    //  If we don't have a parent-visibilitybuffer: we throw up!
    //
    if( !itsVisibilityBufferptr )
    {
	throw( AipsError("Scan::Scan(): No VisibilityBuffer given!") );
    }
}

//
//  De-reference the Experiment-ptr -> usr has RO-access to the
//  experiment properties
//
const Experiment& Scan::getExperiment( void ) const
{
    return itsVisibilityBufferptr->getExperiment();
}

const VisibilityBuffer& Scan::getVisibilityBuffer( void ) const
{
    return *itsVisibilityBufferptr;
}


Scan::~Scan()
{
    //
    //  NOTE: don't delete the experiment-ptr, we only keep that as a
    //  'reference' to our parent, we don't own it!
    //
}

// end of namespace
};

