//
//  SourceInfo:   a class describing the properties of a source
//
#ifndef SOURCEINFO_H
#define SOURCEINFO_H

#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef AIPS_MDIRECTION_H
#include <aips/Measures/MDirection.h>
#endif

class SourceInfo
{
    //
    // Show this station's info neatly formatted on an outputstream
    //
    friend ostream& operator<<( ostream& os, const SourceInfo& siref );

public:
    //
    //  Create an empty SourceInfo
    //
    SourceInfo();
    
    //
    //  The copy ctor
    //
    SourceInfo( const SourceInfo& other);

    //
    //  Create a 'fully' defined antenna info object. 'code' =
    //  something like 3C83 or basically, the search string.
    //
    SourceInfo( const String& name, const String& code, const MDirection& pos=MDirection() );

    //
    //  Assignment
    //
    SourceInfo& operator=( const SourceInfo& other );

    //
    //  Get access to the private parts of this object:
    //
    const String&         getName() const;
    const String&         getCode() const;
    const MDirection&     getDirection() const;
    
    //
    //  The destructor!
    //
    ~SourceInfo();

private:
    //
    // The position of the source....
    //
    MDirection        itsPosition;

    //
    //  The name of this particular source
    //
    String            itsName;
    
    //
    //  The official code of this source
    //
    String            itsCode;
};

#endif
