//
//  Implementation oif the VEXFrequencyDB - class
//
//  Author: Harro Verkouter, 6-7-1999
//
//  $Id: VEXFrequencyDB.cc,v 1.1.1.1 2000/03/20 15:14:35 verkout Exp $
//
//  $Log: VEXFrequencyDB.cc,v $
//  Revision 1.1.1.1  2000/03/20 15:14:35  verkout
//  HV: imported aips++ implement stuff
//
//
//
#ifndef VEXFREQUENCYDB_H
#include <vlbi/j2ms/VEXFrequencyDB.h>
#endif

#ifndef VEXPLUS_H
#include <prep_job/vexplus.h>
#endif

#ifndef FREQUENCYCONFIG_H
#include <vlbi/j2ms/FrequencyConfig.h>
#endif

#ifndef BASEBAND_H
#include <vlbi/j2ms/BaseBand.h>
#endif


#include <iostream.h>
#include <iomanip.h>
#include <ctype.h>




VEXFrequencyDB::VEXFrequencyDB( const String& vexfilename ) :
    myVexFileName( vexfilename )
{
    VexPlus     vexfile( myVexFileName );
    static bool filenameShown( false );    

    filenameShown = false;
    
    //
    // Attempt to parse and read into mem!
    //
    if( !vexfile.parseVex() )
    {
	char*            firststation;
	String           mode;
	BaseBand         bb;
	FrequencyConfig  fc;
	
	firststation = 0;
 	if( vexfile.N_Stations()>=1 )
	{
	    firststation = vexfile.Station( 0 );
	}

	for( int i=0; firststation && i<vexfile.N_Modes(); i++ )
	{
	    int                     bbcnt;
	    FrequencyConfig::PolMap polarizationmap;


	    FrequencyConfig::initPolMap( polarizationmap );
	    
	    //
	    //  Retrieve the name for mode 'i'
	    //
	    mode = vexfile.Mode( i );

	    if( !mode )
	    {
		if( !filenameShown )
		{
		    cout << "Error in file " << myVexFileName << "\n";
		    filenameShown = true;
		}
		
		cout << "No modename for mode #" << setw(2) << i << "\n" << flush;
		continue;
	    }
	    
	    
	    fc = FrequencyConfig( mode );
	    
	    //
	    //  Ok. Fill all basebands....
	    //
	    for( bbcnt=0; bbcnt<vexfile.N_FreqChans(firststation, mode); bbcnt++ )
	    {
		char*              sidebandtxt( vexfile.SideBand(firststation, mode, bbcnt) );
		char*              polarization( vexfile.Pol_by_Freq( firststation, mode, bbcnt) );
		Double             frequency( vexfile.SkyFreq(firststation, mode, bbcnt)*1.0E6 );
		Double             bandwidth( vexfile.BW(firststation, mode, bbcnt) );
		BaseBand::SideBand sideband;

		if( !sidebandtxt || !polarization )
		{
		    if( !filenameShown )
		    {
			cout << "Error in file " << myVexFileName << "\n";
			filenameShown = true;
		    }

		    //
		    //  Error detected...
		    //
		    cout << "Error in mode " << mode << ": chan_def #" << setw(2) << bbcnt << "\n";
		    if( !sidebandtxt )
		    {
			cout << "> Missing sideband information\n";
		    }
		    if( !polarization )
		    {
			cout << "> Missing polarization information\n";
		    }
		    cout << flush;
		    break;
		}
		

		sideband = BaseBand::stringToSideBand( sidebandtxt );
		
		if( sideband==BaseBand::lower )
		{
		    frequency -= bandwidth;
		}
		
		//
		//  Check if we already have a baseband with the same
		//  frequency/bandwidth/sideband; it could be the
		//  other hand of polarization....
		//
		uInt      cnt;
		BaseBand  tmp(frequency, bandwidth, sideband);
		
		for( cnt=0; cnt<fc.getNrBaseBands(); cnt++ )
		{
		    if( tmp==(*(fc.getBaseBand(cnt))) )
		    {
			break;
		    }
		}
		
		if( cnt==fc.getNrBaseBands() )
		{
		    fc.addBaseBand( tmp );
		}

		//
		//  Check polarization
		//
		if( polarizationmap[0] && (polarizationmap[0]!=toupper(*polarization)) )
		{
		    if( !polarizationmap[1] )
		    {
			polarizationmap[1] = toupper( *polarization );
		    }
		    else if( polarizationmap[1]!=toupper(*polarization) )
		    {
			if( !filenameShown )
			{
			    cout << "Error in file " << myVexFileName << "\n";
			    filenameShown = true;
			}
			cout << "Error in VEXfile -> too many polarizations (>2) for\n"
			     << "station=" << firststation << " and mode=" << mode << endl;
		    }
		}
		else
		{
		    polarizationmap[0] = toupper( *polarization );
		}
	    }

	    if( bbcnt!=vexfile.N_FreqChans(firststation, mode) )
	    {
		continue;
	    }

	    //
	    //  Do not forget to set the polarization map
	    //
	    
	    //
	    //  NOTE: If we have find either L,R or both RL pols, make
	    //  sure that pol0 evaluates to L and pol1 evaluates to
	    //  R....  this ordering is implicit built in in
	    //  correlator-control....
	    //
	    int       left = -1;
	    int       right = -1;
	    uint32    polcnt = 0;
	    for( uint32 tmp=0; tmp<2; tmp++ )
	    {
		if( polarizationmap[tmp] )
		{
		    polcnt++;
		    if( polarizationmap[tmp]=='R' )
		    {
			right = tmp;
		    }
		    else if( polarizationmap[tmp]=='L' )
		    {
			left = tmp;
		    }
		}
	    }

	    //
	    //  Ok. We now know 1) how many polarizations were
	    //  configured and b) if L/R were configured (left and/or
	    //  right have values different than -1...
	    //
	    if( polcnt==1 )
	    {
		if( left!=-1 )
		{
		    FrequencyConfig::initPolMap( polarizationmap );
		    polarizationmap[0] = 'L';
		}
		else if( right!=-1 )
		{
		    FrequencyConfig::initPolMap( polarizationmap );
		    polarizationmap[1] = 'R';
		}
	    }
	    else if( polcnt==2 && (left!=-1 && right!=-1) )
	    {
		FrequencyConfig::initPolMap( polarizationmap );
		polarizationmap[0] = 'L';
		polarizationmap[1] = 'R';
	    }
	    
	    fc.setPolMap( polarizationmap );

	    //
	    //  And add it to the list
	    //
	    myConfigurations.add( fc );
	    /*
	    cout << endl 
		 << "Added frequency config (=mode):" << endl
		 << fc << endl;
	    */
	}
	
    }
    //
    //  Done
    //
    
}


FrequencyConfig VEXFrequencyDB::operator[]( const String& key ) const
{
    FrequencyConfig   retval;
    
    for( uint32 i=0; i<myConfigurations.elements(); i++ )
    {
	if( myConfigurations(i).getCode()==key )
	{
	    retval = myConfigurations(i);
	    break;
	}
    }
    return retval;
}


Bool VEXFrequencyDB::searchFrequencyConfig( FrequencyConfig& fcref, const String& key ) const
{
    Bool     retval( True );
    
    fcref = (*this)[key];
    
    if( fcref.getCode()=="<DEF>" )
    {
	retval = False;
    }
    return retval;
}

//
//  Clean up...
//
VEXFrequencyDB::~VEXFrequencyDB()
{
}
