//
//  Implementation of the VEXSourceDB - class
//
//  Author: Harro Verkouter,   6-7-1999
//
//  $Id: VEXSourceDB.cc,v 1.1.1.1 2000/03/20 15:14:36 verkout Exp $
//
//  $Log: VEXSourceDB.cc,v $
//  Revision 1.1.1.1  2000/03/20 15:14:36  verkout
//  HV: imported aips++ implement stuff
//
//
#ifndef VEXSOURCEDB_H
#include <vlbi/j2ms/VEXSourceDB.h>
#endif

#ifndef VEXPLUS_H
#include <prep_job/vexplus.h>
#endif

#ifndef AIPS_VECTOR_H
#include <aips/Arrays/Vector.h>
#endif


#include <iostream.h>
#include <iomanip.h>



VEXSourceDB::VEXSourceDB( const String& vexfilename ) :
    SourceDB(),
    myVexFileName( vexfilename )
{
    VexPlus     vexfile( myVexFileName );
    static bool filenameShown( false );
    
    filenameShown = false;
    
    //
    //  Attempt to open/parse and read VEX-file!
    //
    if( !vexfile.parseVex() )
    {
	char*             srcname;
	char*             srccode;
	char*             epochstr;
	MDirection        srcdir;
	MVDirection       srcvdir;
	const char *      defepoch( "J2000" );
	MDirection::Types epoch;

	for( int i=0; i<vexfile.N_Sources(); i++ )
	{
	    srcname  = vexfile.SourceName( i );
	    srccode  = vexfile.IAUName( i );
	    epochstr = vexfile.RefEpoch( i );

	    if( !srcname || !epochstr )
	    {
		if( !filenameShown )
		{
		    cout << "Error in file " << myVexFileName << "\n";
		    filenameShown = true;
		}
		cout << "Source #" << setw(2) << i << ":\n";
		if( !srcname )
		{
		    cout << "> Missing sourcename\n";
		}
		if( !epochstr )
		{
		    epochstr = defepoch;
		    cout << "> Missing epoch, defaulting to " << epochstr << "\n";
		}
		cout << flush;
		
		//
		//  If the sourcename is missing, then we're in trouble!
		//
		if( !srcname )
		{
		    continue;
		}
	    }
	    
	    //
	    //  SOURCE-code is optional; defaults to sourcename
	    //
	    if( !srccode )
	    {
		srccode = srcname;
		//cout << "> Missing source-code (IAU-name), defaulting to "
		//<< ((srccode!=0)?(srccode):("<NULL>")) << "\n";
	    }

	    if( !(MDirection::getType(epoch, epochstr)) )
	    {
		epoch = MDirection::J2000;
	    }
	    
	    srcvdir = MVDirection( vexfile.Source_RA(i), vexfile.Source_Dec(i) );
	    srcdir  = MDirection( srcvdir, epoch );
	    
	    mySourceList.add( SourceInfo(srcname, srccode, srcdir) );
	}
    }
    //
    //  Done!
    //
}

//
//  Look up stuff!
//
SourceInfo VEXSourceDB::operator[]( const String& key ) const
{
    SourceInfo   retval;
    
    for( uint32 i=0; i<mySourceList.elements(); i++ )
    {
	if( mySourceList(i).getName()==key )
	{
	    retval = mySourceList(i);
	    break;
	}
    }
    return retval;
}

Bool VEXSourceDB::searchSource( SourceInfo& airef, const String& key ) const
{
    Bool   retval( True );
    
    airef = (*this)[ key ];
    
    if( airef.getName()=="" )
    {
	retval = False;
    }
    return retval;
}

//
//  Clean up...
//
VEXSourceDB::~VEXSourceDB()
{
}
