//
//  VEXperiment - implementation of the VEXperiment class
//
//  Author:  Harro Verkouter,  5-7-1999
//
//
//  $Id: VEXperiment.cc,v 1.1.1.1 2000/03/20 15:14:36 verkout Exp $
//
//  $Log: VEXperiment.cc,v $
//  Revision 1.1.1.1  2000/03/20 15:14:36  verkout
//  HV: imported aips++ implement stuff
//
//
#ifndef VEXPERIMENT_H
#include <vlbi/j2ms/VEXperiment.h>
#endif

//
//  VEX-stuff
//
#ifndef VEXPLUS_H
#include <prep_job/vexplus.h>
#endif

//
//  Exception stuff found in -lexception!
//
#ifndef EXCEPTIONOBJECT_H
#include <ExceptionObject/ExceptionObject.h>
#endif

#ifndef AIPS_EXCEPTIONS_H
#include <aips/Exceptions.h>
#endif

#ifndef VEXSOURCEDB_H
#include <vlbi/j2ms/VEXSourceDB.h>
#endif

#ifndef VEXANTENNADB_H
#include <vlbi/j2ms/VEXAntennaDB.h>
#endif

#ifndef VEXFREQUENCYDB_H
#include <vlbi/j2ms/VEXFrequencyDB.h>
#endif

//#ifndef COFSCAN_H
//#include <vlbi/j2ms/COFScan.h>
//#endif
#ifndef COFVISIBILITYBUFFER_H
#include <vlbi/j2ms/COFVisibilityBuffer.h>
#endif


#include <libgen.h>


VEXperiment::VEXperiment( const String& exprootdir ) :
    Experiment( exprootdir ),
    myVexFileptr( 0 ),
    mySourceDBptr( 0 ),
    myAntennaDBptr( 0 ),
    myFrequencyDBptr( 0 ),
    myDestinationDomain( Visibility::frequency )
{
    int        result;
    char       buffer[ 1024 ];
    char       buffer2[ 1024 ];
    char*      basenameptr;
    String     vexfilename;
    ostrstream strm( buffer2, sizeof(buffer2) );
    
    //
    //  Get the directory we're in (this is supposed to be the
    //  epxperiment root-dir) and strip the experiment name from it
    //  (so we can form the name of the .vix file we need to open!
    //
    strcpy( buffer, this->getRootDir() );
    
    if( (basenameptr=basename(buffer))==0 )
    {
	throw( EXCEPTION("VEXperiment - failed to get basename for experimentdirectory: "
			 +this->getRootDir()) );
    }

    //
    // Ok. Let's attempt to open the VEX-file
    //
    strm << this->getRootDir() << "/" << basenameptr << ".vix" << ends;
    vexfilename = String( strm.str() );
    
    myVexFileptr = new VexPlus( vexfilename );
    
    if( (result=myVexFileptr->parseVex())<0 )
    {
	char       errorbuf[ 1024 ];
	ostrstream errstrm( errorbuf, sizeof(errorbuf) );
	
	errstrm << "VEXperiment - ";
	
	switch( result )
	{
	    case -1:
		errstrm << "could not open file " << vexfilename;
		break;
		
	    case -2:
		errstrm << "error parsing vex-file " << vexfilename << " to memory";
		break;
		
	    default:
		errstrm << "Unknown error?! Check VexPlus::parseVex() for error return codes!!";
		break;
	}
	errstrm << ends;
	
	throw( EXCEPTION(errstrm.str()) );
    }

    //
    //  Ok. Vex-file parsed to memory Ok. Now look for the *official*
    //  experiment-code.
    //
    //  NOTE: we don't need the basenameptr anymore; we've got the
    //  VEXfilename in mem by name (it's held in 'vexfilename' )
    //
    if( (basenameptr=myVexFileptr->ExperName())==0 )
    {
	char       errorbuf[ 1024 ];
	ostrstream errstrm( errorbuf, sizeof(errorbuf) );
	
	errstrm << "VEXperiment - Failed to get ExperimentCode from VEX-file "
		<< vexfilename << ends;
	
	throw( EXCEPTION(errstrm.str()) );
    }

    myCode = basenameptr;

    //
    //  Form the MS-name
    //
    myMSname = this->getRootDir()+"/"+myCode+".ms";
    
    //
    //  Fill in the antenna/source list...
    //
    myAntennas = VEXperiment::getAntennaList( *myVexFileptr );
    mySources  = VEXperiment::getSourceList( *myVexFileptr );
    
    //
    //  Form the databases
    //
    mySourceDBptr    = new VEXSourceDB( vexfilename );
    myAntennaDBptr   = new VEXAntennaDB( vexfilename );
    myFrequencyDBptr = new VEXFrequencyDB( vexfilename );

    if( !myFrequencyDBptr )
    {
	throw( AipsError("VEXperiment "+myCode+": No frequency info!") );
    }
    
    
    //
    //  And we're all set?
    //
}


const String& VEXperiment::getCode( void ) const
{
    return myCode;
}

const String& VEXperiment::getMSname( void ) const
{
    return myMSname;
}

void VEXperiment::setMSname( const String& newmsname )
{
    myMSname = newmsname;
    return;
}


Visibility::domain VEXperiment::getDestinationDomain( void ) const
{
    return myDestinationDomain;
}

Bool VEXperiment::setDestinationDomain( Visibility::domain newoutputdomain )
{
    myDestinationDomain = newoutputdomain;
    cout << "VEXperiment " << myCode << " - data will now be written in the "
	 << myDestinationDomain << " domain.\n" << flush;
    return True;
}

const Vector<String>& VEXperiment::getAntennas( void ) const
{
    return myAntennas;
}

const Vector<String>& VEXperiment::getSources( void ) const
{
    return mySources;
}

const FrequencyDB& VEXperiment::getFrequencyDB( void ) const
{
    return *myFrequencyDBptr;
}


Bool VEXperiment::hasAntennaDB( void ) const
{
    return ToBool( myAntennaDBptr!=0 );
}

const AntennaDB& VEXperiment::getAntennaDB( void ) const
{
    if( !myAntennaDBptr )
    {
	throw( AipsError("VEXperiment "+myCode+": attempt access to non-existant AntennaDB") );
    }
    return *myAntennaDBptr;
}


Bool VEXperiment::hasSourceDB( void ) const
{
    return ToBool( mySourceDBptr!=0 );
}

const SourceDB& VEXperiment::getSourceDB( void ) const
{
    if( !mySourceDBptr )
    {
	throw( AipsError("VEXperiment "+myCode+": attempt access to non-existant SourceDB") );
    }
    
    return *mySourceDBptr;
}

/*
Scan* VEXperiment::getScan( const String& scanid )
{
    Scan*   retval( 0 );

    //
    //  Attempt the COFScan...
    //
    try
    {
	uint32   jobid;
	uint32   subjobid;
	
	//
	//  Parse the string; it should be in the form:
	//
	//  <jobid>/<subjobid>
	//
	if( sscanf((const char*)scanid, "%ld/%ld", &jobid, &subjobid)==2 )
	{
	    retval = new COFScan( jobid, subjobid, this);
	}
    }
    catch( const ExceptionObject& y )
    {
	//
	//  Thrown by COF-classes
	//
	cout << "VEXperiment::getScan: " << y.getText() << endl;
	retval = 0;
    }
    return retval;
}
*/

VisibilityBuffer* VEXperiment::getVisibilityBuffer( const String& visbufname )
{
    VisibilityBuffer*   retval( 0 );

    //
    //  Attempt the COFVisibilityBuffer...
    //
    try
    {
	uint32   jobid;
	uint32   subjobid;
	
	//
	//  Parse the string; it should be in the form:
	//
	//  <jobid>/<subjobid>
	//
	if( sscanf((const char*)visbufname, "%ld/%ld", &jobid, &subjobid)==2 )
	{
	    retval = new COFVisibilityBuffer(jobid, subjobid, this);
	}
    }
    catch( const ExceptionObject& y )
    {
	//
	//  Thrown by COF-classes
	//
	cout << "VEXperiment::getVisibilityBuffer: " << y.getText() << endl;
	retval = 0;
    }
    return retval;
}



Bool VEXperiment::hasVernierDelay( void ) const
{
    return True;
}




//
//  Delete allocated resources
//
VEXperiment::~VEXperiment()
{
    delete myVexFileptr;
    
    delete mySourceDBptr;
    delete myAntennaDBptr;
    delete myFrequencyDBptr;
}


//
//  The private methods
//
const Vector<String>& VEXperiment::getAntennaList( const VexPlus& vpref )
{
    //
    //  The return value
    //
    static Vector<String>   antennas;
    
    //
    //  Automatic variable(s)
    //
    int                     cnt;
    
    antennas.resize( vpref.N_Stations() );
    
    for( cnt=0; cnt<vpref.N_Stations(); cnt++ )
    {
	antennas( cnt ) = vpref.Station( cnt );
    }
    return antennas;
}

const Vector<String>& VEXperiment::getSourceList( const VexPlus& vpref )
{
    //
    //  The return value
    //
    static Vector<String>   sources;
    
    //
    //  Automatic variable(s)
    //
    int                     cnt;
    
    sources.resize( vpref.N_Sources() );
    
    for( cnt=0; cnt<vpref.N_Sources(); cnt++ )
    {
	sources( cnt ) = vpref.SourceName( cnt );
    }
    return sources;
}
