//
//  VEXperiment - a specialization from Experiment; reads it's values
//  from a VEX-file
//
//
//  Author:   Harro Verkouter,   05-07-1999
//
//
//   $Id: VEXperiment.h,v 1.1.1.1 2000/03/20 15:14:36 verkout Exp $
//
//
//   $Log: VEXperiment.h,v $
//   Revision 1.1.1.1  2000/03/20 15:14:36  verkout
//   HV: imported aips++ implement stuff
//
//
#ifndef VEXPERIMENT_H
#define VEXPERIMENT_H


//
//  This is our baseclass so we'll need it
//
#ifndef EXPERIMENT_H
#include <vlbi/j2ms/Experiment.h>
#endif

#ifndef VISIBILITY_H
#include <vlbi/j2ms/Visibility.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef AIPS_VECTOR_H
#include <aips/Arrays/Vector.h>
#endif

//
//  Forward declarations
//
class VexPlus;
class VEXSourceDB;
class VEXAntennaDB;
class VEXFrequencyDB;




class VEXperiment :
    public Experiment
{
public:
    //
    //  Create a VEXperiment
    //
    VEXperiment( const String& exprootdir="" );
    
    //
    //  Provide access to the properties
    //
    virtual const String&          getCode( void ) const;

    virtual const String&          getMSname( void ) const;
    virtual void                   setMSname( const String& newmsname );

    virtual Visibility::domain     getDestinationDomain( void ) const;
    virtual Bool                   setDestinationDomain( Visibility::domain newoutputdomain );

    virtual const Vector<String>&  getAntennas( void ) const;
    virtual const Vector<String>&  getSources( void ) const;

    //
    // The experiment must have its own frequency DB!
    //
    virtual const FrequencyDB&     getFrequencyDB( void ) const;

    //
    // The user MAY have specified an antennadatabase.
    //
    // NOTE: attempt to get an AntennaDB whilst none was configure
    // will throw an AipsError!!!!
    //
    virtual Bool                   hasAntennaDB( void ) const;
    virtual const AntennaDB&       getAntennaDB( void ) const;

    //
    //  Also, the user MAY have specified a source DB
    //
    virtual Bool                   hasSourceDB( void ) const;
    virtual const SourceDB&        getSourceDB( void ) const;

    //
    //  VisibilityBuffer's are a part of the experiment, therefore we should ask
    //  the IniExperiment if it would be so kind as to create the
    //  indicated buffer. If it fails to create the buffer, a null-pointer
    //  is returned, otherwise a pointer to the newly created VisibilityBuffer is
    //  returned! 
    //
    //  NOTE: the caller is responsible for deleting the
    //  resources!!!!!!
    //
    virtual VisibilityBuffer*      getVisibilityBuffer( const String& visbufname );

    //
    //  Tells us whether or not the Vernier delay was applied.
    //
    virtual Bool                   hasVernierDelay( void ) const;
    
    //
    //  Delete the resources allocated by this object
    //
    virtual ~VEXperiment();
    
private:
    //
    //  The VEXperiment's private parts
    //
    String             myCode;
    String             myMSname;
    VexPlus*           myVexFileptr;
    Vector<String>     myAntennas;
    Vector<String>     mySources;
    
    //
    //  databases
    //
    VEXSourceDB*       mySourceDBptr;
    VEXAntennaDB*      myAntennaDBptr;
    VEXFrequencyDB*    myFrequencyDBptr;

    //
    //  In what domain should the data be written?
    //
    Visibility::domain myDestinationDomain;
    

    //
    //  Static functions; merely helper stuff. They compile a vector
    //  of strings of the indicated type from the VexFile that you
    //  pass, based on the contents of that file.
    //
    //  NOTE: These functions return a reference to a static Vector
    //  object whose contents may change from function-call to
    //  function-call. Therefore, use this return value as input for
    //  an assignment or copy to your local data-area to ensure proper
    //  operation of your program.
    //
    static const Vector<String>& getAntennaList( const VexPlus& vpref );
    static const Vector<String>& getSourceList( const VexPlus& vpref );    

    //
    //  It's safest to dis-allow these...
    //
    VEXperiment( const VEXperiment& );
    VEXperiment& operator=( const VEXperiment& );
};




#endif
