//
//
//  Implementation of the ms2uvfitsConverter class
//
//
//  Harro Verkouter 11-3-1998
//
//     $Id: ms2uvfitsConverter.cc,v 1.2 2000/08/24 13:56:09 verkout Exp $

//
//  System includes
//
#include <iostream.h>

//
//  Local includes...
//
#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef AIPS_MEASUREMENTSET_H
#include <aips/MeasurementSets/MeasurementSet.h>
#endif

#ifndef HDUOBJECT_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/HDUObject.h>
#endif

#ifndef AIPS_FITS_IO_H
#include <aips/FITS/fitsio.h>
#endif

#ifndef AIPS_FITS_TABLE_H
#include <trial/FITS/FITSTable.h>
#endif

#ifndef MSCONVERSION_H_INC
#include <vlbi/ms2uvfitsimpl/MSConversionInfo.h>
#endif

#ifndef FILEFORMATCONVERTER_H_INC
#include <vlbi/ms2uvfitsimpl/Converters/FileFormatConverter.h>
#endif

#ifndef MS2UVFITSCONVERTER_H_INC
#include <vlbi/ms2uvfitsimpl/Converters/ms2uvfitsConverter.h>
#endif

#ifndef HDUGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/hduGenerator.h>
#endif

#ifndef ANTABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/antabGenerator.h>
#endif

#ifndef MAINGROUPGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/maingroupGenerator.h>
#endif

#ifndef ASCIITABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/asciitabGenerator.h>
#endif

#ifndef FQTABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/fqtabGenerator.h>
#endif

#ifndef TYTABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/tytabGenerator.h>
#endif

#ifndef GCTABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/gctabGenerator.h>
#endif

#ifndef SUTABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/sutabGenerator.h>
#endif

#ifndef IDIHDRGENERATOR_H
#include <vlbi/ms2uvfitsimpl/Generators/idihdrGenerator.h>
#endif

#ifndef IDISOURCEGENERATOR_H
#include <vlbi/ms2uvfitsimpl/Generators/idisourceGenerator.h>
#endif

#ifndef IDIFREQUENCYGENERATOR_H
#include <vlbi/ms2uvfitsimpl/Generators/idifrequencyGenerator.h>
#endif

#ifndef IDIANTENNAGENERATOR_H
#include <vlbi/ms2uvfitsimpl/Generators/idiantennaGenerator.h>
#endif

#ifndef UVTABLEGENERATOR_H
#include <vlbi/ms2uvfitsimpl/Generators/uvtableGenerator.h>
#endif

#ifndef ARRAYGEOMGENERATOR_H
#include <vlbi/ms2uvfitsimpl/Generators/arraygeomGenerator.h>
#endif

#ifndef AIPS_REGULARFILEIO_H
#include <aips/IO/RegularFileIO.h>
#endif

#ifndef AIPS_BLOCK_H
#include <aips/Containers/Block.h>
#endif

#ifndef LINEARCONTAINER_H_INC
#include <vlbi/tmplsrc/LinearContainer.h>
#endif

#ifndef AIPS_SIMORDMAP_H
#include <aips/Containers/SimOrdMap.h>
#endif

#ifndef EXCEPTIONOBJECT_H
#include <vlbi/Utilities/AIPSExceptionObject.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

//
// Need this for the default split function
//
#ifndef DEFSPLIT_H
#include <vlbi/hvplugin/defSplit.h>
#endif

#include <aips/Tables/ExprNode.h>


ostream& operator<<( ostream& os, const ms2uvfitsConverter::generator_t& genref )
{
    switch( genref )
    {
	case MS2UVF::mainprimarygroup:
	    os << "main table -> FITS primary group";
	    break;
	    
	case MS2UVF::antennabinary:
	    os << "array table -> FITS binary AN table";
	    break;
	    
	case MS2UVF::antennaascii:
	    os << "array table -> FITS ASCII AN table";
	    break;
	    
	case MS2UVF::freqbinary:
	    os << "spectral window table -> FITS binary FQ table";
	    break;
	    
	case MS2UVF::gaincurvebinary:
	    os << "syscal table -> FITS binary GC table";
	    break;
	    
	case MS2UVF::tsysbinary:
	    os << "syscal table -> FITS binary TY table";
	    break;
	    
	case MS2UVF::sourcebinary:
	    os << "source+field table -> FITS binary SU table";
	    break;

	case MS2UVF::idiheader:
	    os << "<nothing> -> standard FITS-IDI Header";
	    break;
	    
	case MS2UVF::uvtable:
	    os << "main table -> FITS UV-Binary table";
	    break;

	case MS2UVF::arraygeometry:
	    os << "array table -> FITS-IDI array_geometry table";
	    break;

	case MS2UVF::idisource:
	    os << "source table -> FITS-IDI source table";
	    break;
	    
	case MS2UVF::idifrequency:
	    os << "spectral_window table -> FITS-IDI frequency table";
	    break;
	    
	case MS2UVF::idiantenna:
	    os << "array + antenna table -> FITS-IDI antenna table";
	    break;
	    
	default:
	    os << "<unknown generator>";
	    break;
    }
    return os;
}



//
//  Hmmmh... In the default ctor, we will set up a list (or vector,
//  for that matter) of pointers to hdu generators. All generators in
//  this list will be asked to generate a HDU from the
//  MSConversionInfo object to be translated. So if we want to add a
//  new binary table extension, just write the code that generates a
//  HDU from the MSConversionInfo and add it here to the list!
//
ms2uvfitsConverter::ms2uvfitsConverter() :
    FileFormatConverter(),
    itsEnumToGeneratorMapptr( new SimpleOrderedMap<ms2uvfitsConverter::_generator_t,hduGenerator*>((hduGenerator*)0) ),
    itsFragmentationFn( defSplit )
{
    //
    //  Build the list of hdu's to generate!
    //
    itsAvailableGens.resize( 12 );

    //
    //  First, set everything to 0
    //
    itsAvailableGens=(hduGenerator*)0;

    //
    //  Initialize the pointers to what we want to do
    //
    itsAvailableGens(0)= new maingroupGenerator;
    itsAvailableGens(1)= new antabGenerator;
    itsAvailableGens(2)= new fqtabGenerator;
    itsAvailableGens(3)= new tytabGenerator;
    itsAvailableGens(4)= new gctabGenerator;
    itsAvailableGens(5)= new sutabGenerator;
    itsAvailableGens(6)= new idihdrGenerator;
    itsAvailableGens(7)= new uvtableGenerator;
    itsAvailableGens(8)= new arraygeomGenerator;
    itsAvailableGens(9)= new idifrequencyGenerator;
    itsAvailableGens(10)= new idisourceGenerator;
    itsAvailableGens(11)= new idiantennaGenerator;

    //
    //  Now set up the mapping...
    //
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::mainprimarygroup, itsAvailableGens(0) );
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::antennabinary, itsAvailableGens(1) );
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::freqbinary, itsAvailableGens(2) );
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::gaincurvebinary, itsAvailableGens(4) );
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::tsysbinary, itsAvailableGens(3) );
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::sourcebinary, itsAvailableGens(5) );
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::idiheader, itsAvailableGens(6) );
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::uvtable, itsAvailableGens(7) );
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::arraygeometry, itsAvailableGens(8) );
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::idifrequency, itsAvailableGens(9) );
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::idisource, itsAvailableGens(10) );
    itsEnumToGeneratorMapptr->define( ms2uvfitsConverter::idiantenna, itsAvailableGens(11) );
}


//
//  This function really converts the MS into a UVFITS file....
//
Bool ms2uvfitsConverter::convert( const String& fitsfilename, const String& MSfilename ) const
{
    Bool    retval( False );
    
    if( !itsFragmentationFn )
    {
	cout << "Huh? No fragmentation ptr?! Internal state disrupted!" << endl;
	return retval;
    }
    

    //
    //  Now we face the task of converting the named MS to the named
    //  UVFITS-file...
    //
    try
    {
        cout << "Start conversion attempt.." << endl;

	//
	//  Ok. Now we're facing the task of checking the MS and
	//  eventually splitting it up and then process the separate
	//  parts
	//
	MS*                 msptr;
	char                buffer[ 1024 ];
	uInt                nrpieces;
	ostrstream          strm( buffer, sizeof(buffer) );
	MSConversionInfo**  pieces;
	
	//pieces = MS2UVF::fragmentMS( MSfilename, nrpieces );
	msptr = new MS( MSfilename, Table::Old );
	
	pieces = itsFragmentationFn( *msptr, nrpieces );
	
	//
	//  Reset visibility counter.... brrrr what a SHITTY soln but
	//  hope it works...
	//
	MSConversionInfo::lastVisibility = 0;
	
	for( uInt piececnt=0; piececnt<nrpieces; piececnt++ )
	{
	    //
	    //  Form the current fitsfilename
	    //
	    strm.seekp( 0 );
	    strm << fitsfilename;

	    if( nrpieces>1 )
	    {
		strm << piececnt+1;
	    }
	    strm << ends;

	    //
	    //  Local conversion variables
	    //
	    Int              nrhdus2generate;
	    ByteIO*          ptr2fitsoutput;
	    
	    //
	    //  Ok. First things first: try to create fits-file
	    //
	    ptr2fitsoutput=new RegularFileIO( RegularFile(strm.str()), ByteIO::NewNoReplace );
	    
	    //
	    //  If the fits-file could be created ok, we could try to open
	    //  the MS.
	    //
	    MSConversionInfo& thisconversion( *(pieces[piececnt]) );


	    //
	    //  Now then. For a valid uvfits-file, we need to generate  HDU's from the MS
	    //
	    //  So what we presumably do, is to keep a list of pointers to
	    //  hduGenerators, and execute each of these!
	    //
	    itshduGenerators.shape( nrhdus2generate );
        
	    cout << endl;
	    cout << "--------------------------------" << endl;
	    cout << "FITS HDU's will be generated...." << endl;
	    cout << "Local time: " << Time() << endl;
	    cout << "Data will be written to: " << strm.str() << endl;
	    
	    
	    for( Int i=0; i<nrhdus2generate; i++ )
	    {
		//
		//  The pointer to the produced HDU (if succesfull)
		//
		Vector<HDUObject*>    result;

		if( !itshduGenerators(i) )
		{
		    cout << "No hdu-generator found for HDU #" << i+1 << endl;
		    continue;
		}
		
		Int      nrhdus;
                
		result=itshduGenerators(i)->generate( thisconversion );
		result.shape( nrhdus );
                
                if( !nrhdus )
		{
		    cout << "Generator[" << i+1 << "] did not generate any HDU's...." << endl;
		    continue;
		}

		//
		//  Loop over the generated HDU's
		//
		for( Int hducnt=0; hducnt<nrhdus; hducnt++ )
		{
		    if( !result(hducnt) )
		    {
			cout.form( "Generator #%2d failed to generate HDU #%2d (out of %d)",
				   i+1, hducnt+1, nrhdus) << endl;
			continue;
		    }
		    //
		    //  Write the result to the resulting FITS-output
		    //  file
		    //
		    (*ptr2fitsoutput) << (*(result(hducnt)));
		    cout << "Generator[" << i << "]: HDU " << hducnt << " written" << endl;
		}
		
		//
		//  Ok. The hdu's have been written, now we can delete them!
		//
		for( Int hducnt1=0; hducnt1<nrhdus; hducnt1++ )
		{
		    delete result(hducnt1);
		}
	    }
	    cout << "Ends at " << Time() << endl;
	    cout << "----------------------------------------" << endl << endl;

	    //
	    //  We don't need those anymore!
	    //
	    delete ptr2fitsoutput;
        }

	//
	//  Delete all the conversion info objects...
	//
	for( uInt pcnt=0; pcnt<nrpieces; pcnt++ )
	{
	    delete pieces[ pcnt ];
	}
	delete [] pieces;

	//
	//  Don't need the MS anymore...
	//
	delete msptr;
	
        //
        //  Prepare returncode...
        //
        retval=True;
    }
    catch( AIPSExceptionObject x )
    {
        cout << "Darn. Error: " << x.getMesg() << endl;
	cout << "(" << x.thrownFile() << ":" << x.thrownLine() << ")\n" << flush;
    }
    end_try;
    
    return retval;
}

//
//  Configure ourselves such that the conversion is run according to
//  the user's wishes!
//
//Bool ms2uvfitsConverter::setConversion( const Block<generator_t>& hdus )
Bool ms2uvfitsConverter::setConversion( const LinearContainer<generator_t>& hdus )
{
    //
    // Size the destination vector of pointers according to the user's wishes
    //
    //itshduGenerators.resize( hdus.nelements() , False );
    itshduGenerators.resize( hdus.elements() , False );
    
    //
    //  Now map the enum's to pointers to generators!
    //
    //for( uInt i=0; i<hdus.nelements(); i++ )
    cout << endl;
    cout << "Next conversion will produce, in the following order:" << endl;
    
    for( uInt i=0; i<hdus.elements(); i++ )
    {
	itshduGenerators(i)=(*itsEnumToGeneratorMapptr)( hdus[i] );
	cout.form("%3d: ",i) << hdus[i] << endl;
    }
    return True;
}


Bool ms2uvfitsConverter::setFragmenter( fragmentfn fnptr )
{
    Bool    retval( False );
    
    if( fnptr )
    {
	itsFragmentationFn = fnptr;
	retval = True;
    }
    return retval;
}


//
//  Destruct the conversion engine...(i.e. delete all the MStoHDU
//  converters)
//
ms2uvfitsConverter::~ms2uvfitsConverter()
{
    //
    //  Delete all hdugenerators.
    //
    Int    nrgenerators;
    
    itsAvailableGens.shape( nrgenerators );
    
    for( Int i=0; i<nrgenerators; i++ )
    {
        delete itsAvailableGens(i);
    }

    //
    //  Delete the map!
    //
    delete itsEnumToGeneratorMapptr;
    
    //
    //  That's all folks!
    //
}


MSConversionInfo** ms2uvfitsConverter::fragmentMS( const String& msfilename, uInt& npieces )
{
    MS*                   thems( 0 );
    uInt                  nrowperpiece( 2000 );
    MSConversionInfo**    retval( 0 );
    
    thems = new MeasurementSet( msfilename, Table::Old );
    
    if( !thems )
    {
	return retval;
    }

    npieces = 1;
    retval = new MSConversionInfo*[ npieces ];
    retval[ 0 ] = new MSConversionInfo( *thems, 0, npieces );

    delete thems;
    return retval;

    //
    //  Ok. Produce some subtables...
    //

    /*
    uInt   nrrows( thems->nrow() );
    
    //
    //  for testing purposes: make subtables per 6000 rows....
    //
    npieces = (nrrows/nrowperpiece);
    if( (nrrows%nrowperpiece)!=0 )
    {
	npieces++;
    }
    
    retval = new MSConversionInfo*[ npieces ];
    
    cout << "Breaking " << msfilename << " up into " << npieces << " pieces.." << endl;

    for( uInt p=0; p<npieces; p++ )
    {
	Table    tmp( (*thems)( (thems->nodeRownr()>=Int(p*nrowperpiece)) && 
				(thems->nodeRownr()<Int((p+1)*nrowperpiece)) ) );
	
	retval[ p ] = new MSConversionInfo( tmp, p, npieces );
    }

    delete thems;

    return retval;
    */
}
