//
// Class:
//
//       uvfits2msConverter
//
//
// Purpose:
//
//       To convert a FITS-file into an AIPS++ MeasurementSet (and to
//       boldly go where no man has gone before). This class is a
//       conversion engine. The engine is passed the names of the
//       source FITSfilename and the name of the output MS.
//
//       The object features one public method: convert. This function
//       does all the work of the conversion. It attempts to create
//       all tables that are present in the (private) list of
//       tableGenerators. 
//
//
//
//  Author:
//
//     Harro Verkouter      23-3-1998
//
//  $Id: uvfits2msConverter.h,v 1.1.1.1 2000/03/20 15:16:48 verkout Exp $
//
#ifndef UVFITS2MSCONVERTER_H_INC
#define UVFITS2MSCONVERTER_H_INC

#ifndef FILEFORMATCONVERTER_H_INC
#include <vlbi/ms2uvfitsimpl/Converters/FileFormatConverter.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef AIPS_VECTOR_H
#include <aips/Arrays/Vector.h>
#endif

#ifndef TABLEGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/tableGenerator.h>
#endif

//
//  Forward declarations
//
class MeasurementSet;


//
//  The actual conversion class
//
class uvfits2msConverter :
    public virtual FileFormatConverter
{
public:
    //
    //  Default conversion engine: the only way to go (yet)
    //
    uvfits2msConverter();
    
    //
    //  The convert method! Do the conversion!
    //
    virtual Bool convert( const String& msfilename, const String& fitsfilename ) const;

    //
    //  Destruct the conversion engine
    //
    ~uvfits2msConverter();
    
private:
    //
    //  The private datamembers
    //
    
    //
    //  Here we keep a list of tableGenerators: this is the list of
    //  table the conversion engine should try to generate from the
    //  FITSConversionInfo object.
    //
    Vector<tableGenerator*>  itstableGenerators;


    //
    //  The private methods!
    //

    //
    //  Set up a new MS. The return value is either a valid ptr or an
    //  exception is thrown!
    //
    MeasurementSet*          setupMS( const String& msfilename ) const;

    //
    //  Conversion engines cannot be copied or assigned to.
    //
    uvfits2msConverter( const uvfits2msConverter& );
    uvfits2msConverter& operator=( const uvfits2msConverter& );
};




#endif
