//
//  Implementation of the global ostream<< operator for recordinterface-classes
//
//
//  Author:
//
//         Harro Verkouter     16-04-1998
//
//
//     $Id: DumpRecord.cc,v 1.1.1.1 2000/03/20 15:16:47 verkout Exp $
//
#ifndef DUMPRECORD_H_INC
#include <vlbi/ms2uvfitsimpl/DumpRecord.h>
#endif

#ifndef AIPS_IPOSITION_H
#include <aips/Lattices/IPosition.h>
#endif

#ifndef AIPS_SLICER_H
#include <aips/Lattices/Slicer.h>
#endif

#include <iomanip.h>


//
//  A local 'global' function:
//
//  Compute th number of elements in a shape, basically it returns the
//  product of all axislengths
//
Int calculateNumberElements( const IPosition& arrayshape )
{
    Int   retval( 0 );
    
    for( Int cnt=0; cnt<arrayshape.nelements(); cnt++ )
    {
        //
        //  If we end up here, we are sure that we can initialize
        //  retval correctly!
        //
        if( !cnt )
        {
            //
            //  First element of the arrayshape: initialize retval!
            //
            retval=arrayshape( cnt );
        }
        else
        {
            //
            //  Now we can safely multiply by the axislength
            //
            retval*=arrayshape( cnt );
        }
    }
    return retval;
}

//
//  Make a maximum index (say max. 'maxelements' elements from the
//  IPosition passed as argument)
//
IPosition makeMaximumIndex( const IPosition arrayshape, Int maxelements )
{
    Int         elementsdone( 0 );
    Int         curdimension( 0 ), maxdimension( arrayshape.nelements() );
    Int         nrelements;
    IPosition   retval( maxdimension, 0 );
    
    //
    //  Loop until we have enough elements!
    //
    while( elementsdone<maxelements && curdimension<maxdimension )
    {
        //
        //  Get the number of elements in the current dimension
        //
        nrelements=arrayshape( curdimension );

        if( nrelements>1 )
        {
            //
            // Check if we still can add all elements to the resulting
            // iposition!
            //
            if( (elementsdone*nrelements)>=maxelements )
            {
                break;
            }
            retval(curdimension)= (nrelements-1);

            //
            //  Update loop variables
            //
            if( elementsdone )
            {
                elementsdone*=nrelements;
            }
            else
            {
                elementsdone=retval(curdimension);
            }
        }
        
        curdimension++;
    }
    
    return retval;
    
}



ostream& operator<<( ostream& osref, const RecordInterface& riref )
{
    Int        nrfields( riref.nfields() );

    //
    //  Limit display of array values to 10 elements
    //
    Int        limit( 20 );
    IPosition  currentshape;
    
    
    //
    //  Ok, show all fields of the record!
    //
    for( Int i=0; i<nrfields; i++ )
    {
        //
        //  First, show general info about the field
        //
        osref << setw(2) << i << ": " << riref.name(i) << " [";
        

        //
        //  Make sure that currentshape is set to an empty IPosition
        //  in order to prevent arrayconformance exceptions to be
        //  thrown
        //
        currentshape.resize( 0, False );
        
        //
        //  Do something, depending on type of the record
        //
        switch( riref.type(i) )
        {
            case TpBool:
                osref << riref.asBool( i );
                break;

            case TpUChar:
                osref << riref.asuChar( i );
                break;

            case TpShort:
                osref << riref.asShort( i );
                break;
                
            case TpInt:
                osref << riref.asInt( i );
                break;
                
            case TpUInt:
                osref << riref.asuInt( i );
                break;
                
            case TpFloat:
                osref << riref.asFloat( i );
                break;
                
            case TpDouble:
                osref << riref.asDouble( i );
                break;
                
            case TpComplex:
                osref << riref.asComplex( i );
                break;
                
            case TpDComplex:
                osref << riref.asDComplex( i );
                break;
                
            case TpString:
                osref << riref.asString( i );
                break;
                
                //
                //  Now follow the arrays.....
                //
                
            case TpArrayBool:
                {
                    Array<Bool>    tmp;
                    
                    tmp.reference( riref.asArrayBool(i) );
                    
                    currentshape=tmp.shape();
                    
		    osref << tmp.ac();
		    
//                     if( calculateNumberElements(currentshape)<limit )
//                     {
//                         osref << tmp.ac();
//                     }
//                     else
//                     {
//                         currentshape=makeMaximumIndex( currentshape, limit );
//                         osref << tmp( IPosition(tmp.ndim(), 0), currentshape ).ac() << "...";
//                     }
                }
                break;

            case TpArrayUChar:
                {
                    Array<uChar>    tmp;
                    
                    tmp.reference( riref.asArrayuChar(i) );
                    
                    currentshape=tmp.shape();

                    osref << tmp.ac();
		    
//                     if( calculateNumberElements(currentshape)<limit )
//                     {
//                         osref << tmp.ac();
//                     }
//                     else
//                     {
//                         currentshape=makeMaximumIndex( currentshape, limit );
//                         osref << tmp( IPosition(tmp.ndim(), 0), currentshape ).ac() << "...";
//                     }
                }
                break;
           
            case TpArrayShort:
                {
                    Array<Short>    tmp;
                    
                    tmp.reference( riref.asArrayShort(i) );
                    
                    currentshape=tmp.shape();

                    osref << tmp.ac();
		    
//                     if( calculateNumberElements(currentshape)<limit )
//                     {
//                         osref << tmp.ac();
//                     }
//                     else
//                     {
//                         currentshape=makeMaximumIndex( currentshape, limit );
//                         osref << tmp( IPosition(tmp.ndim(), 0), currentshape ).ac() << "...";
//                     }
                }
                break;

            case TpArrayInt:
                {
                    Array<Int>    tmp;
                    
                    tmp.reference( riref.asArrayInt(i) );
                    
                    currentshape=tmp.shape();
                    
		    osref << tmp.ac();
		    
//                     if( calculateNumberElements(currentshape)<limit )
//                     {
//                         osref << tmp.ac();
//                     }
//                     else
//                     {
//                         currentshape=makeMaximumIndex( currentshape, limit );
//                         osref << tmp( IPosition(tmp.ndim(), 0), currentshape ).ac() << "...";
//                     }
                }
                break;

            case TpArrayUInt:
                {
                    Array<uInt>    tmp;
                    
                    tmp.reference( riref.asArrayuInt(i) );
                    
                    currentshape=tmp.shape();

		    osref << tmp.ac();
		    
//                     if( calculateNumberElements(currentshape)<limit )
//                     {
//                         osref << tmp.ac();
//                     }
//                     else
//                     {
//                         currentshape=makeMaximumIndex( currentshape, limit );
//                         osref << tmp( IPosition(tmp.ndim(), 0), currentshape ).ac() << "...";
//                     }
                }
                break;

            case TpArrayFloat:
                {
                    Array<Float>    tmp;
                    
                    tmp.reference( riref.asArrayFloat(i) );
                    
                    currentshape=tmp.shape();
		    
		    osref << tmp.ac();
                    
//                     if( calculateNumberElements(currentshape)<limit )
//                     {
//                         osref << tmp.ac();
//                     }
//                     else
//                     {
//                         currentshape=makeMaximumIndex( currentshape, limit );
//                         cout << "Showing: " << currentshape << endl;
                        
//                         osref << tmp( IPosition(tmp.ndim(), 0), currentshape ).ac() << "...";
//                     }
                }
                break;

            case TpArrayDouble:
                {
                    Array<Double>    tmp;
                    
                    tmp.reference( riref.asArrayDouble(i) );
                    
                    currentshape=tmp.shape();
                    
		    osref << tmp.ac();
		    
//                     if( calculateNumberElements(currentshape)<limit )
//                     {
//                         osref << tmp.ac();
//                     }
//                     else
//                     {
//                         currentshape=makeMaximumIndex( currentshape, limit );
//                         osref << tmp( IPosition(tmp.ndim(), 0), currentshape ).ac() << "...";
//                     }
                }
                break;

            case TpArrayComplex:
                {
                    Array<Complex>    tmp;
                    
                    tmp.reference( riref.asArrayComplex(i) );
                    
                    currentshape=tmp.shape();
                    
		    osref << tmp.ac();
		    
//                     if( calculateNumberElements(currentshape)<limit )
//                     {
//                         osref << tmp.ac();
//                     }
//                     else
//                     {
//                         currentshape=makeMaximumIndex( currentshape, limit );
//                         osref << tmp( IPosition(tmp.ndim(), 0), currentshape ).ac() << "...";
//                     }
                }
                break;

            case TpArrayDComplex:
                {
                    Array<DComplex>    tmp;
                    
                    tmp.reference( riref.asArrayDComplex(i) );
                    
                    currentshape=tmp.shape();
                    
		    osref << tmp.ac();
		    
//                     if( calculateNumberElements(currentshape)<limit )
//                     {
//                         osref << tmp.ac();
//                     }
//                     else
//                     {
//                         currentshape=makeMaximumIndex( currentshape, limit );
//                         osref << tmp( IPosition(tmp.ndim(), 0), currentshape ).ac() << "...";
//                     }
                }
                break;

            case TpArrayString:
                {
                    Array<String>    tmp;
                    
                    tmp.reference( riref.asArrayString(i) );
                    
                    currentshape=tmp.shape();
                    
		    osref << tmp.ac();
		    
//                     if( calculateNumberElements(currentshape)<limit )
//                     {
//                         osref << tmp.ac();
//                     }
//                     else
//                     {
//                         currentshape=makeMaximumIndex( currentshape, limit );
//                         osref << tmp( IPosition(tmp.ndim(), 0), currentshape ).ac() << "...";
//                     }
                }
                break;

            default:
                osref << "<Unhandled type>";
                break;
        }

        osref << "] " << endl;
    }

    return osref;
}
