//
//
// Class:
//
//        FITSConversionInfo
//
// Purpose:
//
//       The FITSConversionInfo will faciliate reading/converting a
//       FITS-file. The FITSConversionInfo object hides all specific
//       knowledge about the FITS-file. The object holds all the
//       pointers and information necessary to correctly describe the
//       FITS-file in order to keep these values from the user so as
//       they can not be clobbered!
//
//       The object is more a view of the FITS-file rather than an
//       info-object. The object was designed as to faciliate
//       conversion of FITS-files.
//
//       Upon creation of the object, the FITS-conversion object
//       verifies that the file exists and can be opened. An extra
//       check (to see if it really is a FITS-file that's opened)
//       might be added. The user may extract HeaderDataUnit units or
//       tell the object to scan all HeaderDataUnits present in the
//       file.
//
//       The gathered HeaderDataUnits can be accessed by number or by
//       name. From the HeaderDataUnit objects, information can be
//       extracted.
//
//
//  Author:
//
//       Harro Verkouter     23-3-1998
//
//
//      $Id: FITSConversionInfo.h,v 1.1.1.1 2000/03/20 15:16:47 verkout Exp $
//
#ifndef FITSCONVERSIONINFO_H_INC
#define FITSCONVERSIONINFO_H_INC

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif


class FITSConversionInfo
{
public:
    //
    //  The only valid c'tor for this object is the one taking a
    //  String: the name of the FITS-file we want info about.
    //
    FITSConversionInfo( const String& fitsfilename );
    
    //
    //  get the filename for e.g. displaying purposes
    //
    const String& getFileName( void ) const;

    //
    //  Destruct the object: de-allocate all resources used by this
    //  object!
    //
    ~FITSConversionInfo();
    
private:
    //
    //  The object's private parts
    //

    //
    //  The filename
    //
    String     itsFilename;
    

    //
    //  ConversionInfo objects do not have a default state and they
    //  cannot be duplicated/assigned to!
    //
    FITSConversionInfo();
    FITSConversionInfo( const FITSConversionInfo& );
    FITSConversionInfo& operator=( const FITSConversionInfo& );
};


#endif
