//
//  Class: 
//
//       FITSAsciiColumn
//
//  Purpose:
//
//       Object describes a column in a FITSBinaryTable Object. Does
//       correct formatting on get/put optins.
//
//
//
//  Author:
//
//       Harrro Verkoutrer,    18-05-1998
//
//
//  $Id: FITSAsciiColumn.h,v 1.1.1.1 2000/03/20 15:16:48 verkout Exp $
//
#ifndef FITSASCIICOLUMN_H_INC
#define FITSASCIICOLUMN_H_INC

#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef FITSCOLUMN_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSColumn.h>
#endif

//
//  Forward declarations
//
class FITSAsciiTableHDU;
class FitsKeywordList;


class FITSAsciiColumn : 
    public FITSColumn
{
public:
    //
    // The public interface of the FITSAsciiColumn. For explanation
    // of the three different creators, see FITSColumn.h
    //
    FITSAsciiColumn( const String& type, const String& comment, const String& form, const String& unit );
    
    FITSAsciiColumn( Int number, uLong offset, FitsKeywordList& kwsource, FITSAsciiTableHDU* ptr2parent );
    
    FITSAsciiColumn( Int number, uLong offset, const FITSAsciiColumn& other, FITSAsciiTableHDU* ptr2parent );
    
    //
    //  Via recordfieldid! Default to the first field in the record
    //
    virtual Bool            get( uInt rownr, RecordInterface& recordref, 
                                 const RecordFieldId& fieldidx=0 ) const;
    virtual Bool            put( uInt rownr, const RecordInterface& recordref, 
                                 const RecordFieldId& fieldidx=0 );

    //
    //  We have our own method of translating this into FITSkeywords!
    //
    virtual FitsKeywordList asFITSKeys( void ) const;

    //
    //  Width of this column
    //
    virtual uLong           getWidthInBytes( void ) const;

    //
    //  Destruct the column object!
    //
    ~FITSAsciiColumn();

protected:
    //
    // Any protected functions go here.
    //


private:
    //
    //  The BinaryColumn's specific data
    //

    //
    //  We keep track of the width of this column ourselves
    //
    uLong                 itsTotalWidth;

    //
    //  The width of the field
    //
    uInt                  itsFieldWidth;
    
    //
    //  And the precision (nr of digits after the decimal point)
    //
    uInt                  itsPrecision;
    
    //
    //  Number of digits in exponent
    //
    uInt                  itsExponentPrecision;
    
    //
    //  AsciiColumn private methods
    //

    //
    //  This function checks the 'form' string and from there tries
    //  to determine the datatype of the column and the total column
    //  width. If a nonsupported datatype or invalid form-description
    //  was entered, an exception will be thrown!
    //
    void           CheckFormatAndInit( void );

    //  No default ascii columns possible
    FITSAsciiColumn( );
    FITSAsciiColumn( const FITSAsciiColumn& );
    FITSAsciiColumn& operator=( const FITSAsciiColumn& );
};



#endif
