//
//  Class:
//
//        FITSAxis
//
//
//  Purpose:
//
//        This class wil describe the basic properties of an Axis in
//        the description of an array of data. This class does not
//        have to be templated, since the datatype is kept in the
//        HeaderDataUnit-object. If we add axes to that
//        HeaderDataUnit, they will automatically describe data of the
//        format used in the HeaderDataUnit.
//
//
//
//
//  Author:
//
//        Harro Verkouter     24-3-1998
//
//
//      $Id: FITSAxis.h,v 1.1.1.1 2000/03/20 15:16:48 verkout Exp $
//
#ifndef FITSAXIS_H_INC
#define FITSAXIS_H_INC


#ifndef BASICFITSUTILITY_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/BasicFITSUtility.h>
#endif

#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif


#ifndef AIPS_FITS
#include <aips/FITS/fits.h>
#endif


//
//  Forward declaration
//
class FITSArrayHDU;


//
//  The FITSAxis class...
//
class FITSAxis :
    public BasicFITSUtility
{
public:
    //
    //  The only allowed c'tors are those two:
    //
    //   1) Axis is created from known values
    //
    //   2) Axis is created out of a list of FITSKeywords (intended
    //   use: when a hdu has been read from file, we must extract axes
    //   from the FitsKeywordList
    //

    //
    //  This c'tor enables us to create an axis without really having
    //  to specify all it's details! Intended use: if an axis has to
    //  be added to a HDU, the user doesn't know about the axis number
    //  (and shouldn't as well!!). The HeaderDataUnit keeps track of
    //  its own number of axis!
    //
    FITSAxis( const String& type, const String& comment="", Int length=1, Double refpixel=1.0, 
              Double refvalue=0.0, Double increment=1.0, Double rotangle=0.0 );
    

    FITSAxis( Int axisnr, FitsKeywordList& kwsource, FITSArrayHDU* const parent );

    //
    //  This constructor is a special one: it duplicates the contents
    //  of 'other' into the new object, but sets the axisnumber to
    //  'axisnumber'. This constructor is used by the HeaderDataUnit
    //  objects when axes are being added to them. The HeaderDataUnit
    //  object knows what the current axisnumber is. The user
    //  doesn't. So the user creates an axis (from ctor 1) and tells
    //  the HeaderDataUnit to add this one. The HeaderDataUnit calls
    //  this c'tor because it knows all the details.
    //
    //  Note: one of the parameters is the FITSArrayHDU class. We can
    //  make that assumption since arrayHDUs are the only HDU's who
    //  support axis.
    //
    FITSAxis( Int axisnr, const FITSAxis& other, FITSArrayHDU* parent );

    //
    //  Translate the contents of this axis to FitsKeywords!
    //
    virtual FitsKeywordList asFITSKeys( void ) const;

    //
    //  Return the length of this axis
    //
    Int                     getAxisLength( void ) const;
    void                    setAxisLength( Int newlength );

    //
    //  Return the axis-parameters, naming convention reselmbles FITS
    //  nomenclature.
    //
    Int                     axisnr( void ) const;
    Int                     naxis( void ) const;
    Double                  crpix( void ) const;
    Double                  crval( void ) const;
    Double                  cdelt( void ) const;
    Double                  crota( void ) const;
    const String&           ctype( void ) const;

    //
    //  Destruct the axis
    //
    virtual ~FITSAxis();
    
private:
    //
    //  These values describe the FITS-axis
    //
    
    //
    // The type of this axis
    //
    String    itsAxistype;
    
    //
    //  Comment associated with this axis...
    //
    String    itsComment;

    //
    //  The length of this axis
    //
    Int       itsAxislength;

    //
    //  The reference pixel of this axis
    //
    Double    itsReferencepixel;
    
    //
    //  The reference value for this axis
    //
    Double    itsReferencevalue;
    
    //
    //  Axis increment (per pixel)
    //
    Double    itsIncrement;
    
    //
    //  The rotation  angle
    //
    Double    itsRotationangle;

    //
    //  The number of this axis in the current HDU
    //
    Int       itsAxisnumber;

    //
    //  Private methods
    //

    //
    //  We forbid copying/assigning of axes!
    //
    FITSAxis( const FITSAxis& );
    FITSAxis& operator=( const FITSAxis& );
};

//
//  The global output operator....
//
ostream& operator<<( ostream& osref, const FITSAxis& axisref );

#endif
