//
//  Class:
//
//        FITSBinaryTableHDU
//
//
//  Purpose:
//
//        Specialization of the FITSTableHDU class, representing a
//        binary table in FITS format.
//
//
//  Author:
//
//        Harro Verkouter,   18-05-1998
//
//
//      $Id: FITSBinaryTableHDU.h,v 1.1.1.1 2000/03/20 15:16:48 verkout Exp $  
//
#ifndef FITSBINARYTABLEHDU_H_INC
#define FITSBINARYTABLEHDU_H_INC

//
//  Need this: it's our baseclass
//
#ifndef FITSTABLEHDU_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSTableHDU.h>
#endif

#ifndef AIPS_REGEX_H
#include <aips/Utilities/Regex.h>
#endif

//
//  The class itself
//
class FITSBinaryTableHDU :
    public FITSTableHDU
{
    //
    //  The public interface
    //
public:
    
    //
    //  A Binary table! Create in memory
    //
    FITSBinaryTableHDU( const String& name, uInt version );
    
    //
    //  Supposedly: create it from a disk/tape (a stream, rather)
    //
    FITSBinaryTableHDU( FitsKeywordList& kwlist, ByteIO* ptr2stream );
    

    //
    //  For binary table HDU's we allow adding of binary columns!
    //
    //
    //  Tables do have the possibility of adding columns. FITSTable
    //  objects should implement this function, in order for them to
    //  get the pointer to the parent right!
    //
    //  IMPORTANT NOTE:
    //
    //  The column object that is passed as argument is COPIED before
    //  it is added internally. So any changes one performs on the
    //  local variable are NOT reflected in the internal column
    //  object. The 'getColumn()' methods should be used to get a
    //  reference to the actual column object. Binding information (if
    //  the column was bound to something) is copied. So if you
    //  already did that, there's no need to redo that after the
    //  column object is added to the HDU.
    //
    //  The return value is the actual column number that was assigned
    //  to this column. If less than zero: an error occurred and the
    //  column could not be added!
    //
    Int           addColumn( const FITSBinaryColumn& column2add );

    //
    //  What type of hdu are we?
    //
    virtual HDUObject::hdu_t getHDUtype( void ) const;
    

    //
    // Retrieve columns. If the columns are not found, an exception
    // will be thrown!
    //
    FITSBinaryColumn&   getColumn( const String& name, uInt occurrence=0 ) const;
    FITSBinaryColumn&   getColumn( uInt columnnr ) const;

    //
    //  Return the extension type of this table!
    //
    String              xtension( void ) const;

    //
    //  Return the regex that matches the description of this extname
    //
    static Regex        fitsname( void );

    //
    //  Destruct the HDU!
    //
    ~FITSBinaryTableHDU();

protected:
    //
    //  The protected functions go in here. Like the header keywords!
    //
    virtual FitsKeywordList   getDataDescriptionKeywords( void ) const;
    
private:
    //
    //  The BinaryTable's private parts go here!
    //

    //
    //  Undefined and inaccessible
    //
    FITSBinaryTableHDU();
    FITSBinaryTableHDU( const FITSBinaryTableHDU& );
    FITSBinaryTableHDU& operator=( const FITSBinaryTableHDU& );
};

#endif

