//
//  Implementation of the FITSIDIDHeader class!
//
//
//  Author: Harro Verkouter, 13-09-1999
//
//
//   $Id: FITSIDIHeader.cc,v 1.1.1.1 2000/03/20 15:16:48 verkout Exp $
//
//  $Log: FITSIDIHeader.cc,v $
//  Revision 1.1.1.1  2000/03/20 15:16:48  verkout
//  HV: Imported aips++ implement-stuff
//
//
#ifndef FITSIDIHEADER_H
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSIDIHeader.h>
#endif


FITSIDIHeader::FITSIDIHeader() :
    HDUObject( TpUChar )
{
}

FITSIDIHeader::FITSIDIHeader( FitsKeywordList& kwlist ) :
    HDUObject( kwlist )
{
    FitsKeyword*   keywordptr;
    
    //
    //  Ok. Check if this is an IDI header...
    //
    if( (keywordptr=kwlist(FITS::NAXIS))==0 )
    {
        //
        //  Huh? An array HDU without NAXIS keyword!?!
        //
        throw( AipsError("No NAXIS keyword present in IDI-Header keywordlist!") );
    }
    
    //
    //  Ok. delete this keyword from the kwlist!
    //
    kwlist.del();

    //
    //  Retrieve the axis info!
    //
    Int       i;
    Int       nraxes( 2 );
    Int       valuesneeded[2] = { 777777701, 0 };
    
    for( i=0; i<nraxes; i++ )
    {
        //
        //  Let all the axes get their info from the keywordlist!
        //
	if( (keywordptr=kwlist(FITS::NAXIS, i+1))==0 )
	{
	    break;
	}
	if( keywordptr->asInt()!=valuesneeded[i] )
	{
	    break;
	}
	kwlist.del();
    }

    if( i<nraxes )
    {
	//
	//  Something wrong!!!
	//
	throw( AipsError("FITSIDIHeader: signature in keywordlist not NOT a FITS-IDI one!") );
    }

    //
    //  We're left with all the other keywords!
    //
    this->setExtraKeywords( kwlist );
    
    //
    //  Done!
    //    
}


uLong FITSIDIHeader::getCurrentDataVolumeInBytes( void ) const
{
    return 0;
}

HDUObject::hdu_t FITSIDIHeader::getHDUtype( void ) const
{
    return HDUObject::defaulthdu;
}


FITSIDIHeader::~FITSIDIHeader()
{    
}


//
//
//  The protected section...
//
//
FitsKeywordList FITSIDIHeader::getHeaderKeywords( void ) const
{
    FitsKeywordList   retval;
    
    retval.mk( FITS::SIMPLE, True, "Yep! It's a FITS file..." );
    retval.mk( FITS::BITPIX,    8 );
    return retval;
}

FitsKeywordList FITSIDIHeader::getDataDescriptionKeywords( void ) const
{
    FitsKeywordList   retval;

    retval.mk( FITS::NAXIS,     0 );
    retval.mk( FITS::EXTEND,    True );
    retval.mk( FITS::GROUPS,    True );
    retval.mk( FITS::GCOUNT,    0 );
    retval.mk( FITS::PCOUNT,    0 );
    
    //retval.mk( 1, FITS::NAXIS,  777777701, "Signature code for UV data in table" );
    //retval.mk( 2, FITS::NAXIS,          0, "No data in primary array" );

    return retval;
}
