//
//  Implementation of the FITSImageExtension class
//
//  Author:
//
//         Harro Verkouter,   26-05-1998
//
//
//     $Id: FITSImageExtension.cc,v 1.1.1.1 2000/03/20 15:16:48 verkout Exp $
//
#ifndef FITSIMAGEEXTENSION_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSImageExtension.h>
#endif

#ifndef FITSAXIS_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSAxis.h>
#endif

#ifndef AIPS_RECORDINTERFACE_H
#include <aips/Containers/RecordInterface.h>
#endif

#ifndef FREEFUNCTIONS_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/freefunctions.h>
#endif

#ifndef FITSDATASEGMENT_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSDataSegment.h>
#endif

#ifndef AIPS_RECORDDESC_H
#include <aips/Containers/RecordDesc.h>
#endif

#ifndef EXCEPTIONOBJECT_H
#include <vlbi/Utilities/AIPSExceptionObject.h>
#endif


FITSImageExtension::FITSImageExtension( DataType basicfitstype ) :
    FITSArrayHDU( basicfitstype ),
    itsExtensionName( "BLAH" ),
    itsExtensionVersion( 1 )
{
}


FITSImageExtension::FITSImageExtension( FitsKeywordList& kwlist, ByteIO* ptr2stream ) :
    FITSArrayHDU( kwlist, ptr2stream ),
    itsExtensionName( "BLAH" ),
    itsExtensionVersion( 1 )
{
    FitsKeyword*   keywordptr;
    
    //
    //  Take out the extname/extver keywords!
    //
    if( (keywordptr=kwlist(FITS::EXTNAME))!=0 )
    {
        itsExtensionName=keywordptr->asString();
        kwlist.del();
    }
    
    if( (keywordptr=kwlist(FITS::EXTVER))!=0 )
    {
        itsExtensionVersion=keywordptr->asInt();
        kwlist.del();
    }

    //
    //  After taking out all the keywords for axes etc. we're left
    //  with notjing but the 'extra' keywords. Therefore we copy
    //  what's left in the keywordlist into our own list of extra
    //  keywords.
    //
    this->setExtraKeywords( kwlist );

    //
    //  Now's the time to make the datasegment
    //
    this->makeDataSegment( ptr2stream );
}

HDUObject::hdu_t FITSImageExtension::getHDUtype( void ) const
{
    return HDUObject::imageextension;
}

//
//  We use the recordinterface to pass the array. Assume the array is
//  in field 0 of the recordinterface
//
Bool FITSImageExtension::put( const RecordInterface& recordref )
{
    Bool             retval( False );
    
    //
    //  datatype not compatible....
    //
    if( recordref.type(0)!=MakeArrayType(this->getDataType()) )
    {
        return retval;
    }


    //
    //  Put the complete array!
    //
    if( !this->hasDataSegment() )
    {
        //
        //  No data allocated yet....
        //
        this->makeDataSegment();
        
        if( !this->hasDataSegment() )
        {
            throw( AipsError("Aargh. Failed to create datasegment for PrimaryArray!") );
        }
    }

    //
    //  Flush the recordfield to the datasegment
    //
    retval = flushRecordField( recordref, 0, this->getDataSegment(), 0L );
    
    return retval;
}

Bool FITSImageExtension::get( RecordInterface& recordref ) const
{
    Bool             retval( False );

    //
    //  If no datasegment, return False
    //
    if( !this->hasDataSegment() )
    {
        return retval;
    }
    
    //
    //  Make a record desc so we can restructure the user's record to
    //  match the description of this array!
    //
    RecordDesc   thisarray;
    
    thisarray.addField( "arraydata", MakeArrayType(this->getDataType()), this->shape() );
    

    Bool   except( False );
    
    try
    {
        recordref.restructure( thisarray );
    }
    catch( AIPSExceptionObject x )
    {
        x.getMesg();
        except=True;
    }
    end_try;

    //
    //  If we failed to reshape the Record -> get out!
    //
    if( except )
    {
        return retval;
    }
    
    //
    //  Fill the recordfield from the datasegment
    //
    retval = fillRecordField( recordref, 0, this->getDataSegment(), 0L );

    return retval;
}


String FITSImageExtension::xtension( void ) const
{
    return "IMAGE";
}

String FITSImageExtension::extname( void ) const
{
    return itsExtensionName;
}

Int FITSImageExtension::extver( void ) const
{
    return itsExtensionVersion;
}

Regex FITSImageExtension::fitsname( void )
{
    return Regex( "^IMAGE[ \t]*$" );
}



FITSImageExtension::~FITSImageExtension()
{
    //
    //  Nothing to do yet!
    //
}

FitsKeywordList FITSImageExtension::getHeaderKeywords( void ) const
{
    FitsKeywordList  retval;
    
    retval.mk( FITS::XTENSION, this->xtension(), "This is an IMAGE extension" );
    retval.mk( FITS::EXTNAME,  this->extname() );
    retval.mk( FITS::EXTVER,   this->extver() );

    return retval;
}

    
