//
//  Class:
//
//          FITSImageExtension
//
//  Purpose:
//
//          This class describes an ImageExtension in a FITS file!
//
//
//  Author:
//
//          Harro Verkouter,   26-05-1998
//
//
//     $Id: FITSImageExtension.h,v 1.1.1.1 2000/03/20 15:16:48 verkout Exp $
//
#ifndef FITSIMAGEEXTENSION_H_INC
#define FITSIMAGEEXTENSION_H_INC


//
//  We need this include since it is our baseclass!
//
#ifndef FITSARRAYHDU_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSArrayHDU.h>
#endif

//
// Forward declarations
//
class RecordInterface;


//
//  The primary array hdu class. It isA FITSArrayHDU, therefore we
//  derive it from that class
//
class FITSImageExtension :
    public FITSArrayHDU
{
public:
    //
    //  The public interface
    //

    //
    //  Create a primary array of a given type (basic fits type: the
    //  type of data items),
    //
    FITSImageExtension( DataType basicfitstype );

    //
    //  Create the primary array from a fitskeywordlist, that's just
    //  been read from an existing fitsfile represented by
    //  'ptr2stream'
    //
    FITSImageExtension( FitsKeywordList& kwlist, ByteIO* ptr2stream );

    //
    //  The type of this hdu
    //
    virtual HDUObject::hdu_t getHDUtype( void ) const;

    //
    // Define get/put actions on the array!
    //
    // Note: these functions are NOT defined on the baseclass, and
    // they are virtual in THIS class -> derived classes from this
    // object may overload the get/put actions.
    //
    virtual Bool      put( const RecordInterface& recordref );
    
    virtual Bool      get( RecordInterface& recordref ) const;

    //
    //  Implement the 'extension HDU' specific access function
    //

    //
    // Returns 'IMAGE'
    //
    String            xtension( void ) const;

    //
    //  Do images have an extname/extver? 
    //
    String            extname( void ) const;
    Int               extver( void ) const;
    
    //
    //  Return the regex that matches the description of this extname
    //
    static Regex      fitsname( void );
    

    //
    //  Destruct the primary array
    //
    virtual ~FITSImageExtension();
    
protected:
    //
    //  Protected functions
    //
    virtual FitsKeywordList getHeaderKeywords( void ) const;

private:
    //
    //  The primary arrays private parts
    //
    String            itsExtensionName;
    Int               itsExtensionVersion;
    

    //
    //  The private methods
    //

    //
    //  These are undefined and inaccessible
    //
    FITSImageExtension();
    FITSImageExtension( const FITSImageExtension& );
    FITSImageExtension& operator=( const FITSImageExtension& );
};



#endif
