//
//  Derived class FITSMatrixAxis -> describes an axis in a matrix in a
//  row in binary table... (doesn't that tell you what you always
//  wanted to know but where afraid to ask?)
//
//  Author:  Harro Verkouter,  13-09-1999
//
//  $Id: FITSMatrixAxis.h,v 1.1.1.1 2000/03/20 15:16:48 verkout Exp $
//
//  $Log: FITSMatrixAxis.h,v $
//  Revision 1.1.1.1  2000/03/20 15:16:48  verkout
//  HV: Imported aips++ implement-stuff
//
//
#ifndef FITSMATRIXAXIS_H
#define FITSMATRIXAXIS_H


#ifndef BASICFITSUTILITY_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/BasicFITSUtility.h>
#endif

#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif


#ifndef AIPS_FITS
#include <aips/FITS/fits.h>
#endif


//
//  Forward declaration!
//
class FITSMatrixColumn;


class FITSMatrixAxis :
    public BasicFITSUtility
{
public:
    //
    //  Create a fully defined matrix axis...
    //
    FITSMatrixAxis( const String& type, const String& comment="", Int length=1,
		    Double refpixel=1.0, Double refvalue=0.0, Double increment=1.0,
		    Double rotangle=0.0 );
    

    FITSMatrixAxis( Int axisnr, FitsKeywordList& kwsource, FITSMatrixColumn* const parent );

    //
    //  This constructor is a special one: it duplicates the contents
    //  of 'other' into the new object, but sets the axisnumber to
    //  'axisnumber'. This constructor is used by the MatrixColumn
    //  objects when axes are being added to them. The MatrixColumn
    //  object knows what the current axisnumber is. The user
    //  doesn't. So the user creates an axis (from ctor 1) and tells
    //  the MatrixColumn to add this one. The MatrixColumn calls this
    //  c'tor because it knows all the details.
    //
    //  Note: one of the parameters is the MatrixColumn class. We can
    //  make that assumption since MatrixColumns are the only columns's who
    //  support axis.
    //
    FITSMatrixAxis( Int axisnr, const FITSMatrixAxis& other, FITSMatrixColumn* parent );

    //
    //  Translate the contents of this axis to FitsKeywords!
    //
    virtual FitsKeywordList asFITSKeys( void ) const;

    //
    //  Return the length of this axis
    //
    Int                     getAxisLength( void ) const;
    void                    setAxisLength( Int newlength );

    //
    //  Return the axis-parameters, naming convention reselmbles FITS
    //  nomenclature.
    //
    Int                     axisnr( void ) const;
    Int                     naxis( void ) const;
    Double                  crpix( void ) const;
    Double                  crval( void ) const;
    Double                  cdelt( void ) const;
    Double                  crota( void ) const;
    const String&           ctype( void ) const;

    //
    //  De-allocate resources (if any)
    //
    virtual ~FITSMatrixAxis();
    
    
private:
    //
    //  The private parts
    //

    //
    // The type of this axis
    //
    String    itsAxistype;
    
    //
    //  Comment associated with this axis...
    //
    String    itsComment;

    //
    //  The length of this axis
    //
    Int       itsAxislength;

    //
    //  The reference pixel of this axis
    //
    Double    itsReferencepixel;
    
    //
    //  The reference value for this axis
    //
    Double    itsReferencevalue;
    
    //
    //  Axis increment (per pixel)
    //
    Double    itsIncrement;
    
    //
    //  The rotation  angle
    //
    Double    itsRotationangle;

    //
    //  The number of this axis in the current HDU
    //
    Int       itsAxisnumber;

    //
    //  These are undefined hence we make'm inaccessible
    //
    FITSMatrixAxis();
    FITSMatrixAxis( const FITSMatrixAxis& );
    FITSMatrixAxis& operator=( const FITSMatrixAxis& );
};


//
//  global output operator
//
ostream& operator<<( ostream& os, const FITSMatrixAxis& axisref );

#endif
