//
//  Specialization of a binary column-> the matrix column. This (NOT
//  STANDARD) fits stuff is needed to make data available to classic
//  AIPS in the FITS-IDI format.
//
//  This format makes use of matrices in binary table columns -> THEY
//  COME WITH AN AXIS DESCRIPTION, as if it were a PrimaryArray, only
//  the keywords are somewhat malformed...
//
//  Author: Harro Verkouter, 13-09-1999
//
//
//  $Id: FITSMatrixColumn.h,v 1.1.1.1 2000/03/20 15:16:48 verkout Exp $
//
//  $Log: FITSMatrixColumn.h,v $
//  Revision 1.1.1.1  2000/03/20 15:16:48  verkout
//  HV: Imported aips++ implement-stuff
//
//
#ifndef FITSMATRIXCOLUMN_H
#define FITSMATRIXCOLUMN_H


//
//  We're some sort of a binary columns... thus....
//
#ifndef FITSBINARYCOLUMN_H
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryColumn.h>
#endif

#ifndef CONTAINER_H
#include <vlbi/tmplsrc/Container.h>
#endif

#ifndef AIPS_IPOSITION
#include <aips/Lattices/IPosition.h>
#endif

#ifndef AIPS_FITS
#include <aips/FITS/fits.h>
#endif

//
//  Forward declaration
//
class FITSMatrixAxis;



class FITSMatrixColumn :
    public FITSBinaryColumn
{
public:
    //
    //  Create a matrix column
    //

    //
    //  One must at least specify the datatype of this column -> the
    //  shape will be determined from the axes....  and the type
    //  (=name of the column) and unit. Comment is optional
    //
    FITSMatrixColumn( const String& type, DataType tp, const String& unit="",
		      const String& comment="" );

    
    //
    //  These c'tors should NOT be used by the public. We could have
    //  made them protected, but then we need to make the
    //  FITSBinaryTable class (or member functions thereof) friends of
    //  this clas, which means that they have access to ALL members
    //  and methods of this class, which I REALLY DONT LIKE!!! Better
    //  to let them have controlled acces with an object the doesn't
    //  do anything rather than giving them access to our private
    //  parts where they can mess up things whilst we don't know
    //  anything about it!
    //
    FITSMatrixColumn( Int number, uLong offset, FitsKeywordList& kwsource,
		      FITSBinaryTableHDU* ptr2parent );
    
    FITSMatrixColumn( Int number, uLong offset, const FITSMatrixColumn& other,
		      FITSBinaryTableHDU* ptr2parent );


    //
    //  Get RO-access to some of the properties
    //
    uInt                     nrAxes( void ) const;
    const FITSMatrixAxis*    getAxis( uInt axnr ) const;

    //
    //  Give usr. possibility to add an axis (if we're not bound to a
    //  table, that is!)
    //
    Bool                     addAxis( const FITSMatrixAxis& axis );

    //
    //  Return the shape of the column as an IPosition object
    //
    IPosition                shape( void ) const;

    //
    //  Translate ourselves into FITSKeywords
    //
    virtual FitsKeywordList  asFITSKeys( void ) const;

    //
    //  De-allocate resources..
    //
    virtual ~FITSMatrixColumn();
    

private:
    //
    //  Our private parts
    //
    
    //
    //  What data type are we???
    //
    //  NOTE: we keep the scalar form -> the column description will
    //  make it into an array.
    //
    DataType                    myDataType;

    //
    //  We keep a list of matrix axes, describing the data in this
    //  matrix...
    //
    Container<FITSMatrixAxis*>  myAxes;
    
    
    //
    //  These are undefined and hence we make'm inaccessible
    //
    FITSMatrixColumn();
    FITSMatrixColumn( const FITSMatrixColumn& );
    FITSMatrixColumn& operator=( const FITSMatrixColumn& );
};


#endif
