//
//  Some global functions to test if DataTypes match etc. See
//  descriptions that go with the functions for detals.
//
//  Also conversion from DataType -> BITPIX and vice-versa
//
//  And test for equality of two FitsKeywords (match on name+(index))
//
//
//
//  Author:      
//
//               Harro Verkouter,     20-04-1998
//
//   $Id: freefunctions.h,v 1.1.1.1 2000/03/20 15:16:48 verkout Exp $
//
#ifndef FREEFUNCTIONS_H_INC
#define FREEFUNCTIONS_H_INC


#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef AIPS_DATATYPE_H
#include <aips/Utilities/DataType.h>
#endif

//
//  Forward declarations
//
class FitsKeyword;
class IPosition;
class istream;


//
//  This is a local free function, specially suited for the
//  FITSParameter class. it takes two DataTypes and decides whether or
//  not the source type can be changed in the destination type, by
//  either casting or indexing (especially for arrays, indexed by
//  IPositions) or taking the real/imaginary part or any combination
//  of the actions.
//
//  It basically checks if the BASIC types are equal/compatible, and
//  only for those basic types that are supported in FITS!
//
Bool dataTypesMatch( DataType destinationtype, DataType sourcetype );


//
//  Test if the given datatype is a basic fits-compatible datatype
//
Bool isBasicFitsType( DataType sometype );

//
// Make an array type of a basic type (if sometype is a valid basic
// fitstype!)
//
DataType MakeArrayType( DataType sometype );

//
//  Convert DataTypes to bitpix value (and the reverse)
//

//
//  Return value is TpOther if the 'bitpixvalue' is not recognized as
//  a valid value for the 'BITPIX' keyword.
//
DataType  BitpixToDataType( Int bitpixvalue );

//
//  Return value is 0 if 'sometype' is not a basic fitstype!
//
Int       DataTypeToBitpix( DataType sometype );

//
//  Return True if the names of the two keywords (and the indices, if
//  applicable) match!
//
Bool operator==( const FitsKeyword& lval, const FitsKeyword& rval );

//
//  Read an IPosition from a stream, in ASCII format; the inverse from
//  operator<<(IPosition)
//
istream& operator>>( istream& is, IPosition& rval );

#endif
