// antabGenerator.cc
//
// Implementation of the antabGenerator class. See antabGenerator.h for details.
//
//
// Author:
//
//      Harro Verkouter   12-3-1998
//
// $Id: antabGenerator.cc,v 1.7 2000/09/22 13:17:12 verkout Exp $
#ifndef ANTABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/antabGenerator.h>
#endif


//
//  Aips++ includes
//
#ifndef HDUOBJECT_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/HDUObject.h>
#endif

#ifndef AIPS_MSCOLUMNS_H
#include <aips/MeasurementSets/MSColumns.h>
#endif

//
//  My includes
//
#ifndef HDUGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/hduGenerator.h>
#endif

#ifndef MSCONVERSIONINFO_H_INC
#include <vlbi/ms2uvfitsimpl/MSConversionInfo.h>
#endif

#ifndef FITSAXIS_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSAxis.h>
#endif

#ifndef FITSPARAMETER_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSParameter.h>
#endif

#ifndef FITSBINARYCOLUMN_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryColumn.h>
#endif

#ifndef AIPS_ARRAYMATH_H
#include <aips/Arrays/ArrayMath.h>
#endif

#ifndef FITSBINARYTABLEHDU_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryTableHDU.h>
#endif

#ifndef AIPS_RECORD_H
#include <aips/Containers/Record.h>
#endif

#ifndef AIPS_RECORDFIELD_H
#include <aips/Containers/RecordField.h>
#endif

#ifndef AIPS_EXPRNODE_H
#include <aips/Tables/ExprNode.h>
#endif

#ifndef AIPS_FITS_UTIL_H
#include <trial/FITS/FITSUtil.h>
#endif

#ifndef EXCEPTIONOBJECT_H
#include <vlbi/Utilities/AIPSExceptionObject.h>
#endif

#include <vlbi/ms2uvfitsimpl/DumpRecord.h>

#include <aips/Measures.h>
#include <aips/Measures/MeasTable.h>
#include <aips/Quanta/Euler.h>
#include <aips/Tables/TableRecord.h>


//
//  Create a default antab generator, nothing to do yet...
//
antabGenerator::antabGenerator() :
    hduGenerator()
{
}

//
//  This function will do all the work of getting all the necessary
//  antenna information out of the MSConversionInfo object. It will
//  then create a binary table extension HDU and pass that back to the
//  caller!
//
Bool antabGenerator::convert( Vector<HDUObject*>& ref2hduptr, MSConversionInfo& inforef ) const
{
    cout << "Making AN table" << endl;
    //
    //  First, make a list off all the stuff that we may need!
    //
    Int                    nrantennas;
    uInt                   currentArrayNumber;
    String                 arrayname;
    String                 rdate;
    MSFeed                 orgfeed( inforef.getMS().feed() );
    MSArray                arrayTable( inforef.getMS().array() );
    MSAntenna              organtenna( inforef.getMS().antenna() );
    Vector<Double>         arraypos;
    FITSBinaryColumn       anname( "ANNAME", "The antennaname", "8A", "" );
    FITSBinaryColumn       stabxyz( "STABXYZ", "", "3D", "METERS" );
    FITSBinaryColumn       orbparm( "ORBPARM", "Orbital parameters", "0D", "" );
    FITSBinaryColumn       nosta( "NOSTA", "Antenna number", "1J", "" );
    FITSBinaryColumn       mntsta( "MNTSTA", "Mount", "1J", "" );
    FITSBinaryColumn       staxof( "STAXOF", "x-offset", "1E", "METERS" );
    FITSBinaryColumn       poltya( "POLTYA", "Polarization type", "1A", "" );
    FITSBinaryColumn       polaa( "POLAA", "Polarization angle", "1E", "DEGREES" );
    FITSBinaryColumn       polcala( "POLCALA", "", "1E", "" );
    FITSBinaryColumn       poltyb( "POLTYB", "Polarization type", "1A", "" );
    FITSBinaryColumn       polab( "POLAB", "Polarization angle", "1E", "DEGREES" );
    FITSBinaryColumn       polcalb( "POLCALB", "", "1E", "" );


    //
    //  All the MS columns that we need
    //
    ROArrayColumn<Double>  arrayposcol( arrayTable, MSArray::columnName(MSArray::POSITION) );
    ROScalarColumn<String> arraynamecol( arrayTable, MSArray::columnName(MSArray::NAME) );
    ROScalarColumn<Double> timecol( inforef.getMS(), MS::columnName(MS::TIME) );


    //
    //  We may as well already resize the return va;ue vector
    //
    ref2hduptr.resize( arrayTable.nrow() );
    ref2hduptr=(HDUObject*)0;

    //
    //  Loop over all arraycombinations
    //
    for( currentArrayNumber=0; currentArrayNumber<arrayTable.nrow(); currentArrayNumber++ )
    {
        //
        //  Create a new Binary Table object
        //
        FITSBinaryTableHDU*    hduptr( new FITSBinaryTableHDU("AIPS AN", currentArrayNumber+1) );

        //
        // Retrieve important info
        //
        if( arrayposcol.isDefined(currentArrayNumber) )
        {
            arraypos=arrayposcol( currentArrayNumber );
        }
        else
        {
            cout.form("Array #%3d: ", currentArrayNumber) << "Arraypos not defined, default to (0,0,0)" << endl;
            arraypos=0.0;
        }
        
        if( arraynamecol.isDefined(currentArrayNumber) )
        {
            arrayname=arraynamecol( currentArrayNumber );
        }
        else
        {
            String        tmp;
            ostrstream    defarrayname;
            
            defarrayname << "ARRAY_" << currentArrayNumber << ends;
            tmp=String( defarrayname.str() );
            
            cout.form("Array #%3d: ", currentArrayNumber) << "Arrayname not defined, default to "
                                              << defarrayname.str() << endl;
            
            arrayname=tmp;
        }
        

        String    timesys;
    
        try
        {
            FITSDateUtil::toFITS( rdate, timesys, inforef.observingDate().day() );
        }
        catch( AIPSExceptionObject x )
        {
            x.getMesg();


            FITSDateUtil::toFITS( rdate, timesys, (Double)MVTime(Time()) );
            cout.form("Array #%3d: ", currentArrayNumber) << "DATE-OBS can't be represented in FITS, default to "
                                                          << rdate << endl;
        }
        end_try;
        


        //
        //  Configure the binary table
        //
        hduptr->addColumn( anname );
        hduptr->addColumn( stabxyz );
        hduptr->addColumn( orbparm );
        hduptr->addColumn( nosta );
        hduptr->addColumn( mntsta );
        hduptr->addColumn( staxof );
        hduptr->addColumn( poltya );
        hduptr->addColumn( polaa );
        hduptr->addColumn( polcala );
        hduptr->addColumn( poltyb );
        hduptr->addColumn( polab );
        hduptr->addColumn( polcalb );
        
        //
        //
        //  Now it's safe to SELECT antennas for the current sub-array
        //
        //
        MSAntenna     antennaTable( organtenna( organtenna.col(MSAntenna::columnName(MSAntenna::ARRAY_ID))
                                                ==Int(currentArrayNumber)) );

        //
        // SELECT feeds for the current sub-array.
        //
        MSFeed        feedTable( orgfeed( orgfeed.col(MSFeed::columnName(MSFeed::ARRAY_ID))
                                          ==Int(currentArrayNumber)) );
        
        //
        // Now we create column objects that refence into the SELECTED table
        //
        ROScalarColumn<String> antname( antennaTable, MSAntenna::columnName(MSAntenna::NAME) );
        ROScalarColumn<String> antmount( antennaTable, MSAntenna::columnName(MSAntenna::MOUNT) );
        ROArrayColumn<Double>  antposition( antennaTable, MSAntenna::columnName(MSAntenna::POSITION) );
        ROArrayColumn<Double>  antoffset( antennaTable, MSAntenna::columnName(MSAntenna::OFFSET) );
        ROArrayColumn<String>  poltype( feedTable, MSFeed::columnName(MSFeed::POLARIZATION_TYPE) );
        ROScalarColumn<Int>    antid( feedTable, MSFeed::columnName(MSFeed::ANTENNA_ID) );
        
        //
        //  The number of rows
        //
        nrantennas=antennaTable.nrow();
    
        hduptr->setNumberRows( nrantennas );
        
        
        //
        //  Restructure the record
        //
        Record               tablerow( hduptr->getRowDescription() );
        
        //
        //  Now we can have recordfieldptrs!
        //
        RecordFieldPtr<String>          nameptr( tablerow, "ANNAME" );
        RecordFieldPtr<Array<Double> >  stabptr( tablerow, "STABXYZ" );
        RecordFieldPtr<Int>             nostaptr( tablerow, "NOSTA" );
        RecordFieldPtr<Int>             mntstaptr( tablerow, "MNTSTA" );
        RecordFieldPtr<Float>           staxofptr( tablerow, "STAXOF" );
        RecordFieldPtr<uChar>           poltyaptr( tablerow, "POLTYA" );
        RecordFieldPtr<Float>           polaaptr( tablerow, "POLAA" );
        RecordFieldPtr<Float>           polcalaptr( tablerow, "POLCALA" );
        RecordFieldPtr<uChar>           poltybptr( tablerow, "POLTYB" );
        RecordFieldPtr<Float>           polabptr( tablerow, "POLAB" );    
        RecordFieldPtr<Float>           polcalbptr( tablerow, "POLCALB" );

        //
        //  Fill in recordfields that do not change across rows (if any)
        //
        *polaaptr=(Float)0.0;
        *polabptr=(Float)0.0;
        *polcalaptr=(Float)0.0;
        *polcalbptr=(Float)0.0;
        
        //
        //  Get the columns....
        //
        Int              tmpint;
        String           tmpstring;

        for( uInt i=0; i<(uInt)nrantennas; i++ )
        {
            //
            //  Name
            //
            *nameptr="";
            
            if( antname.isDefined(i) )
            {
                *nameptr = upcase( antname(i) );
            }
            
            if( *nameptr=="" )
            {
                String       str;
                ostrstream   defantname;
                
                //defantname << "AR" << currentArrayNumber << "AN" << i << ends;
		defantname << "AN" << i << ends;
                str=defantname.str();
                
                cout.form("Row #%3d: ",i) << "Antennaname not defined. Default to "
                                          << defantname.str() << endl;
                
                *nameptr=str;
            }

            //
            // Antenna position
            //
            if( antposition.isDefined(i) )
            {
				//
				// Baah! Need to flip the sign of the Y-coordinate of the
				// antennaposition because of Left-Hand coordsys in AIPS?!
				//
				Array<Double>  ap = antposition(i);
	
				ap( IPosition(1, 1) ) = -ap( IPosition(1, 1) );
                *stabptr = ap - arraypos;
            }
            else
            {
                cout.form("Row #%3d: ",i) << "Antennaposition not defined. Default to "
                                          << "(0,0,0)" << endl;
                *stabptr  = 0.0;
                *stabptr -= arraypos;
            }
            
            
            //
            //  Number ( in FITS, antenna nrs are 1-relative)
            //
            *nostaptr= i+1;
            
            //
            // Translate the MOUNT-string to Classic AIPS MNTSTA-number
            //
            if( antmount.isDefined(i) )
            {
                tmpstring=downcase( antmount(i) );
            }
            else
            {
                tmpstring="<unknown>";
                
                cout.form("Row #%3d: ",i) << "Antennamount not defined. Defaulting to "
                                          << tmpstring << endl;
            }
            
            
            if( tmpstring.contains("alt-az") )
            {
                tmpint=0;
            }
            else if( tmpstring.contains("equatorial") )
            {
                tmpint=1;
            }
            else if( tmpstring.contains("X-Y") )
            {
                tmpint=2;
            }
            else if( tmpstring.contains("orbiting") )
            {
                tmpint=3;
            }
            else
            {
                tmpint=0;
            }
            
            //
            //  Do put the value
            //
            *mntstaptr=tmpint;
            
            //
            //  The station X-offset
            //
            if( antoffset.isDefined(i) )
            {
                *staxofptr = (Float)( antoffset(i)(IPosition(1,0)) );
            }
            else
            {
                *staxofptr=0.0;
                cout.form("Row #%3d :",i) << "Antenna x-offset not defined, default to "
                                          << *staxofptr << endl;
            }
            
            
            //
            //  We need to fill in the polarization
            //
            Bool          found( False );
            const uInt    nmax( feedTable.nrow() );
            
            //
            //  First set to default value
            //
            *poltyaptr = ' ';
            *poltybptr = ' ';
            for( uInt j=0; j<nmax; j++ )
            {
                if( antid.isDefined(j) && i==antid(j) && poltype.isDefined(j) )
                {
                    Vector<String>  poltypes( poltype(j) );
                    
                    found = True;
                    
                    if( poltypes.nelements()>=1 )
                    {
                        *poltyaptr=poltypes(0)[0];
                    }
                    if( poltypes.nelements()>=2 )
                    {
                        *poltybptr=poltypes(1)[0];
                    }
                }
            }
            if( !found )
            {
                cout.form("Row #%3d: ",i) << "Polarization types could not be determined" << endl;
            }
            
            //
            //  Do write the row!
            //
            hduptr->putrow( i, tablerow );
        }
        
        //
        //  add some extra keywords....
        //
        FitsKeyword*      kwptr;
        FitsKeywordList   ek;
        
	//
        //  Before we add them, let's calculate them!
	//
	//  Calculate GSTIA0, DEGPDY, UT1UTC, and IATUTC.
	//
	String          timeUnit = timecol.keywordSet().asString("UNIT");
	Quantum<Double> stime(timecol(0), timeUnit);
	MEpoch          utctime(stime, MEpoch::UTC );
	MEpoch          iattime = MEpoch::Convert(utctime, MEpoch::IAT)();
	MEpoch          ut1time = MEpoch::Convert(utctime, MEpoch::UT1)();
	Double          utcsec  = utctime.get("s").getValue();
	Double          ut1sec  = ut1time.get("s").getValue();
	Double          iatsec  = iattime.get("s").getValue();
	
	//
	//  Use the beginning of the IAT day to calculate the GMST.
	//
	Double          utcday = floor(utctime.get("d").getValue());
	Double          iatday = floor(iattime.get("d").getValue());
	Double          gstday, gstday1;
	{
		//
		// Our timesys==UTC so GSTIA0 should be GSTUTC0....
		//
	    Quantum<Double>   itime(utcday, "d");
	    MEpoch            ut0time(itime, MEpoch::UTC);
	    MEpoch            gsttime = MEpoch::Convert(ut0time, MEpoch::GMST)();

	    gstday = gsttime.get("d").getValue();
	}

	Double           gstdeg = 360 * (gstday - floor(gstday));
	{
	    //
	    // #degrees/IATday is the difference between this and the next day.
	    //
	    Quantum<Double>   itime(iatday+1, "d");
	    MEpoch            ia0time(itime, MEpoch::IAT);
	    MEpoch            gsttime = MEpoch::Convert(ia0time, MEpoch::GMST)();

	    gstday1 = gsttime.get("d").getValue();
	}

	Double degpdy = 360 * (gstday1 - gstday);

	//
	//  PolarMotion gives -x and -y.
	//  Need to be multiplied by earth radius to get them in meters.
	//
	const Euler&          polarMotion = MeasTable::polarMotion( utcday );

	//
	//  Phew.. having done that, we can now safely fill them in...
	//
        ek.mk( "ARRAYX", arraypos(0) );
        ek.mk( "ARRAYY", arraypos(1) );
        ek.mk( "ARRAYZ", arraypos(2) );
        ek.mk( "GSTIA0", gstdeg );
        ek.mk( "DEGPDY", degpdy );
        ek.mk( "FREQ",   inforef.getReferenceFrequency(MSConversionInfo::CenterChannel) );
        ek.mk( "RDATE",  rdate );
        ek.mk( "POLARX", -polarMotion(0)*6356752.31 );
        ek.mk( "POLARY", -polarMotion(1)*6356752.31 );
        ek.mk( "UT1UTC", ut1sec-utcsec );
        ek.mk( "IATUTC", ::round2int(iatsec-utcsec) );
	ek.mk( "TIMSYS", "UTC" );
        ek.mk( "ARRNAM", arrayname );
        ek.mk( "NUMORB", 0 );
        ek.mk( "NOPCAL", 0 );
        ek.mk( "POLTYPE", "        " );

	//
	//  Transfer them to the HDU!
	//
        ek.first();
        ek.next();
        
        kwptr = ek.curr();
        while( kwptr )
        {
            hduptr->addExtraKeyword( *kwptr );
            kwptr = ek.next();
        }

        //
        //  And add the HDUObject to the return value vector
        //
        ref2hduptr(currentArrayNumber)=hduptr;
    }
    
    Int     length;

    ref2hduptr.shape( length );
    
    return ToBool( length!=0 );
}



antabGenerator::~antabGenerator()
{
}

