//
//  Really generate an ARRAY_GEOMETERY binary table extension!
//
//  Author: Harro Verkouter,  20-09-1999
//
//  $Id: arraygeomGenerator.cc,v 1.4 2000/08/24 14:32:16 verkout Exp $
//
//  $Log: arraygeomGenerator.cc,v $
//  Revision 1.4  2000/08/24 14:32:16  verkout
//  HV: Shoot.. forgot to round IATUTC to int (which it is by defn)..
//
//  Revision 1.3  2000/08/24 13:54:29  verkout
//  HV: Dang! Forgot to do REF_FREQ issues... they been done now!
//
//  Revision 1.2  2000/08/24 11:20:43  verkout
//  HV: Needed a couple of changes for conformance to AIPS standards: sign of antenna Y-position in AIPS AN table flipped, GSTIA0 is really GST(TIMSYS)0 (in both UVFITS/IDIFITS), BANDWIDTH column in AIPS SU table is really CHANNELWIDTH, IAT-UTC == integer by def'n: made sure it is
//
//  Revision 1.1.1.1  2000/03/20 15:16:47  verkout
//  HV: Imported aips++ implement-stuff
//
//
#ifndef ARRAYGEOMGENERATOR_H
#include <vlbi/ms2uvfitsimpl/Generators/arraygeomGenerator.h>
#endif

//
//  Aips++ includes
//
#ifndef HDUOBJECT_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/HDUObject.h>
#endif

#ifndef MSCONVERSIONINFO_H_INC
#include <vlbi/ms2uvfitsimpl/MSConversionInfo.h>
#endif

#ifndef FITSBINARYCOLUMN_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryColumn.h>
#endif

#ifndef AIPS_ARRAYMATH_H
#include <aips/Arrays/ArrayMath.h>
#endif

#ifndef FITSBINARYTABLEHDU_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryTableHDU.h>
#endif

#ifndef AIPS_RECORD_H
#include <aips/Containers/Record.h>
#endif

#ifndef AIPS_RECORDFIELD_H
#include <aips/Containers/RecordField.h>
#endif

#ifndef AIPS_EXPRNODE_H
#include <aips/Tables/ExprNode.h>
#endif

#ifndef AIPS_FITS_UTIL_H
#include <trial/FITS/FITSUtil.h>
#endif

#ifndef EXCEPTIONOBJECT_H
#include <vlbi/Utilities/AIPSExceptionObject.h>
#endif

#ifndef AIPS_MSFEED_H
#include <aips/MeasurementSets/MSFeed.h>
#endif

#ifndef AIPS_MSARRAY_H
#include <aips/MeasurementSets/MSArray.h>
#endif

#ifndef AIPS_MSANTENNA_H
#include <aips/MeasurementSets/MSAntenna.h>
#endif

#ifndef AIPS_MEASUREMENTSET_H
#include <aips/MeasurementSets/MeasurementSet.h>
#endif

#include <vlbi/ms2uvfitsimpl/DumpRecord.h>

#include <aips/Quanta/Euler.h>
#include <aips/Measures/MeasTable.h>

//
//  Create a default arraygeom generator, nothing to do yet...
//
arraygeomGenerator::arraygeomGenerator() :
    hduGenerator()
{
}

//
//  This function will do all the work of getting all the necessary
//  antenna information out of the MSConversionInfo object. It will
//  then create a binary table extension HDU and pass that back to the
//  caller!
//
Bool arraygeomGenerator::convert( Vector<HDUObject*>& ref2hduptr, MSConversionInfo& inforef ) const
{
    cout << "Making ARRAY_GEOMETRY table" << endl;
    //
    //  First, make a list off all the stuff that we may need!
    //
    Int                    nrantennas;
    uInt                   currentArrayNumber;
    String                 arrayname;
    String                 rdate;
    MSFeed                 orgfeed( inforef.getMS().feed() );
    MSArray                arrayTable( inforef.getMS().array() );
    MSAntenna              organtenna( inforef.getMS().antenna() );
    Vector<Double>         arraypos;
    FITSBinaryColumn       anname( "ANNAME", "The antennaname", "8A", "" );
    FITSBinaryColumn       stabxyz( "STABXYZ", "", "3D", "METERS" );
    FITSBinaryColumn       derxyz( "DERXYZ", "", "3E", "METER/S" );
    FITSBinaryColumn       orbparm( "ORBPARM", "Orbital parameters", "0D", "" );
    FITSBinaryColumn       nosta( "NOSTA", "Antenna number", "1J", "" );
    FITSBinaryColumn       mntsta( "MNTSTA", "Mount", "1J", "" );
    FITSBinaryColumn       staxof( "STAXOF", "x-offset", "1J", "METERS" );

    //
    //  All the MS columns that we need
    //
    ROArrayColumn<Double>  arrayposcol( arrayTable, MSArray::columnName(MSArray::POSITION) );
    ROScalarColumn<String> arraynamecol( arrayTable, MSArray::columnName(MSArray::NAME) );
    ROScalarColumn<Double> timecol( inforef.getMS(), MS::columnName(MS::TIME) );


    //
    //  We may as well already resize the return va;ue vector
    //
    ref2hduptr.resize( arrayTable.nrow() );
    ref2hduptr=(HDUObject*)0;

    //
    //  Loop over all arraycombinations
    //
    for( currentArrayNumber=0; currentArrayNumber<arrayTable.nrow(); currentArrayNumber++ )
    {
        //
        //  Create a new Binary Table object
        //
        FITSBinaryTableHDU*    hduptr( new FITSBinaryTableHDU("ARRAY_GEOMETRY", currentArrayNumber+1) );

        //
        // Retrieve important info
        //
        if( arrayposcol.isDefined(currentArrayNumber) )
        {
            arraypos=arrayposcol( currentArrayNumber );
        }
        else
        {
            cout.form("Array #%3d: ", currentArrayNumber) 
		<< "Arraypos not defined, default to (0,0,0)" << endl;
            arraypos=0.0;
        }
        
        if( arraynamecol.isDefined(currentArrayNumber) )
        {
            arrayname=arraynamecol( currentArrayNumber );
        }
        else
        {
            String        tmp;
            ostrstream    defarrayname;
            
            defarrayname << "ARRAY_" << currentArrayNumber << ends;
            tmp=String( defarrayname.str() );
            
            cout.form("Array #%3d: ", currentArrayNumber) << "Arrayname not defined, default to "
                                              << defarrayname.str() << endl;
            
            arrayname=tmp;
        }
        

	MVTime    actualobstime;
        String    timesys;
    
        try
        {
	    actualobstime = inforef.observingDate();
            FITSDateUtil::toFITS( rdate, timesys, inforef.observingDate().day() );
        }
        catch( AIPSExceptionObject x )
        {
            x.getMesg();

	    actualobstime = MVTime( Time() );
            FITSDateUtil::toFITS( rdate, timesys, (Double)MVTime(Time()) );
            cout.form("Array #%3d: ", currentArrayNumber) << "DATE-OBS can't be represented in FITS, default to "
                                                          << rdate << endl;
        }
        end_try;
        
	
	char         tmpbuf[ 256 ];
	ostrstream   tmpstrm( tmpbuf, sizeof(tmpbuf) );

	tmpstrm.form( "%04d-%02d-%02d", actualobstime.year(), actualobstime.month(),
		      actualobstime.monthday() )
			  << ends;
	rdate = tmpstrm.str();



        //
        //  Configure the binary table
        //
        hduptr->addColumn( anname );
        hduptr->addColumn( stabxyz );
        hduptr->addColumn( derxyz );
        hduptr->addColumn( orbparm );
        hduptr->addColumn( nosta );
        hduptr->addColumn( mntsta );
        hduptr->addColumn( staxof );
        
        //
        //
        //  Now it's safe to SELECT antennas for the current sub-array
        //
        //
        MSAntenna     antennaTable( organtenna( organtenna.col(MSAntenna::columnName(MSAntenna::ARRAY_ID))
                                                ==Int(currentArrayNumber)) );

        //
        // SELECT feeds for the current sub-array.
        //
        MSFeed        feedTable( orgfeed( orgfeed.col(MSFeed::columnName(MSFeed::ARRAY_ID))
                                          ==Int(currentArrayNumber)) );
        
        //
        // Now we create column objects that refence into the SELECTED table
        //
        ROScalarColumn<String> antname( antennaTable, MSAntenna::columnName(MSAntenna::NAME) );
        ROScalarColumn<String> antmount( antennaTable, MSAntenna::columnName(MSAntenna::MOUNT) );
        ROArrayColumn<Double>  antposition( antennaTable, MSAntenna::columnName(MSAntenna::POSITION) );
        ROArrayColumn<Double>  antoffset( antennaTable, MSAntenna::columnName(MSAntenna::OFFSET) );
        ROArrayColumn<String>  poltype( feedTable, MSFeed::columnName(MSFeed::POLARIZATION_TYPE) );
        ROScalarColumn<Int>    antid( feedTable, MSFeed::columnName(MSFeed::ANTENNA_ID) );
        
        //
        //  The number of rows
        //
        nrantennas=antennaTable.nrow();
    
        hduptr->setNumberRows( nrantennas );
        
        //
        //  Restructure the record
        //
        Record               tablerow( hduptr->getRowDescription() );
        
        //
        //  Now we can have recordfieldptrs!
        //
        RecordFieldPtr<String>          nameptr( tablerow, "ANNAME" );
        RecordFieldPtr<Array<Double> >  stabptr( tablerow, "STABXYZ" );
        RecordFieldPtr<Array<Float> >   derptr( tablerow, "DERXYZ" );
        RecordFieldPtr<Int>             nostaptr( tablerow, "NOSTA" );
        RecordFieldPtr<Int>             mntstaptr( tablerow, "MNTSTA" );
        RecordFieldPtr<Int>             staxofptr( tablerow, "STAXOF" );

        //
        //  Fill in recordfields that do not change across rows (if any)
        //
        *derptr=(Float)0.0;
        
        //
        //  Get the columns....
        //
        Int              tmpint;
        String           tmpstring;

        for( uInt i=0; i<(uInt)nrantennas; i++ )
        {
            //
            //  Name
            //
            *nameptr="";
            
            if( antname.isDefined(i) )
            {
                *nameptr = upcase( antname(i) );
            }
            
            if( *nameptr=="" )
            {
                String       str;
                ostrstream   defantname;
                
                //defantname << "AR" << currentArrayNumber << "AN" << i << ends;
		defantname << "AN" << i << ends;
                str=defantname.str();
                
                cout.form("Row #%3d: ",i) << "Antennaname not defined. Default to "
                                          << defantname.str() << endl;
                
                *nameptr=str;
            }

            //
            // Antenna position
            //
            if( antposition.isDefined(i) )
            {
                *stabptr=antposition(i)-arraypos.ac();
            }
            else
            {
                cout.form("Row #%3d: ",i) << "Antennaposition not defined. Default to "
                                          << "(0,0,0)" << endl;
                *stabptr=0.0;
                *stabptr-=arraypos.ac();
            }
            
            
            //
            //  Number ( in FITS, antenna nrs are 1-relative)
            //
            *nostaptr= i+1;
            
            //
            // Translate the MOUNT-string to Classic AIPS MNTSTA-number
            //
            if( antmount.isDefined(i) )
            {
                tmpstring=downcase( antmount(i) );
            }
            else
            {
                tmpstring="<unknown>";
                
                cout.form("Row #%3d: ",i) << "Antennamount not defined. Defaulting to "
                                          << tmpstring << endl;
            }
            
            
            if( tmpstring.contains("alt-az") )
            {
                tmpint=0;
            }
            else if( tmpstring.contains("equatorial") )
            {
                tmpint=1;
            }
            else if( tmpstring.contains("orbiting") )
            {
                tmpint=2;
            }
            else
            {
                tmpint=3;
            }
            
            //
            //  Do put the value
            //
            *mntstaptr=tmpint;
            
            //
            //  The station X-offset
            //
            if( antoffset.isDefined(i) )
            {
                *staxofptr = (Int)( antoffset(i)(IPosition(1,0)) );
            }
            else
            {
                *staxofptr=0;
                cout.form("Row #%3d :",i) << "Antenna x-offset not defined, default to "
                                          << *staxofptr << endl;
            }
            
            //
            //  Do write the row!
            //
            hduptr->putrow( i, tablerow );
        }
        
        //
        //  add some extra keywords....
        //
        FitsKeyword*      kwptr;
        FitsKeywordList   ek;
        
        ek.mk( "FRAME", "GEOCENTRIC" );
        ek.mk( "ARRAYX", arraypos(0) );
        ek.mk( "ARRAYY", arraypos(1) );
        ek.mk( "ARRAYZ", arraypos(2) );
        ek.mk( "NUMORB", 0 );


	//
        //  Before we add them, let's calculate them!
	//
	//  Calculate GSTIA0, DEGPDY, UT1UTC, and IATUTC.
	//
	String          timeUnit = timecol.keywordSet().asString("UNIT");
	Quantum<Double> stime(timecol(0), timeUnit);
	MEpoch          utctime(stime, MEpoch::UTC );
	MEpoch          iattime = MEpoch::Convert(utctime, MEpoch::IAT)();
	MEpoch          ut1time = MEpoch::Convert(utctime, MEpoch::UT1)();
	Double          utcsec  = utctime.get("s").getValue();
	Double          ut1sec  = ut1time.get("s").getValue();
	Double          iatsec  = iattime.get("s").getValue();
	
	//
	//  Use the beginning of the IAT day to calculate the GMST.
	//
	Double          utcday = floor(utctime.get("d").getValue());
	Double          iatday = floor(iattime.get("d").getValue());
	Double          gstday, gstday1;
	{
		//
		// Our TIMSYS=UTC so GSTIA0 really means GSTUTC0
		//
	    Quantum<Double>   itime(utcday, "d");
	    MEpoch            ut0time(itime, MEpoch::UTC);
	    MEpoch            gsttime = MEpoch::Convert(ut0time, MEpoch::GMST)();

	    gstday = gsttime.get("d").getValue();
	}

	Double           gstdeg = 360 * (gstday - floor(gstday));
	{
	    //
	    // #degrees/IATday is the difference between this and the next day.
	    //
	    Quantum<Double>   itime(iatday+1, "d");
	    MEpoch            ia0time(itime, MEpoch::IAT);
	    MEpoch            gsttime = MEpoch::Convert(ia0time, MEpoch::GMST)();

	    gstday1 = gsttime.get("d").getValue();
	}

	Double degpdy = 360 * (gstday1 - gstday);

	//
	//  PolarMotion gives -x and -y.
	//  Need to be multiplied by earth radius to get them in meters.
	//
	const Euler&          polarMotion = MeasTable::polarMotion( utcday );
	
	//
	//  Phew.. having done that, we can now safely fill them in...
	//
	ek.mk( "FRAME",  "GEOCENTRIC" );
        ek.mk( "ARRAYX", 0.0 );
        ek.mk( "ARRAYY", 0.0 );
        ek.mk( "ARRAYZ", 0.0 );
        ek.mk( "GSTIA0", gstdeg );
        ek.mk( "DEGPDY", degpdy );
        ek.mk( "FREQ",   inforef.getReferenceFrequency(MSConversionInfo::CenterChannel) );
        ek.mk( "RDATE",  rdate );
        ek.mk( "POLARX", -polarMotion(0)*6356752.31 );
        ek.mk( "POLARY", -polarMotion(1)*6356752.31 );
        ek.mk( "UT1UTC", ut1sec-utcsec );
        ek.mk( "IATUTC", ::round2int(iatsec-utcsec) );
	ek.mk( "TIMSYS", "UTC" );
        ek.mk( "ARRNAM", "EVN" );
        ek.mk( "NUMORB", 0 );

	//
	//  The common keywords
	//
	ek.mk( "TABREV", (Int)1 );
	ek.mk( "OBSCODE", inforef.getObsCode() );
	ek.mk( "NO_STKD", (Int)inforef.nrPolarizations() );
	ek.mk( "STK_1", inforef.getFirstStokes() );
	ek.mk( "NO_BAND", (Int)inforef.nrIFs() );
	ek.mk( "NO_CHAN", (Int)inforef.nrChannels() );
    ek.mk( "REF_FREQ", inforef.getReferenceFrequency(MSConversionInfo::CenterChannel) );	
	ek.mk( "CHAN_BW", inforef.getChannelwidth() );
	ek.mk( "REF_PIXL", (Float)1 );
	

	//
	//  Transfer'm to the HDU!
	//
        ek.first();
        ek.next();
        
        kwptr = ek.curr();
        while( kwptr )
        {
            hduptr->addExtraKeyword( *kwptr );
            kwptr = ek.next();
        }

        //
        //  And add the HDUObject to the return value vector
        //
        ref2hduptr(currentArrayNumber) = hduptr;
    }
    
    Int     length;

    ref2hduptr.shape( length );
    
    return ToBool( length!=0 );
}



arraygeomGenerator::~arraygeomGenerator()
{
}


