//
//  Create the array_geometery binary table extension to be used when
//  writing FITS-ID files. Ref. AIPS Memo #102, 'The FITS
//  Interferometry Data Interchange Format' by Chris Flatters
//  (www.nrao.edu/~cflatter)
//
//   Author:  Harro Verkouter,  20-09-1999
//
//   $Id: arraygeomGenerator.h,v 1.2 2000/08/24 11:20:43 verkout Exp $
//
//   $Log: arraygeomGenerator.h,v $
//   Revision 1.2  2000/08/24 11:20:43  verkout
//   HV: Needed a couple of changes for conformance to AIPS standards: sign of antenna Y-position in AIPS AN table flipped, GSTIA0 is really GST(TIMSYS)0 (in both UVFITS/IDIFITS), BANDWIDTH column in AIPS SU table is really CHANNELWIDTH, IAT-UTC == integer by def'n: made sure it is
//
//   Revision 1.1.1.1  2000/03/20 15:16:47  verkout
//   HV: Imported aips++ implement-stuff
//
//
#ifndef ARRAYGEOMGENERATOR_H
#define ARRAYGEOMGENERATOR_H

//
//  Aips++ includes
//


//
// My includes
//
#ifndef HDUGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/hduGenerator.h>
#endif

//
// Need this to fool compiler?
//
#include <aips/Tables/TableRecord.h>
#include <aips/Measures/MeasConvert.h>
#include <aips/Measures/MEpoch.h>
#include <aips/Quanta/MVEpoch.h>

typedef MeasConvert<MEpoch,MVEpoch,MCEpoch> stoopidGcc;


//
//  The arraygeom generator is a specific kind of HDU generator....
//
class arraygeomGenerator :
    public virtual hduGenerator
{
    //
    //  The public section:
    //
public:

    //
    //  The default arraygeomGenerator...
    //
    arraygeomGenerator();
    

    //
    // Special interface function: we can ask for the HDU for the
    // specified ARRAY number; every ARRAY has its own ARRAYGEOM-hdu.
    //
    // If the HDU was succesfully generated, the return value will be
    // non-null.
    //
    // NOTE: The caller is responsible for (eventually) releasing the
    // allocated resources! (i.e. caller must delete the pointer!)
    //
    Vector<HDUObject*>   generate( MSConversionInfo& inforef );
    

    //
    //  Delete the generator
    //
    virtual ~arraygeomGenerator();
    
protected:
    //
    //  This virtual function is called by the base class: it is a
    //  request to actually generate a arraygeomHDU!
    //
    virtual Bool  convert( Vector<HDUObject*>& ref2hduptr, MSConversionInfo& inforef ) const;
    

private:
    //
    //  The object's private parts go here:
    //

    //
    //  Prevent copying and assignment
    //
    arraygeomGenerator( const arraygeomGenerator& );
    arraygeomGenerator& operator=( const arraygeomGenerator& );
};


#endif
