// asciitabGenerator.cc
//
// Implementation of the asciitabGenerator class. See asciitabGenerator.h for details.
//
//
// Author:
//
//      Harro Verkouter   12-3-1998
//
// $Id: asciitabGenerator.cc,v 1.1.1.1 2000/03/20 15:16:47 verkout Exp $
#ifndef ASCIITABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/asciitabGenerator.h>
#endif


//
//  Aips++ includes
//
#ifndef HDUOBJECT_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/HDUObject.h>
#endif

#ifndef AIPS_MSCOLUMNS_H
#include <aips/MeasurementSets/MSColumns.h>
#endif

//
//  My includes
//
#ifndef MSCONVERSIONINFO_H_INC
#include <vlbi/ms2uvfitsimpl/MSConversionInfo.h>
#endif

#ifndef FITSASCIITABLEHDU_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSAsciiTableHDU.h>
#endif

#ifndef FITSASCIICOLUMN_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSAsciiColumn.h>
#endif

#ifndef AIPS_RECORD_H
#include <aips/Containers/Record.h>
#endif

#ifndef AIPS_RECORDFIELD_H
#include <aips/Containers/RecordField.h>
#endif

#include <vlbi/ms2uvfitsimpl/DumpRecord.h>



//
//  Create a default asciitab generator, nothing to do yet...
//
asciitabGenerator::asciitabGenerator() :
    hduGenerator()
{
}


//
//  This function will do all the work of getting all the necessary
//  antenna information out of the MSConversionInfo object. It will
//  then create a Ascii table extension HDU and pass that back to the
//  caller!
//
Bool asciitabGenerator::convert( Vector<HDUObject*>& ref2hduptr, MSConversionInfo& inforef ) const
{
    //
    //  Initialize the return value
    //
    ref2hduptr.resize( 1 );
    ref2hduptr=(HDUObject*)0;
    

    //
    // Do it!
    //
    Int                 nrantennas;
    Record              tablerow;
    FITSAsciiColumn     anname( "ANNAME", "The antennaname", "A8", "" );
    FITSAsciiColumn     stabxyz( "STABXYZ", "", "F12.3", "METERS" );
    FITSAsciiColumn     derxyz( "DERXYZ", "", "D7.2E3", "METERS/SEC" );
    FITSAsciiTableHDU*  hduptr( 0 );


    //
    //  Now do create the object and return the pointer to the caller!
    //
    hduptr=new FITSAsciiTableHDU( "ANTAB", 1 );

    hduptr->addColumn( anname );
    hduptr->addColumn( stabxyz );
    hduptr->addColumn( derxyz );

    hduptr->setNumberRows( 10 );
    
    //
    //  Restructure the record
    //
    tablerow.restructure( hduptr->getRowDescription() );

    //
    //  Now we can have recordfieldptrs!
    //
    RecordFieldPtr<String>   nameptr( tablerow, "ANNAME" );
    RecordFieldPtr<Float>    stabptr( tablerow, "STABXYZ" );
    RecordFieldPtr<Double>   derxyzptr( tablerow, "DERXYZ" );


    *nameptr="Blah";
    //FITSColumn&    col0( hduptr->getColumn("STABXYZ") );
    //FITSColumn&    col1( hduptr->getColumn("ANNAME") );
    
    for( Int k=0; k<5; k++ )
    {
        *stabptr=(Float)k*3.14;
        (*derxyzptr)=-1.0*k*3.14;
        hduptr->putrow( k, tablerow );
    }
    
    /*
    for( Int i=0; i<5; i++ )
    {
        *stabptr=(Double)i*3.14;
        *nameptr="St. Croix";

        hduptr->put( i, tablerow );
    }
    */

    ref2hduptr(0)=hduptr;

    return True;
}



asciitabGenerator::~asciitabGenerator()
{
}

