// fqtabGenerator.cc
//
// Implementation of the fqtabGenerator class. See fqtabGenerator.h for details.
//
//
// Author:
//
//      Harro Verkouter   12-3-1998
//
// $Id: fqtabGenerator.cc,v 1.2 2000/08/24 13:54:29 verkout Exp $

#ifndef FQTABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/fqtabGenerator.h>
#endif


//
//  Aips++ includes
//
#ifndef HDUOBJECT_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/HDUObject.h>
#endif

#ifndef AIPS_MSCOLUMNS_H
#include <aips/MeasurementSets/MSColumns.h>
#endif

//
//  My includes
//
#ifndef HDUGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/hduGenerator.h>
#endif

#ifndef MSCONVERSIONINFO_H_INC
#include <vlbi/ms2uvfitsimpl/MSConversionInfo.h>
#endif

#ifndef FITSAXIS_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSAxis.h>
#endif

#ifndef FITSPARAMETER_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSParameter.h>
#endif

#ifndef FITSBINARYCOLUMN_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryColumn.h>
#endif

#include <aips/Arrays/Matrix.h>
#include <aips/Arrays/ArrayUtil.h>

#ifndef FITSBINARYTABLEHDU_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryTableHDU.h>
#endif

#ifndef AIPS_RECORD_H
#include <aips/Containers/Record.h>
#endif

#ifndef AIPS_RECORDFIELD_H
#include <aips/Containers/RecordField.h>
#endif

#ifndef AIPS_MATH_H
#include <aips/Mathematics/Math.h>
#endif

#include <vlbi/ms2uvfitsimpl/DumpRecord.h>



//
//  Create a default fqtab generator, nothing to do yet...
//
fqtabGenerator::fqtabGenerator() :
    hduGenerator()
{
}


//
//  This function will do all the work of getting all the necessary
//  frequency information out of the MSConversionInfo object. It will
//  then create a binary table extension HDU and pass that back to the
//  caller!
//
Bool fqtabGenerator::convert( Vector<HDUObject*>& ref2hduptr, MSConversionInfo& inforef ) const
{
    cout << "Making FQ Table" << endl;

    //
    //  This is the stuff we need
    //
    uInt                   nrifs( inforef.nrIFs() );
    uInt                   nrfreqgrps;
    Double                 refFreq( inforef.getReferenceFrequency(MSConversionInfo::CenterChannel) );
    IPosition              ifshape( IPosition(1,inforef.nrIFs()) );
    FITSBinaryColumn       freqsel( "FREQSEL", "", "1J", "" );
    FITSBinaryColumn       iffreq( "IF FREQ", TpArrayDouble, ifshape, "HZ" );
    FITSBinaryColumn       chwidth( "CH WIDTH", TpArrayFloat, ifshape, "HZ" );
    FITSBinaryColumn       totbw( "TOTAL BANDWIDTH", TpArrayFloat, ifshape, "HZ" );
    FITSBinaryColumn       sidebandcol( "SIDEBAND", TpArrayInt, ifshape );
    MSSpectralWindow       specTable( inforef.getMS().spectralWindow() );
    FITSBinaryTableHDU*    hduptr( new FITSBinaryTableHDU("AIPS FQ", 1) );
    ROArrayColumn<Double>  inchanfreq( specTable, MSSpectralWindow::columnName(MSSpectralWindow::CHAN_FREQ) );
    ROScalarColumn<Double> intotbw( specTable, MSSpectralWindow::columnName(MSSpectralWindow::TOTAL_BANDWIDTH) );
    

    //
    //  Resize the retval
    //
    ref2hduptr.resize( 1 );
    ref2hduptr=(HDUObject*)0;
    

    //
    //  Add columns to the binary table object
    //
    hduptr->addColumn( freqsel );
    hduptr->addColumn( iffreq );
    hduptr->addColumn( chwidth );
    hduptr->addColumn( totbw );
    hduptr->addColumn( sidebandcol );


    //
    //  The number of frequency-groups. As of MS vs. 2.0 we can
    //  determine that. For now, we set it to one.
    //
    nrfreqgrps = 1;
    
    hduptr->setNumberRows( nrfreqgrps );
    
    //
    //  We now can create a record with the correct rowdescription and
    //  create RecordFieldPtrs.
    //
    Record                 datarecord( hduptr->getRowDescription() );
    
    RecordFieldPtr<Int>            freqselptr( datarecord, "FREQSEL");
    RecordFieldPtr<Array<Int> >    sidebandptr( datarecord, "SIDEBAND");
    RecordFieldPtr<Array<Float> >  ifwidthptr( datarecord, "CH WIDTH");
    RecordFieldPtr<Array<Float> >  totbwptr( datarecord, "TOTAL BANDWIDTH");
    RecordFieldPtr<Array<Double> > iffreqptr( datarecord, "IF FREQ");

    //
    //  Introduce some Arrays that can be used to fill and later on
    //  put in the datarecord
    //
    IPosition       index( IPosition(1) );
    Array<Int>      sideband( ifshape );
    Array<Float>    channelwidth( ifshape );
    Array<Float>    totalbandwidth( ifshape );
    Array<Double>   frequencyoffset( ifshape );
    

    //
    // Do write all the frequency groups. In MS vs. 2.0, we should
    // write all the FREQ_GRP. For now, we assume that all rows in the
    // SpectralWindow table make up one frequency-group.
    //
    for( uInt i=0; i<nrfreqgrps; i++ )
    {
        //
        // The freq-grp 'ID'
        //
	*freqselptr=i+1;

	//
	//  In the future, we should make a subtable, containing rows
	//  for all 'IF's; we should select based upon FREQ_GRP.
	//
	Table    grptab( specTable );
	//Table    grptab( specTable(specTable.col(MSSpectralWindow::columnName(MSSpectralWindow::FREQ_GRP))==i) );
	
	//
	//  If the number of rows in this table is not equal to the
	//  number of IFs, we're in trouble!
	//
	if( grptab.nrow()!=nrifs )
	{
	    cout << "*** Frequency group " << i << " has non-conformant nr of IFs;" << endl;
	    cout << "*** found " << grptab.nrow() << ", needed: " << nrifs << endl;
	    cout << "*** ignoring this one." << endl;
	    continue;
	}

	//
	//  We want the rownrs in the original spectralwindow table
	//
	uInt           currow;
	Vector<uInt>   orgrownr( grptab.rowNumbers() );

	//
	//  Now, for every IF, we need to do the following:
	//
	for( index(0)=0; index(0)<nrifs; index(0)++ )
	{
	    Int   cursideband( 1 );
	    
	    //
	    //  'cache' the rownr in the original spectralwindow table
	    //
	    currow = orgrownr( index(0) );

	    //
	    //  Let's try to find the offset of this IF wrt inforef.getReferenceFrequency()
	    //
	    Vector<Double>  freqs;
        
	    if( inchanfreq.isDefined(currow) )
	    {
		freqs=inchanfreq(currow);
	    }
	    else
	    {
		freqs.resize( IPosition(1,1) );
		
		freqs=refFreq;
		
		cout.form("Row #%3d: ",currow) << "No channel frequencies found, default to " 
					  << refFreq << endl;
	    }
	    
	    //
	    // Determine the offset w.r.t. the reference frequency
	    //
	    frequencyoffset( index ) = freqs(0)-refFreq;

	    //
	    //  Attempt to derive the channel bandwidth?
	    //
	    if( freqs.nelements()>1 )
	    {
		//*ifwidthptr = freqs(1) - freqs(0);
		channelwidth( index ) = fabs( freqs(1)-freqs(0) );
		
		if( freqs(1)>freqs(0) )
		{
		    cursideband = 1;
		}
		else
		{
		    cursideband = -1;
		}
	    }
	    else
	    {
		//
		//  We can't derive it from thge channel frequencies;
		//  attempt to get it from the total bandwidth
		//
		if( intotbw.isDefined(currow) )
		{
		    channelwidth( index ) = fabs( intotbw(currow)/inforef.nrChannels() );
		    //*ifwidthptr = intotbw(i)/inforef.nrChannels();
		}
		else
		{
		    //
		    //  Default
		    //
		    //*ifwidthptr = 1.0;
		    channelwidth( index ) = 1.0;
		    
		    cout.form("Row #%3d: ",currow) << "No channel bandwidth info found, default to " 
					      << channelwidth( index ) << endl;
		}
	    }
	    //*ifwidthptr = fabs( inforef.getTotalBandwidth()/inforef.nrChannels() );

	    //
	    //  Get the totalbandwidth
	    //
	    if( intotbw.isDefined(currow) )
	    {
		//*totbwptr = intotbw(i);
		totalbandwidth( index ) = intotbw(currow);
	    }
	    else
	    {
		//
		//  Attempt to derive from frequencies
		//
		Int      shp;
		
		freqs.shape( shp );
		
		if( shp==inforef.nrChannels() )
		{
		    totalbandwidth( index ) = fabs( freqs(inforef.nrChannels()-1) - freqs(0) );
		}
		else
		{
		    //*totbwptr = 1.0;
		    totalbandwidth( index ) = 1.0;
		    cout.form("Row #%3d: ",i) << "No total bandwidth info found, default to " 
					      << totalbandwidth( index ) << endl;
		}
	    }

	    //
	    //  The sideband
	    //
	    sideband( index ) = cursideband;
	    //*sidebandptr = 1;	    
	}

	//
	//  Fill in the datarecord
	//
        *sidebandptr = sideband;
	*totbwptr = totalbandwidth;
	*ifwidthptr = channelwidth;
	*iffreqptr = frequencyoffset;

        //
	// Write the current row
        //
        hduptr->putrow( i, datarecord );
    }

    //
    //  add some extra keywords....
    //
    FitsKeyword*      kwptr;
    FitsKeywordList   ek;

    ek.mk( "NO_IF", (Int)nrifs );

    ek.first();
    ek.next();
    kwptr=ek.curr();
    while( kwptr )
    {
        hduptr->addExtraKeyword( *kwptr );
        kwptr=ek.next();
    }

    //
    //  Copy the HDUObject pointer into the retval
    //
    ref2hduptr(0)=hduptr;
    
    return True;
}



fqtabGenerator::~fqtabGenerator()
{
}

