// gctabGenerator.cc
//
// Implementation of the gctabGenerator class. See gctabGenerator.h for details.
//
//
// Author:
//
//      Harro Verkouter   12-3-1998
//
// $Id: gctabGenerator.cc,v 1.2 2000/08/24 13:54:30 verkout Exp $

#ifndef GCTABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/gctabGenerator.h>
#endif


//
//  Aips++ includes
//
#ifndef HDUOBJECT_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/HDUObject.h>
#endif

#ifndef AIPS_MSCOLUMNS_H
#include <aips/MeasurementSets/MSColumns.h>
#endif

//
//  My includes
//
#ifndef HDUGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/hduGenerator.h>
#endif

#ifndef MSCONVERSIONINFO_H_INC
#include <vlbi/ms2uvfitsimpl/MSConversionInfo.h>
#endif

#ifndef FITSPARAMETER_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSParameter.h>
#endif

#ifndef FITSBINARYCOLUMN_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryColumn.h>
#endif

#ifndef AIPS_VECTOR_H
#include <aips/Arrays/Vector.h>
#endif

#ifndef AIPS_MATRIX_H
#include <aips/Arrays/Matrix.h>
#endif


#ifndef FITSBINARYTABLEHDU_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryTableHDU.h>
#endif

#ifndef AIPS_RECORD_H
#include <aips/Containers/Record.h>
#endif

#ifndef AIPS_RECORDFIELD_H
#include <aips/Containers/RecordField.h>
#endif

#ifndef AIPS_CONSTANTS_H
#include <aips/Mathematics/Constants.h>
#endif

#ifndef AIPS_TABLEITERATOR_H
#include <aips/Tables/TableIter.h>
#endif

#ifndef AIPS_EXPR_NODE_H
#include <aips/Tables/ExprNode.h>
#endif

//
//  Create a default gctab generator, nothing to do yet...
//
gctabGenerator::gctabGenerator() :
    hduGenerator()
{
}


//
//  This function will do all the work of getting all the necessary
//  TSYS information out of the MSConversionInfo object. It will
//  then create a binary table extension HDU and pass that back to the
//  caller!
//
Bool gctabGenerator::convert( Vector<HDUObject*>& ref2hduptr, MSConversionInfo& inforef ) const
{
    cout << "Making GC table" << endl;
    //
    //  First, we make sure the return-value is empty!
    //
    ref2hduptr.resize( 0 );

    //
    //  Local variables we need
    //
    Table               syscal( inforef.getMS().sysCal() );
    Table               sortedcal;
    Table               sortedcalunique;
    Record              rowdata;
    ROMSColumns         mscol( inforef.getMS() );
    Block<String>       sortnames(3);
    Vector<Float>       tsysval;
    ROMSSysCalColumns   sysCalColumns( inforef.getMS().sysCal() );
    ROMSAntennaColumns  anttab( inforef.getMS().antenna() );
    FITSBinaryTableHDU* gctabptr( new FITSBinaryTableHDU("AIPS GC", 1) );

    //
    //  Counters etc.
    //
    Int                 nrantennas;
    Int                 npol;
    Int                 nchan,nstk;
    Int                 nsamplesperantenna;
    uInt                i;
    Double              sttim;
    Double              reftime;
    Double              hourangle;
    IPosition           shp;
    IPosition           shape;
    const uInt          nrow( sysCalColumns.time().nrow() );
    Vector<Float>       havec;
    TableIterator       tabiter;
    
    

    if( !nrow )
    {
        cout << "No syscal info" << endl;
        return False;
    }

    //
    // Determine if we have to skip the first time.
    // This is needed for WSRT MS's, where the first time in the SYSCAL
    // table is the average at the middle of the observation.
    //
    sttim = sysCalColumns.time()( 0 );
    for( i=0; i<nrow; i++ )
    {
	Double tim( sysCalColumns.time()(i) );

	if( tim!=sttim )
        {
	    if( tim<sttim )
            {
                cout << "GCtab: First time in SYSCAL table is an average and will be skipped" << endl;
                syscal = syscal( syscal.nodeRownr()>=Int(i) );
	    }
	    break;
	}
    }

    //
    //  Having done this, we need to sort the syscal table
    //
    sortnames[0]= "ANTENNA_ID";
    sortnames[1]= "ARRAY_ID";
    sortnames[2]= "SPECTRAL_WINDOW_ID";
    
    sortedcal = syscal.sort( sortnames );
    
    //
    //  Redo it, and remove duplicates...
    //
    sortedcalunique = sortedcal.sort( sortnames, Sort::Ascending, Sort::InsSort+Sort::NoDuplicates );

    //
    //  Now we can (finally) deduce how many antennas there are
    //
    nrantennas = sortedcalunique.nrow();
    
    if( !nrantennas )
    {
        cout << "No syscal GC info!" << endl;
        return False;
    }
    
    //
    //  Infer the number of polarizations from the shape of the
    //  trx-column
    //
    npol = (sysCalColumns.trx().shape(0))( 0 );
    
    cout << "Found " << nrantennas << " GC table entries" << endl;

    //
    // Find out the hour-angle?
    //
    shp=mscol.data().shape( 0 );
    
    nstk=shp( 0 );
    nchan=shp( 1 );
    
    reftime=(inforef.observingDate()).second();

    Int            rownr( -1 );
    Double         d( 0.0 );
    Vector<Double> uvw;
    
    while( d==0.0 )
    {
        rownr++;
        
        Vector<Double>  pos1( anttab.position()(mscol.antenna1()(rownr)) );
        Vector<Double>  pos2( anttab.position()(mscol.antenna2()(rownr)) );
       
        d = pos2(0)-pos1(0);
    }
    
    mscol.uvw().get( rownr, uvw );
    
    //
    //  We have uvw, now backtrace to hour-angle
    //
    hourangle= acos(uvw(0)/d)/C::degree;

    //
    // acos returns a value between 0 and pi.
    // Find the correct quadrant by looking at the sign of v.
    // Note that v=d*sin(ha)*sin(dec), thus sign(v)==sign(sin(ha))
    // (since dec>0 for WSRT).
    //
    if( uvw(1)<0 )
    {
        hourangle = -hourangle;
    }


    //
    // Now step through the sorted NOT unique syscal table to find out the
    // 'chunks'?
    //
    tabiter= TableIterator( sortedcal, sortnames, TableIterator::Ascending, TableIterator::NoSort );
    
    Table             tablechunk( tabiter.table() );
    MSSysCal          chunkcal( tablechunk );
    ROMSSysCalColumns chunkcalcols( chunkcal );
    uInt              n( tablechunk.nrow() );
    
    shape=IPosition(1,n);
    
    havec.resize( n );
    for( uInt k=0; k<n; k++ )
    {
        havec(k)= hourangle+( (chunkcalcols.time()(i)-reftime)/C::hour * 15.0 );
    }


    //
    //  Now seems to be a reasonable time to start building the table!
    //
    gctabptr->addColumn( FITSBinaryColumn("ANTENNA_NO", TpInt) );
    gctabptr->addColumn( FITSBinaryColumn("SUBARRAY", TpInt) );
    gctabptr->addColumn( FITSBinaryColumn("FREQ ID", TpInt) );
    gctabptr->addColumn( FITSBinaryColumn("TYPE_1", TpInt) );
    gctabptr->addColumn( FITSBinaryColumn("NTERM_1", TpInt) );
    gctabptr->addColumn( FITSBinaryColumn("X_TYP_1", TpInt) );
    gctabptr->addColumn( FITSBinaryColumn("Y_TYP_1", TpInt) );
    gctabptr->addColumn( FITSBinaryColumn("X_VAL_1", TpFloat) );
    gctabptr->addColumn( FITSBinaryColumn("Y_VAL_1", TpArrayFloat, shape, "DEGREES") );
    gctabptr->addColumn( FITSBinaryColumn("GAIN_1", TpArrayFloat, shape) );
    gctabptr->addColumn( FITSBinaryColumn("SENS_1", TpFloat, IPosition(1,1), "K/JY") );

    if( npol==2 )
    {
        gctabptr->addColumn( FITSBinaryColumn("TYPE_2", TpInt) );
        gctabptr->addColumn( FITSBinaryColumn("NTERM_2", TpInt) );
        gctabptr->addColumn( FITSBinaryColumn("X_TYP_2", TpInt) );
        gctabptr->addColumn( FITSBinaryColumn("Y_TYP_2", TpInt) );
        gctabptr->addColumn( FITSBinaryColumn("X_VAL_2", TpFloat) );
        gctabptr->addColumn( FITSBinaryColumn("Y_VAL_2", TpArrayFloat, shape, "DEGREES") );
        gctabptr->addColumn( FITSBinaryColumn("GAIN_2", TpArrayFloat, shape) );
        gctabptr->addColumn( FITSBinaryColumn("SENS_2", TpFloat, IPosition(1,1), "K/JY") );
    }

    //
    //  Restructure the record so it will hold all of the recordfields!
    //
    rowdata.restructure( gctabptr->getRowDescription() );
    

    RecordFieldPtr<Int>           antenna( rowdata, "ANTENNA_NO");
    RecordFieldPtr<Int>           arrayId( rowdata, "SUBARRAY");
    RecordFieldPtr<Int>           spwId( rowdata, "FREQ ID");
    RecordFieldPtr<Int>           type1( rowdata, "TYPE_1");
    RecordFieldPtr<Int>           nterm1( rowdata, "NTERM_1");
    RecordFieldPtr<Int>           xtype1( rowdata, "X_TYP_1");
    RecordFieldPtr<Int>           ytype1( rowdata, "Y_TYP_1");
    RecordFieldPtr<Float>         xval1( rowdata, "X_VAL_1");
    RecordFieldPtr<Array<Float> > yval1( rowdata, "Y_VAL_1");
    RecordFieldPtr<Array<Float> > gain1( rowdata, "GAIN_1");
    RecordFieldPtr<Float>         sens1( rowdata, "SENS_1");
    RecordFieldPtr<Int>           type2; 
    RecordFieldPtr<Int>           nterm2; 
    RecordFieldPtr<Int>           xtype2;
    RecordFieldPtr<Int>           ytype2;
    RecordFieldPtr<Float>         xval2; 
    RecordFieldPtr<Array<Float> > yval2;
    RecordFieldPtr<Array<Float> > gain2;
    RecordFieldPtr<Float>         sens2;

    if( npol==2 )
    {
	type2  = RecordFieldPtr<Int>( rowdata, "TYPE_2");
	nterm2 = RecordFieldPtr<Int>( rowdata, "NTERM_2");
	xtype2 = RecordFieldPtr<Int>( rowdata, "X_TYP_2");
	ytype2 = RecordFieldPtr<Int>( rowdata, "Y_TYP_2");
	xval2  = RecordFieldPtr<Float>( rowdata, "X_VAL_2");
	yval2  = RecordFieldPtr<Array<Float> >( rowdata, "Y_VAL_2");
	gain2  = RecordFieldPtr<Array<Float> >( rowdata, "GAIN_2");
	sens2  = RecordFieldPtr<Float>( rowdata, "SENS_2");
    }

    //
    //  Count how many rows in the resulting table.....
    //
    uInt    nrfitsrows( 0 );
    
    while( !tabiter.pastEnd() )
    {
        nrfitsrows++;
        tabiter++;
    }
    
    gctabptr->setNumberRows( nrfitsrows );

    //
    // Restart the iterator
    //
    tabiter.reset();
    
    //
    //  Write out the table!
    //

    //
    // the number of rows in the table has already been set, so now we
    // can use the variable to index into the fits-table!
    //
    nrfitsrows=0;
    
    nsamplesperantenna = -1;

    while( !tabiter.pastEnd() )
    {
        //
        //  These variables only live during this loop....
        //
	Table             tableChunk( tabiter.table() );
	MSSysCal          syscal( tableChunk );
	Vector<Float>     vec;
	Matrix<Float>     gaindata;
	ROMSSysCalColumns sysCalColumns( syscal );

        //
        //  Fill in the rowrecord:
        //
    	*antenna = sysCalColumns.antennaId()(0)+1;
	*arrayId = sysCalColumns.arrayId()(0)+1;
	*spwId   = sysCalColumns.spectralWindowId()(0)+1;

	if( Int(tableChunk.nrow())!=shape(0) )
        {
	    cout <<"SysCal table is irregular!\nMismatching #rows for antenna " << *antenna << endl;
            return False;
	}
        
	*type1 = 1;               // tabulated values
	*nterm1 = sysCalColumns.antennaId().nrow();
	*xtype1 = 0;              // none
	*ytype1 = 3;              // hourangle
	*xval1 = 0;

	//
	// Get the number of samples per anttena, so we can fill in
	// the extra keyword 'NO_TABS' later on
	//
	if( nsamplesperantenna<0 )
	{
	    nsamplesperantenna = *nterm1;
	}
	

	sysCalColumns.trx().getColumn( gaindata );

        vec=gaindata.row(0);

	*yval1 = havec;
	*gain1 = vec;
	*sens1 = 1;

	if( npol==2 )
        {
	    *type2 = 1;               // tabulated values
	    *nterm2 = sysCalColumns.antennaId().nrow();
	    *xtype2 = 0;
	    *ytype2 = 3;
	    *xval2 = 0;
	    *yval2 = havec;
            *gain2 = gaindata.row(1);
	    *sens2 = 1;
	}
        //
	// Write the current row
        //
        gctabptr->putrow( nrfitsrows, rowdata );
        
	tabiter++;
        nrfitsrows++;
    }

    //
    //  Add extra keywords....
    //
    FitsKeyword*      kwptr;
    FitsKeywordList   ek;

    ek.mk( "OBSCODE", "" );
    ek.mk( "NO_POL", npol );
    ek.mk( "NO_STKD", nstk );
    ek.mk( "STK_1", -5 );
    ek.mk( "NO_BAND", 1 );
    ek.mk( "NO_CHAN", nchan );
    ek.mk( "REF_FREQ", inforef.getReferenceFrequency(MSConversionInfo::CenterChannel) );
    ek.mk( "CHAN_BW", inforef.getChannelwidth() );
    ek.mk( "REF_PIXL", (Float)1 );
    ek.mk( "NO_TABS", nsamplesperantenna );
    ek.mk( "TABREV", 2 );
    

    ek.first();
    ek.next();
    kwptr=ek.curr();
    while( kwptr )
    {
        gctabptr->addExtraKeyword( *kwptr );
        kwptr=ek.next();
    }


    //
    //  Prepare return value...
    //
    ref2hduptr.resize( 1 );
    ref2hduptr(0)=gctabptr;


    return True;
}


gctabGenerator::~gctabGenerator()
{
}

