// class:
//
//     hduGenerator
//
//
// purpose:
//
//
//     To generate HDU's (+ data!) from a MS, which can then be
//     written to a fitsfile! This class merely serves a s a
//     base-class: the specific kind of hdu generated depends on the
//     type of hduGenerator used. As an example, a antabGenerator will
//     be created. This Generator takes a MS and retrieves from it the
//     info it needs to create a valid Classic aips ANTAB binary table
//     extension HDU!
//
//
// Author:      Harro Verkouter  12-3-1998
//
//   $Id: hduGenerator.h,v 1.1.1.1 2000/03/20 15:16:47 verkout Exp $
#ifndef HDUGENERATOR_H_INC
#define HDUGENERATOR_H_INC

#ifndef CONVERTERBASE_H_INC
#include <vlbi/tmplsrc/ConverterBase.h>
#endif

#ifndef HDUOBJECT_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/HDUObject.h>
#endif

#ifndef AIPS_VECTOR_H
#include <aips/Arrays/Vector.h>
#endif

//
//  I think we can get away with these as forward declarations: we're
//  not going to use them other than by reference!
//
class MSConversionInfo;


class hduGenerator :
    public virtual ConverterBase<Vector<HDUObject*>, MSConversionInfo>
{
public:
    //
    //  The default ctor: nothing to do!
    //
    hduGenerator();
    
    //
    // Ok. The only public method: generate the HDU(s) (below the
    // surface, we use the 'convert', which is virtual)
    //
    //  Upon succesful generation, returns non-empty vector. Otherwise
    //  vector with no elements is returned! 
    //
    //  Note: 
    //        +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //
    //        The caller is responsible for deleting the HDU object!!!!!!
    //
    //        +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    Vector<HDUObject*>  generate( MSConversionInfo& inforef ) const;
    
    //
    //  The dtor: (nothing to do)
    //
    virtual ~hduGenerator();

protected:
    //
    //  This function call is protected: not to be used by the caller
    //  directly: we supply the generate method for that! Derived
    //  classes should implement this method!
    //
    virtual Bool convert( Vector<HDUObject*>& ref2hduptr, MSConversionInfo& inforef ) const;

    
private:
    //
    //  The objects private parts
    //

    //
    //  Prevent copying/assignment
    //
    hduGenerator( const hduGenerator& );
    hduGenerator& operator=( const hduGenerator& );
};


#endif
