//
//  Really do generate an idiantenna HDU...
//
//  Author:  Harro Verkouter,  22-09-1999
//
//
//  $Id: idiantennaGenerator.cc,v 1.2 2000/08/24 13:54:30 verkout Exp $
//
//  $Log: idiantennaGenerator.cc,v $
//  Revision 1.2  2000/08/24 13:54:30  verkout
//  HV: Dang! Forgot to do REF_FREQ issues... they been done now!
//
//  Revision 1.1.1.1  2000/03/20 15:16:47  verkout
//  HV: Imported aips++ implement-stuff
//
//
#ifndef IDIANTENNAGENERATOR_H
#include <vlbi/ms2uvfitsimpl/Generators/idiantennaGenerator.h>
#endif

//
//  Aips++ includes
//
#ifndef HDUOBJECT_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/HDUObject.h>
#endif

#ifndef AIPS_MSCOLUMNS_H
#include <aips/MeasurementSets/MSColumns.h>
#endif

//
//  My includes
//
#ifndef MSCONVERSIONINFO_H_INC
#include <vlbi/ms2uvfitsimpl/MSConversionInfo.h>
#endif

#ifndef FITSAXIS_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSAxis.h>
#endif

#ifndef FITSPARAMETER_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSParameter.h>
#endif

#ifndef FITSBINARYCOLUMN_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryColumn.h>
#endif

#ifndef AIPS_ARRAYMATH_H
#include <aips/Arrays/ArrayMath.h>
#endif

#ifndef FITSBINARYTABLEHDU_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryTableHDU.h>
#endif

#ifndef AIPS_RECORD_H
#include <aips/Containers/Record.h>
#endif

#ifndef AIPS_RECORDFIELD_H
#include <aips/Containers/RecordField.h>
#endif

#ifndef AIPS_EXPRNODE_H
#include <aips/Tables/ExprNode.h>
#endif

#ifndef AIPS_FITS_UTIL_H
#include <trial/FITS/FITSUtil.h>
#endif

#ifndef EXCEPTIONOBJECT_H
#include <vlbi/Utilities/AIPSExceptionObject.h>
#endif

#ifndef AIPS_STOKES_H
#include <aips/Measures/Stokes.h>
#endif

//#include <vlbi/ms2uvfitsimpl/DumpRecord.h>


//
//  Create a default idiantenna generator, nothing to do yet...
//
idiantennaGenerator::idiantennaGenerator() :
    hduGenerator()
{
}

//
//  This function will do all the work of getting all the necessary
//  antenna information out of the MSConversionInfo object. It will
//  then create a binary table extension HDU and pass that back to the
//  caller!
//
Bool idiantennaGenerator::convert( Vector<HDUObject*>& ref2hduptr, MSConversionInfo& inforef ) const
{
    cout << "Making ANTENNA table" << endl;
    //
    //  First, make a list off all the stuff that we may need!
    //
    Int                    nrantennas;
    uInt                   currentArrayNumber;
    uInt                   currentAntennaNumber;
    uInt                   nopcal( 0 );
    uInt                   nRows;
    String                 arrayname;
    String                 rdate;
    MSFeed                 orgfeed( inforef.getMS().feed() );
    MSArray                arrayTable( inforef.getMS().array() );
    MSAntenna              organtenna( inforef.getMS().antenna() );
    Vector<Double>         arraypos;
    FITSBinaryColumn       time( "TIME", "Midpoint of period covered", "1D", "DAYS" );
    FITSBinaryColumn       interval( "TIME_INTERVAL", "Duration of period covered", "1D", "DAYS" );
    FITSBinaryColumn       anname( "ANNAME", "The antennaname", "8A", "" );
    FITSBinaryColumn       anno( "ANTENNA_NO", "The antenna nr", "1J", "" );
    FITSBinaryColumn       array( "ARRAY", "Arraynumber", "1J", "" );
    FITSBinaryColumn       freqid( "FREQID", "Frequency ID", "1J", "" );
    FITSBinaryColumn       diglev( "NO_LEVELS", "Nr of digitizer levels", "1J", "" );
    FITSBinaryColumn       poltya( "POLTYA", "Polarization type", "1A", "" );
    FITSBinaryColumn       polaa( "POLAA", TpArrayFloat, IPosition(1, inforef.nrIFs()), "DEGREES" );
    FITSBinaryColumn       polcala( "POLCALA", TpArrayFloat, IPosition(2, nopcal, inforef.nrIFs()) );
    FITSBinaryColumn       poltyb( "POLTYB", "Polarization type", "1A", "" );
    FITSBinaryColumn       polab( "POLAB", TpArrayFloat, IPosition(1, inforef.nrIFs()), "DEGREES" );
    FITSBinaryColumn       polcalb( "POLCALB", TpArrayFloat, IPosition(2, nopcal, inforef.nrIFs()) );

    //
    //  Create a new Binary Table object
    //
    FITSBinaryTableHDU*    hduptr( new FITSBinaryTableHDU("ANTENNA", 1) );

    //
    //  All the MS columns that we need
    //
    ROScalarColumn<Double> timecol( inforef.getMS(), MS::columnName(MS::TIME) );

    //
    //  We may as well already resize the return va;ue vector
    //
    ref2hduptr.resize( 1 );
    ref2hduptr=(HDUObject*)0;

    //
    //  Configure the binary table
    //
    hduptr->addColumn( time );
    hduptr->addColumn( interval );	
    hduptr->addColumn( anname );
    hduptr->addColumn( anno );
    hduptr->addColumn( array );
    hduptr->addColumn( freqid );
    hduptr->addColumn( diglev );
    hduptr->addColumn( poltya );
    hduptr->addColumn( polaa );
    hduptr->addColumn( polcala );
    hduptr->addColumn( poltyb );
    hduptr->addColumn( polab );
    hduptr->addColumn( polcalb );

    //
    //  Let's count how many rows the resulting table should have.....
    //
    nRows = 0;
    for( currentArrayNumber=0; currentArrayNumber<arrayTable.nrow(); currentArrayNumber++ )
    {
	Table    antsperarray( organtenna(organtenna.col(MSAntenna::columnName(MSAntenna::ARRAY_ID))
					  ==Int(currentArrayNumber)) );
	
	nRows += antsperarray.nrow();
    }
    
    hduptr->setNumberRows( nRows );
    
    //
    //  Restructure the row-record and make RecordFieldPtrs that point
    //  into the record....
    //
    Record                          tablerow( hduptr->getRowDescription() );
    RecordFieldPtr<Double>          timeptr( tablerow, "TIME" );
    RecordFieldPtr<Double>          intervalptr( tablerow, "TIME_INTERVAL" );    
    RecordFieldPtr<String>          nameptr( tablerow, "ANNAME" );
    RecordFieldPtr<Int>             annoptr( tablerow, "ANTENNA_NO" );
    RecordFieldPtr<Int>             arrayptr( tablerow, "ARRAY" );
    RecordFieldPtr<Int>             freqidptr( tablerow, "FREQID" );
    RecordFieldPtr<Int>             diglevptr( tablerow, "NO_LEVELS" );
    RecordFieldPtr<uChar>           poltyaptr( tablerow, "POLTYA" );
    RecordFieldPtr<Array<Float> >   polaaptr( tablerow, "POLAA" );
    RecordFieldPtr<Array<Float> >   polcalaptr( tablerow, "POLCALA" );
    RecordFieldPtr<uChar>           poltybptr( tablerow, "POLTYB" );
    RecordFieldPtr<Array<Float> >   polabptr( tablerow, "POLAB" );
    RecordFieldPtr<Array<Float> >   polcalbptr( tablerow, "POLCALB" );

    Array<Float>                    arflt( IPosition(2, nopcal, inforef.nrIFs()) );
    arflt = (Float)0.0;
    
    //
    //  Ok. Write everything!
    //
    nRows = 0;
    for( currentArrayNumber=0; currentArrayNumber<arrayTable.nrow(); currentArrayNumber++ )
    {
        //
        //
        //  Now it's safe to SELECT antennas for the current sub-array
        //
        //
        MSAntenna     antennaTable( organtenna( organtenna.col(MSAntenna::columnName(MSAntenna::ARRAY_ID))
                                                ==Int(currentArrayNumber)) );

        //
        // SELECT feeds for the current sub-array.
        //
        MSFeed        feedTable( orgfeed( orgfeed.col(MSFeed::columnName(MSFeed::ARRAY_ID))
                                          ==Int(currentArrayNumber)) );
        
        //
        // Now we create column objects that refence into the SELECTED table
        //
        ROScalarColumn<String> antname( antennaTable, MSAntenna::columnName(MSAntenna::NAME) );
        ROScalarColumn<Int>    antid( antennaTable, MSAntenna::columnName(MSAntenna::ANTENNA_ID) );
        ROArrayColumn<String>  poltype( feedTable, MSFeed::columnName(MSFeed::POLARIZATION_TYPE) );
        
        //
        //  Fill in recordfields that do not change across rows (if any)
        //
        *polaaptr=(Float)0.0;
        *polabptr=(Float)0.0;
        *polcalaptr=arflt;
        *polcalbptr=arflt;

	//
	//  According to doc. values are:
	//
	//  2      -> for MkII/MkIII terminals
	//  2 or 4 -> for VLBA depending on observing mode
	//
        *diglevptr=2;

	*arrayptr    = currentArrayNumber+1;
	*freqidptr   = 1;
	*timeptr     = 0.0;
	*intervalptr = 1.0;

        //
        //  Get the columns....
        //
        for( uInt i=0; i<(uInt)antennaTable.nrow(); i++ )
        {
            //
            //  Name
            //
            *nameptr="";
            
            if( antname.isDefined(i) )
            {
                *nameptr = upcase( antname(i) );
            }
            
            if( *nameptr=="" )
            {
                String       str;
                ostrstream   defantname;
                
                //defantname << "AR" << currentArrayNumber << "AN" << i << ends;
		defantname << "AN" << i << ends;
                str=defantname.str();
                
                cout.form("Row #%3d: ",i) << "Antennaname not defined. Default to "
                                          << defantname.str() << endl;
                
                *nameptr=str;
            }

            //
            //  Number
            //
            *annoptr = antid(i) + 1;
            
            //
            //  We need to fill in the polarization
            //
            Bool          found( False );
            const uInt    nmax( feedTable.nrow() );
            
            //
            //  First set to default value
            //
	    Stokes::StokesTypes  stktyp( Stokes::fromFITSValue(inforef.getFirstStokes()) );
	    String               stkstr( Stokes::name(stktyp) );

            *poltyaptr = stkstr[0];
            *poltybptr = stkstr[1];
	    
	    /*
            for( uInt j=0; j<nmax; j++ )
            {
                if( antid.isDefined(j) && i==antid(j) && poltype.isDefined(j) )
                {
                    Vector<String>  poltypes( poltype(j) );
                    
                    found = True;
                    
                    if( poltypes.nelements()>=1 )
                    {
                        *poltyaptr=poltypes(0)[0];
                    }
                    if( poltypes.nelements()>=2 )
                    {
                        *poltybptr=poltypes(1)[0];
                    }
                }
            }
            if( !found )
            {
                cout.form("Row #%3d: ",i) << "Polarization types could not be determined" << endl;
            }
            */

            //
            //  Do write the row!
            //
            hduptr->putrow( nRows, tablerow );
	    nRows++;
        }        
    }

    //
    //  add some extra keywords....
    //
    FitsKeyword*      kwptr;
    FitsKeywordList   ek;
    
    //
    //  The common keywords
    //
    ek.mk( "TABREV", (Int)1 );
    ek.mk( "OBSCODE", inforef.getObsCode() );
    ek.mk( "NO_STKD", (Int)inforef.nrPolarizations() );
    ek.mk( "STK_1", inforef.getFirstStokes() );
    ek.mk( "NO_BAND", (Int)inforef.nrIFs() );
    ek.mk( "NO_CHAN", (Int)inforef.nrChannels() );
    ek.mk( "REF_FREQ", inforef.getReferenceFrequency(MSConversionInfo::CenterChannel) );	
    ek.mk( "CHAN_BW", inforef.getChannelwidth() );
    ek.mk( "REF_PIXL", (Float)1 );	
    ek.mk( "NOPCAL", (Int)nopcal );
    ek.mk( "POLTYPE", "APPROX" );
    
    ek.first();
    ek.next();
    
    kwptr=ek.curr();
    while( kwptr )
    {
	hduptr->addExtraKeyword( *kwptr );
	kwptr=ek.next();
    }
    
    //
    //  And add the HDUObject to the return value vector
    //
    ref2hduptr(0)=hduptr;
    
    return True;
}



idiantennaGenerator::~idiantennaGenerator()
{
}

