//
//  Do generate the source table!
//
//
//
//
#ifndef IDISOURCEGENERATOR_H
#include <vlbi/ms2uvfitsimpl/Generators/idisourceGenerator.h>
#endif

#ifndef AIPS_MSCOLUMNS_H
#include <aips/MeasurementSets/MSColumns.h>
#endif

//
//  My includes
//
#ifndef MSCONVERSIONINFO_H_INC
#include <vlbi/ms2uvfitsimpl/MSConversionInfo.h>
#endif

#ifndef FITSPARAMETER_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSParameter.h>
#endif

#ifndef FITSBINARYCOLUMN_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryColumn.h>
#endif

#ifndef AIPS_VECTOR_H
#include <aips/Arrays/Vector.h>
#endif

#ifndef AIPS_MATRIX_H
#include <aips/Arrays/Matrix.h>
#endif

#ifndef FITSBINARYTABLEHDU_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryTableHDU.h>
#endif

#ifndef AIPS_RECORD_H
#include <aips/Containers/Record.h>
#endif

#ifndef AIPS_TABLERECORD_H
#include <aips/Tables/TableRecord.h>
#endif

#ifndef AIPS_RECORDFIELD_H
#include <aips/Containers/RecordField.h>
#endif

#ifndef AIPS_CONSTANTS_H
#include <aips/Mathematics/Constants.h>
#endif

#ifndef AIPS_TABLEITERATOR_H
#include <aips/Tables/TableIter.h>
#endif

#ifndef AIPS_EXPR_NODE_H
#include <aips/Tables/ExprNode.h>
#endif

#ifndef AIPS_TABLEDESC_H
#include <aips/Tables/TableDesc.h>
#endif

#ifndef AIPS_MEASURES_H
#include <aips/Measures.h>
#endif

#ifndef AIPS_MDIRECTION_H
#include <aips/Measures/MDirection.h>
#endif

#ifndef AIPS_MEPOCH_H
#include <aips/Measures/MEpoch.h>
#endif

#ifndef AIPS_MVANGLE_H
#include <aips/Quanta/MVAngle.h>
#endif

#ifndef AIPS_MEASUREMENTSET_H
#include <aips/MeasurementSets/MeasurementSet.h>
#endif

//
//  Include this for the getEpoch-function
//
#ifndef SUTABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/sutabGenerator.h>
#endif


//
//  Create a default idisource generator, nothing to do yet...
//
idisourceGenerator::idisourceGenerator() :
    hduGenerator()
{
}


//
//  This function will do all the work of getting all the necessary
//  TSYS information out of the MSConversionInfo object. It will
//  then create a binary table extension HDU and pass that back to the
//  caller!
//

//
//
//  The strategy for the SOURCE NAME is as follows:
//
//  * If the field has a source attached ( source_id!=-1 ), and it's
//  the only field with this source_id, we write the name from the
//  source table.
//
//  * If the field has a source attached AND there is more than one
//  field, we first look if the 'NAME' column is present in the
//  fieldtable. If so, the value in this field will be used. If the
//  'NAME' column is absent (MS vs. < 2.0) we use the string in the
//  'CODE' column as sourcename for the current field.
//
//     In either case the following holds: if the value in the field
//     is empty or not defined, we make up a sourcename ourselves and
//     tell the user we did it.
//
//
//  NOTE: On SOURCE NUMBERING.
//
//  Since Classic AIPS cannot deal with multiple pointings per source
//  and the users still want their data in Classic AIPS, we do the
//  following: every field in the field-table is regarded as a single
//  source (as seen from Classic AIPS point of view). The RA/DEC are
//  taken from the field-table. If there happens to be a physical
//  source of radiation present in the field (indicated by
//  source_id!=-1), the velocity/ IQUV-fluxes etc. are taken from the
//  SOURCE table, otherwise they will be filled in with defaults.
//
//  Errr. I'm drifting from the subject. The source numbering. In the
//  main-table generator, we have to take into account too the fact
//  that we're dealing with multi-source: we add the SOURCE ID-random
//  parameter to the list of parameters. The value we fill in here is
//  the FIELD_ID from the main table. So we do NOT use the SOURCE_ID
//  from the SOURCE-table in aips++.
//
Bool idisourceGenerator::convert( Vector<HDUObject*>& ref2hduptr, MSConversionInfo& inforef ) const
{
    cout << "Making SOURCE table" << endl;
    //
    //  First, we make sure the return-value is empty!
    //
    ref2hduptr.resize( 0 );


    //
    //  Local variables we need
    //
    Int                    unknownsource( 0 );
    Int                    currentsourceid;
    Int                    rownrinsourcetab;
    uInt                   nrifs( inforef.nrIFs() );
    uInt                   nrrows; // The number of rows in the destination table!
    uInt                   pointingspersrc;
    Bool                   hasFieldNameCol;
    Table                  sourcetable( inforef.getMS().source() );
    Table                  fieldtable( inforef.getMS().field() );
    Double                 currentepoch;
    String                 sourcename;
    ROScalarColumn<Int>    sourceidcol( sourcetable, "SOURCE_ID" );
    ROScalarColumn<Int>    fieldsourceid( fieldtable, "SOURCE_ID" );
    ROScalarColumn<Int>    fieldidcol( fieldtable, "FIELD_ID" );
    ROScalarColumn<String> sourcenamecol( sourcetable, "NAME" );
    ROScalarColumn<String> fieldnamecol;
    ROScalarColumn<String> fieldcodecol( fieldtable, "CODE" );
    ROArrayColumn<Double>  sourcedirectioncol( sourcetable, "DIRECTION" );
    ROArrayColumn<Double>  fielddirectioncol( fieldtable, "POINTING_DIR" );
    ROArrayColumn<Double>  propermotioncol( sourcetable, "PROPER_MOTION" );
    ROScalarColumn<Double> sysvelcol( sourcetable, "SYSVEL" );
    ROScalarColumn<Int>    calgroupcol( sourcetable, "CALIBRATION_GROUP" );
    ROScalarColumn<String> sourcecodecol( sourcetable, "CODE" );
    FITSBinaryTableHDU*    sutabptr( new FITSBinaryTableHDU("SOURCE", 1) );

    
    //
    //  Let's try to find out the total number of rows in the
    //  destination SU-table. This number is necessarily equal to the
    //  number of pointings. We will treat (in principle) every
    //  pointing as a different source.
    //
    nrrows=fieldtable.nrow();
    
    //
    //  If, after all checking etc. nrrows==0, it means that there's
    //  nothing here for us!
    //
    if( !nrrows )
    {
	cout << "No source/field information found." << endl;
	return ToBool( false );
    }

    //
    // Check if a fieldname-column is present
    //
    hasFieldNameCol=False;
    
    if( fieldtable.tableDesc().isColumn("NAME") )
    {
	fieldnamecol.attach( fieldtable, "NAME" );
	hasFieldNameCol=True;
    }
    
    //
    //  Having verified that we have to write some stuff, we need to
    //  setup the table!
    //
    IPosition     ifshape( 1, nrifs );
    Array<Float>  nullswithifshape( ifshape );
    Array<uChar>  charswithifshape( ifshape );
    Array<uChar>  fourchars( IPosition(1,4) );
    Array<Double> dnullswithifshape( ifshape );
    
    nullswithifshape=(Float)0.0f;
    charswithifshape=((uChar)' ');
    fourchars=((uChar)' ');
    dnullswithifshape=(Double)0.0;

    sutabptr->addColumn( FITSBinaryColumn("SOURCE_ID", "Source ID", "1J", "") );
    sutabptr->addColumn( FITSBinaryColumn("SOURCE", "Source name", "16A", "") );
    sutabptr->addColumn( FITSBinaryColumn("QUAL", "", "1J", "") );
    sutabptr->addColumn( FITSBinaryColumn("CALCODE", TpArrayUChar, IPosition(1,4)) );
    //sutabptr->addColumn( FITSBinaryColumn("CALCODE", "Calibrator code", "4A", "" ) );
    sutabptr->addColumn( FITSBinaryColumn("FREQID", "", "1J", "") );
    sutabptr->addColumn( FITSBinaryColumn("IFLUX", TpArrayFloat, ifshape, "JY") );
    sutabptr->addColumn( FITSBinaryColumn("QFLUX", TpArrayFloat, ifshape, "JY") );
    sutabptr->addColumn( FITSBinaryColumn("UFLUX", TpArrayFloat, ifshape, "JY") );
    sutabptr->addColumn( FITSBinaryColumn("VFLUX", TpArrayFloat, ifshape, "JY") );
    sutabptr->addColumn( FITSBinaryColumn("ALPHA", TpArrayFloat, ifshape, "") );
    sutabptr->addColumn( FITSBinaryColumn("FREQOFF", TpArrayFloat, ifshape, "HZ") );
    sutabptr->addColumn( FITSBinaryColumn("RAEPO", "RA of epoch", "1D", "DEGREES") );
    sutabptr->addColumn( FITSBinaryColumn("DECEPO", "DEC of epoch", "1D", "DEGREES") );
    sutabptr->addColumn( FITSBinaryColumn("EQUINOX", "Mean equinox", "8A", "") );    
    sutabptr->addColumn( FITSBinaryColumn("RAAPP", "Apparent RA", "1D", "DEGREES") );
    sutabptr->addColumn( FITSBinaryColumn("DECAPP", "Apparent DEC", "1D", "DEGREES") );
    sutabptr->addColumn( FITSBinaryColumn("SYSVEL", TpArrayDouble, ifshape, "M/SEC") );    
    sutabptr->addColumn( FITSBinaryColumn("VELTYP", "Velocity type", "8A", "") );    
    sutabptr->addColumn( FITSBinaryColumn("VELDEF", "Velocity definition", "8A", "") );    
    sutabptr->addColumn( FITSBinaryColumn("RESTFREQ", TpArrayDouble, ifshape, "M/SEC") );
    sutabptr->addColumn( FITSBinaryColumn("PMRA", "", "1D", "DEG/DAY") );
    sutabptr->addColumn( FITSBinaryColumn("PMDEC", "", "1D", "DEG/DAY") );
    sutabptr->addColumn( FITSBinaryColumn("PARALLAX", "Parallax of source", "1E", "ARCSEC") );

    //
    //  Tell the table how many rows it should hold!
    //
    //  NOTE: For every field (or SOURCE, to speak in Classic AIPS
    //  terms) we need to write 'nrfreqids' rows, one for every
    //  frequency setup... At the moment, that's one, may change later
    //  on...
    //
    sutabptr->setNumberRows( nrrows );
    

    //
    //  Having done that, we can now start to use the datarecord and
    //  make recordfieldpointers!
    //
    Record     rowdata( sutabptr->getRowDescription() );

    //
    //  The recordfieldpointers
    //
    RecordFieldPtr<Int>            sourceidptr( rowdata, "SOURCE_ID" );
    RecordFieldPtr<String>         nameptr( rowdata, "SOURCE" );
    RecordFieldPtr<Int>            qualptr( rowdata, "QUAL" );
    //RecordFieldPtr<String>         calcodeptr( rowdata, "CALCODE" );
    //
    //  HV: 05-04-2000
    //  Changed template type of RecordFieldPtr for CALCODE from 
    //  String to Array<uChar> since we must make sure that we
    //  get 4 spaces there!
    //
    RecordFieldPtr<Array<uChar> >  calcodeptr( rowdata, "CALCODE" );
    RecordFieldPtr<Int>            freqidptr( rowdata, "FREQID" );
    RecordFieldPtr<Array<Float> >  ifluxptr( rowdata, "IFLUX" );
    RecordFieldPtr<Array<Float> >  qfluxptr( rowdata, "QFLUX" );
    RecordFieldPtr<Array<Float> >  ufluxptr( rowdata, "UFLUX" );
    RecordFieldPtr<Array<Float> >  vfluxptr( rowdata, "VFLUX" );
    RecordFieldPtr<Array<Float> >  alphaptr( rowdata, "ALPHA" );
    RecordFieldPtr<Array<Float> >  freqoffptr( rowdata, "FREQOFF" );
    RecordFieldPtr<Double>         raepoptr( rowdata, "RAEPO" );
    RecordFieldPtr<Double>         decepoptr( rowdata, "DECEPO" );
    RecordFieldPtr<String>         equinoxptr( rowdata, "EQUINOX" );
    RecordFieldPtr<Double>         raappptr( rowdata, "RAAPP" );
    RecordFieldPtr<Double>         decappptr( rowdata, "DECAPP" );
    RecordFieldPtr<Array<Double> > sysvelptr( rowdata, "SYSVEL" );
    RecordFieldPtr<String>         veltypptr( rowdata, "VELTYP" );
    RecordFieldPtr<String>         veldefptr( rowdata, "VELDEF" );
    RecordFieldPtr<Array<Double> > restfreqptr( rowdata, "RESTFREQ" );
    RecordFieldPtr<Double>         pmraptr( rowdata, "PMRA" );
    RecordFieldPtr<Double>         pmdecptr( rowdata, "PMDEC" );
    RecordFieldPtr<Float>          parallaxptr( rowdata, "PARALLAX" );
    
    //
    //  Now loop over all fields!
    //
    for( uInt fieldnr=0; fieldnr<fieldtable.nrow(); fieldnr++ )
    {
	//
	//  Fill in default stuff
	//
	*qualptr = 1;
	//*calcodeptr = "    ";
	*calcodeptr = fourchars;

	*ifluxptr = nullswithifshape;
	*qfluxptr = nullswithifshape;
	*ufluxptr = nullswithifshape;
	*vfluxptr = nullswithifshape;

	*freqoffptr = nullswithifshape;
	*freqidptr  = 1;

	*alphaptr = nullswithifshape;
	*restfreqptr = dnullswithifshape;
	
	*raepoptr = 0.0;
	*decepoptr = 0.0;

	*pmraptr = 0.0;
	*pmdecptr = 0.0;

	*raappptr = 0.0;
	*decappptr = 0.0;

	*sysvelptr = dnullswithifshape;
	*veltypptr = "LSR";
	*veldefptr = "RADIO";
	*parallaxptr = 0.0f;
	
	*equinoxptr = "";

	//
	// Stuff into source id the current fieldid
	//
	if( !fieldidcol.isDefined(fieldnr) )
	{
	    *sourceidptr = fieldnr+1;
	    
	    cout.form("field %03d: ",fieldnr) << "No fieldid found, defaulting to " << *sourceidptr << endl;
	}
	else
	{
	    *sourceidptr = (fieldidcol(fieldnr)+1);
	}
	
	//
	//  How about the name of the source? Let's first find out if
	//  there's a source 'attached' to this field and if so, if
	//  this is the only field 'attached' to that source
	//
	sourcename="";
	currentsourceid=-1;
	currentepoch=0.0;
	rownrinsourcetab=-1;
	pointingspersrc=0;

	if( fieldsourceid.isDefined(fieldnr) )
	{
	    currentsourceid=fieldsourceid( fieldnr );
	}

	if( currentsourceid!=-1 )
	{
	    //
	    //  Select.... i.e. try to find the row in the source
	    //  table that describes the source.
	    //
	    Table   pointings( fieldtable(fieldtable.col("SOURCE_ID")==currentsourceid) );
	    Table   src( sourcetable(sourcetable.col("SOURCE_ID")==currentsourceid) );	    

	    pointingspersrc=pointings.nrow();
	    
	    //
	    //  Theoretically there should be only one entry in the
	    //  sourcetable with SOURCE_ID==currentsourceid
	    //
	    if( src.nrow()==1 )
	    {
		rownrinsourcetab = src.rowNumbers()( 0 );
	    }
	    
	    //
	    //  If an entry was found in the sourcetable, we take source specific info
	    //  from the sourcetable....
	    //
	    if( rownrinsourcetab>=0 )
	    {
		//
		//  SYSVEL
		//
		if( sysvelcol.isDefined(rownrinsourcetab) )
		{
		    //
		    //  <NOTE> Maybe we need to check the velocity
		    //  reference to (when needed) convert the
		    //  radial velocity to LSR velocity!
		    //
		    *sysvelptr = sysvelcol( rownrinsourcetab );
		}
		
		//
		//  Proper motion
		//
		if( propermotioncol.isDefined(rownrinsourcetab) )
		{
		    Vector<Double>   pm;
		    
		    propermotioncol.get( rownrinsourcetab, pm );
		    
		    *pmraptr = pm(0);
		    *pmdecptr = pm(1);

// 		    *pmraptr = propermotioncol(rownrinsourcetab)( 0 );
// 		    *pmdecptr = propermotioncol(rownrinsourcetab)( 1 );
		}
		
		//
		//  Calibration group
		//
		if( calgroupcol.isDefined(rownrinsourcetab) )
		{
		    *qualptr = calgroupcol( rownrinsourcetab );
		}

		//
		//  Source code:
		//
		//  NOTE: in the MS this is a string; the length of the string should be
		//  equal to the number of IFs, one character/IF!
		//
		if( sourcecodecol.isDefined(rownrinsourcetab) )
		{
		    const String&  tmpdata( sourcecodecol(rownrinsourcetab) );
		    
		    //
		    //  Copy at most 4 characters...
		    //
		    *calcodeptr = fourchars;
		    for( int ccnt=0; ccnt<4 && ccnt<tmpdata.length(); ccnt++ )
		    {
			(*calcodeptr)( IPosition(1,ccnt) ) = tmpdata[ccnt];
		    }
		}
	    }
	}

	//
	//  Now let's see. We first check if there's a fieldname
	//  column. If not present or the field is empty, we check the
	//  fieldcode-column. If that field is empty, we can fall back
	//  on the source table (if a source was attached). If, after
	//  that, the sourcename is still empty, we make up something.
	//
	if( hasFieldNameCol && fieldnamecol.isDefined(fieldnr) )
	{
	    sourcename = upcase( fieldnamecol(fieldnr) );
	}
	
	if( sourcename=="" && fieldcodecol.isDefined(fieldnr) )
	{
	    sourcename = upcase( fieldcodecol(fieldnr) );
	}

	if( sourcename=="" && pointingspersrc==1 )
	{
	    //
	    //  One pointing per source: we can use the sourcename as
	    //  a unique name. But tell the user we did it.
	    //
	    if( rownrinsourcetab>=0 && sourcenamecol.isDefined(rownrinsourcetab) )
	    {
		sourcename = upcase( sourcenamecol(rownrinsourcetab) );
		
		if( sourcename!="" )
		{
		    cout.form("field %03d: ",fieldnr) 
			<< "No fieldname found. Using unique name from SOURCE table: "
			<< sourcename << endl;
		}
	    }
	}
	
	//
	//  If the sourcename is STILL empty, we have to make
	//  something of our own!
	//
	if( sourcename=="" )
	{
	    char        buffer[32];
	    ostrstream  namestr( buffer, 32 );
	    
	    namestr << "FIELD" << unknownsource++ << ends;
	    
	    sourcename = namestr.str();
	    
	    //
	    //  Tell usr. what we did....
	    //
	    cout.form("field %03d: ",fieldnr) << "No fieldname found, defaulting to " << sourcename << endl;
	}

	//
	//  Ok. Over to the field position now!
	//
	if( fielddirectioncol.isDefined(fieldnr) )
	{
		const MEpoch       rdate( inforef.observingDate().get(), MEpoch::UTC );
		MDirection         earthcenter( MVPosition(0.0, 0.0, 0.0), MDirection::ITRF );
		MeasFrame          appframe( earthcenter, rdate );
		Vector<Double>     apppos;
		MDirection         deldir;
		MDirection         appdir;
		MDirection::Types  type;
	    String             refkw( "MEASURE_REFERENCE" );
	    const TableRecord& keywordset( fielddirectioncol.keywordSet() );
    
	    Vector<Double>     fieldpos;
	    MVAngle            fieldra;
	    MVAngle            fielddec;

	    fielddirectioncol.get( fieldnr, fieldpos );

	    //
	    // 'fieldpos' in MS is in units af radians; in FITS we expect DEG(rees)
	    //
	    fieldra  = MVAngle( fieldpos(0) );
	    fielddec = MVAngle( fieldpos(1) );
	    
		type = MDirection::J2000;
    	if( keywordset.isDefined(refkw) && keywordset.dataType(refkw)==TpString )
	    {
    		String             tmp;

			keywordset.get( refkw, tmp );

			MDirection::getType( type, tmp );
	    }

	   	deldir = MDirection( MVDirection(fieldra.get(), fielddec.get()), type );
		appdir = MDirection::Convert( deldir,
									  MDirection::Ref(MDirection::APP, appframe) )();		
 
	    *raepoptr  = fieldra.degree();
	    *decepoptr = fielddec.degree();
	    
		apppos     = (appdir.getValue()).get();
		fieldra    = MVAngle( apppos(0) );
		fielddec   = MVAngle( apppos(1) );

	    *raappptr  = fieldra.degree();
	    *decappptr = fielddec.degree();
	}
	else
	{
	    cout.form("field %03d: ",fieldnr) << "No fieldposition found, defaulting to "
					      << "(" << *raappptr << "," << *decappptr << ")"
					      << endl;
	}

	//
	//  The epoch
	//
	String    equinoxstring;
	
	currentepoch = getEpoch( fielddirectioncol );
	
	switch( (Int)currentepoch )
	{
	    case 1950:
		equinoxstring = "1950.0B";
		break;
		
	    case 2000:
		equinoxstring = "J2000";
		break;
		
	    default:
		equinoxstring = "J2000";		
		cout.form("field %03d: ",fieldnr) << "No EPOCH found, defaulting to "
						  << equinoxstring << endl;
		break;
	}
	
	//
	//  Do write the name in the record!
	//
	*nameptr = sourcename;
	*equinoxptr = equinoxstring;

	//
	//  Do not forget to write the row....
	//
	sutabptr->putrow( fieldnr, rowdata );
    }

    //
    // extract velocity definition/frame info from the sysvel column
    //
    String             tmp;
    String             refkw( "MEASURE_REFERENCE" );
    const TableRecord& keywordset( sysvelcol.keywordSet() );
    
    if( keywordset.isDefined(refkw) && keywordset.dataType(refkw)==TpString )
    {
	keywordset.get( refkw, tmp );
    }

    //
    //  Add extra keywords....
    //
    FitsKeyword*      kwptr;
    FitsKeywordList   ek;

    //
    //  The common keywords
    //
    ek.mk( "TABREV", (Int)1 );
    ek.mk( "OBSCODE", inforef.getObsCode() );
    ek.mk( "NO_STKD", (Int)inforef.nrPolarizations() );
    ek.mk( "STK_1", inforef.getFirstStokes() );
    ek.mk( "NO_BAND", (Int)inforef.nrIFs() );
    ek.mk( "NO_CHAN", (Int)inforef.nrChannels() );
    ek.mk( "REF_FREQ", inforef.getReferenceFrequency(MSConversionInfo::CenterChannel) );	
    ek.mk( "CHAN_BW", inforef.getChannelwidth() );
    ek.mk( "REF_PIXL", (Float)1 );
    
    //
    //  Add the keywords to the HDU!
    //
    ek.first();
    ek.next();
    kwptr=ek.curr();
    while( kwptr )
    {
        sutabptr->addExtraKeyword( *kwptr );
        kwptr=ek.next();
    }


    //
    //  Prepare return value...
    //
    ref2hduptr.resize( 1 );
    ref2hduptr(0)=sutabptr;
    
    return True;
}


idisourceGenerator::~idisourceGenerator()
{
}

