//
//  Implementation of class dummyhduGenerator
//
//
//
//   Author:   Harro Verkouter
//             05-06-1998
// 
//   $Id: maingroupGenerator.cc,v 1.5 2000/11/16 23:36:31 verkout Exp $
//
//   $Log: maingroupGenerator.cc,v $
//   Revision 1.5  2000/11/16 23:36:31  verkout
//   HV: There is something about standards. We thought we were writing consistent FITS, unfortunately not up to the Classic AIPS expectations. Stuff has been fixed now: phases inverted as well as all of the signs of U, V and W.
//
//   Revision 1.4  2000/09/22 13:17:13  verkout
//   HV: Updated time -> day/dayfraction calculation. There was a bug. Showed up in FITS-IDI files. It's ok now!
//
//   Revision 1.3  2000/08/24 13:54:30  verkout
//   HV: Dang! Forgot to do REF_FREQ issues... they been done now!
//
//   Revision 1.2  2000/03/21 08:51:22  verkout
//   HV: By popular demand added the RECEIVER=JIVE keyword to the generators of the main data HDUs
//
//   Revision 1.1.1.1  2000/03/20 15:16:47  verkout
//   HV: Imported aips++ implement-stuff
//
//
//  My includes
//
#ifndef MAINGROUPGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/maingroupGenerator.h>
#endif

//
//  AIPS++ includes
//
#ifndef FITSPRIMARYGROUP_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSPrimaryGroup.h>
#endif


#ifndef MSCONVERSION_H_INC
#include <vlbi/ms2uvfitsimpl/MSConversionInfo.h>
#endif

#ifndef FITSAXIS_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSAxis.h>
#endif

#ifndef FITSPARAMETER_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSParameter.h>
#endif

#ifndef AIPS_RECORD_H
#include <aips/Containers/Record.h>
#endif

#ifndef AIPS_RECORDFIELDID_H
#include <aips/Containers/RecordFieldId.h>
#endif

#ifndef AIPS_RECORDFIELD_H
#include <aips/Containers/RecordField.h>
#endif

#ifndef AIPS_VECTOR_H
#include <aips/Arrays/Vector.h>
#endif

#ifndef AIPS_MATRIX_H
#include <aips/Arrays/Matrix.h>
#endif

#include <aips/Arrays/ArrayMath.h>

#ifndef AIPS_MEASUREMENTSET_H
#include <aips/MeasurementSets/MeasurementSet.h>
#endif

#ifndef AIPS_MSCOLUMNS_H
#include <aips/MeasurementSets/MSColumns.h>
#endif

#ifndef AIPS_MSANTENNACOLUMNS_H
#include <aips/MeasurementSets/MSAntennaColumns.h>
#endif

#ifndef AIPS_FITS_UTIL_H
#include <trial/FITS/FITSUtil.h>
#endif

#ifndef AIPS_TABLEDESC_H
#include <aips/Tables/TableDesc.h>
#endif

#ifndef AIPS_TABLERECORD_H
#include <aips/Tables/TableRecord.h>
#endif

#ifndef AIPS_MVTIME_H
#include <aips/Measures/MVTime.h>
#endif

#ifndef AIPS_STOKES_H
#include <aips/Measures/Stokes.h>
#endif

#ifndef AIPS_GENSORT_H
#include <aips/Utilities/GenSort.h>
#endif

#ifndef AIPS_TABLEITER_H
#include <aips/Tables/TableIter.h>
#endif

#ifndef EXCEPTIONOBJECT_H
#include <vlbi/Utilities/AIPSExceptionObject.h>
#endif


#include <vlbi/ms2uvfitsimpl/DumpRecord.h>

#include <strstream.h>
#include <fstream.h>

#define KOUT( a )  (cout << #a << "=" << a << endl)


//
// Translate time (in MJD seconds) to JD number and dayfraction
//
void maingroupGenerator::timeToDay( Double& day, Double& dayfraction, Double time )
{
    const Double  JDofMJD0( 2400000.5 );
	
	//
	//  First, go from MJD(s) to MJD(d)
	//
	time /= C::day;

	//
	//  Now split into whole days and day fraction
	//
	day         =  ::floor(time);
	dayfraction =  time - ::floor(time);

	//
	//  Go from whole MJD days to whole JD days, ie
	//  add offset...
	//
	day         += JDofMJD0;

    //time /= C::day; // now in days;
    //time += JDofMJD0; // now in JD
    
    //day = (Int)::floor(time);
    //dayfraction = time - ::floor(time);
    return;
}





//
//  Default c'tor: nothing to do
//
maingroupGenerator::maingroupGenerator()
{
}


//
//  This function performs all the dirty work. It creates a dummy hdu
//  object! It is the callers response to delete the object if he/she
//  no longer desires to use the object!
//
Bool maingroupGenerator::convert( Vector<HDUObject*>& ref2hduptr, MSConversionInfo& inforef ) const
{
    cout << "Making Maingroup" << endl;
    //
    //  The idea is that we first attempt to gather info, then create
    //  the axes/parameters, configure the hdu, out of speed
    //  considerations create the ROColumnObjects for the columns we
    //  need to access, set up RecordFieldPtrs for fields we want to
    //  write and then: Off we go!
    //


    //
    //  Resize the return value vector
    //
    ref2hduptr.resize( 1 );
    ref2hduptr=(HDUObject*)0;


    //
    //  Some general info, like ra/dec, sourcename
    //
    Bool                   usewavelength( False );
    Bool                   hasWeightSpectrum( False );
    Bool                   delSrcdataptr;
    Bool                   delWeightptr;
    Bool                   delFlagptr;
    Bool                   delDstdataptr;
    Bool                   delStokesIndptr;
	Double                 day;
    Double                 epoche( 2000.0 );  // NOTE: default epoch=J2000!
    Double                 dayFraction;
    Double                 wavelength( C::c/inforef.getReferenceFrequency(MSConversionInfo::CenterChannel) );
    String                 objectname;
    String                 bunit( "UNCALIB" );
    String                 telescope;
    String                 instrument;
    String                 uuname;
    String                 vvname;
    String                 wwname;
    MSField                fieldTable( inforef.getMS().field() );
    MSArray                arrayTable( inforef.getMS().array() );
    Vector<Int>            stokes( inforef.nrPolarizations() );
    const Double           oneOverC( 1.0 / C::c );
    Vector<uInt>           stokesindex( inforef.nrPolarizations() );
    Vector<Double>         radec;
    Vector<Double>         uvw( 3 );
    MSSpectralWindow       spwinTable( inforef.getMS().spectralWindow() );
    FITSPrimaryGroup*      newhdu( new FITSPrimaryGroup(TpFloat) );

    //
    //  Do create column references for all the columns that we need to access
    //
    ROArrayColumn<Double>  indir( fieldTable, MSField::columnName(MSField::PHASE_DIR) );
    ROScalarColumn<String> inname( fieldTable, MSField::columnName(MSField::NAME) );
    ROArrayColumn<Complex> datacolumn( inforef.getMS(), inforef.getDataColumnName() );
    ROScalarColumn<String> inarrayname( arrayTable, MSArray::columnName(MSArray::NAME) );    
    ROArrayColumn<Int>     stokesTypes( spwinTable, MSSpectralWindow::columnName(MSSpectralWindow::CORR_TYPE) );
    ROScalarColumn<Float>  weight( inforef.getMS(), MS::columnName(MS::WEIGHT) );
    ROArrayColumn<Float>   weightspectrum;
    ROScalarColumn<Bool>   rowflag( inforef.getMS(), MS::columnName(MS::FLAG_ROW) );
    ROArrayColumn<Bool>    dataflag( inforef.getMS(), MS::columnName(MS::FLAG) );
    ROArrayColumn<Double>  uvwcol( inforef.getMS(), MS::columnName(MS::UVW) );
    ROScalarColumn<Double> time( inforef.getMS(), MS::columnName(MS::TIME) );
    ROScalarColumn<Int>    ant1( inforef.getMS(), MS::columnName(MS::ANTENNA1) );
    ROScalarColumn<Int>    ant2( inforef.getMS(), MS::columnName(MS::ANTENNA2) );
    ROScalarColumn<Int>    array_id( inforef.getMS(), MS::columnName(MS::ARRAY_ID) );
    ROScalarColumn<Int>    spwinid( inforef.getMS(), MS::columnName(MS::SPECTRAL_WINDOW_ID) );
    ROScalarColumn<Int>    fieldid( inforef.getMS(), MS::columnName(MS::FIELD_ID) );
    ROScalarColumn<Double> exposure( inforef.getMS(), MS::columnName(MS::EXPOSURE) );

    //
    //  Lets see if we have a weightspectrum. If so, we need to attach
    //  the ROColumn..object to the column in the MS. Otherwise we
    //  leave it un-attached.
    //
    hasWeightSpectrum=inforef.getMS().tableDesc().isColumn(MS::columnName(MS::WEIGHT_SPECTRUM));

    if( hasWeightSpectrum )
    {
	weightspectrum.attach( inforef.getMS(), MS::columnName(MS::WEIGHT_SPECTRUM) );
    }


    //
    //  Do get the RA and DEC and sourcename from the MS
    //
    if( fieldTable.nrow() && indir.isDefined(0) )
    {
        radec=indir(0);
    }
    else
    {
	cout << "++++ No source pos found. Defaulting to (0,0)" << endl;
    
        radec=Vector<Double>(2);
        radec=(Double)0.0;
    }
    
    if( fieldTable.nrow() && inname.isDefined(0) )
    {
        objectname=inname(0);
    }
    else
    {
        String       tmp;
        ostrstream   defname;
        
        defname << "SOURCE_" << 0 << ends;

        tmp=String( defname.str() );
        
	cout << "++++ No source name found, defaulting to `" << defname.str() << "'" << endl;
        objectname=tmp;
    }

    //
    //  Check if the DATE-OBS can be constructed validly.
    //
    String            date;
    String            timesys;
    MVTime            actualobstime;

    try
    {
        FITSDateUtil::toFITS( date, timesys, inforef.observingDate().day() );
        actualobstime=inforef.observingDate();
    }
    catch( AIPSExceptionObject x )
    {
        x.getMesg();

        actualobstime=MVTime( Time() );

        FITSDateUtil::toFITS( date, timesys, (Double)actualobstime );
	cout << "++++ DATE-OBS could not be represented in FITS, defaulting to "
             << date << endl;
    }
    end_try;

    //
    //  Attempt to determine the epoch for the coordinates
    //
    Bool               foundEpoch( False );
    String             mref( "MEASURE_REFERENCE" );
    const TableRecord& fkeys( indir.keywordSet() );

    if( fkeys.isDefined(mref) && fkeys.dataType(mref)==TpString )
    {
        //
        //  Ok. Get the reference:
        //
	String     dirtype;

	fkeys.get( mref, dirtype );
        
	if( dirtype.contains("2000") )
        {
            epoche=2000.0;
	    foundEpoch=True;
	}
        else if( dirtype.contains("1950") )
        {
            epoche=1950.0;
	    foundEpoch=True;
	}
    }
    //
    //  If not found, default to 2000.0
    //
    if( !foundEpoch )
    {
	cout << "++++ Couldn't find MS epoch. Assuming J2000." << endl;
	epoche=2000.0;
    }

    
    //
    //  Get the telescope and instrument strings:
    //
    if( arrayTable.nrow() && inarrayname.isDefined(0) )
    {
        telescope=inarrayname(0);
        instrument=inarrayname(0);
    }
    else
    {
        String      tmp;
        ostrstream  defarrname;
        
        defarrname << "ARRAY_" << 0 << ends;
        tmp=String( defarrname.str() );
        
	cout << "++++ No arrayname found, defaulting to `" << defarrname.str() << "'" << endl;
        
        telescope=tmp;
        instrument=tmp;
    }
    
    //
    //  See if we have to deviate from the default unit ("UNCALIB")
    //
    if( datacolumn.keywordSet().isDefined("UNIT") && 
	datacolumn.keywordSet().dataType("UNIT")==TpString )
    {
	datacolumn.keywordSet().get("UNIT", bunit);
	bunit = upcase(bunit);
    }

    //
    //  We need to check all the stokes stuff (and determine the
    //  FITS-indices)
    //

    //
    //  Read the vector of stokestypes from the spectralwindow table.
    //
    if( spwinTable.nrow() && stokesTypes.isDefined(0) )
    {
        stokes=stokesTypes(0);
    }
    else
    {
	cout << "++++ No polarization info found, defaulting to: " << endl;
        
        stokes.resize( inforef.nrPolarizations() );
        for( uInt polcnt=0; polcnt<inforef.nrPolarizations(); polcnt++ )
        {
            stokes(polcnt)=Stokes::type( (Int)Stokes::RR + (Int)polcnt );
	    cout.form("     Polarization #%2d -> ",polcnt) << Stokes::name( Stokes::type(stokes(polcnt)) ) << endl;
        }
    }
    

    //
    // OK, turn these into FITS values.
    //
    for( uInt i=0; i<inforef.nrPolarizations(); i++)
    {
	stokes(i)=Stokes::FITSValue( Stokes::StokesTypes(stokes(i)) );
    }

    //
    //  OK, get an index vector that sorts these in ascending order if
    //  stokes(0) >= 0, or descending order if < 0.
    //
    if( stokes(0)>=0 )
    {
	GenSortIndirect<Int>::sort( stokesindex, stokes );
    }
    else
    {
	GenSortIndirect<Int>::sort( stokesindex, stokes, Sort::Descending );
    }

    //
    // Now we need to make sure we can represent the stokes in FITS
    //
    if( stokes.nelements()>2 )
    {
	Int  initialdiff( stokes(stokesindex(1))-stokes(stokesindex(0)) );
	
        for( uInt j=2; j<stokes.nelements(); j++)
        {
	    if( stokes(stokesindex(j))-stokes(stokesindex(j-1))!=initialdiff )
            {
                throw( AipsError("The STOKES parameters are not representable in FITS.") );
	    }
	}
    }

    //
    // These are the axes that describe the datastructure
    //
    FITSAxis          ax_cmplx( "COMPLEX", "Re/Im/Wght", 3 );
    FITSAxis          ax_stokes( "STOKES", "Polarizaton axis", 
                              inforef.nrPolarizations(),
                              1.0, stokes(stokesindex(0))*1.0,
                              ((stokes.nelements()>1)?(stokes(stokesindex(1))-stokes(stokesindex(0))*1.0):(1.0)) );
    FITSAxis          ax_freq( "FREQ", "Frequency axis", inforef.nrChannels(), 1.0,
                            inforef.getReferenceFrequency(MSConversionInfo::CenterChannel),
							inforef.getChannelincrement() );
    FITSAxis          ax_if( "IF", "IF axis", inforef.nrIFs(), 1.0, 1.0 );
    FITSAxis          ax_ra( "RA", "Position axis (historically)", 1, 1.0, radec(0)/C::degree );
    FITSAxis          ax_declination( "DEC", "Position axis (historically)", 1, 1.0, radec(1)/C::degree );

    
    //
    // Ok. "Let's doohooohoo it!" (ref. Lord Flasheart in `BlackAdder
    // goes forth!')
    //
    newhdu->addAxis( ax_cmplx );
    newhdu->addAxis( ax_stokes );
    newhdu->addAxis( ax_freq );
    newhdu->addAxis( ax_if );
    newhdu->addAxis( ax_ra );
    newhdu->addAxis( ax_declination );
    
    //
    //  Decide on randomparameter-uvw-units
    //
    Double    uvwscale;
    
    if( usewavelength )
    {
	uuname="UU-L-";
	vvname="VV-L-";
	wwname="WW-L-";
	uvwscale=wavelength;
    }
    else
    {
	//
	//  Unit is time
	//
	uuname="UU--";
	vvname="VV--";
	wwname="WW--";
	uvwscale=1.0;
    }
    
    //
    //  The data is preceded by parameters:
    //
    newhdu->addParameter( FITSParameter(uuname, "", uvwscale) );
    newhdu->addParameter( FITSParameter(vvname, "", uvwscale) );
    newhdu->addParameter( FITSParameter(wwname, "", uvwscale) );
    newhdu->addParameter( FITSParameter("DATE", "Day number") );
    newhdu->addParameter( FITSParameter("DATE", "Day fraction") );
    newhdu->addParameter( FITSParameter("BASELINE") );
    newhdu->addParameter( FITSParameter("FREQSEL") );
    newhdu->addParameter( FITSParameter("SOURCE") );
    newhdu->addParameter( FITSParameter("INTTIM") );

    //
    //  Maybe now's the time to start filling in the data!
    //


    //
    //  This is the record we're going to use to pass the data to the
    //  FITS-stuff!
    //
    Record       datarecord( newhdu->getGroupDescription() );

    //
    //  Make recordfieldptrs
    //
    RecordFieldPtr<Float>         uptr( datarecord, uuname );
    RecordFieldPtr<Float>         vptr( datarecord, vvname );
    RecordFieldPtr<Float>         wptr( datarecord, wwname );
    RecordFieldPtr<Float>         dayptr( datarecord, 3 );
    RecordFieldPtr<Float>         dayfracptr( datarecord, 4 );
    RecordFieldPtr<Float>         blptr( datarecord, "BASELINE" );
    RecordFieldPtr<Float>         freqselptr( datarecord, "FREQSEL" );
    RecordFieldPtr<Float>         sourceidptr( datarecord, "SOURCE" );
    RecordFieldPtr<Float>         inttimptr( datarecord, "INTTIM" );
    //
    //  Tricky......
    //
    RecordFieldPtr<Array<Float> > datamatrixptr( datarecord, 9 );

    //
    //  Loop over all groups
    //
    uInt    nrgroups( inforef.getMS().nrow()/inforef.nrIFs() );

    //
    //  Tell the HDU how many groups there are.
    //
    //  NOTE: the nr of rows in the maintable isn't the number of
    //  groups; it's nrrows/nrIFs -> for every group there are nrIF
    //  rows in the maintable, each containing a matrix of nrchannels
    //  X nrpolarizations
    //
    newhdu->setNumberOfGroups( nrgroups );

    //
    // Do get the storage to the arrays for speed reasons
    //

    //
    //  The source data
    //
    IPosition          srcindex( IPosition(2,0) );
    Array<Complex>     srcdata( IPosition(2, inforef.nrPolarizations(), inforef.nrChannels()) );
    const Complex*     srcdataptr( srcdata.getStorage(delSrcdataptr) );

    //
    //  The weight data
    //
    Array<Float>       weightdata( IPosition(1, inforef.nrChannels()) );
    const Float*       weightdataptr( weightdata.getStorage(delWeightptr) );

    //
    //  The flags
    //
    Array<Bool>        flagdata( IPosition(2, inforef.nrPolarizations(), inforef.nrChannels()) );
    const Bool*        flagdataptr( flagdata.getStorage(delFlagptr) );

    //
    //  The destination data
    //
    IPosition          dstindex( IPosition(6,0) );
    const Float*       destdataptr( (*datamatrixptr).getStorage(delDstdataptr) );

    //
    //  Last but not least: the stokes index!
    //
    const uInt*        stokesindptr( stokesindex.getStorage(delStokesIndptr) );

    //
    //  Loop and go!
    //
    uInt               nrchannels( inforef.nrChannels() );
    uInt               nrpolarizations( inforef.nrPolarizations() );
    uInt               nrifs( inforef.nrIFs() );
    Bool               rowFlag;
    Float              rowWeight;

    //
    //  Ok. We start anew. The dest'matrix' and srcmatrix have
    //  different shapes. The source-matrix is ( (nchannel *
    //  npolcombis) Complex ), whilst the dest'matrix' is ( (nrifs *
    //  nrchannels * nrpolcombis * 3) Float )
    //
    //  What we do is the following: we resort the maintable, based on
    //  time, baseline and sp_win_id. Then we start iterating over the
    //  maintable in steps of chunks of successive
    //  'time,baseline'-pairs.
    //
    Block<String>     sortnames( 5 );
    Block<String>     iternames( 4 );

    sortnames[0]="TIME";
    iternames[0]="TIME";
    
    sortnames[1]="ANTENNA1";
    iternames[1]="ANTENNA1";
    
    sortnames[2]="ANTENNA2";
    iternames[2]="ANTENNA2";
    
    sortnames[3]="FIELD_ID";
    iternames[3]="FIELD_ID";
    
    sortnames[4]="SPECTRAL_WINDOW_ID";
    
    //
    //  sort the maintable
    //
    // HV: 17-03-2000 - Removed this; let the iterator sort stuff; we don't need to
    //                  keep the table sorted so lets not waste a lot of (CPU)time 
    //                  on that
    //cout << "Start maintable sort..." << flush;
    //Table             maintable( inforef.getMS().sort(sortnames, Sort::Ascending) );
    //cout << "done." << endl;

    //
    //  Now we make a tableiterator. We iterate over equal times and
    //  baseline. In that way, we get a 'table' (reference) for every
    //  time/baseline pair. If the nr of rows in such a table is NOT
    //  equal to the nr of ifs, then we have aserious problem!
    //
    //TableIterator     maintableiterator( maintable, iternames, TableIterator::Ascending );
    TableIterator     maintableiterator( inforef.getMS(), iternames, TableIterator::Ascending );
    
    //
    // Dtermine the number of valid groupss
    //
    uLong        nrskipped;
    
    nrskipped = 0;

    //
    // HV: 17-03-2000 - Removed the following section. On *large* MSs this takes
    //                  a *lot* of time (however it ensures that the correct nr
    //                  nr of groups is determined...) so let's make a dumb guess
    //                  at the nr-of-groups and add/skip them on the fly... :-(
    //
    /*
    nrgroups = 0;
    nrskipped = 0;
    
    while( !maintableiterator.pastEnd() )
    {
        Table    grouptable( maintableiterator.table() );
        
        if( grouptable.nrow()==nrifs )
        {
            nrgroups++;
        }
        else
        {
	    //cout << "*** Visibility has non-conformant nr of IFs" << endl;
	    //cout << "*** found " << grouptable.nrow() << " should have found: " << nrifs << endl;
            nrskipped++;
        }
        maintableiterator.next();
    }

    //
    // Reset number of groups in the HDU
    //
    newhdu->setNumberOfGroups( nrgroups );
    */
    
    cout << "++++ Found " << nrgroups << " valid visibilities; dropped " 
         << nrskipped << endl;
    

    //
    // Manually count groups
    //
    uLong             groupcnt( 0 );
    nrskipped = 0;
    
    //
    // Reset the table iterator to start
    //
    maintableiterator.reset();
    
    //
    //  Loop until either of the following conditions hold;
    //
    //  either no more groups or we've reached the number of groups
    //  that were allocated for this HDU....
    //
    while( !maintableiterator.pastEnd() && groupcnt<nrgroups )
    {
	//
	//  Get the current table....
	//
	Table        grouptable( maintableiterator.table() );

	//
	// The datasize of one IF in this group (in Float's)
	//
	uLong        IFsize( nrchannels*nrpolarizations*3 );

	//
	//  sanitycheck....
	//
	if( grouptable.nrow()!=nrifs )
	{
	    //cout << "*** Visibility has non-conformant nr of IFs" << endl;
	    //cout << "*** found " << grouptable.nrow() << " should have found: " << nrifs << endl;
	    maintableiterator.next();
	    nrskipped++;
	    continue;
	}

	//
	//  Attach the column objects to the grouptable...
	//
	datacolumn.attach( grouptable, inforef.getDataColumnName() );
	rowflag.attach( grouptable, MS::columnName(MS::FLAG_ROW) );
	dataflag.attach( grouptable, MS::columnName(MS::FLAG) );
	weight.attach( grouptable, MS::columnName(MS::WEIGHT) );

	if( hasWeightSpectrum )
	{
	    weightspectrum.attach( grouptable, MS::columnName(MS::WEIGHT_SPECTRUM) );
	}
	else
	{
	    weightspectrum.reference( ROArrayColumn<Float>() );
	}
	
	uvwcol.attach( grouptable, MS::columnName(MS::UVW) );
	time.attach( grouptable, MS::columnName(MS::TIME) );
	ant1.attach( grouptable, MS::columnName(MS::ANTENNA1) );
	ant2.attach( grouptable, MS::columnName(MS::ANTENNA2) );
	array_id.attach( grouptable, MS::columnName(MS::ARRAY_ID) );
	spwinid.attach( grouptable, MS::columnName(MS::SPECTRAL_WINDOW_ID) );
	fieldid.attach( grouptable, MS::columnName(MS::FIELD_ID) );
	exposure.attach( grouptable, MS::columnName(MS::EXPOSURE) );

	//
	//  Ok. Every row in the 'grouptable' contains all the data for a specific IF...
	//
	for( uInt ifcnt=0, orgrow=0; ifcnt<nrifs; ifcnt++, orgrow++ )
	{
	    //
	    // DATA matrix
	    //
	    if( datacolumn.isDefined(orgrow) )
	    {
		datacolumn.get( orgrow, srcdata );
		//cout << "*************** SRCDATA **********" << endl;
		//cout << "IF=" << ifcnt << ": " << srcdata.ac() << endl;
		//cout << "**********************************" << endl;
	    }
	    else
	    {
		cout.form("Row #%6d: ", orgrow) << "Datamatrix not defined, default to Cmplx(0,0)" << endl;
		srcdata=Complex(0.0f, 0.0f);
	    }

		//
		//  HV: 19-10-2000 - Denise Gabuzda uncovered cross polarization
		//                   problems with our data. It appeared that we
		//                   had some minus-signs left out (i.e. we didn't do
		//                   the math correctly; Denise did...). For
		//                   mapping this was not a problem; for parallactic
		//                   angle stuff is was disastrous :/
		//                   So it was decided that in the UVF files:
		//
		//                   1) the sign of the phases should be inverted
		//                   2) the signs of all U,V and W should be inverted
		//
		//
		//  This is step 1)
		//
		srcdata = conj( srcdata );

	    //
	    // FLAG_ROW
	    //
	    if( rowflag.isDefined(orgrow) )
	    {
		rowFlag=rowflag(orgrow);
	    }
	    else
	    {
		rowFlag=False;
		//cout.form("Row #%6d: ", orgrow) << "Rowflag not defined, default to False" << endl;
	    }
	    
	    //
	    // FLAG
	    //
	    if( dataflag.isDefined(orgrow) )
	    {
		dataflag.get( orgrow, flagdata );
	    }
	    else
	    {
		//cout.form("Row #%6d: ", orgrow) << "Dataflags not defined, default to False" << endl;
		flagdata=False;
	    }
	    
	    //
	    // WEIGHT
	    //
	    if( weight.isDefined(orgrow) )
	    {
		rowWeight=weight(orgrow);
	    }
	    else
	    {
		rowWeight=1.0f;
		//cout.form("Row #%6d: ", orgrow) << "Rowweight not defined, default to " << rowWeight << endl;
	    }
	    
	    //
	    // WEIGHT_SPECTRUM
	    //
	    if( hasWeightSpectrum )
	    {
		if( weightspectrum.isDefined(orgrow) )
		{
		    weightspectrum.get( orgrow, weightdata );
		}
		else
		{
		    //cout.form("Row #%6d: ", orgrow) << "Weightspectrum not defined, default to 1.0" << endl;
		    weightdata=1.0f;
		}
	    }
	    else
	    {
		for( uInt ii = 0; ii<nrchannels; ii++ )
		{
		    ((Float*)weightdataptr)[ii]=rowWeight;
		}
	    }

	    //
	    // We need to re-sort the data, since in the MS the data
	    // matrix is channels x polarizations, whereas in the FITS
	    // file, we have polarization x channels x IFs
	    //
	    uLong     count( 0 );
	    uLong     srcoffset;
	    uLong     dstoffset;

	    for( uInt chancnt=0; chancnt<nrchannels; chancnt++ )
	    {
		for( uInt polcnt=0; polcnt<nrpolarizations; polcnt++, count++ )
		{
		    //
		    //  Calc source offset (we're adressing in a
		    //  matrix of dimension
		    //  nrchannels*nrpolarizations), so the current
		    //  channel/pol combi can be found at:
		    //  (current. pol.)*nrchannels + (current
		    //  channelnr)
		    //
		    
		    //srcindex(0)=stokesindptr[j];
		    //srcindex(1)=k;
		    
		    //dstindex(1)=j;
		    //dstindex(2)=k;
		    //dstindex(3)=ifcnt;
		    //dstindex(4)=0;
		    //dstindex(5)=0;

		    //
		    //  Matrix is in FORTRAN order in memory... (*sigh*)
		    //
		    //srcoffset=stokesindptr[polcnt]*nrchannels+chancnt;
		    srcoffset=chancnt*nrpolarizations+stokesindptr[polcnt];
		    
		    //
		    //  Reshuffle data and set weight
		    //
		    //  The destinationoffset is somewhat more
		    //  difficult: the organisation within a random
		    //  group is:
		    //
		    //   [RR ..LL] [RR..LL] [RR..LL] [RR..LL] [RR..LL].........
		    //    |                     |       |
		    //    -----------------------       --------------->
		    //           nrchannel                   nrchannel
		    //               |                           |
		    //               -----------------------------
		    //                           nrif
		    //
		    //
		    //   where [RR..LL] basically is:
		    //
		    //   [Real Imag Weight] .... [Real Imag Weight]
		    //           |                       |
		    //           -------------------------
		    //                     nrstokes
		    //
		    //  So the destination calculation becomes:
		    //
		    //  (current IF)*IFdatasize + (current channel)*(nrstokes*3) + (current stokesindex)
		    //    + (0,1 or 2, depending on writing Real,Imag or weight)
		    //
		    //
		    dstoffset = ifcnt*IFsize + chancnt*(nrpolarizations*3) + polcnt*3;
		    

		    //dstindex(0)=0;
		    //(*datamatrixptr)(dstindex)  = srcdata(srcindex).real();
		    //dstindex(0)=1;
		    //(*datamatrixptr)(dstindex) = srcdata(srcindex).imag();

		    //dstindex(0)=2;
		    ((Float*)destdataptr)[ dstoffset ]   = srcdataptr[srcoffset].real();
		    ((Float*)destdataptr)[ dstoffset+1 ] = srcdataptr[srcoffset].imag();


// 		    ((Float*)destdataptr)[ ifcnt*IFsize + 3*count]   = srcdataptr[offset].real();
// 		    ((Float*)destdataptr)[ ifcnt*IFsize + 3*count+1] = srcdataptr[offset].imag();
		    
                    //((Float*)destdataptr)[ dstoffset+2 ] =  1.0;
		    if( rowFlag || flagdataptr[srcoffset] )
		    {
			// FLAGged
			//(*datamatrixptr)(dstindex) = -weightdataptr[k];
                    	((Float*)destdataptr)[ dstoffset+2 ] = -weightdataptr[chancnt];
//			((Float*)destdataptr)[ ifcnt*IFsize + 3*count + 2] = -weightdataptr[k];
		    }
		    else
		    {
			// NOT FLAGged
			//(*datamatrixptr)(dstindex) = weightdataptr[k];
                        ((Float*)destdataptr)[ dstoffset+2 ] =  weightdataptr[chancnt];
//			((Float*)destdataptr)[ ifcnt*IFsize + 3*count + 2] =  weightdataptr[k];
		    }
		}
	    }
	}

//  	cout << (*datamatrixptr).ac() << endl;
	
	//
	//  Ok. All if's have been processed....now get all the random
	//  parameters!
	//

        //
	// Random parameters
	// UU VV WW
        //
        if( uvwcol.isDefined(0) )
        {
            uvwcol.get( 0, uvw );
        }
        else
        {
	    //cout.form("Row #%6d: ", originalrownr(0)) << "UVW  not defined, default to (0,0,0)" << endl;
	    uvw=0.0;
        }

	//
	//  This is step 2) of the explanation above (see note "HV: 19-10-2000 - ...") above
	//
	uvw *= -1.0;

	//
	//  We want to get the uvw in units of wavelengths. From the
	//  MS we get it in meters. So we divide through by the
	//  wavelength to get it in units of wavelength
	//
	if( usewavelength )
	{
	    *uptr = uvw(0) / wavelength;
	    *vptr = uvw(1) / wavelength;
	    *wptr = uvw(2) / wavelength;
	}
	else
	{
	    *uptr = uvw(0) * oneOverC;
	    *vptr = uvw(1) * oneOverC;
	    *wptr = uvw(2) * oneOverC;
	}
	

	// TIME
        if( time.isDefined(0) )
        {
            timeToDay( day, dayFraction, time(0) );
        }
        else
        {
            timeToDay( day, dayFraction, actualobstime );

	    //cout.form("Row #%6d: ", 0) << "Time not defined, default to day=" << day 
	    //		      << ", frac=" << dayFraction << endl;
        }
	*dayptr     = day;
	*dayfracptr = dayFraction;

	// BASELINE
        if( ant1.isDefined(0) && ant2.isDefined(0) )
        {
            *blptr = (ant1(0)+1)*256 + ant2(0) + 1;
            
            if( array_id.isDefined(0) )
            {
                *blptr+=( array_id(0)*0.01 );
            }
            else
            {
		//cout.form("Row #%6d: ", 0) << "Warning: baseline could not cater for (possible) sub-array."
		//                                << endl;
            }
        }
        else
        {
            *blptr=0.0;
            // cout.form("Row #%6d: ", 0) << "Baseline could not be constructed, default to "
	    //                                << *blptr << endl;
        }
        

	// FREQSEL ( in the future this will be FREQ_GRP+1, for now, it's 1 )
	*freqselptr = 1;
	
	/*
        if( spwinid.isDefined(0) )
        {
            *freqselptr = spwinid(0) + 1;
        }
        else
        {
            *freqselptr = 1.0;
	    cout.form("Row #%6d: ", 0) << "Spectral window ID not defined, default to " 
                                              << *freqselptr << endl;
        }
*/
        
	// SOURCE ID
	if( fieldid.isDefined(0) )
	{
	    *sourceidptr = (fieldid(0)+1);
	}
	else
	{
	    *sourceidptr = 1;
	    
	    //cout.form("Row #%6d: ", 0) << "Sourceid not defined, default to " 
	    //                                 << *sourceidptr << endl;
	}
	

        // INTTIM
        if( exposure.isDefined(0) )
        {
            *inttimptr = exposure(0);
        }
        else
        {
            *inttimptr=1.0;
	    //cout.form("Row #%6d: ", 0) << "Exposure not defined, default to " 
            //                                  << *inttimptr << endl;
        }


        //
        //  Put the group!
        //
        newhdu->put( groupcnt, datarecord );

	if( !(groupcnt%100) )
	{
	    cout.form( "Done: %4.1lf%%", ((Double)groupcnt/(Double)nrgroups)*100.0 ) << "\r" << flush;
	}
	groupcnt++;

	//
	//  DON't forget to advance the tableiterator!!!!!!
	//
	maintableiterator.next();
    }
    cout << endl;

    cout << "Wrote " << groupcnt << " groups out of anticipated " << nrgroups 
	 << ", skipped " << nrskipped << " groups." << endl;
    
    //
    //  Check if we need to add some groups...
    //
    if( groupcnt<nrgroups )
    {
	cout << "Drat! Not enough groups were written. Adding " << (nrgroups-groupcnt) 
	     << " empty groups to validate fits-file...." << flush;
	
	//
	//  Set up empty group
	//
	(*datamatrixptr) = 0.0f;//Complex( 0.0f, 0.0f );

	*uptr = 0.0;
	*vptr = 0.0;
	*wptr = 0.0;
	
	*dayptr      = 0.0;
	*dayfracptr  = 0.0;
	
	*blptr       = 0.0;
	*freqselptr  = 1;
	*sourceidptr = 1;
	*inttimptr   = 1.0;
	
	
	while( groupcnt<nrgroups )
	{
	    newhdu->put( groupcnt, datarecord );
	    groupcnt++;
	}
	cout << "ok." << endl;
    }
    

    //
    //  Don't forget to delete (if necessary) the storage for the
    //  arrays!
    //
    srcdata.freeStorage( srcdataptr, delSrcdataptr );
    weightdata.freeStorage( weightdataptr, delWeightptr );
    flagdata.freeStorage( flagdataptr, delFlagptr );
    (*datamatrixptr).freeStorage( destdataptr, delDstdataptr );
    stokesindex.freeStorage( stokesindptr, delStokesIndptr );

    //
    //  Add some extra-keywords.....
    //
    char              buffer[ 1024 ];
    ostrstream        strm( buffer, sizeof(buffer) );
    FitsKeyword*      ptr2kw;
    FitsKeywordList   tmp;
    

    tmp.mk( FITS::OBJECT, objectname, "Multisource" );
    tmp.mk( FITS::EPOCH, epoche, "Epoch" );
    tmp.mk( FITS::DATE_OBS, date );
    tmp.mk( FITS::TELESCOP, telescope );
    tmp.mk( FITS::OBSERVER, "JIVE" );
    tmp.mk( FITS::INSTRUME, instrument );
    tmp.mk( "RECEIVER", "JIVE" );
    tmp.mk( FITS::BSCALE, 1.0 );
    tmp.mk( FITS::BZERO, 0.0 );
    tmp.mk( FITS::BUNIT, bunit );

    //
    //  Check for multiple pieces....
    //
    if( inforef.maxPieceNr()>1 )
    {
	strm.seekp( 0 );
	strm << "AIPS  IPIECE= " << inforef.itsPieceNr()+1 << "  NPIECE=" 
	     << inforef.maxPieceNr() << ends;
	
	tmp.history( strm.str() );

	//
	//  We need to write first/last visibility....
	//
	strm.seekp( 0 );
	strm << "AIPS  FIRSTVIS= " << MSConversionInfo::lastVisibility+1 << ends;
	tmp.history( strm.str() );

	MSConversionInfo::lastVisibility += groupcnt;

	strm.seekp( 0 );
	strm << "AIPS  LASTVIS = " << MSConversionInfo::lastVisibility+1 << ends;
	tmp.history( strm.str() );	
    }

    tmp.first();
    tmp.next();
    ptr2kw=tmp.curr();
    while( ptr2kw )
    {
        newhdu->addExtraKeyword( *ptr2kw );
        ptr2kw=tmp.next();
    }
    
    

    //
    //  Fill in the parameter
    //
    ref2hduptr(0)=newhdu;
    
    //
    //  And return!
    //
    return True;
}



//
//  The d'tor: nothing to destruct at this moment.....
//
maingroupGenerator::~maingroupGenerator()
{
}











