// sutabGenerator.cc
//
// Implementation of the sutabGenerator class. See sutabGenerator.h for details.
//
//
// Author:
//
//      Harro Verkouter   10-8-1998
//
// $Id: sutabGenerator.cc,v 1.4 2000/11/17 10:30:41 verkout Exp $
#ifndef SUTABGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/sutabGenerator.h>
#endif


//
//  Aips++ includes
//
#ifndef HDUOBJECT_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/HDUObject.h>
#endif

#ifndef AIPS_MSCOLUMNS_H
#include <aips/MeasurementSets/MSColumns.h>
#endif

//
//  My includes
//
#ifndef HDUGENERATOR_H_INC
#include <vlbi/ms2uvfitsimpl/Generators/hduGenerator.h>
#endif

#ifndef MSCONVERSIONINFO_H_INC
#include <vlbi/ms2uvfitsimpl/MSConversionInfo.h>
#endif

#ifndef FITSPARAMETER_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSParameter.h>
#endif

#ifndef FITSBINARYCOLUMN_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryColumn.h>
#endif

#ifndef AIPS_VECTOR_H
#include <aips/Arrays/Vector.h>
#endif

#ifndef AIPS_MATRIX_H
#include <aips/Arrays/Matrix.h>
#endif

#ifndef FITSBINARYTABLEHDU_H_INC
#include <vlbi/ms2uvfitsimpl/FITSUtils/FITSBinaryTableHDU.h>
#endif

#ifndef AIPS_RECORD_H
#include <aips/Containers/Record.h>
#endif

#ifndef AIPS_TABLERECORD_H
#include <aips/Tables/TableRecord.h>
#endif

#ifndef AIPS_RECORDFIELD_H
#include <aips/Containers/RecordField.h>
#endif

#ifndef AIPS_CONSTANTS_H
#include <aips/Mathematics/Constants.h>
#endif

#ifndef AIPS_TABLEITERATOR_H
#include <aips/Tables/TableIter.h>
#endif

#ifndef AIPS_EXPR_NODE_H
#include <aips/Tables/ExprNode.h>
#endif

#ifndef AIPS_TABLEDESC_H
#include <aips/Tables/TableDesc.h>
#endif

#ifndef AIPS_MEASURES_H
#include <aips/Measures.h>
#endif

#ifndef AIPS_MDIRECTION_H
#include <aips/Measures/MDirection.h>
#endif

#ifndef AIPS_MVANGLE_H
#include <aips/Quanta/MVAngle.h>
#endif

#ifndef AIPS_MEPOCH_H
#include <aips/Measures/MEpoch.h>
#endif

#ifndef AIPS_MEASUREMENTSET_H
#include <aips/MeasurementSets/MeasurementSet.h>
#endif

#include <vlbi/ms2uvfitsimpl/DumpRecord.h>

//
//  DEFINES
//
#define SHOWVAR( a ) \
    cout << #a << ": " << a << flush;



//
//  Dirty? A local global function to extract the epoch from a
//  column. It attempts to look for 2000.0 or 1950.0. If not found,
//  0.0 is returned.
//
Double  getEpoch( const ROTableColumn& column )
{
    Double             retval( 0.0 );
    String             keyword( "MEASURE_REFERENCE" );
    const TableRecord& columnkw( column.keywordSet() );
    
    if( columnkw.isDefined(keyword) && columnkw.dataType(keyword)==TpString )
    {
        //
        //  Ok. Get the reference:
        //
	String     dirtype;

	columnkw.get( keyword, dirtype );
        
	if( dirtype.contains("2000") )
        {
            retval=2000.0;
	}
        else if( dirtype.contains("1950") )
        {
            retval=1950.0;
	}
    }
    return retval;
}




//
//  Create a default sutab generator, nothing to do yet...
//
sutabGenerator::sutabGenerator() :
    hduGenerator()
{
}


//
//  This function will do all the work of getting all the necessary
//  TSYS information out of the MSConversionInfo object. It will
//  then create a binary table extension HDU and pass that back to the
//  caller!
//

//
//
//  The strategy for the SOURCE NAME is as follows:
//
//  * If the field has a source attached ( source_id!=-1 ), and it's
//  the only field with this source_id, we write the name from the
//  source table.
//
//  * If the field has a source attached AND there is more than one
//  field, we first look if the 'NAME' column is present in the
//  fieldtable. If so, the value in this field will be used. If the
//  'NAME' column is absent (MS vs. < 2.0) we use the string in the
//  'CODE' column as sourcename for the current field.
//
//     In either case the following holds: if the value in the field
//     is empty or not defined, we make up a sourcename ourselves and
//     tell the user we did it.
//
//
//  NOTE: On SOURCE NUMBERING.
//
//  Since Classic AIPS cannot deal with multiple pointings per source
//  and the users still want their data in Classic AIPS, we do the
//  following: every field in the field-table is regarded as a single
//  source (as seen from Classic AIPS point of view). The RA/DEC are
//  taken from the field-table. If there happens to be a physical
//  source of radiation present in the field (indicated by
//  source_id!=-1), the velocity/ IQUV-fluxes etc. are taken from the
//  SOURCE table, otherwise they will be filled in with defaults.
//
//  Errr. I'm drifting from the subject. The source numbering. In the
//  main-table generator, we have to take into account too the fact
//  that we're dealing with multi-source: we add the SOURCE ID-random
//  parameter to the list of parameters. The value we fill in here is
//  the FIELD_ID from the main table. So we do NOT use the SOURCE_ID
//  from the SOURCE-table in aips++.
//
Bool sutabGenerator::convert( Vector<HDUObject*>& ref2hduptr, MSConversionInfo& inforef ) const
{
    cout << "Making SU table" << endl;
    //
    //  First, we make sure the return-value is empty!
    //
    ref2hduptr.resize( 0 );


    //
    //  Local variables we need
    //
    Int                  unknownsource( 0 );
    Int                  currentsourceid;
    Int                  rownrinsourcetab;
    uInt                 nrifs( inforef.nrIFs() );
    uInt                 nrrows; // The number of rows in the destination table!
    uInt                 pointingspersrc;
    Bool                 hasFieldNameCol;
    Table                sourcetable( inforef.getMS().source() );
    Table                fieldtable( inforef.getMS().field() );
    Double               currentepoch;
    String               sourcename;
    ROScalarColumn<Int>    sourceidcol( sourcetable, "SOURCE_ID" );
    ROScalarColumn<Int>    fieldsourceid( fieldtable, "SOURCE_ID" );
    ROScalarColumn<Int>    fieldidcol( fieldtable, "FIELD_ID" );
    ROScalarColumn<String> sourcenamecol( sourcetable, "NAME" );
    ROScalarColumn<String> fieldnamecol;
    ROScalarColumn<String> fieldcodecol( fieldtable, "CODE" );
    ROArrayColumn<Double>  sourcedirectioncol( sourcetable, "DIRECTION" );
    ROArrayColumn<Double>  fielddirectioncol( fieldtable, "POINTING_DIR" );
    ROArrayColumn<Double>  propermotioncol( sourcetable, "PROPER_MOTION" );
    ROScalarColumn<Double> sysvelcol( sourcetable, "SYSVEL" );
    ROScalarColumn<Int>    calgroupcol( sourcetable, "CALIBRATION_GROUP" );
    ROScalarColumn<String> sourcecodecol( sourcetable, "CODE" );
    FITSBinaryTableHDU*  sutabptr( new FITSBinaryTableHDU("AIPS SU", 1) );

    
    //
    //  Let's try to find out the total number of rows in the
    //  destination SU-table. This number is necessarily equal to the
    //  number of pointings. We will treat (in principle) every
    //  pointing as a different source.
    //
    nrrows=fieldtable.nrow();
    
    //
    //  If, after all checking etc. nrrows==0, it means that there's
    //  nothing here for us!
    //
    if( !nrrows )
    {
	cout << "No source/field information found." << endl;
	return ToBool( false );
    }

    //
    // Check if a fieldname-column is present
    //
    hasFieldNameCol=False;
    
    if( fieldtable.tableDesc().isColumn("NAME") )
    {
	fieldnamecol.attach( fieldtable, "NAME" );
	hasFieldNameCol=True;
    }
    
    //
    //  Having verified that we have to write some stuff, we need to
    //  setup the table!
    //
    IPosition     ifshape( 1, nrifs );
    Array<Float>  nullswithifshape( ifshape );
    Array<uChar>  charswithifshape( ifshape );
    Array<uChar>  fourchars( IPosition(1, 4) );
    Array<Double> dnullswithifshape( ifshape );
    
    nullswithifshape=(Float)0.0f;
    charswithifshape=((uChar)' ');
    fourchars=((uChar)' ');
    dnullswithifshape=(Double)0.0;

    sutabptr->addColumn( FITSBinaryColumn("ID. NO.", "Source ID", "1J", "") );
    sutabptr->addColumn( FITSBinaryColumn("SOURCE", "Source name", "16A", "") );
    sutabptr->addColumn( FITSBinaryColumn("QUAL", "", "1J", "") );
    //sutabptr->addColumn( FITSBinaryColumn("CALCODE", TpArrayUChar, ifshape) );
    sutabptr->addColumn( FITSBinaryColumn("CALCODE", TpArrayUChar, IPosition(1,4) ) );
    sutabptr->addColumn( FITSBinaryColumn("IFLUX", TpArrayFloat, ifshape, "JY") );
    sutabptr->addColumn( FITSBinaryColumn("QFLUX", TpArrayFloat, ifshape, "JY") );
    sutabptr->addColumn( FITSBinaryColumn("UFLUX", TpArrayFloat, ifshape, "JY") );
    sutabptr->addColumn( FITSBinaryColumn("VFLUX", TpArrayFloat, ifshape, "JY") );
    sutabptr->addColumn( FITSBinaryColumn("FREQOFF", TpArrayDouble, ifshape, "HZ") );
    sutabptr->addColumn( FITSBinaryColumn("BANDWIDTH", "", "1D", "HZ") );
    sutabptr->addColumn( FITSBinaryColumn("RAEPO", "RA of epoch", "1D", "DEGREES") );
    sutabptr->addColumn( FITSBinaryColumn("DECEPO", "DEC of epoch", "1D", "DEGREES") );
    sutabptr->addColumn( FITSBinaryColumn("EPOCH", "", "1D", "YEARS") );
    sutabptr->addColumn( FITSBinaryColumn("RAAPP", "Apparent RA", "1D", "DEGREES") );
    sutabptr->addColumn( FITSBinaryColumn("DECAPP", "Apparent DEC", "1D", "DEGREES") );
    sutabptr->addColumn( FITSBinaryColumn("LSRVEL", TpArrayDouble, ifshape, "M/SEC") );
    sutabptr->addColumn( FITSBinaryColumn("RESTFREQ", "", "1D", "HZ") );
    sutabptr->addColumn( FITSBinaryColumn("PMRA", "", "1D", "DEG/DAY") );
    sutabptr->addColumn( FITSBinaryColumn("PMDEC", "", "1D", "DEG/DAY") );

    //
    //  Tell the table how many rows it should hold!
    //
    sutabptr->setNumberRows( nrrows );
    

    //
    //  Having done that, we can now start to use the datarecord and
    //  make recordfieldpointers!
    //
    Record     rowdata( sutabptr->getRowDescription() );

    //
    //  The recordfieldpointers
    //
    RecordFieldPtr<Int>            sourceidptr( rowdata, "ID. NO." );
    RecordFieldPtr<String>         nameptr( rowdata, "SOURCE" );
    RecordFieldPtr<Int>            qualptr( rowdata, "QUAL" );
    RecordFieldPtr<Array<uChar> >  calcodeptr( rowdata, "CALCODE" );
    RecordFieldPtr<Array<Float> >  ifluxptr( rowdata, "IFLUX" );
    RecordFieldPtr<Array<Float> >  qfluxptr( rowdata, "QFLUX" );
    RecordFieldPtr<Array<Float> >  ufluxptr( rowdata, "UFLUX" );
    RecordFieldPtr<Array<Float> >  vfluxptr( rowdata, "VFLUX" );
    RecordFieldPtr<Array<Double> > freqoffptr( rowdata, "FREQOFF" );
    RecordFieldPtr<Double>         bandwidthptr( rowdata, "BANDWIDTH" );
    RecordFieldPtr<Double>         raepoptr( rowdata, "RAEPO" );
    RecordFieldPtr<Double>         decepoptr( rowdata, "DECEPO" );
    RecordFieldPtr<Double>         epochptr( rowdata, "EPOCH" );
    RecordFieldPtr<Double>         raappptr( rowdata, "RAAPP" );
    RecordFieldPtr<Double>         decappptr( rowdata, "DECAPP" );
    RecordFieldPtr<Array<Double> > lsrvelptr( rowdata, "LSRVEL" );
    RecordFieldPtr<Double>         restfreqptr( rowdata, "RESTFREQ" );
    RecordFieldPtr<Double>         pmraptr( rowdata, "PMRA" );
    RecordFieldPtr<Double>         pmdecptr( rowdata, "PMDEC" );

    
    //
    //  Now loop over all fields!
    //
    for( uInt fieldnr=0; fieldnr<fieldtable.nrow(); fieldnr++ )
    {
	//
	//  Fill in default stuff
	//
	*qualptr = 1;
	//*calcodeptr = charswithifshape;
	*calcodeptr = fourchars;
	*ifluxptr = nullswithifshape;
	*qfluxptr = nullswithifshape;
	*ufluxptr = nullswithifshape;
	*vfluxptr = nullswithifshape;
	*freqoffptr = 0.0;
	*bandwidthptr = inforef.getChannelwidth();
	//*bandwidthptr = inforef.getTotalBandwidth();
	*raepoptr = 0.0;
	*decepoptr = 0.0;
	*epochptr = 0.0;
	*pmraptr = 0.0;
	*pmdecptr = 0.0;
	*raappptr = 0.0;
	*decappptr = 0.0;
	*lsrvelptr = dnullswithifshape;
	*restfreqptr = 0.0;

	//
	// Stuff in source id the current fieldid
	//
	if( !fieldidcol.isDefined(fieldnr) )
	{
	    *sourceidptr = fieldnr+1;
	    
	    cout.form("field %03d: ",fieldnr) << "No fieldid found, defaulting to " << *sourceidptr << endl;
	}
	else
	{
	    *sourceidptr = (fieldidcol(fieldnr)+1);
	}
	
	//
	//  How about the name of the source? Let's first find out if
	//  there's a source 'attached' to this field and if so, if
	//  this is the only field 'attached' to that source
	//
	sourcename="";
	currentsourceid=-1;
	currentepoch=0.0;
	rownrinsourcetab=-1;
	pointingspersrc=0;

	if( fieldsourceid.isDefined(fieldnr) )
	{
	    currentsourceid=fieldsourceid( fieldnr );
	}

	if( currentsourceid!=-1 )
	{
	    //
	    //  Select.... i.e. try to find the row in the source
	    //  table that describes the source.
	    //
	    Table   pointings( fieldtable(fieldtable.col("SOURCE_ID")==currentsourceid) );
	    Table   src( sourcetable(sourcetable.col("SOURCE_ID")==currentsourceid) );	    

	    pointingspersrc=pointings.nrow();
	    
	    //
	    //  Theoretically there should be only one entry in the
	    //  sourcetable with SOURCE_ID==currentsourceid
	    //
	    if( src.nrow()==1 )
	    {
		rownrinsourcetab = src.rowNumbers()( 0 );
	    }
	    
	    //
	    //  If an entry was found in the sourcetable, we take source specific info
	    //  from the sourcetable....
	    //
	    if( rownrinsourcetab>=0 )
	    {
		//
		//  SYSVEL
		//
		if( sysvelcol.isDefined(rownrinsourcetab) )
		{
		    //
		    //  <NOTE> Maybe we need to check the velocity
		    //  reference to (when needed) convert the
		    //  radial velocity to LSR velocity!
		    //
		    *lsrvelptr = sysvelcol( rownrinsourcetab );
		}
		
		//
		//  Proper motion
		//
		if( propermotioncol.isDefined(rownrinsourcetab) )
		{
		    Vector<Double>   pm;
		    
		    propermotioncol.get( rownrinsourcetab, pm );
		    
		    *pmraptr = pm(0);
		    *pmdecptr = pm(1);

// 		    *pmraptr = propermotioncol(rownrinsourcetab)( 0 );
// 		    *pmdecptr = propermotioncol(rownrinsourcetab)( 1 );
		}
		
		//
		//  Calibration group
		//
		if( calgroupcol.isDefined(rownrinsourcetab) )
		{
		    *qualptr = calgroupcol( rownrinsourcetab );
		}

		//
		//  Source code:
		//
		//  NOTE: in the MS this is a string; the length of the string should be
		//  equal to the number of IFs, one character/IF!
		//
		if( sourcecodecol.isDefined(rownrinsourcetab) )
		{
		    const String&  tmpdata( sourcecodecol(rownrinsourcetab) );
		    
		    if( tmpdata.length()==nrifs )
		    {
			for( uInt i=0; i<nrifs; i++ )
			{
			    (*calcodeptr)( IPosition(1,i) ) = tmpdata[ i ];
			}
		    }
		    else
		    {
			cout.form("field %03d: ",fieldnr) 
			    << "CODE not conformant with nr. of IFs. Skipping."
			    << endl;

		    }
		    //uChar  thecode( sourcecodecol(rownrinsourcetab)[0] );
		    //(*calcodeptr).ac() = thecode;
		}
	    }
	}

	//
	//  Now let's see. We first check if there's a fieldname
	//  column. If not present or the field is empty, we check the
	//  fieldcode-column. If that field is empty, we can fall back
	//  on the source table (if a source was attached). If, after
	//  that, the sourcename is still empty, we make up something.
	//
	if( hasFieldNameCol && fieldnamecol.isDefined(fieldnr) )
	{
	    sourcename = upcase( fieldnamecol(fieldnr) );
	}
	
	if( sourcename=="" && fieldcodecol.isDefined(fieldnr) )
	{
	    sourcename = upcase( fieldcodecol(fieldnr) );
	}

	if( sourcename=="" && pointingspersrc==1 )
	{
	    //
	    //  One pointing per source: we can use the sourcename as
	    //  a unique name. But tell the user we did it.
	    //
	    if( rownrinsourcetab>=0 && sourcenamecol.isDefined(rownrinsourcetab) )
	    {
		sourcename = upcase( sourcenamecol(rownrinsourcetab) );
		
		if( sourcename!="" )
		{
		    cout.form("field %03d: ",fieldnr) 
			<< "No fieldname found. Using unique name from SOURCE table: "
			<< sourcename << endl;
		}
	    }
	}
	
	//
	//  If the sourcename is STILL empty, we have to make
	//  something of our own!
	//
	if( sourcename=="" )
	{
	    char        buffer[32];
	    ostrstream  namestr( buffer, 32 );
	    
	    namestr << "FIELD" << unknownsource++ << ends;
	    
	    sourcename = namestr.str();
	    
	    //
	    //  Tell usr. what we did....
	    //
	    cout.form("field %03d: ",fieldnr) << "No fieldname found, defaulting to " << sourcename << endl;
	}

	//
	//  Ok. Over to the field position now!
	//
	if( fielddirectioncol.isDefined(fieldnr) )
	{
		const MEpoch       rdate( inforef.observingDate().get(), MEpoch::UTC );
		MDirection         earthcenter( MVPosition(0.0, 0.0, 0.0), MDirection::ITRF );
		MeasFrame          appframe( earthcenter, rdate );
		Vector<Double>     apppos;
		MDirection         deldir;
		MDirection         appdir;
		MDirection::Types  type;
	    String             refkw( "MEASURE_REFERENCE" );
	    const TableRecord& keywordset( fielddirectioncol.keywordSet() );
    
	    Vector<Double>     fieldpos;
	    MVAngle            fieldra;
	    MVAngle            fielddec;

	    fielddirectioncol.get( fieldnr, fieldpos );

	    //
	    // 'fieldpos' in MS is in units af radians; in FITS we expect DEG(rees)
	    //
	    fieldra  = MVAngle( fieldpos(0) );
	    fielddec = MVAngle( fieldpos(1) );
	    
		type = MDirection::J2000;
    	if( keywordset.isDefined(refkw) && keywordset.dataType(refkw)==TpString )
	    {
    		String             tmp;

			keywordset.get( refkw, tmp );

			MDirection::getType( type, tmp );
	    }

	   	deldir = MDirection( MVDirection(fieldra.get(), fielddec.get()), type );
		appdir = MDirection::Convert( deldir,
									  MDirection::Ref(MDirection::APP, appframe) )();		
 
	    *raepoptr  = fieldra.degree();
	    *decepoptr = fielddec.degree();
	    
		apppos     = (appdir.getValue()).get();
		fieldra    = MVAngle( apppos(0) );
		fielddec   = MVAngle( apppos(1) );

	    *raappptr  = fieldra.degree();
	    *decappptr = fielddec.degree();
	    //Vector<Double>     fieldpos;
	    //MVAngle            fieldra;
	    //MVAngle            fielddec;

	    //fielddirectioncol.get( fieldnr, fieldpos );

	    //
	    // 'fieldpos' in MS is in units af radians; in FITS we expect DEG(rees)
	    //
	    //fieldra=MVAngle( fieldpos(0) );
	    //fielddec=MVAngle( fieldpos(1) );
	    
	    //*raepoptr = fieldra.degree();
	    //*decepoptr = fielddec.degree();
	}
	else
	{
	    cout.form("field %03d: ",fieldnr) << "No fieldposition found, defaulting to "
					      << "(" << *raappptr << "," << *decappptr << ")"
					      << endl;
	}

	//
	//  The epoch
	//
	currentepoch=getEpoch( fielddirectioncol );
	
	if( currentepoch==0.0 )
	{
	    currentepoch=2000.0;
	    
	    cout.form("field %03d: ",fieldnr) << "No EPOCH found, defaulting to " << currentepoch << endl;
	}
	
	//
	//  Do write the name in the record!
	//
	*nameptr = sourcename;
	*epochptr = currentepoch;

	//
	//  Do not forget to write the row....
	//
	sutabptr->putrow( fieldnr, rowdata );
    }

    cout << "Entries processed...!" << endl;
    

    //
    // extract velocity definition/frame info from the sysvel column
    //
    String             tmp;
    String             refkw( "MEASURE_REFERENCE" );
    const TableRecord& keywordset( sysvelcol.keywordSet() );
    
    if( keywordset.isDefined(refkw) && keywordset.dataType(refkw)==TpString )
    {
	keywordset.get( refkw, tmp );
    }

    //
    //  Add extra keywords....
    //
    FitsKeyword*      kwptr;
    FitsKeywordList   ek;

    ek.mk( "NO_IF", (Int)nrifs );
    //ek.mk( "VELTYP", );
    //ek.mk( "VELDEF", );
    ek.mk( "FREQID", 1 );
    
    //
    //  Add the keywords to the HDU!
    //
    ek.first();
    ek.next();
    kwptr=ek.curr();
    while( kwptr )
    {
        sutabptr->addExtraKeyword( *kwptr );
        kwptr=ek.next();
    }


    //
    //  Prepare return value...
    //
    ref2hduptr.resize( 1 );
    ref2hduptr(0)=sutabptr;
    
    return True;
}


sutabGenerator::~sutabGenerator()
{
}

