//
//  Implementation of the MSConversionInfo class. See
//  MSConversionInfo.h for comments
//
//
//
//    Author:   Harro Verkouter    12-3-1998
//
//    $Id: MSConversionInfo.cc,v 1.3 2000/08/24 14:33:16 verkout Exp $
#define MSCONVERSIONINFO_CC

#ifndef MSCONVERSIONINFO_H_INC
#include <vlbi/ms2uvfitsimpl/MSConversionInfo.h>
#endif

//
//  AIPS++ includes
//
#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

#ifndef AIPS_STRING_H
#include <aips/Utilities/String.h>
#endif

#ifndef AIPS_MEASUREMENTSET_H
#include <aips/MeasurementSets/MeasurementSet.h>
#endif

#ifndef AIPS_MSCOLUMNS_H
#include <aips/MeasurementSets/MSColumns.h>
#endif

#ifndef AIPS_MSSPECTRALWINDOW_H
#include <aips/MeasurementSets/MSSpectralWindow.h>
#endif

#ifndef AIPS_MSSPWINDOWCOLUMNS_H
#include <aips/MeasurementSets/MSSpWindowColumns.h>
#endif

#ifndef AIPS_ARRAYLOGICAL_H
#include <aips/Arrays/ArrayLogical.h>
#endif

#ifndef AIPS_CONSTANTS_H
#include <aips/Mathematics/Constants.h>
#endif

#ifndef AIPS_FITS_UTIL_H
#include <trial/FITS/FITSUtil.h>
#endif

#ifndef AIPS_TIME_H
#include <aips/OS/Time.h>
#endif

#ifndef AIPS_STOKES_H
#include <aips/Measures/Stokes.h>
#endif

#ifndef AIPS_GENSORT_H
#include <aips/Utilities/GenSort.h>
#endif


//#include <trial/MeasurementSet/MSReader.h>
#include <aips/Measures.h>

#include <math.h>

//
//  My own define. If MS vs 2.0 comes into action, we may have use the
//  LAG_DATA column. Therefore, define this to make this code able to
//  deal with MSv2.0
//
//#define  HV_MSv2_DEFINED

Double round2int( Double r )
{
	Double   retval;
	Double   delta;

	if( r<0.0 )
	{
		retval = ::ceil( r );
		delta  = -1.0;
	}
	else
	{
		retval = ::floor( r );
		delta  = 1.0;
	}

	if( ::fabs(r-retval)>=0.5 )
	{
		retval += delta;
	}
	return retval;
}




//
//  Attempt to create the MS. The MeasurementSet ctor throws an
//  exception if it fails to open
//
MSConversionInfo::MSConversionInfo( const String& msname ) :
    itsMSname( msname ),
    itsMSptr( 0 ),
    itsROColumnsptr( 0 ),
    itsNumberOfPolarizations( 0 ),
    itsNumberOfChannels( 0 ),
    itsNumberOfIFs( 0 ),
    itsReferenceFrequency( 0.0 ),
    itsTotalBandwidth( 0.0 ),
    itsChannelincrement( 0.0 ),
    myPieceNr( 0 ),
    nrPieces( 1 ),
    myFirstStokes( 0 )
{
    MS*    tmpptr = new MeasurementSet(msname,Table::Old);
    
    this->attachMS( *tmpptr );

}

MSConversionInfo::MSConversionInfo( MS& msref, uInt thispiece, uInt maxpiece ) :
    itsMSptr( 0 ),
    itsROColumnsptr( 0 ),
    itsNumberOfPolarizations( 0 ),
    itsNumberOfChannels( 0 ),
    itsNumberOfIFs( 0 ),
    itsReferenceFrequency( 0.0 ),
    itsTotalBandwidth( 0.0 ),
    itsChannelincrement( 0.0 ),
    myPieceNr( thispiece ),
    nrPieces( maxpiece ),
    myFirstStokes( 0 )
{
    this->attachMS( msref );
}


//
//  The name of the MS
//
const String& MSConversionInfo::getMSname( void ) const
{
    return itsMSname;
}

ROMSColumns& MSConversionInfo::getROColumns( void ) const
{
    return *itsROColumnsptr;
}

//
//  Ok. Return a ref. to the MS contained in this object!
//
const MeasurementSet& MSConversionInfo::getMS( void ) const
{
    return *itsMSptr;
}

//
//  Read-only access to cached derived quantities
//
Int MSConversionInfo::getFirstStokes( void ) const
{
    return myFirstStokes;
}


uInt MSConversionInfo::nrPolarizations( void ) const
{
    return itsNumberOfPolarizations;
}

uInt MSConversionInfo::nrChannels( void ) const
{
    return itsNumberOfChannels;
}

uInt MSConversionInfo::nrIFs( void ) const
{
    return itsNumberOfIFs;
}


const MVTime& MSConversionInfo::observingDate( void ) const
{
    return itsObservingDate;
}

const String& MSConversionInfo::getObsCode( void ) const
{
    return myObsCode;
}



Double MSConversionInfo::getReferenceFrequency( MSConversionInfo::reference r ) const
{
	Double   retval = itsReferenceFrequency;

	//
	// Internally, the reference frequency is the band-edge, so if
	// user wants the band-center, give it to him/her
	//
	switch( r )
	{
		case MSConversionInfo::CenterChannel:
			retval += this->getChannelwidth()/2.0;
			break;
			
		case MSConversionInfo::CenterBand:
			retval += itsTotalBandwidth/2.0;
			break;

		case MSConversionInfo::Edge:
		default:
			break;
	}
    return retval;
}

Double MSConversionInfo::getTotalBandwidth( void ) const
{
    return itsTotalBandwidth;
}

Double MSConversionInfo::getChannelwidth( void ) const
{
    Double  retval( 0.0 );
    
    if( itsNumberOfChannels )
    {
	retval = itsTotalBandwidth/itsNumberOfChannels;
    }
    
    return retval;
}


Double MSConversionInfo::getChannelincrement( void ) const
{
    return itsChannelincrement;
}


const String& MSConversionInfo::getDataColumnName( void ) const
{
    return itsDataColumnName;
}

uInt MSConversionInfo::itsPieceNr( void ) const
{
    return myPieceNr;
}

uInt MSConversionInfo::maxPieceNr( void ) const
{
    return nrPieces;
}



//
//  Delete the object...
//
MSConversionInfo::~MSConversionInfo()
{
    delete itsROColumnsptr;
    delete itsMSptr;
}


//
//  The private section: attemp to attach a MS. Return False if
//  something wrong (including: we were already attached...) I know
//  it's a lousy construction but since it's hidden from the outside
//  world ... :-)
//
Bool MSConversionInfo::attachMS( MS& msref )
{
    if( itsMSptr )
    {
	return False;
    }

    
    itsMSptr = new MS( msref );

    
    itsMSname = itsMSptr->tableName();
	
    cout << endl << "********************************************************************************" 
         << endl 
         << endl << Time() << ": Inspection of `" << itsMSname << "' starts" << endl;


    if( !itsMSptr->nrow() )
    {
        throw( AipsError("Empty MS. Nothing to do!") );
    }

    //
    // If we end up here, the MS is opened. Now we can start
    // referencing the columns!
    //
    itsROColumnsptr=new ROMSColumns( *itsMSptr );
    
    //
    //  Attempt to fill in some of the more general info that is
    //  expected to be accessed more than once....
    //
    myObsCode = "tConvert (c)";

    //
    //  Find out the spectralwindow table, see what's in there. If
    //  there are no entries, fall backt to the raw data!
    //
    uInt   nrrows;
    
    if( (nrrows=itsMSptr->spectralWindow().nrow())!=0 )
    {
        MSSpectralWindow       spectralTable( itsMSptr->spectralWindow() );

	//
	// Ok. Since the purpose is to convert to UVFITS, we assume
	// that every entry in the SP_WINDOW_TABLE is an IF (in
	// classic AIPS terminology).
	//
	itsNumberOfIFs=nrrows;
        
        //
        //  Create some column objects
        //
        ROScalarColumn<Int>    numcorr( spectralTable, 
                                        MSSpectralWindow::columnName(MSSpectralWindow::NUM_CORR));
        ROScalarColumn<Int>    numchan(spectralTable, 
                                       MSSpectralWindow::columnName(MSSpectralWindow::NUM_CHAN));
        ROArrayColumn<Double>  frequencies(spectralTable,
                                           MSSpectralWindow::columnName(MSSpectralWindow::CHAN_FREQ));
        ROArrayColumn<Int>     stokesTypes(spectralTable,
                                           MSSpectralWindow::columnName(MSSpectralWindow::CORR_TYPE));
        ROScalarColumn<Double> totalbw(spectralTable,
                                       MSSpectralWindow::columnName(MSSpectralWindow::TOTAL_BANDWIDTH));
        
        //
        // Ok. Scan the table!
        //
        Int    numstokes;
        Int    numchannels;
        uInt   i;
	Bool   deltafound( False );
	Bool   bandwidthfound( False );
        //Double delta( 0.0 );
	Double delta0( 0.0 );
        Double f0( 0.0 );
	Double bandwidth;


        //
        //  Initialize with the entries in the first row of the table!
        //
        if( frequencies.isDefined(0) )
        {
	    Vector<Double>  row0;
	    
	    row0 = frequencies( 0 );
	    
	    f0 = row0( 0 );
	    
	    if( row0.nelements()>=2 )
	    {
		deltafound = True;
		delta0 = row0(1) - row0(0);
	    }
            //f0=frequencies(0)(IPosition(1,0));
        }
                   
        numstokes=numcorr( 0 );
        numchannels=numchan( 0 );

	if( totalbw.isDefined(0) )
	{
	    bandwidthfound = True;
	    bandwidth = totalbw(0);
	}
	    

        //
        //  Check all other rows
        //
        for( i=0; i<nrrows; i++ )
        {
            if( numcorr(i)!=numstokes )
            {
                cout << "Number of correlations varies in the MS" << endl;
                break;
            }
            if( numchan(i)!=numchannels )
            {
                cout << "Number of channels varies in the MS" << endl;
                break;
            }
            
            //
            // Check if the frequency channels can be expressed as f0+i*delta
            //
	    uInt             j;
            Vector<Double>   freqs;
            
            if( frequencies.isDefined(i) )
            {
                freqs = frequencies(i);
            }
            
            //if( freqs.nelements()>=2 )
            //{
            //    delta = freqs(1) - freqs(0);
	    //	deltafound=True;
            //}
            //for( uInt j=2; j<freqs.nelements(); j++ )
            for( j=1; j<freqs.nelements(); j++ )
            {
                //if( !near(delta, freqs(j)-freqs(j-1)) )
                if( !near(delta0, freqs(j)-freqs(j-1)) )
                {
                    cout << "Channel spacing varies across the band" << endl;
                    break;
                }
            }
	    
	    if( j<freqs.nelements() )
	    {
		// Something must have gone wrong
		break;
	    }
	    
	    //
	    // Check totalbandwith
	    //
	    if( totalbw.isDefined(i) )
	    {
		if( !near(totalbw(i), bandwidth) )
		{
		    cout << "Bandwidth varies across spectral windows" << endl;
		    break;
		}
	    }
        }
        
        //
        //  If the loop was prematurely ended, something went wrong.
        //
        if( i<nrrows )
        {
            throw( AipsError("Impossible to continue.") );
        }
        
        if( !deltafound )
        {
            delta0=totalbw(0);
	    if( numchannels )
	    {
		delta0/=numchannels;
		cout << "Deriving bandwidth from totalbandwitdth/nrchannels" << endl;
	    }
        }
        
        //
        //  Check the polarizations
        //
        Vector<Int>  stokes(numstokes);
        
        if( stokesTypes.isDefined(0) )
        {
            stokes=stokesTypes(0);
        }
        
        //
        // Find out what the Stokes are and make sure that they are the same
        // throughout the MS. In principle we could handle the same stokes in
        // different order by transposing, but this may well never happen.
        //
        for( i=0; i<nrrows; i++ )
        {
            if( stokesTypes.isDefined(i) && !allEQ(stokes.ac(), stokesTypes(i).ac()) )
            {
                cout << "Stokes types vary for different spectral windows" << endl;
                break;
            }
        }

	//
	//  Sort the stokes stuff and make sure they can be represented in FITS?!
	//
	//
	// OK, turn these into FITS values.
	//
	for( uInt i=0; i<numstokes; i++)
	{
	    stokes(i)=Stokes::FITSValue( Stokes::StokesTypes(stokes(i)) );
	}

	Vector<uInt>           stokesindex( numstokes );
	//
	//  OK, get an index vector that sorts these in ascending order if
	//  stokes(0) >= 0, or descending order if < 0.
	//
	if( stokes(0)>=0 )
	{
	    GenSortIndirect<Int>::sort( stokesindex, stokes );
	}
	else
	{
	    GenSortIndirect<Int>::sort( stokesindex, stokes, Sort::Descending );
	}
	
	//
	// Now we need to make sure we can represent the stokes in FITS
	//
	if( stokes.nelements()>2 )
	{
	    Int  initialdiff( stokes(stokesindex(1))-stokes(stokesindex(0)) );
	    
	    for( uInt j=2; j<stokes.nelements(); j++)
	    {
		if( stokes(stokesindex(j))-stokes(stokesindex(j-1))!=initialdiff )
		{
		    throw( AipsError("The STOKES parameters are not representable in FITS.") );
		}
	    }
	}
	//
	//  Save the first stokes value!
	//
	myFirstStokes = stokes( stokesindex(0) );

        //
        // Throw up if something went wrong
        //
        if( i<nrrows )
        {
            throw( AipsError("Impossible to continue.") );
        }

        //
        //  Fill in the private datamembers
        //
        itsNumberOfPolarizations=numstokes;
        itsNumberOfChannels=numchannels;
        itsReferenceFrequency = (f0-delta0/2);
        //itsTotalBandwidth=delta;
        itsTotalBandwidth=bandwidth;
	itsChannelincrement=delta0;
    }
    else
    {
        throw( AipsError("Empty SPECTRAL_WINDOW subtable. Impossible to continue.") );
    }


    //
    //  Perform sanity checking/defaulting for key-values
    //
    if( !itsNumberOfPolarizations || !itsNumberOfChannels || !itsNumberOfIFs )
    {
        //
        //  These cannot be defaulted: throw up!
        //
        throw( AipsError("No polarizations and/or spectral channels and/or IFs: data matrices are empty.") );
    }
    
    if( !itsReferenceFrequency )
    {
        //
        //  This one could be defaulted, though it may not be
        //  correct. Tell usr.
        //
        Float    deffreq( 1.0 );
        
        cout << "No reference frequency found. Using default of " << deffreq << "Hz" << endl;
        itsReferenceFrequency=deffreq;
    }

    if( !itsTotalBandwidth )
    {
        //
        //  This one could be defaulted, though it may not be
        //  correct. Tell usr.
        //
        Float    defbw( 1.0 );
        
        cout << "No bandwidth found. Using default of " << defbw << "Hz" << endl;
        itsTotalBandwidth=defbw;
    }
    

    //
    //  Use shapes of the raw data!
    //
    
    //
    //  Ok. The odds are that the DATA column is the column
    //  containing the data. We may as well find 'FLOAT_DATA'
    //  and/or 'LAG_DATA', but mainly it will be 'DATA'.
    //
    Bool   hasdata( itsMSptr->isColumn(MS::DATA) );
    Bool   hasfloatdata( itsMSptr->isColumn(MS::FLOAT_DATA) );
    //
    //  When MSv2.0 comes into action this can be executed
    //
#ifdef HV_MSv2_DEFINED
    Bool   haslagdata( itsMSptr->isColumn(MS::LAG_DATA) );
#else
    Bool   haslagdata( False );
#endif
    
    //
    //  Introduce some help-booleans
    //
    Bool   apparentSDdata( ToBool( ToBool(hasdata &&  hasfloatdata && !haslagdata) || 
                           ToBool(hasdata && !hasfloatdata && !haslagdata) ||
                           ToBool(!hasdata &&  hasfloatdata && !haslagdata) ) );
    Bool   apparentIFdata( ToBool(hasdata && !hasfloatdata && !haslagdata) );
    Bool   apparentRawdata( ToBool(!hasdata && !hasfloatdata && haslagdata) );
    
    
    if( apparentIFdata )
    {
        cout << "MS appears to contain interferometer data. Using cmplx 'DATA' column" << endl;
        itsDataColumnName="DATA";
    }
    else if( apparentSDdata )
    {
        if( !hasdata )
        {
            cout << "MS appears to contain SingleDish data. Using float 'FLOAT_DATA' column." << endl;
            itsDataColumnName="FLOAT_DATA";
        }
        else
        {
            cout << "MS appears to contain SingleDish data. Using cmplx 'DATA' column." << endl;
            itsDataColumnName="DATA";
        }
    }
    else if( apparentRawdata )
    {
        cout << "MS appears to contain raw correlator data. Using cmplx 'LAG_DATA' column" << endl;
        itsDataColumnName="LAG_DATA";
    }
    else
    {
        throw( AipsError("Sorry, the converter could not determine the type of data this MS represents.") );
    }


    //
    //  Do not forget to do a sanity check on the values found!
    //
    if( !itsNumberOfPolarizations || !itsNumberOfChannels )
    {
        throw( AipsError("The converter found no channels and/or no polarizations.\n"
                         "Impossible to continue conversion.") );
    }
    else
    {
        cout << "ConversionInfo deduced " << itsNumberOfChannels
             << " channels x " << itsNumberOfPolarizations
             << " polarizations." << endl;
	cout << "                       ref. freq: " << itsReferenceFrequency << endl
	     << "                       bandwidth: " << itsTotalBandwidth << endl
	     << "                       ch. inc. : " << itsChannelincrement << endl;
	
    }

    //
    //  We need to find out the observing date!
    //
    if( itsROColumnsptr->time().isDefined(0) )
    {
        Double   mjdinseconds( itsROColumnsptr->time()(0) );
        
        //
        //  MVTime assumes units of MJD to be in days, from MS we get
        //  seconds....
        //
        itsObservingDate=MVTime( mjdinseconds/C::day );
    }
    else
    {
        cout <<"Could not deduce Observing date; first cell in TIME column not defined!"
             << endl << "Using today as default." << endl;

        itsObservingDate=MVTime( Time() );
    }
    
    cout << "Deduced observing date: " << itsObservingDate << endl;

    //
    // Show user... 
    //
    cout << endl << Time() << ": Inspection of `" << itsMSname << "' ends"
         << endl << "********************************************************************************" 
         << endl ;    

    return True;
}
