/********************************************************************************

  class:     
             templated ConverterBase 
             (templated on input-type and output-type)

             NOTE:

             When doing the template instantation yourself, be sure to
             instantiate the following templates to ensure smooth
             operating:

             template class ConverterBase<[your output-type],[your input-type]>;


  Purpose:

  A generic base-class for conversions to/from specific
  data-types. This is to be interpreted in the wider sende of the
  word, i.e. the only public function of the converter will take two
  string arguments (on the whole, two filenames, one for the source
  file and one for the destination file) and the caller of the method
  expects that the translated form of the inputfile is written into
  the outputfile. The base class implements a VERY basic conversion:
  the copy conversion: it copies the contents of infile to outfile....


  Use:

  If you want to have a specific conversion; derive a class from this
  baseclass. Implement a member function that takes two strings and
  returns a boolean. Do all the work you need in there.

  Afterwards, just call this public method from within the application
  and tada: off you go!



  $Id: ConverterBase.h,v 1.1.1.1 2000/03/20 15:15:58 verkout Exp $


  Author: Harro Verkouter (9-3-1998)

 ********************************************************************************/
#ifndef CONVERTERBASE_H_INC
#define CONVERTERBASE_H_INC

//
//  Includes for aips++ datatypes
//
#include <aips/aips.h>

template <class tOuttype, class tIntype>
class ConverterBase
{
public:
    //
    //  default c'tor -> nothing more needed...
    //
    ConverterBase();

    //
    //  The only public method: do the conversion. We follow the
    //  convention of specifying the destination first, then the
    //  source (ref. memcpy, strcpy etc. )
    //
    virtual Bool convert( tOuttype&, tIntype& ) const;
    virtual Bool convert( const tOuttype&, const tIntype& ) const;    

    //
    //  The virtual desctructor
    //
    virtual ~ConverterBase();

private:
    //
    //  No private datamembers (yet)
    //

    //
    //  The private method section
    //

    //
    //  Prevent copying and assignment
    //
    ConverterBase( const ConverterBase<tOuttype, tIntype>& );
    ConverterBase<tOuttype,tIntype>& operator=( const ConverterBase<tOuttype,tIntype>& );
};


#endif

