//
//  class:
//
//         LinearContainer <>
//
//  purpose:
//
//         a templated lightweigth, slightly optimized storage container which
//         can be indexed linearly
//
//         NOTE:
//
//         When doing the template instantiations yourself, be sure to
//         instantiate the following templates to ensure smoot
//         operation:
//
//         template class LinearContainer<[your type]>;
//         template ostream& operator<<( ostream&, const LinearContainer<[your type]>& );
//
//
//
//  Author:
//
//         Harro Verkouter
//
//
//      $Id: LinearContainer.h,v 1.1.1.1 2000/03/20 15:15:58 verkout Exp $
//
#ifndef LINEARCONTAINER_H_INC
#define LINEARCONTAINER_H_INC


#ifndef AIPS_AIPS_H
#include <aips/aips.h>
#endif

//
//  Forward declarations
//
class ostream;

//
// The obvious templated container-class: a lightweight one!
//
template <class T>
class LinearContainer
{

public:
    
    //
    //  Default c'tor: a container of zero length
    //
    LinearContainer();

    //
    //  Provide storage for 'initiallength' elements of type T and set
    //  them initially to initialval
    //
    LinearContainer( uInt initiallength, const T& initialval=T() );
    
    //
    //  The copy c'tor
    //
    LinearContainer( const LinearContainer<T>& other );

    //
    //  Resize the vector, user has control over whether or not as
    //  much elements as possibleare to be copied. It is a defaulted
    //  parameter, by default as much elements as possible are copied.
    //
    //  NOTE: this parameter is only used when enlarging the
    //  container!
    //
    void resize( uInt newlength, const T& initval=T() );

    //
    //  Add a COPY of the argument to the container!
    //
    void add( const T& newval );
    

    //
    //  Gain access to the members.
    //   The [] forms performs boundschecking
    //   The () forms does not perform boundschecking!
    //
    T& operator[]( uInt idx );
    T& operator()( uInt idx );
    
    const T& operator[]( uInt idx ) const;
    const T& operator()( uInt idx ) const;
    
    //
    //  Use at own risk: get direct access to the storage
    //
    T*       getstorage( void );
    const T* getstorage( void ) const;

    //
    //  Some general interface functions
    //
    uInt   elements( void ) const;
    
    //
    //  Assignment
    //
    LinearContainer<T>& operator=( const LinearContainer<T>& rval );

    //
    //  The destructor... for all object stored in the container, the
    //  desctructor is called!
    //
    ~LinearContainer();
    
private:
    //
    //  Private members are the array of elements and the num,ber of
    //  elements
    //
    T*     storage;
    
    uInt   nrelements;
    uInt   nrallocated;

    //
    //  Private methods...
    //
    void   copyElements( T* dst, const T* src, uInt nr2copy );
    void   setElements( T* dst, const T& val2set, uInt nr2set );

    T*     allocateStorage( const uInt nr2allocate, uInt& reallyallocated );
};

//
// Nice to have a function that dumps a container on a stream
//
template <class T>
ostream& operator<<( ostream& os, const LinearContainer<T>& vref ); 



#ifdef LINEARCONTAINER_CC

#endif


#endif
